using System;

using Server.Network;

namespace Server
{
	public class Effects
	{
		public Effects() : base()
		{
		}

		public static void PlaySound(IPoint3D p, Map map, int soundID)
		{
			Packet packet1;
			IPooledEnumerable enumerable1;
			NetState state1;
			IEnumerator enumerator1;
			IDisposable disposable1;
			if (soundID == -1)
			{
				return; 
			}
			if (map != null)
			{
				packet1 = null;
				enumerable1 = map.GetClientsInRange(new Point3D(p));
				enumerator1 = enumerable1.GetEnumerator();
				try
				{
					while (enumerator1.MoveNext())
					{
						state1 = ((NetState) enumerator1.Current);
						state1.Mobile.ProcessDelta();
						if (packet1 == null)
						{
							packet1 = new PlaySound(soundID, p);
 
						}
						state1.Send(packet1);
 
					}
 
				}
				finally
				{
					disposable1 = (enumerator1 as IDisposable);
					if (disposable1 != null)
					{
						disposable1.Dispose();
 
					}
 
				}
				enumerable1.Free();
 
			}
 
		}
		
		public static void SendBoltEffect(IEntity e)
		{
			Effects.SendBoltEffect(e, true, 0);
 
		} 

		public static void SendBoltEffect(IEntity e, bool sound)
		{
			Effects.SendBoltEffect(e, sound, 0);
 
		} 

		public static void SendBoltEffect(IEntity e, bool sound, int hue)
		{
			Map map1;
			Packet packet1;
			Packet packet2;
			Packet packet3;
			IPooledEnumerable enumerable1;
			NetState state1;
			IEnumerator enumerator1;
			IDisposable disposable1;
			map1 = e.Map;
			if (map1 == null)
			{
				return; 
			}
			if ((e as Item) != null)
			{
				((Item) e).ProcessDelta();
 
			}
			else
			{
				if ((e as Mobile) != null)
				{
					((Mobile) e).ProcessDelta();
 
				}
 
			}
			packet1 = null;
			packet2 = null;
			packet3 = null;
			enumerable1 = map1.GetClientsInRange(e.Location);
			enumerator1 = enumerable1.GetEnumerator();
			try
			{
				while (enumerator1.MoveNext())
				{
					state1 = ((NetState) enumerator1.Current);
					if (state1.Mobile.CanSee(e))
					{
						if (Effects.SendParticlesTo(state1))
						{
							if (packet1 == null)
							{
								packet1 = new TargetParticleEffect(e, 0, 10, 5, 0, 0, 5031, 3, 0);
 
							}
							state1.Send(packet1);
 
						}
						if (packet2 == null)
						{
							packet2 = new BoltEffect(e, hue);
 
						}
						state1.Send(packet2);
						if (sound)
						{
							if (packet3 == null)
							{
								packet3 = new PlaySound(41, e);
 
							}
							state1.Send(packet3);
 
						}
 
					}
 
				}
 
			}
			finally
			{
				disposable1 = (enumerator1 as IDisposable);
				if (disposable1 != null)
				{
					disposable1.Dispose();
 
				}
 
			}
			enumerable1.Free();
 
		}
		
		public static void SendLocationEffect(IPoint3D p, Map map, int itemID, int duration)
		{
			Effects.SendLocationEffect(p, map, itemID, duration, 10, 0, 0);
 
		} 

		public static void SendLocationEffect(IPoint3D p, Map map, int itemID, int duration, int speed)
		{
			Effects.SendLocationEffect(p, map, itemID, duration, speed, 0, 0);
 
		} 

		public static void SendLocationEffect(IPoint3D p, Map map, int itemID, int duration, int hue, int renderMode)
		{
			Effects.SendLocationEffect(p, map, itemID, duration, 10, hue, renderMode);
 
		} 

		public static void SendLocationEffect(IPoint3D p, Map map, int itemID, int duration, int speed, int hue, int renderMode)
		{
			Effects.SendPacket(p, map, new LocationEffect(p, itemID, speed, duration, hue, renderMode));
 
		} 

		public static void SendLocationParticles(IEntity e, int itemID, int speed, int duration, int effect)
		{
			Effects.SendLocationParticles(e, itemID, speed, duration, 0, 0, effect, 0);
 
		} 

		public static void SendLocationParticles(IEntity e, int itemID, int speed, int duration, int effect, int unknown)
		{
			Effects.SendLocationParticles(e, itemID, speed, duration, 0, 0, effect, unknown);
 
		} 

		public static void SendLocationParticles(IEntity e, int itemID, int speed, int duration, int hue, int renderMode, int effect, int unknown)
		{
			Map map1;
			Packet packet1;
			Packet packet2;
			IPooledEnumerable enumerable1;
			NetState state1;
			IEnumerator enumerator1;
			IDisposable disposable1;
			map1 = e.Map;
			if (map1 != null)
			{
				packet1 = null;
				packet2 = null;
				enumerable1 = map1.GetClientsInRange(e.Location);
				enumerator1 = enumerable1.GetEnumerator();
				try
				{
					while (enumerator1.MoveNext())
					{
						state1 = ((NetState) enumerator1.Current);
						state1.Mobile.ProcessDelta();
						if (Effects.SendParticlesTo(state1))
						{
							if (packet1 == null)
							{
								packet1 = new LocationParticleEffect(e, itemID, speed, duration, hue, renderMode, effect, unknown);
 
							}
							state1.Send(packet1);
 
						}
						else
						{
							if (itemID != 0)
							{
								if (packet2 == null)
								{
									packet2 = new LocationEffect(e, itemID, speed, duration, hue, renderMode);
 
								}
								state1.Send(packet2);
 
							}
 
						}
 
					}
 
				}
				finally
				{
					disposable1 = (enumerator1 as IDisposable);
					if (disposable1 != null)
					{
						disposable1.Dispose();
 
					}
 
				}
				enumerable1.Free();
 
			}
 
		}
		
		public static void SendMovingEffect(IEntity from, IEntity to, int itemID, int speed, int duration, bool fixedDirection, bool explodes)
		{
			Effects.SendMovingEffect(from, to, itemID, speed, duration, fixedDirection, explodes, 0, 0);
 
		} 

		public static void SendMovingEffect(IEntity from, IEntity to, int itemID, int speed, int duration, bool fixedDirection, bool explodes, int hue, int renderMode)
		{
			if ((from as Mobile) != null)
			{
				((Mobile) from).ProcessDelta();
 
			}
			if ((to as Mobile) != null)
			{
				((Mobile) to).ProcessDelta();
 
			}
			Effects.SendPacket(from.Location, from.Map, new MovingEffect(from, to, itemID, speed, duration, fixedDirection, explodes, hue, renderMode));
 
		} 

		public static void SendMovingParticles(IEntity from, IEntity to, int itemID, int speed, int duration, bool fixedDirection, bool explodes, int effect, int explodeEffect, int explodeSound)
		{
			Effects.SendMovingParticles(from, to, itemID, speed, duration, fixedDirection, explodes, 0, 0, effect, explodeEffect, explodeSound, 0);
 
		} 

		public static void SendMovingParticles(IEntity from, IEntity to, int itemID, int speed, int duration, bool fixedDirection, bool explodes, int effect, int explodeEffect, int explodeSound, int unknown)
		{
			Effects.SendMovingParticles(from, to, itemID, speed, duration, fixedDirection, explodes, 0, 0, effect, explodeEffect, explodeSound, unknown);
 
		} 

		public static void SendMovingParticles(IEntity from, IEntity to, int itemID, int speed, int duration, bool fixedDirection, bool explodes, int hue, int renderMode, int effect, int explodeEffect, int explodeSound, EffectLayer layer, int unknown)
		{
			Map map1;
			Packet packet1;
			Packet packet2;
			IPooledEnumerable enumerable1;
			NetState state1;
			IEnumerator enumerator1;
			IDisposable disposable1;
			if ((from as Mobile) != null)
			{
				((Mobile) from).ProcessDelta();
 
			}
			if ((to as Mobile) != null)
			{
				((Mobile) to).ProcessDelta();
 
			}
			map1 = from.Map;
			if (map1 != null)
			{
				packet1 = null;
				packet2 = null;
				enumerable1 = map1.GetClientsInRange(from.Location);
				enumerator1 = enumerable1.GetEnumerator();
				try
				{
					while (enumerator1.MoveNext())
					{
						state1 = ((NetState) enumerator1.Current);
						state1.Mobile.ProcessDelta();
						if (Effects.SendParticlesTo(state1))
						{
							if (packet1 == null)
							{
								packet1 = new MovingParticleEffect(from, to, itemID, speed, duration, fixedDirection, explodes, hue, renderMode, effect, explodeEffect, explodeSound, layer, unknown);
 
							}
							state1.Send(packet1);
 
						}
						else
						{
							if (itemID > 1)
							{
								if (packet2 == null)
								{
									packet2 = new MovingEffect(from, to, itemID, speed, duration, fixedDirection, explodes, hue, renderMode);
 
								}
								state1.Send(packet2);
 
							}
 
						}
 
					}
 
				}
				finally
				{
					disposable1 = (enumerator1 as IDisposable);
					if (disposable1 != null)
					{
						disposable1.Dispose();
 
					}
 
				}
				enumerable1.Free();
 
			}
 
		}
		
		public static void SendMovingParticles(IEntity from, IEntity to, int itemID, int speed, int duration, bool fixedDirection, bool explodes, int hue, int renderMode, int effect, int explodeEffect, int explodeSound, int unknown)
		{
			Effects.SendMovingParticles(from, to, itemID, speed, duration, fixedDirection, explodes, hue, renderMode, effect, explodeEffect, explodeSound, 255, unknown);
 
		} 

		public static void SendPacket(IPoint3D origin, Map map, Packet p)
		{
			IPooledEnumerable enumerable1;
			NetState state1;
			IEnumerator enumerator1;
			IDisposable disposable1;
			if (map != null)
			{
				enumerable1 = map.GetClientsInRange(new Point3D(origin));
				enumerator1 = enumerable1.GetEnumerator();
				try
				{
					while (enumerator1.MoveNext())
					{
						state1 = ((NetState) enumerator1.Current);
						state1.Mobile.ProcessDelta();
						state1.Send(p);
 
					}
 
				}
				finally
				{
					disposable1 = (enumerator1 as IDisposable);
					if (disposable1 != null)
					{
						disposable1.Dispose();
 
					}
 
				}
				enumerable1.Free();
 
			}
 
		} 

		public static void SendPacket(Point3D origin, Map map, Packet p)
		{
			IPooledEnumerable enumerable1;
			NetState state1;
			IEnumerator enumerator1;
			IDisposable disposable1;
			if (map != null)
			{
				enumerable1 = map.GetClientsInRange(origin);
				enumerator1 = enumerable1.GetEnumerator();
				try
				{
					while (enumerator1.MoveNext())
					{
						state1 = ((NetState) enumerator1.Current);
						state1.Mobile.ProcessDelta();
						state1.Send(p);
 
					}
 
				}
				finally
				{
					disposable1 = (enumerator1 as IDisposable);
					if (disposable1 != null)
					{
						disposable1.Dispose();
 
					}
 
				}
				enumerable1.Free();
 
			}
 
		} 

		public static bool SendParticlesTo(NetState state)
		{
			if (Effects.m_ParticleSupportType != 0)
			{
				if ((Effects.m_ParticleSupportType == 1) && (state.Version != null))
				{
					return (state.Version.Type == 1); 
				}
				return false; 
			}
			return true; 
		} 

		public static void SendTargetEffect(IEntity target, int itemID, int duration)
		{
			Effects.SendTargetEffect(target, itemID, duration, 0, 0);
 
		} 

		public static void SendTargetEffect(IEntity target, int itemID, int speed, int duration)
		{
			Effects.SendTargetEffect(target, itemID, speed, duration, 0, 0);
 
		} 

		public static void SendTargetEffect(IEntity target, int itemID, int duration, int hue, int renderMode)
		{
			Effects.SendTargetEffect(target, itemID, 10, duration, hue, renderMode);
 
		} 

		public static void SendTargetEffect(IEntity target, int itemID, int speed, int duration, int hue, int renderMode)
		{
			if ((target as Mobile) != null)
			{
				((Mobile) target).ProcessDelta();
 
			}
			Effects.SendPacket(target.Location, target.Map, new TargetEffect(target, itemID, speed, duration, hue, renderMode));
 
		} 

		public static void SendTargetParticles(IEntity target, int itemID, int speed, int duration, int effect, EffectLayer layer)
		{
			Effects.SendTargetParticles(target, itemID, speed, duration, 0, 0, effect, layer, 0);
 
		} 

		public static void SendTargetParticles(IEntity target, int itemID, int speed, int duration, int effect, EffectLayer layer, int unknown)
		{
			Effects.SendTargetParticles(target, itemID, speed, duration, 0, 0, effect, layer, unknown);
 
		} 

		public static void SendTargetParticles(IEntity target, int itemID, int speed, int duration, int hue, int renderMode, int effect, EffectLayer layer, int unknown)
		{
			Map map1;
			Packet packet1;
			Packet packet2;
			IPooledEnumerable enumerable1;
			NetState state1;
			IEnumerator enumerator1;
			IDisposable disposable1;
			if ((target as Mobile) != null)
			{
				((Mobile) target).ProcessDelta();
 
			}
			map1 = target.Map;
			if (map1 != null)
			{
				packet1 = null;
				packet2 = null;
				enumerable1 = map1.GetClientsInRange(target.Location);
				enumerator1 = enumerable1.GetEnumerator();
				try
				{
					while (enumerator1.MoveNext())
					{
						state1 = ((NetState) enumerator1.Current);
						state1.Mobile.ProcessDelta();
						if (Effects.SendParticlesTo(state1))
						{
							if (packet1 == null)
							{
								packet1 = new TargetParticleEffect(target, itemID, speed, duration, hue, renderMode, effect, layer, unknown);
 
							}
							state1.Send(packet1);
 
						}
						else
						{
							if (itemID != 0)
							{
								if (packet2 == null)
								{
									packet2 = new TargetEffect(target, itemID, speed, duration, hue, renderMode);
 
								}
								state1.Send(packet2);
 
							}
 
						}
 
					}
 
				}
				finally
				{
					disposable1 = (enumerator1 as IDisposable);
					if (disposable1 != null)
					{
						disposable1.Dispose();
 
					}
 
				}
				enumerable1.Free();
 
			}
 
		}

		public static ParticleSupportType ParticleSupportType 
		{ 
			get
			{
				return Effects.m_ParticleSupportType;
			}
			set
			{
				Effects.m_ParticleSupportType = value;
			}
		}
		
		private static ParticleSupportType m_ParticleSupportType = ParticleSupportType.Detect;
	}
}