using System;

using Server;

namespace Server.Gumps
{
	public class GumpAlphaRegion : GumpEntry
	{
		public GumpAlphaRegion(int x, int y, int width, int height) : base()
		{
			this.m_X = x;
			this.m_Y = y;
			this.m_Width = width;
			this.m_Height = height;
		} 

		public override void AppendTo(DisplayGumpFast disp)
		{
			disp.AppendLayout(GumpAlphaRegion.m_LayoutName);
			disp.AppendLayout(this.m_X);
			disp.AppendLayout(this.m_Y);
			disp.AppendLayout(this.m_Width);
			disp.AppendLayout(this.m_Height);
		} 

		public override string Compile()
		{
			object[] array1;
			array1 = new object[4];
			array1[0] = this.m_X;
			array1[1] = this.m_Y;
			array1[2] = this.m_Width;
			array1[3] = this.m_Height;
			return string.Format("{{ checkertrans {0} {1} {2} {3} }}", array1); 
		} 
		
		public int Height 
		{ 
			get
			{
				return this.m_Height;
			}
			set
			{
				base.Delta(ref this.m_Height, value);
			}
		}

		public int Width 
		{ 
			get
			{
				return this.m_Width;
			}
			set
			{
				//TRF base.Delta(&(this.m_Width), value);
				base.Delta(ref this.m_Width, value);
			}
		}

		public int X 
		{ 
			get
			{
				return this.m_X;
			}
			set
			{
				base.Delta(ref this.m_X, value);
			}
		}

		public int Y 
		{ 
			get
			{
				return this.m_Y;
			}
			set
			{
				base.Delta(ref this.m_Y, value);
			}
		}

		private int m_Height;
		private static byte[] m_LayoutName = Gump.StringToBuffer("checkertrans");
		private int m_Width;
		private int m_X;
		private int m_Y;
	}
 
	public class GumpBackground : GumpEntry
	{
		public GumpBackground(int x, int y, int width, int height, int gumpID) : base()
		{
			this.m_X = x;
			this.m_Y = y;
			this.m_Width = width;
			this.m_Height = height;
			this.m_GumpID = gumpID;
		} 

		public override void AppendTo(DisplayGumpFast disp)
		{
			disp.AppendLayout(GumpBackground.m_LayoutName);
			disp.AppendLayout(this.m_X);
			disp.AppendLayout(this.m_Y);
			disp.AppendLayout(this.m_GumpID);
			disp.AppendLayout(this.m_Width);
			disp.AppendLayout(this.m_Height);
		} 

		public override string Compile()
		{
			object[] array1;
			array1 = new object[5];
			array1[0] = this.m_X;
			array1[1] = this.m_Y;
			array1[2] = this.m_GumpID;
			array1[3] = this.m_Width;
			array1[4] = this.m_Height;
			return string.Format("{{ resizepic {0} {1} {2} {3} {4} }}", array1); 
		} 
		
		public int GumpID 
		{ 
			get
			{
				return this.m_GumpID;
			}
			set
			{
				base.Delta(ref this.m_GumpID, value);
			}
		}

		public int Height 
		{ 
			get
			{
				return this.m_Height;
			}
			set
			{
				base.Delta(ref this.m_Height, value);
			}
		}

		public int Width 
		{ 
			get
			{
				return this.m_Width;
			}
			set
			{
				base.Delta(ref this.m_Width, value);
			}
		}

		public int X 
		{ 
			get
			{
				return this.m_X;
			}
			set
			{
				base.Delta(ref this.m_X, value);
			}
		}

		public int Y 
		{ 
			get
			{
				return this.m_Y;
			}
			set
			{
				base.Delta(ref this.m_Y, value);
			}
		}
		
		private int m_GumpID;
		private int m_Height;
		private static byte[] m_LayoutName = Gump.StringToBuffer("resizepic");
		private int m_Width;
		private int m_X;
		private int m_Y;
	}
 
	public class GumpButton : GumpEntry
	{
		public GumpButton(int x, int y, int normalID, int pressedID, int buttonID, GumpButtonType type, int param) : base()
		{
			this.m_X = x;
			this.m_Y = y;
			this.m_ID1 = normalID;
			this.m_ID2 = pressedID;
			this.m_ButtonID = buttonID;
			this.m_Type = type;
			this.m_Param = param;
		} 

		public override void AppendTo(DisplayGumpFast disp)
		{
			disp.AppendLayout(GumpButton.m_LayoutName);
			disp.AppendLayout(this.m_X);
			disp.AppendLayout(this.m_Y);
			disp.AppendLayout(this.m_ID1);
			disp.AppendLayout(this.m_ID2);
			disp.AppendLayout((int)this.m_Type);
			disp.AppendLayout(this.m_Param);
			disp.AppendLayout(this.m_ButtonID);
		} 

		public override string Compile()
		{
			object[] array1;
			array1 = new object[7];
			array1[0] = this.m_X;
			array1[1] = this.m_Y;
			array1[2] = this.m_ID1;
			array1[3] = this.m_ID2;
			array1[4] = this.m_Type;
			array1[5] = this.m_Param;
			array1[6] = this.m_ButtonID;
			return string.Format("{{ button {0} {1} {2} {3} {4} {5} {6} }}", array1); 
		} 

		public int ButtonID 
		{ 
			get
			{
				return this.m_ButtonID;
			}
			set
			{
				base.Delta(ref this.m_ButtonID, value);
			}
		}

		public int NormalID 
		{ 
			get
			{
				return this.m_ID1;
			}
			set
			{
				base.Delta(ref this.m_ID1, value);
			}
		}

		public int Param 
		{ 
			get
			{
				return this.m_Param;
			}
			set
			{
				base.Delta(ref this.m_Param, value);
			}
		}

		public int PressedID 
		{ 
			get
			{
				return this.m_ID2;
			}
			set
			{
				base.Delta(ref this.m_ID2, value);
			}
		}

		public GumpButtonType Type 
		{ 
			get
			{
				return this.m_Type;
			}
			set
			{
				Gump gump1;
				if (this.m_Type != value)
				{
					this.m_Type = value;
					gump1 = base.Parent;
					if (gump1 != null)
					{
						gump1.Invalidate();
					}
				}
			}
		}

		public int X 
		{ 
			get
			{
				return this.m_X;
			}
			set
			{
				base.Delta(ref this.m_X, value);
			}
		}

		public int Y 
		{ 
			get
			{
				return this.m_Y;
			}
			set
			{
				base.Delta(ref this.m_Y, value);
			}
		}
		
		private int m_ButtonID;
		private int m_ID1;
		private int m_ID2;
		private static byte[] m_LayoutName = Gump.StringToBuffer("button");
		private int m_Param;
		private GumpButtonType m_Type;
		private int m_X;
		private int m_Y;
	}
 
	public enum GumpButtonType
	{
		Page = 0,
		Reply = 1,
	}
 
	public class GumpCheck : GumpEntry
	{
		public GumpCheck(int x, int y, int inactiveID, int activeID, bool initialState, int switchID) : base()
		{
			this.m_X = x;
			this.m_Y = y;
			this.m_ID1 = inactiveID;
			this.m_ID2 = activeID;
			this.m_InitialState = initialState;
			this.m_SwitchID = switchID;
		}
		
		public override void AppendTo(DisplayGumpFast disp)
		{
			disp.AppendLayout(GumpCheck.m_LayoutName);
			disp.AppendLayout(this.m_X);
			disp.AppendLayout(this.m_Y);
			disp.AppendLayout(this.m_ID1);
			disp.AppendLayout(this.m_ID2);
			disp.AppendLayout(this.m_InitialState);
			disp.AppendLayout(this.m_SwitchID);
		} 

		public override string Compile()
		{
			object[] array1;
			array1 = new object[6];
			array1[0] = this.m_X;
			array1[1] = this.m_Y;
			array1[2] = this.m_ID1;
			array1[3] = this.m_ID2;
			array1[4] = (this.m_InitialState ? 1 : 0);
			array1[5] = this.m_SwitchID;
			return string.Format("{{ checkbox {0} {1} {2} {3} {4} {5} }}", array1); 
		} 

		public int ActiveID 
		{ 
			get
			{
				return this.m_ID2;
			}
			set
			{
				base.Delta(ref this.m_ID2, value);
			}
		}

		public int InactiveID 
		{ 
			get
			{
				return this.m_ID1;
			}
			set
			{
				base.Delta(ref this.m_ID1, value);
			}
		}

		public bool InitialState 
		{ 
			get
			{
				return this.m_InitialState;
			}
			set
			{
				base.Delta(ref this.m_InitialState, value);
			}
		}

		public int SwitchID 
		{ 
			get
			{
				return this.m_SwitchID;
			}
			set
			{
				base.Delta(ref this.m_SwitchID, value);
			}
		}

		public int X 
		{ 
			get
			{
				return this.m_X;
			}
			set
			{
				base.Delta(ref this.m_X, value);
			}
		}

		public int Y 
		{ 
			get
			{
				return this.m_Y;
			}
			set
			{
				base.Delta(ref this.m_Y, value);
			}
		}
		
		private int m_ID1;
		private int m_ID2;
		private bool m_InitialState;
		private static byte[] m_LayoutName = Gump.StringToBuffer("checkbox");
		private int m_SwitchID;
		private int m_X;
		private int m_Y;
	}
 
	public abstract class GumpEntry
	{
		public GumpEntry() : base()
		{
		}

		public abstract void AppendTo(DisplayGumpFast disp);
		public abstract string Compile();
		
		protected void Delta(ref bool var, bool val)
		{
			if (var != val)
			{
				var = val;
				if (this.m_Parent != null)
				{
					this.m_Parent.Invalidate();
				}
			}
		} 

		protected void Delta(ref int var, int val)
		{
			if (var != val)
			{
				var = val;
				if (this.m_Parent != null)
				{
					this.m_Parent.Invalidate();
 
				}
 
			}
 
		} 

		protected void Delta(ref string var, string val)
		{
			if (var != val)
			{
				var = val;
				if (this.m_Parent != null)
				{
					this.m_Parent.Invalidate();
 
				}
 
			}
 
		} 

		public Gump Parent 
		{ 
			get
			{
				return this.m_Parent;
			}
			set
			{
				if (this.m_Parent != value)
				{
					if (this.m_Parent != null)
					{
						this.m_Parent.Remove(this);
					}
					this.m_Parent = value;
					this.m_Parent.Add(this);
				}
			}
		}

		private Gump m_Parent;
	}
 
	public class GumpGroup : GumpEntry
	{
		public GumpGroup(int group) : base()
		{
			this.m_Group = group;
		}

		public override void AppendTo(DisplayGumpFast disp)
		{
				disp.AppendLayout(GumpGroup.m_LayoutName);
			disp.AppendLayout(this.m_Group);
 
		} 

		public override string Compile()
		{
				return string.Format("{{ group {0} }}", this.m_Group); 
		} 

		public int Group { 
			get
			{
				return this.m_Group;
			}
			set
			{
				base.Delta(ref this.m_Group, value);
			}
		}
		
		private int m_Group;
		private static byte[] m_LayoutName = Gump.StringToBuffer("group");
	}
 
	public class GumpHtml : GumpEntry
	{
		public GumpHtml(int x, int y, int width, int height, string text, bool background, bool scrollbar) : base()
		{
			this.m_X = x;
			this.m_Y = y;
			this.m_Width = width;
			this.m_Height = height;
			this.m_Text = text;
			this.m_Background = background;
			this.m_Scrollbar = scrollbar;
		} 

		public override void AppendTo(DisplayGumpFast disp)
		{
			disp.AppendLayout(GumpHtml.m_LayoutName);
			disp.AppendLayout(this.m_X);
			disp.AppendLayout(this.m_Y);
			disp.AppendLayout(this.m_Width);
			disp.AppendLayout(this.m_Height);
			disp.AppendLayout(base.Parent.Intern(this.m_Text));
			disp.AppendLayout(this.m_Background);
			disp.AppendLayout(this.m_Scrollbar);
		} 

		public override string Compile()
		{
			object[] array1;
			array1 = new object[7];
			array1[0] = this.m_X;
			array1[1] = this.m_Y;
			array1[2] = this.m_Width;
			array1[3] = this.m_Height;
			array1[4] = base.Parent.Intern(this.m_Text);
			array1[5] = (this.m_Background ? 1 : 0);
			array1[6] = (this.m_Scrollbar ? 1 : 0);
			return string.Format("{{ htmlgump {0} {1} {2} {3} {4} {5} {6} }}", array1); 
		} 

		public bool Background { 
			get
			{
				return this.m_Background;
			}
			set
			{
				base.Delta(ref this.m_Background, value);
			}
		}

		public int Height 
		{ 
			get
			{
				return this.m_Height;
			}
			set
			{
				base.Delta(ref this.m_Height, value);
			}
		}

		public bool Scrollbar 
		{ 
			get
			{
				return this.m_Scrollbar;
			}
			set
			{
				base.Delta(ref this.m_Scrollbar, value);
			}
		}

		public string Text 
		{ 
			get
			{
				return this.m_Text;
			}
			set
			{
				base.Delta(ref this.m_Text, value);
			}
		}

		public int Width 
		{ 
			get
			{
				return this.m_Width;
			}
			set
			{
				base.Delta(ref this.m_Width, value);
			}
		}

		public int X 
		{ 
			get
			{
				return this.m_X;
			}
			set
			{
				base.Delta(ref this.m_X, value);
			}
		}

		public int Y 
		{ 
			get
			{
				return this.m_Y;
			}
			set
			{
				base.Delta(ref this.m_Y, value);
			}
		}

		private bool m_Background;
		private int m_Height;
		private static byte[] m_LayoutName = Gump.StringToBuffer("htmlgump");
		private bool m_Scrollbar;
		private string m_Text;
		private int m_Width;
		private int m_X;
		private int m_Y;
	}
 
	public class GumpHtmlLocalized : GumpEntry
	{
		public GumpHtmlLocalized(int x, int y, int width, int height, int number, bool background, bool scrollbar) : base()
		{
			this.m_X = x;
			this.m_Y = y;
			this.m_Width = width;
			this.m_Height = height;
			this.m_Number = number;
			this.m_Background = background;
			this.m_Scrollbar = scrollbar;
		} 

		public GumpHtmlLocalized(int x, int y, int width, int height, int number, int color, bool background, bool scrollbar) : base()
		{
			this.m_X = x;
			this.m_Y = y;
			this.m_Width = width;
			this.m_Height = height;
			this.m_Number = number;
			this.m_Background = background;
			this.m_Scrollbar = scrollbar;
			this.m_Color = color;
			this.m_UseColor = true;
 
		} 

		public override void AppendTo(DisplayGumpFast disp)
		{
			disp.AppendLayout((this.m_UseColor ? GumpHtmlLocalized.m_LayoutNameColor : GumpHtmlLocalized.m_LayoutName));
			disp.AppendLayout(this.m_X);
			disp.AppendLayout(this.m_Y);
			disp.AppendLayout(this.m_Width);
			disp.AppendLayout(this.m_Height);
			disp.AppendLayout(this.m_Number);
			disp.AppendLayout(this.m_Background);
			disp.AppendLayout(this.m_Scrollbar);
			if (this.m_UseColor)
			{
				disp.AppendLayout(this.m_Color);
			}
		} 

		public override string Compile()
		{
			object[] array1;
			if (this.m_UseColor)
			{
				array1 = new object[8];
				array1[0] = this.m_X;
				array1[1] = this.m_Y;
				array1[2] = this.m_Width;
				array1[3] = this.m_Height;
				array1[4] = this.m_Number;
				array1[5] = (this.m_Background ? 1 : 0);
				array1[6] = (this.m_Scrollbar ? 1 : 0);
				array1[7] = this.m_Color;
				return string.Format("{{ xmfhtmlgumpcolor {0} {1} {2} {3} {4} {5} {6} {7} }}", array1); 
			}
			array1 = new object[7];
			array1[0] = this.m_X;
			array1[1] = this.m_Y;
			array1[2] = this.m_Width;
			array1[3] = this.m_Height;
			array1[4] = this.m_Number;
			array1[5] = (this.m_Background ? 1 : 0);
			array1[6] = (this.m_Scrollbar ? 1 : 0);
			return string.Format("{{ xmfhtmlgump {0} {1} {2} {3} {4} {5} {6} }}", array1); 
		} 
		
		public bool Background 
		{ 
			get
			{
				return this.m_Background;
			}
			set
			{
				base.Delta(ref this.m_Background, value);
			}
		}

		public int Color 
		{ 
			get
			{
				return this.m_Color;
			}
			set
			{
				base.Delta(ref this.m_Color, value);
			}
		}

		public int Height 
		{ 
			get
			{
				return this.m_Height;
			}
			set
			{
				base.Delta(ref this.m_Height, value);
			}
		}

		public int Number 
		{ 
			get
			{
				return this.m_Number;
			}
			set
			{
				base.Delta(ref this.m_Number, value);
			}
		}

		public bool Scrollbar 
		{ 
			get
			{
				return this.m_Scrollbar;
			}
			set
			{
				base.Delta(ref this.m_Scrollbar, value);
			}
		}

		public bool UseColor 
		{ 
			get
			{
				return this.m_UseColor;
			}
			set
			{
				base.Delta(ref this.m_UseColor, value);
			}
		}

		public int Width 
		{ 
			get
			{
				return this.m_Width;
			}
			set
			{
				base.Delta(ref this.m_Width, value);
			}
		}

		public int X 
		{ 
			get
			{
				return this.m_X;
			}
			set
			{
				base.Delta(ref this.m_X, value);
			}
		}

		public int Y 
		{ 
			get
			{
				return this.m_Y;
			}
			set
			{
				base.Delta(ref this.m_Y, value);
			}
		}

		private bool m_Background;
		private int m_Color;
		private int m_Height;
		private static byte[] m_LayoutName = Gump.StringToBuffer("xmfhtmlgump");
		private static byte[] m_LayoutNameColor = Gump.StringToBuffer("xmfhtmlgumpcolor");
		private int m_Number;
		private bool m_Scrollbar;
		private bool m_UseColor;
		private int m_Width;
		private int m_X;
		private int m_Y;
	}
 
	public class GumpImage : GumpEntry
	{
		public GumpImage(int x, int y, int gumpID) : this(x, y, gumpID, 0)
		{
		}

		public GumpImage(int x, int y, int gumpID, int hue) : base()
		{
			this.m_X = x;
			this.m_Y = y;
			this.m_GumpID = gumpID;
			this.m_Hue = hue;
		} 

		public override void AppendTo(DisplayGumpFast disp)
		{
				disp.AppendLayout(GumpImage.m_LayoutName);
			disp.AppendLayout(this.m_X);
			disp.AppendLayout(this.m_Y);
			disp.AppendLayout(this.m_GumpID);
			if (this.m_Hue != 0)
			{
				disp.AppendLayout(GumpImage.m_HueEquals);
				disp.AppendLayoutNS(this.m_Hue);
 
			}
 
		}
		
		public override string Compile()
		{
				object[] array1;
			if (this.m_Hue == 0)
			{
				return string.Format("{{ gumppic {0} {1} {2} }}", this.m_X, this.m_Y, this.m_GumpID); 
			}
			array1 = new object[4];
			array1[0] = this.m_X;
			array1[1] = this.m_Y;
			array1[2] = this.m_GumpID;
			array1[3] = this.m_Hue;
			return string.Format("{{ gumppic {0} {1} {2} hue={3} }}", array1); 
		} 

		public int GumpID { 
			get
			{
				return this.m_GumpID;
			}
			set
			{
				base.Delta(ref this.m_GumpID, value);
			}
		}

		public int Hue 
		{ 
			get
			{
				return this.m_Hue;
			}
			set
			{
				base.Delta(ref this.m_Hue, value);
			}
		}

		public int X 
		{ 
			get
			{
				return this.m_X;
			}
			set
			{
				base.Delta(ref this.m_X, value);
			}
		}

		public int Y 
		{ 
			get
			{
				return this.m_Y;
			}
			set
			{
				base.Delta(ref this.m_Y, value);
			}
		}

		private int m_GumpID;
		private int m_Hue;
		private static byte[] m_HueEquals = Gump.StringToBuffer(" hue=");
		private static byte[] m_LayoutName = Gump.StringToBuffer("gumppic");
		private int m_X;
		private int m_Y;
	}
 
	public class GumpImageTiled : GumpEntry
	{
		public GumpImageTiled(int x, int y, int width, int height, int gumpID) : base()
		{
			this.m_X = x;
			this.m_Y = y;
			this.m_Width = width;
			this.m_Height = height;
			this.m_GumpID = gumpID;
		} 

		public override void AppendTo(DisplayGumpFast disp)
		{
			disp.AppendLayout(GumpImageTiled.m_LayoutName);
			disp.AppendLayout(this.m_X);
			disp.AppendLayout(this.m_Y);
			disp.AppendLayout(this.m_Width);
			disp.AppendLayout(this.m_Height);
			disp.AppendLayout(this.m_GumpID);
		} 

		public override string Compile()
		{
			object[] array1;
			array1 = new object[5];
			array1[0] = this.m_X;
			array1[1] = this.m_Y;
			array1[2] = this.m_Width;
			array1[3] = this.m_Height;
			array1[4] = this.m_GumpID;
			return string.Format("{{ gumppictiled {0} {1} {2} {3} {4} }}", array1); 
		} 
		
		public int GumpID { 
			get
			{
				return this.m_GumpID;
			}
			set
			{
				base.Delta(ref this.m_GumpID, value);
			}
		}

		public int Height 
		{ 
			get
			{
				return this.m_Height;
			}
			set
			{
				base.Delta(ref this.m_Height, value);
			}
		}

		public int Width 
		{ 
			get
			{
				return this.m_Width;
			}
			set
			{
				base.Delta(ref this.m_Width, value);
			}
		}

		public int X 
		{ 
			get
			{
				return this.m_X;
			}
			set
			{
				base.Delta(ref this.m_X, value);
			}
		}

		public int Y 
		{ 
			get
			{
				return this.m_Y;
			}
			set
			{
				base.Delta(ref this.m_Y, value);
			}
		}

		private int m_GumpID;
		private int m_Height;
		private static byte[] m_LayoutName = Gump.StringToBuffer("gumppictiled");
		private int m_Width;
		private int m_X;
		private int m_Y;
	}
 
	public class GumpItem : GumpEntry
	{
		public GumpItem(int x, int y, int itemID) : this(x, y, itemID, 0)
		{
		}

		public GumpItem(int x, int y, int itemID, int hue) : base()
		{
			this.m_X = x;
			this.m_Y = y;
			this.m_ItemID = itemID;
			this.m_Hue = hue;
		} 

		public override void AppendTo(DisplayGumpFast disp)
		{
				disp.AppendLayout(((this.m_Hue == 0) ? GumpItem.m_LayoutName : GumpItem.m_LayoutNameHue));
			disp.AppendLayout(this.m_X);
			disp.AppendLayout(this.m_Y);
			disp.AppendLayout(this.m_ItemID);
			if (this.m_Hue != 0)
			{
				disp.AppendLayout(this.m_Hue);
 
			}
 
		} 

		public override string Compile()
		{
				object[] array1;
			if (this.m_Hue == 0)
			{
				return string.Format("{{ tilepic {0} {1} {2} }}", this.m_X, this.m_Y, this.m_ItemID); 
			}
			array1 = new object[4];
			array1[0] = this.m_X;
			array1[1] = this.m_Y;
			array1[2] = this.m_ItemID;
			array1[3] = this.m_Hue;
			return string.Format("{{ tilepichue {0} {1} {2} {3} }}", array1); 
		} 

		public int Hue { 
			get
			{
				return this.m_Hue;
			}
			set
			{
				base.Delta(ref this.m_Hue, value);
			}
		}

		public int ItemID 
		{ 
			get
			{
				return this.m_ItemID;
			}
			set
			{
				base.Delta(ref this.m_ItemID, value);
			}
		}

		public int X 
		{ 
			get
			{
				return this.m_X;
			}
			set
			{
				base.Delta(ref this.m_X, value);
			}
		}

		public int Y 
		{ 
			get
			{
				return this.m_Y;
			}
			set
			{
				base.Delta(ref this.m_Y, value);
			}
		}

		private int m_Hue;
		private int m_ItemID;
		private static byte[] m_LayoutName = Gump.StringToBuffer("tilepic");
		private static byte[] m_LayoutNameHue = Gump.StringToBuffer("tilepichue");
		private int m_X;
		private int m_Y;
	}
 
	public class GumpLabel : GumpEntry
	{
		public GumpLabel(int x, int y, int hue, string text)
		{
			this.m_X = x;
			this.m_Y = y;
			this.m_Hue = hue;
			this.m_Text = text;
		}

		public override void AppendTo(DisplayGumpFast disp)
		{
			disp.AppendLayout(GumpLabel.m_LayoutName);
			disp.AppendLayout(this.m_X);
			disp.AppendLayout(this.m_Y);
			disp.AppendLayout(this.m_Hue);
			disp.AppendLayout(base.Parent.Intern(this.m_Text));
		} 

		public override string Compile()
		{
			object[] array1;
			array1 = new object[4];
			array1[0] = this.m_X;
			array1[1] = this.m_Y;
			array1[2] = this.m_Hue;
			array1[3] = base.Parent.Intern(this.m_Text);
			return string.Format("{{ text {0} {1} {2} {3} }}", array1); 
		} 
		
		public int Hue { 
			get
			{
				return this.m_Hue;
			}
			set
			{
				base.Delta(ref this.m_Hue, value);
			}
		}

		public string Text 
		{ 
			get
			{
				return this.m_Text;
			}
			set
			{
				base.Delta(ref this.m_Text, value);
			}
		}

		public int X 
		{ 
			get
			{
				return this.m_X;
			}
			set
			{
				base.Delta(ref this.m_X, value);
			}
		}

		public int Y 
		{ 
			get
			{
				return this.m_Y;
			}
			set
			{
				base.Delta(ref this.m_Y, value);
			}
		}

		private int m_Hue;
		private static byte[] m_LayoutName = Gump.StringToBuffer("text");
		private string m_Text;
		private int m_X;
		private int m_Y;
	}
 
	public class GumpLabelCropped : GumpEntry
	{
		public GumpLabelCropped(int x, int y, int width, int height, int hue, string text) : base()
		{
			this.m_X = x;
			this.m_Y = y;
			this.m_Width = width;
			this.m_Height = height;
			this.m_Hue = hue;
			this.m_Text = text;
		}
		
		public override void AppendTo(DisplayGumpFast disp)
		{
			disp.AppendLayout(GumpLabelCropped.m_LayoutName);
			disp.AppendLayout(this.m_X);
			disp.AppendLayout(this.m_Y);
			disp.AppendLayout(this.m_Width);
			disp.AppendLayout(this.m_Height);
			disp.AppendLayout(this.m_Hue);
			disp.AppendLayout(base.Parent.Intern(this.m_Text));
		}
		
		public override string Compile()
		{
			object[] array1;
			array1 = new object[6];
			array1[0] = this.m_X;
			array1[1] = this.m_Y;
			array1[2] = this.m_Width;
			array1[3] = this.m_Height;
			array1[4] = this.m_Hue;
			array1[5] = base.Parent.Intern(this.m_Text);
			return string.Format("{{ croppedtext {0} {1} {2} {3} {4} {5} }}", array1); 
		} 

		public int Height { 
			get
			{
				return this.m_Height;
			}
			set
			{
				base.Delta(ref this.m_Height, value);
			}
		}

		public int Hue 
		{ 
			get
			{
				return this.m_Hue;
			}
			set
			{
				base.Delta(ref this.m_Hue, value);
			}
		}

		public string Text 
		{ 
			get
			{
				return this.m_Text;
			}
			set
			{
				base.Delta(ref this.m_Text, value);
			}
		}

		public int Width 
		{ 
			get
			{
				return this.m_Width;
			}
			set
			{
				base.Delta(ref this.m_Width, value);
			}
		}

		public int X 
		{ 
			get
			{
				return this.m_X;
			}
			set
			{
				base.Delta(ref this.m_X, value);
			}
		}

		public int Y 
		{ 
			get
			{
				return this.m_Y;
			}
			set
			{
				base.Delta(ref this.m_Y, value);
			}
		}
		
		private int m_Height;
		private int m_Hue;
		private static byte[] m_LayoutName = Gump.StringToBuffer("croppedtext");
		private string m_Text;
		private int m_Width;
		private int m_X;
		private int m_Y;
	}
 
	public class GumpPage : GumpEntry
	{
		public GumpPage(int page) : base()
		{
			this.m_Page = page;
		}

		public override void AppendTo(DisplayGumpFast disp)
		{
			disp.AppendLayout(GumpPage.m_LayoutName);
			disp.AppendLayout(this.m_Page);
		} 

		public override string Compile()
		{
			return string.Format("{{ page {0} }}", this.m_Page); 
		} 

		public int Page { 
			get
			{
				return this.m_Page;
			}
			set
			{
				base.Delta(ref this.m_Page, value);
			}
		}
		
		private static byte[] m_LayoutName = Gump.StringToBuffer("page");
		private int m_Page;
	}
 
	public class GumpRadio : GumpEntry
	{
		public GumpRadio(int x, int y, int inactiveID, int activeID, bool initialState, int switchID) : base()
		{
			this.m_X = x;
			this.m_Y = y;
			this.m_ID1 = inactiveID;
			this.m_ID2 = activeID;
			this.m_InitialState = initialState;
			this.m_SwitchID = switchID;
 
		} 

		public override void AppendTo(DisplayGumpFast disp)
		{
			disp.AppendLayout(GumpRadio.m_LayoutName);
			disp.AppendLayout(this.m_X);
			disp.AppendLayout(this.m_Y);
			disp.AppendLayout(this.m_ID1);
			disp.AppendLayout(this.m_ID2);
			disp.AppendLayout(this.m_InitialState);
			disp.AppendLayout(this.m_SwitchID);
 		} 

		public override string Compile()
		{
			object[] array1;
			array1 = new object[6];
			array1[0] = this.m_X;
			array1[1] = this.m_Y;
			array1[2] = this.m_ID1;
			array1[3] = this.m_ID2;
			array1[4] = (this.m_InitialState ? 1 : 0);
			array1[5] = this.m_SwitchID;
			return string.Format("{{ radio {0} {1} {2} {3} {4} {5} }}", array1); 
		} 
		
		public int ActiveID { 
			get
			{
				return this.m_ID2;
			}
			set
			{
				base.Delta(ref this.m_ID2, value);
			}
		}

		public int InactiveID 
		{ 
			get
			{
				return this.m_ID1;
			}
			set
			{
				base.Delta(ref this.m_ID1, value);
			}
		}

		public bool InitialState 
		{ 
			get
			{
				return this.m_InitialState;
			}
			set
			{
				base.Delta(ref this.m_InitialState, value);
			}
		}

		public int SwitchID 
		{ 
			get
			{
				return this.m_SwitchID;
			}
			set
			{
				base.Delta(ref this.m_SwitchID, value);
			}
		}

		public int X 
		{ 
			get
			{
				return this.m_X;
			}
			set
			{
				base.Delta(ref this.m_X, value);
			}
		}

		public int Y 
		{ 
			get
			{
				return this.m_Y;
			}
			set
			{
				base.Delta(ref this.m_Y, value);
			}
		}

		private int m_ID1;
		private int m_ID2;
		private bool m_InitialState;
		private static byte[] m_LayoutName = Gump.StringToBuffer("radio");
		private int m_SwitchID;
		private int m_X;
		private int m_Y;
	}
 
	public class GumpTextEntry : GumpEntry
	{
		public GumpTextEntry(int x, int y, int width, int height, int hue, int entryID, string initialText) : base()
		{
			this.m_X = x;
			this.m_Y = y;
			this.m_Width = width;
			this.m_Height = height;
			this.m_Hue = hue;
			this.m_EntryID = entryID;
			this.m_InitialText = initialText;
		} 

		public override void AppendTo(DisplayGumpFast disp)
		{
			disp.AppendLayout(GumpTextEntry.m_LayoutName);
			disp.AppendLayout(this.m_X);
			disp.AppendLayout(this.m_Y);
			disp.AppendLayout(this.m_Width);
			disp.AppendLayout(this.m_Height);
			disp.AppendLayout(this.m_Hue);
			disp.AppendLayout(this.m_EntryID);
			disp.AppendLayout(base.Parent.Intern(this.m_InitialText));
		} 

		public override string Compile()
		{
			object[] array1;
			array1 = new object[7];
			array1[0] = this.m_X;
			array1[1] = this.m_Y;
			array1[2] = this.m_Width;
			array1[3] = this.m_Height;
			array1[4] = this.m_Hue;
			array1[5] = this.m_EntryID;
			array1[6] = base.Parent.Intern(this.m_InitialText);
			return string.Format("{{ textentry {0} {1} {2} {3} {4} {5} {6} }}", array1); 
		} 

		public int EntryID { 
			get
			{
				return this.m_EntryID;
			}
			set
			{
				base.Delta(ref this.m_EntryID, value);
			}
		}

		public int Height 
		{ 
			get
			{
				return this.m_Height;
			}
			set
			{
				base.Delta(ref this.m_Height, value);
			}
		}

		public int Hue 
		{ 
			get
			{
				return this.m_Hue;
			}
			set
			{
				base.Delta(ref this.m_Hue, value);
			}
		}

		public string InitialText 
		{ 
			get
			{
				return this.m_InitialText;
			}
			set
			{
				base.Delta(ref this.m_InitialText, value);
			}
		}

		public int Width 
		{ 
			get
			{
				return this.m_Width;
			}
			set
			{
				base.Delta(ref this.m_Width, value);
			}
		}

		public int X 
		{ 
			get
			{
				return this.m_X;
			}
			set
			{
				base.Delta(ref this.m_X, value);
			}
		}

		public int Y 
		{ 
			get
			{
				return this.m_Y;
			}
			set
			{
				base.Delta(ref this.m_Y, value);
			}
		}

		private int m_EntryID;
		private int m_Height;
		private int m_Hue;
		private string m_InitialText;
		private static byte[] m_LayoutName = Gump.StringToBuffer("textentry");
		private int m_Width;
		private int m_X;
		private int m_Y;
	}
 
	public class RelayInfo
	{
		public RelayInfo(int buttonID, int[] switches, TextRelay[] textEntries) : base()
		{
			this.m_ButtonID = buttonID;
			this.m_Switches = switches;
			this.m_TextEntries = textEntries;
		} 

		public TextRelay GetTextEntry(int entryID)
		{
				int num1;
			for (num1 = 0; (num1 < this.m_TextEntries.Length); num1 = (num1 + 1))
			{
				if (this.m_TextEntries[num1].EntryID == entryID)
				{
					return this.m_TextEntries[num1]; 
				}
 
			}
			return null; 
		} 

		public bool IsSwitched(int switchID)
		{
				int num1;
			for (num1 = 0; (num1 < this.m_Switches.Length); num1 = (num1 + 1))
			{
				if (this.m_Switches[num1] == switchID)
				{
					return true; 
				}
 
			}
			return false; 
		} 

		public int ButtonID { 
			get
			{
				return this.m_ButtonID;
			}
		}

		public int[] Switches 
		{ 
			get
			{
				return this.m_Switches;
			}
		}

		public TextRelay[] TextEntries 
		{ 
			get
			{
				return this.m_TextEntries;
			}
		}

		private int m_ButtonID;
		private int[] m_Switches;
		private TextRelay[] m_TextEntries;
	}
 
	public class TextRelay
	{
		public TextRelay(int entryID, string text) : base()
		{
			this.m_EntryID = entryID;
			this.m_Text = text;
		}

		public int EntryID { 
			get
			{
				return this.m_EntryID;
			}
		}

		public string Text { 
			get
			{
				return this.m_Text;
			}
		}
		
		private int m_EntryID;
		private string m_Text;
	}
}