using System;
using System.Collections;
using System.Diagnostics;

using Server.Targeting;
using Server.Items;
using Server.Network;

namespace Server
{
	public class Item : IEntity, IHued
	{
		public Item() : base()
		{
			this.m_Serial = Serial.NewItem;
			this.m_Items = new ArrayList(1);
			this.Visible = true;
			this.Movable = true;
			this.Amount = 1;
			this.m_Map = Map.Internal;
			this.SetLastMoved();
			World.AddItem(this);
 
		} 

		public Item(int itemID) : base()
		{
			this.m_ItemID = itemID;
 
		} 

		public Item(Serial serial) : base()
		{
			this.m_Serial = serial;
		} 

		public virtual void AddBlessedForProperty(ObjectPropertyList list, Mobile m)
		{
			list.Add(1062203, "{0}", m.Name);
 
		} 

		public virtual void AddItem(Item item)
		{
			int num1;
			object[] array1;
			Serial serial1;
			if (((item == null) || item.Deleted) || (item.m_Parent == this))
			{
				return; 
			}
			if (item == this)
			{
				array1 = new object[4];
				serial1 = this.Serial;
				array1[0] = serial1.Value;
				array1[1] = base.GetType().Name;
				serial1 = item.Serial;
				array1[2] = serial1.Value;
				array1[3] = item.GetType().Name;
				Console.WriteLine("Warning: Adding item to itself: [0x{0:X} {1}].AddItem( [0x{2:X} {3}] )", array1);
				Console.WriteLine(new StackTrace());
				return; 
			}
			if (this.IsChildOf(item))
			{
				array1 = new object[4];
				serial1 = this.Serial;
				array1[0] = serial1.Value;
				array1[1] = base.GetType().Name;
				serial1 = item.Serial;
				array1[2] = serial1.Value;
				array1[3] = item.GetType().Name;
				Console.WriteLine("Warning: Adding parent item to child: [0x{0:X} {1}].AddItem( [0x{2:X} {3}] )", array1);
				Console.WriteLine(new StackTrace());
				return; 
			}
			if ((item.m_Parent as Mobile) != null)
			{
				((Mobile) item.m_Parent).RemoveItem(item);
 
			}
			else
			{
				if ((item.m_Parent as Item) != null)
				{
					((Item) item.m_Parent).RemoveItem(item);
					goto L_0132;
 
				}
				item.SendRemovePacket();
 
			}
 
			L_0132:
				item.Parent = this;
			item.Map = this.m_Map;
			num1 = this.m_Items.Count;
			this.m_Items.Add(item);
			this.TotalItems = ((((this.TotalItems - num1) + this.m_Items.Count) + item.TotalItems) - (item.IsVirtualItem ? 1 : 0));
			this.TotalWeight = (this.TotalWeight + (item.TotalWeight + item.PileWeight));
			this.TotalGold = (this.TotalGold + item.TotalGold);
			item.Delta(ItemDelta.Update);
			item.OnAdded(this);
			this.OnItemAdded(item);
 
		}
		
		public virtual void AddLockedDownProperty(ObjectPropertyList list)
		{
			list.Add(501643);
 
		} 

		public virtual void AddLootTypeProperty(ObjectPropertyList list)
		{
			if (this.m_LootType == LootType.Blessed)
			{
				list.Add(1038021);
				return; 
			}
			if (this.m_LootType == LootType.Cursed)
			{
				list.Add(1049643);
				return; 
			}
			if (this.Insured)
			{
				list.Add(1061682);
 
			}
 
		} 

		public virtual void AddNameProperties(ObjectPropertyList list)
		{
			this.AddNameProperty(list);
			if (this.IsSecure)
			{
				this.AddSecureProperty(list);
 
			}
			else
			{
				if (this.IsLockedDown)
				{
					this.AddLockedDownProperty(list);
 
				}
 
			}
			if ((this.m_BlessedFor != null) && !this.m_BlessedFor.Deleted)
			{
				this.AddBlessedForProperty(list, this.m_BlessedFor);
 
			}
			if (this.DisplayLootType)
			{
				this.AddLootTypeProperty(list);
 
			}
 
		} 

		public virtual void AddNameProperty(ObjectPropertyList list)
		{
			if (this.m_Name == null)
			{
				if (this.m_Amount <= 1)
				{
					list.Add(this.LabelNumber);
					return; 
				}
				list.Add(1050039, "{0} #{1}", this.m_Amount, this.LabelNumber);
				return; 
			}
			if (this.m_Amount <= 1)
			{
				list.Add(this.m_Name);
				return; 
			}
			list.Add(1050039, "{0} {1}", this.m_Amount, this.Name);
 
		} 

		public virtual void AddResistanceProperties(ObjectPropertyList list)
		{
			int num1;
			num1 = this.PhysicalResistance;
			if (num1 != 0)
			{
				list.Add(1060448, num1.ToString());
 
			}
			num1 = this.FireResistance;
			if (num1 != 0)
			{
				list.Add(1060447, num1.ToString());
 
			}
			num1 = this.ColdResistance;
			if (num1 != 0)
			{
				list.Add(1060445, num1.ToString());
 
			}
			num1 = this.PoisonResistance;
			if (num1 != 0)
			{
				list.Add(1060449, num1.ToString());
 
			}
			num1 = this.EnergyResistance;
			if (num1 != 0)
			{
				list.Add(1060446, num1.ToString());
 
			}
 
		} 

		public virtual void AddSecureProperty(ObjectPropertyList list)
		{
			list.Add(501644);
 
		} 

		public virtual bool AllowEquipedCast(Mobile from)
		{
			return false; 
		} 

		public virtual bool AllowSecureTrade(Mobile from, Mobile to, Mobile newOwner, bool accepted)
		{
			return true; 
		} 

		public bool AtPoint(int x, int y)
		{
			if (this.m_Location.m_X == x)
			{
				return (this.m_Location.m_Y == y); 
			}
			return false; 
		} 

		public bool AtWorldPoint(int x, int y)
		{
			if ((this.m_Parent == null) && (this.m_Location.m_X == x))
			{
				return (this.m_Location.m_Y == y); 
			}
			return false; 
		} 

		public void Bounce(Mobile from)
		{
			object obj1;
			if ((this.m_Parent as Item) != null)
			{
				((Item) this.m_Parent).RemoveItem(this);
 
			}
			else
			{
				if ((this.m_Parent as Mobile) != null)
				{
					((Mobile) this.m_Parent).RemoveItem(this);
 
				}
 
			}
			this.m_Parent = null;
			if (this.m_Bounce != null)
			{
				obj1 = this.m_Bounce.m_Parent;
				if (((obj1 as Item) != null) && !((Item) obj1).Deleted)
				{
					this.Location = this.m_Bounce.m_Location;
					((Item) obj1).AddItem(this);
					goto L_0101;
 
				}
				if (((obj1 as Mobile) != null) && !((Mobile) obj1).Deleted)
				{
					if (((Mobile) obj1).EquipItem(this))
					{
						goto L_0101;
 
					}
					this.MoveToWorld(this.m_Bounce.m_WorldLoc, this.m_Bounce.m_Map);
					goto L_0101;
 
				}
				this.MoveToWorld(this.m_Bounce.m_WorldLoc, this.m_Bounce.m_Map);
 
			}
			else
			{
				this.MoveToWorld(from.Location, from.Map);
 
			}
 
			L_0101:
				this.m_Bounce = null;
 
		} 

		public virtual bool CanEquip(Mobile m)
		{
			if (this.m_Layer != 0)
			{
				return (m.FindItemOnLayer(this.m_Layer) == null); 
			}
			return false; 
		} 

		public virtual bool CheckBlessed(Mobile m)
		{
			if ((this.m_LootType == LootType.Blessed) || (Mobile.InsuranceEnabled && this.Insured))
			{
				return true; 
			}
			if (m != null)
			{
				return (m == this.m_BlessedFor); 
			}
			return false; 
		} 

		public virtual bool CheckBlessed(object obj)
		{
			return this.CheckBlessed((obj as Mobile)); 
		} 

		public virtual bool CheckConflictingLayer(Mobile m, Item item, Layer layer)
		{
			return (this.m_Layer == layer); 
		} 

		public virtual bool CheckItemUse(Mobile from, Item item)
		{
			if ((this.m_Parent as Item) != null)
			{
				return ((Item) this.m_Parent).CheckItemUse(from, item); 
			}
			if ((this.m_Parent as Mobile) != null)
			{
				return ((Mobile) this.m_Parent).CheckItemUse(from, item); 
			}
			return true; 
		} 

		public virtual bool CheckLift(Mobile from, Item item)
		{
			if ((this.m_Parent as Item) != null)
			{
				return ((Item) this.m_Parent).CheckLift(from, item); 
			}
			if ((this.m_Parent as Mobile) != null)
			{
				return ((Mobile) this.m_Parent).CheckLift(from, item); 
			}
			return true; 
		} 

		public virtual bool CheckNewbied()
		{
			return (this.m_LootType == LootType.Newbied); 
		} 

		public virtual bool CheckPropertyConfliction(Mobile m)
		{
			return false; 
		} 

		public virtual bool CheckTarget(Mobile from, Target targ, object targeted)
		{
			if ((this.m_Parent as Item) != null)
			{
				return ((Item) this.m_Parent).CheckTarget(from, targ, targeted); 
			}
			if ((this.m_Parent as Mobile) != null)
			{
				return ((Mobile) this.m_Parent).CheckTarget(from, targ, targeted); 
			}
			return true; 
		} 

		public void ClearBounce()
		{
			this.m_Bounce = null;
 
		} 

		public virtual void Consume()
		{
			this.Consume(1);
 
		} 

		public virtual void Consume(int amount)
		{
			this.Amount = (this.Amount - amount);
			if (this.Amount <= 0)
			{
				this.Delete();
 
			}
 
		} 

		public virtual void Delete()
		{
			int num1;
			if (this.Deleted)
			{
				return; 
			}
			if (!World.OnDelete(this))
			{
				return; 
			}
			this.OnDelete();
			for (num1 = (this.m_Items.Count - 1); (num1 >= 0); num1 = (num1 - 1))
			{
				if (num1 < this.m_Items.Count)
				{
					((Item) this.m_Items[num1]).OnParentDeleted(this);
 
				}
 
			}
			this.SendRemovePacket();
			this.SetFlag(ImplFlag.Deleted, true);
			if ((this.Parent as Mobile) != null)
			{
				((Mobile) this.Parent).RemoveItem(this);
 
			}
			else
			{
				if ((this.Parent as Item) != null)
				{
					((Item) this.Parent).RemoveItem(this);
 
				}
 
			}
			if (this.m_Map != null)
			{
				this.m_Map.OnLeave(this);
				this.m_Map = null;
 
			}
			World.RemoveItem(this);
			this.OnAfterDelete();
			this.m_RemovePacket = null;
			this.m_WorldPacket = null;
			this.m_OPLPacket = null;
			this.m_PropertyList = null;
 
		} 

		public void Delta(ItemDelta flags)
		{
			if ((this.m_Map == null) || (this.m_Map == Map.Internal))
			{
				return; 
			}
			this.m_DeltaFlags = (this.m_DeltaFlags | flags);
			if (!this.GetFlag(ImplFlag.InQueue))
			{
				this.SetFlag(ImplFlag.InQueue, true);
				Item.m_DeltaQueue.Add(this);
 
			}
 
		} 

		public virtual void Deserialize(GenericReader reader)
		{
			int num1;
			SaveFlag flag1;
			int num2;
			int num3;
			int num4;
			int num5;
			Serial serial1;
			SaveFlag flag2;
			Serial serial2;
			Serial serial3;
			int num6;
			int num7;
			Item item1;
			int num8;
			num1 = reader.ReadInt();
			this.SetLastMoved();
			num8 = num1;
			switch (num8)
			{
				case 0:
				{
					goto L_06B3;
 
				}
				case 1:
				{
					goto L_06A7;
 
				}
				case 2:
				{
					goto L_068F;
 
				}
				case 3:
				{
					goto L_0682;
 
				}
				case 4:
				{
					goto L_0682;
 
				}
				case 5:
				{
					goto L_03E4;
 
				}
				case 6:
				{
					goto L_0038;
 
				}
				case 7:
				{
					goto L_0038;
 
				}
 
			}
			return; 
			L_0038:
				flag1 = (SaveFlag)reader.ReadInt();
			if (num1 < 7)
			{
				this.LastMoved = reader.ReadDeltaTime();
 
			}
			else
			{
				num2 = reader.ReadEncodedInt();
				try
				{
					this.LastMoved = DateTime.Now - TimeSpan.FromMinutes(((double) num2));
 
				}
				catch /*(Exception ignored)*/
				{
					this.LastMoved = DateTime.Now;
 
				}
 
			}
			if (Item.GetSaveFlag(flag1, SaveFlag.Direction))
			{
				this.m_Direction = (Direction)reader.ReadByte();
 
			}
			if (Item.GetSaveFlag(flag1, SaveFlag.Bounce))
			{
				this.m_Bounce = BounceInfo.Deserialize(reader);
 
			}
			if (Item.GetSaveFlag(flag1, SaveFlag.LootType))
			{
				this.m_LootType = (LootType)reader.ReadByte();
 
			}
			num3 = 0;
			num4 = 0;
			num5 = 0;
			if (Item.GetSaveFlag(flag1, SaveFlag.LocationFull))
			{
				num3 = reader.ReadEncodedInt();
				num4 = reader.ReadEncodedInt();
				num5 = reader.ReadEncodedInt();
 
			}
			else
			{
				if (Item.GetSaveFlag(flag1, SaveFlag.LocationByteXY))
				{
					num3 = reader.ReadByte();
					num4 = reader.ReadByte();
 
				}
				else
				{
					if (Item.GetSaveFlag(flag1, SaveFlag.LocationShortXY))
					{
						num3 = reader.ReadShort();
						num4 = reader.ReadShort();
 
					}
 
				}
				if (Item.GetSaveFlag(flag1, SaveFlag.LocationSByteZ))
				{
					num5 = reader.ReadSByte();
 
				}
 
			}
			this.m_Location = new Point3D(num3, num4, num5);
			if (Item.GetSaveFlag(flag1, SaveFlag.ItemID))
			{
				this.m_ItemID = reader.ReadEncodedInt();
 
			}
			if (Item.GetSaveFlag(flag1, SaveFlag.Hue))
			{
				this.m_Hue = reader.ReadEncodedInt();
 
			}
			if (Item.GetSaveFlag(flag1, SaveFlag.Amount))
			{
				this.m_Amount = reader.ReadEncodedInt();
 
			}
			else
			{
				this.m_Amount = 1;
 
			}
			if (Item.GetSaveFlag(flag1, SaveFlag.Layer))
			{
				this.m_Layer = (Layer)reader.ReadByte();
 
			}
			if (Item.GetSaveFlag(flag1, SaveFlag.Name))
			{
				this.m_Name = reader.ReadString();
 
			}
			if (!Item.GetSaveFlag(flag1, SaveFlag.Parent))
			{
				goto L_0235;
 
			}
			serial1 = reader.ReadInt();
			if (serial1.IsMobile)
			{
				this.m_Parent = World.FindMobile(serial1);
 
			}
			else
			{
				if (serial1.IsItem)
				{
					this.m_Parent = World.FindItem(serial1);
					goto L_0215;
 
				}
				this.m_Parent = null;
 
			}
 
			L_0215:
				if ((this.m_Parent == null) && (serial1.IsMobile || serial1.IsItem))
				{
					this.Delete();
 
				}
 
			L_0235:
				if (Item.GetSaveFlag(flag1, SaveFlag.Items))
				{
					this.m_Items = reader.ReadItemList();
 
				}
				else
				{
					this.m_Items = new ArrayList(1);
 
				}
			if (Item.GetSaveFlag(flag1, SaveFlag.IntWeight))
			{
				this.m_Weight = ((double) reader.ReadEncodedInt());
 
			}
			else
			{
				if (Item.GetSaveFlag(flag1, SaveFlag.WeightNot1or0))
				{
					this.m_Weight = reader.ReadDouble();
					goto L_02C0;
 
				}
				if (Item.GetSaveFlag(flag1, SaveFlag.WeightIs0))
				{
					this.m_Weight = 0;
					goto L_02C0;
 
				}
				this.m_Weight = 1;
 
			}
 
			L_02C0:
				if (Item.GetSaveFlag(flag1, SaveFlag.Map))
				{
					this.m_Map = reader.ReadMap();
 
				}
				else
				{
					this.m_Map = Map.Internal;
 
				}
			if (Item.GetSaveFlag(flag1, SaveFlag.Visible))
			{
				this.SetFlag(ImplFlag.Visible, reader.ReadBool());
 
			}
			else
			{
				this.SetFlag(ImplFlag.Visible, true);
 
			}
			if (Item.GetSaveFlag(flag1, SaveFlag.Movable))
			{
				this.SetFlag(ImplFlag.Movable, reader.ReadBool());
 
			}
			else
			{
				this.SetFlag(ImplFlag.Movable, true);
 
			}
			if (Item.GetSaveFlag(flag1, SaveFlag.Stackable))
			{
				this.SetFlag(ImplFlag.Stackable, reader.ReadBool());
 
			}
			if (Item.GetSaveFlag(flag1, SaveFlag.ImplFlags))
			{
				this.m_Flags = (ImplFlag)reader.ReadEncodedInt();
 
			}
			if (Item.GetSaveFlag(flag1, SaveFlag.InsuredFor))
			{
				reader.ReadMobile();
 
			}
			if (Item.GetSaveFlag(flag1, SaveFlag.BlessedFor))
			{
				this.m_BlessedFor = reader.ReadMobile();
 
			}
			if (Item.GetSaveFlag(flag1, SaveFlag.HeldBy))
			{
				this.m_HeldBy = reader.ReadMobile();
 
			}
			if (Item.GetSaveFlag(flag1, SaveFlag.SavedFlags))
			{
				this.m_SavedFlags = reader.ReadEncodedInt();
 
			}
			if ((this.m_Map == null) || (this.m_Parent != null))
			{
				goto L_081D;
 
			}
			this.m_Map.OnEnter(this);
			return; 
			L_03E4:
				flag2 = (SaveFlag)reader.ReadInt();
			this.LastMoved = reader.ReadDeltaTime();
			if (Item.GetSaveFlag(flag2, SaveFlag.Direction))
			{
				this.m_Direction = (Direction)reader.ReadByte();
 
			}
			if (Item.GetSaveFlag(flag2, SaveFlag.Bounce))
			{
				this.m_Bounce = BounceInfo.Deserialize(reader);
 
			}
			if (Item.GetSaveFlag(flag2, SaveFlag.LootType))
			{
				this.m_LootType = (LootType)reader.ReadByte();
 
			}
			if (Item.GetSaveFlag(flag2, SaveFlag.LocationFull))
			{
				this.m_Location = reader.ReadPoint3D();
 
			}
			if (Item.GetSaveFlag(flag2, SaveFlag.ItemID))
			{
				this.m_ItemID = reader.ReadInt();
 
			}
			if (Item.GetSaveFlag(flag2, SaveFlag.Hue))
			{
				this.m_Hue = reader.ReadInt();
 
			}
			if (Item.GetSaveFlag(flag2, SaveFlag.Amount))
			{
				this.m_Amount = reader.ReadInt();
 
			}
			else
			{
				this.m_Amount = 1;
 
			}
			if (Item.GetSaveFlag(flag2, SaveFlag.Layer))
			{
				this.m_Layer = (Layer)reader.ReadByte();
 
			}
			if (Item.GetSaveFlag(flag2, SaveFlag.Name))
			{
				this.m_Name = reader.ReadString();
 
			}
			if (!Item.GetSaveFlag(flag2, SaveFlag.Parent))
			{
				goto L_0544;
 
			}
			serial2 = reader.ReadInt();
			if (serial2.IsMobile)
			{
				this.m_Parent = World.FindMobile(serial2);
 
			}
			else
			{
				if (serial2.IsItem)
				{
					this.m_Parent = World.FindItem(serial2);
					goto L_0524;
 
				}
				this.m_Parent = null;
 
			}
 
			L_0524:
				if ((this.m_Parent == null) && (serial2.IsMobile || serial2.IsItem))
				{
					this.Delete();
 
				}
 
			L_0544:
				if (Item.GetSaveFlag(flag2, SaveFlag.Items))
				{
					this.m_Items = reader.ReadItemList();
 
				}
				else
				{
					this.m_Items = new ArrayList(1);
 
				}
			if (Item.GetSaveFlag(flag2, SaveFlag.IntWeight))
			{
				this.m_Weight = ((double) reader.ReadEncodedInt());
 
			}
			else
			{
				if (Item.GetSaveFlag(flag2, SaveFlag.WeightNot1or0))
				{
					this.m_Weight = reader.ReadDouble();
					goto L_05D3;
 
				}
				if (Item.GetSaveFlag(flag2, SaveFlag.WeightIs0))
				{
					this.m_Weight = 0;
					goto L_05D3;
 
				}
				this.m_Weight = 1;
 
			}
 
			L_05D3:
				if (Item.GetSaveFlag(flag2, SaveFlag.Map))
				{
					this.m_Map = reader.ReadMap();
 
				}
				else
				{
					this.m_Map = Map.Internal;
 
				}
			if (Item.GetSaveFlag(flag2, SaveFlag.Visible))
			{
				this.SetFlag(ImplFlag.Visible, reader.ReadBool());
 
			}
			else
			{
				this.SetFlag(ImplFlag.Visible, true);
 
			}
			if (Item.GetSaveFlag(flag2, SaveFlag.Movable))
			{
				this.SetFlag(ImplFlag.Movable, reader.ReadBool());
 
			}
			else
			{
				this.SetFlag(ImplFlag.Movable, true);
 
			}
			if (Item.GetSaveFlag(flag2, SaveFlag.Stackable))
			{
				this.SetFlag(ImplFlag.Stackable, reader.ReadBool());
 
			}
			if ((this.m_Map == null) || (this.m_Parent != null))
			{
				goto L_081D;
 
			}
			this.m_Map.OnEnter(this);
			return; 
			L_0682:
				this.m_Direction = (Direction)reader.ReadInt();
 
			L_068F:
				this.m_Bounce = BounceInfo.Deserialize(reader);
			this.LastMoved = reader.ReadDeltaTime();
 
			L_06A7:
				this.m_LootType = (LootType)reader.ReadByte();
 
			L_06B3:
				this.m_Location = reader.ReadPoint3D();
			this.m_ItemID = reader.ReadInt();
			this.m_Hue = reader.ReadInt();
			this.m_Amount = reader.ReadInt();
			this.m_Layer = (Layer)reader.ReadByte();
			this.m_Name = reader.ReadString();
			serial3 = reader.ReadInt();
			if (serial3.IsMobile)
			{
				this.m_Parent = World.FindMobile(serial3);
 
			}
			else
			{
				if (serial3.IsItem)
				{
					this.m_Parent = World.FindItem(serial3);
					goto L_073F;
 
				}
				this.m_Parent = null;
 
			}
 
			L_073F:
				if ((this.m_Parent == null) && (serial3.IsMobile || serial3.IsItem))
				{
					this.Delete();
 
				}
			num6 = reader.ReadInt();
			this.m_Items = new ArrayList(num6);
			for (num7 = 0; (num7 < num6); num7 = (num7 + 1))
			{
				item1 = reader.ReadItem();
				if (item1 != null)
				{
					this.m_Items.Add(item1);
 
				}
 
			}
			this.m_Weight = reader.ReadDouble();
			if (num1 <= 3)
			{
				reader.ReadInt();
				reader.ReadInt();
				reader.ReadInt();
 
			}
			this.m_Map = reader.ReadMap();
			this.SetFlag(ImplFlag.Visible, reader.ReadBool());
			this.SetFlag(ImplFlag.Movable, reader.ReadBool());
			if (num1 <= 3)
			{
				reader.ReadBool();
 
			}
			this.Stackable = reader.ReadBool();
			if ((this.m_Map != null) && (this.m_Parent == null))
			{
				this.m_Map.OnEnter(this);
 
			}
 
			L_081D:
				return; 
		}
		
		public virtual bool DropToItem(Mobile from, Item target, Point3D p)
		{
			object obj1;
			if (((this.Deleted || from.Deleted) || (target.Deleted || (from.Map != target.Map))) || ((from.Map == null) || (target.Map == null)))
			{
				return false; 
			}
			obj1 = target.RootParent;
			if ((from.AccessLevel < AccessLevel.GameMaster) && !from.InRange(target.GetWorldLocation(), 2))
			{
				return false; 
			}
			if (!from.CanSee(target) || !from.InLOS(target))
			{
				return false; 
			}
			if (!target.IsAccessibleTo(from))
			{
				return false; 
			}
			if (((obj1 as Mobile) != null) && !((Mobile) obj1).CheckNonlocalDrop(from, this, target))
			{
				return false; 
			}
			if (!from.OnDroppedItemToItem(this, target, p))
			{
				return false; 
			}
			if (((target as Container) != null) && ((p.m_X != -1) && (p.m_Y != -1)))
			{
				return this.OnDroppedInto(from, ((Container) target), p); 
			}
			return this.OnDroppedOnto(from, target); 
		} 

		public virtual bool DropToMobile(Mobile from, Mobile target, Point3D p)
		{
			if (((this.Deleted || from.Deleted) || (target.Deleted || (from.Map != target.Map))) || ((from.Map == null) || (target.Map == null)))
			{
				return false; 
			}
			if ((from.AccessLevel < AccessLevel.GameMaster) && !from.InRange(target.Location, 2))
			{
				return false; 
			}
			if (!from.CanSee(target) || !from.InLOS(target))
			{
				return false; 
			}
			if (!from.OnDroppedItemToMobile(this, target))
			{
				return false; 
			}
			if (!this.OnDroppedToMobile(from, target))
			{
				return false; 
			}
			if (!target.OnDragDrop(from, this))
			{
				return false; 
			}
			return true; 
		} 

		public virtual bool DropToWorld(Mobile from, Point3D p)
		{
			Map map1;
			int num1;
			int num2;
			int num3;
			int num4;
			Tile tile1;
			TileFlag flag1;
			int num5;
			int num6;
			int num7;
			Tile[] array1;
			int num8;
			Tile tile2;
			ItemData data1;
			int num9;
			ArrayList list1;
			IPooledEnumerable enumerable1;
			Item item1;
			ItemData data2;
			int num10;
			int num11;
			int num12;
			Tile tile3;
			ItemData data3;
			int num13;
			int num14;
			int num15;
			int num16;
			int num17;
			int num18;
			Item item2;
			ItemData data4;
			int num19;
			int num20;
			int num21;
			int num22;
			int num23;
			int num24;
			int num25;
			bool flag2;
			int num26;
			int num27;
			Tile tile4;
			ItemData data5;
			int num28;
			int num29;
			int num30;
			Item item3;
			ItemData data6;
			int num31;
			int num32;
			IEnumerator enumerator1;
			IDisposable disposable1;
			ItemData data7;
			if ((this.Deleted || from.Deleted) || (from.Map == null))
			{
				return false; 
			}
			if (!from.InRange(p, 2))
			{
				return false; 
			}
			map1 = from.Map;
			if (map1 == null)
			{
				return false; 
			}
			num1 = p.m_X;
			num2 = p.m_Y;
			num3 = -2147483648;
			num4 = (from.Z + 16);
			tile1 = map1.Tiles.GetLandTile(num1, num2);
			flag1 = TileData.LandTable[(tile1.ID & 16383)].Flags;
			num5 = 0;
			num6 = 0;
			num7 = 0;
			map1.GetAverageZ(num1, num2, ref num5, ref num6, ref num7);
			if ((!tile1.Ignored && ((flag1 & TileFlag.Impassable) == 0)) && (num6 <= num4))
			{
				num3 = num6;
 
			}
			array1 = map1.Tiles.GetStaticTiles(num1, num2, true);
			for (num8 = 0; (num8 < array1.Length); num8 = (num8 + 1))
			{
				tile2 = array1[num8];
				data1 = TileData.ItemTable[(tile2.ID & 16383)];
				if (data1.Surface)
				{
					num9 = (tile2.Z + data1.CalcHeight);
					if ((num9 <= num4) && (num9 >= num3))
					{
						num3 = num9;
 
					}
 
				}
 
			}
			list1 = new ArrayList();
			enumerable1 = map1.GetItemsInRange(p, 0);
			enumerator1 = enumerable1.GetEnumerator();
			try
			{
				while (enumerator1.MoveNext())
				{
					item1 = ((Item) enumerator1.Current);
					if (item1.ItemID < 16384)
					{
						list1.Add(item1);
						data2 = item1.ItemData;
						if (data2.Surface)
						{
							num10 = (item1.Z + data2.CalcHeight);
							if ((num10 <= num4) && (num10 >= num3))
							{
								num3 = num10;
 
							}
 
						}
 
					}
 
				}
 
			}
			finally
			{
				disposable1 = (enumerator1 as IDisposable);
				if (disposable1 != null)
				{
					disposable1.Dispose();
 
				}
 
			}
			enumerable1.Free();
			if (num3 == -2147483648)
			{
				return false; 
			}
			if (num3 > num4)
			{
				return false; 
			}
			Item.m_OpenSlots = 1048575;
			num11 = num3;
			for (num12 = 0; (num12 < array1.Length); num12 = (num12 + 1))
			{
				tile3 = array1[num12];
				data3 = TileData.ItemTable[(tile3.ID & 16383)];
				num13 = tile3.Z;
				num14 = (num13 + data3.CalcHeight);
				if ((num14 == num13) && !data3.Surface)
				{
					num14 = (num14 + 1);
 
				}
				num15 = (num13 - num3);
				num16 = (num14 - num3);
				if ((num15 < 20) && (num16 >= 0))
				{
					if (num15 < 0)
					{
						num15 = 0;
 
					}
					if (num16 > 19)
					{
						num16 = 19;
 
					}
					num17 = (num16 - num15);
					//TRF Item.m_OpenSlots = (Item.m_OpenSlots & !(((1 << (num17 & 31)) - 1) << (num15 & 31)));
					Item.m_OpenSlots = (Item.m_OpenSlots & ~(((1 << (num17 & 31)) - 1) << (num15 & 31)));
				}
			}
			for (num18 = 0; (num18 < list1.Count); num18 = (num18 + 1))
			{
				item2 = ((Item) list1[num18]);
				data4 = item2.ItemData;
				num19 = item2.Z;
				num20 = (num19 + data4.CalcHeight);
				if ((num20 == num19) && !data4.Surface)
				{
					num20 = (num20 + 1);
 
				}
				num21 = (num19 - num3);
				num22 = (num20 - num3);
				if ((num21 < 20) && (num22 >= 0))
				{
					if (num21 < 0)
					{
						num21 = 0;
 
					}
					if (num22 > 19)
					{
						num22 = 19;
 
					}
					num23 = (num22 - num21);
					//TRF Item.m_OpenSlots = (Item.m_OpenSlots & !(((1 << (num23 & 31)) - 1) << (num21 & 31)));
					Item.m_OpenSlots = (Item.m_OpenSlots & ~(((1 << (num23 & 31)) - 1) << (num21 & 31)));
				}
 
			}
			data7 = this.ItemData;
			num24 = data7.Height;
			if (num24 == 0)
			{
				num24 = (num24 + 1);
 
			}
			if (num24 > 30)
			{
				num24 = 30;
 
			}
			num25 = ((1 << (num24 & 31)) - 1);
			flag2 = false;
			for (num26 = 0; (num26 < 20); num26 = (num26 + 1))
			{
				if ((num26 + num24) > 20)
				{
					num25 = (num25 >> 1);
 
				}
				flag2 = (((Item.m_OpenSlots >> (num26 & 31)) & num25) == num25);
				if (flag2)
				{
					num3 = (num3 + num26);
					goto L_03B9;
 
				}
 
			}
 
			L_03B9:
				if (!flag2)
				{
					return false; 
				}
			data7 = this.ItemData;
			num24 = data7.Height;
			if (num24 == 0)
			{
				num24 = (num24 + 1);
 
			}
			if ((num6 > num3) && ((num3 + num24) > num5))
			{
				return false; 
			}
			if (((flag1 & TileFlag.Impassable) != 0) && ((num6 > num11) && ((num3 + num24) > num5)))
			{
				return false; 
			}
			for (num27 = 0; (num27 < array1.Length); num27 = (num27 + 1))
			{
				tile4 = array1[num27];
				data5 = TileData.ItemTable[(tile4.ID & 16383)];
				num28 = tile4.Z;
				num29 = (num28 + data5.CalcHeight);
				if ((num29 > num3) && ((num3 + num24) > num28))
				{
					return false; 
				}
				if ((data5.Surface || data5.Impassable) && ((num29 > num11) && ((num3 + num24) > num28)))
				{
					return false; 
				}
 
			}
			for (num30 = 0; (num30 < list1.Count); num30 = (num30 + 1))
			{
				item3 = ((Item) list1[num30]);
				data6 = item3.ItemData;
				num31 = item3.Z;
				if (((item3.Z + data6.CalcHeight) > num3) && ((num3 + num24) > item3.Z))
				{
					return false; 
				}
 
			}
			p = new Point3D(num1, num2, num3);
			if (!from.InLOS(new Point3D(num1, num2, (num3 + 1))))
			{
				return false; 
			}
			if (!from.OnDroppedItemToWorld(this, p))
			{
				return false; 
			}
			if (!this.OnDroppedToWorld(from, p))
			{
				return false; 
			}
			num32 = this.GetDropSound();
			this.MoveToWorld(p, from.Map);
			from.SendSound(((num32 == -1) ? 66 : num32), this.GetWorldLocation());
			return true; 
		}
		
		public virtual Item Dupe(int amount)
		{
			return this.Dupe(new Item(), amount); 
		} 

		public virtual Item Dupe(Item item, int amount)
		{
			item.Visible = this.Visible;
			item.Movable = this.Movable;
			item.LootType = this.LootType;
			item.Direction = this.Direction;
			item.Hue = this.Hue;
			item.ItemID = this.ItemID;
			item.Location = this.Location;
			item.Layer = this.Layer;
			item.Name = this.Name;
			item.Weight = this.Weight;
			item.Amount = amount;
			item.Map = this.Map;
			if ((this.Parent as Mobile) != null)
			{
				((Mobile) this.Parent).AddItem(item);
 
			}
			else
			{
				if ((this.Parent as Item) != null)
				{
					((Item) this.Parent).AddItem(item);
 
				}
 
			}
			item.Delta(ItemDelta.Update);
			return item; 
		} 

		public virtual void FreeCache()
		{
			this.m_RemovePacket = null;
			this.m_WorldPacket = null;
			this.m_OPLPacket = null;
			this.m_PropertyList = null;
 
		} 

		public BounceInfo GetBounce()
		{
			return this.m_Bounce; 
		} 

		public virtual void GetChildProperties(ObjectPropertyList list, Item item)
		{
			if ((this.m_Parent as Item) != null)
			{
				((Item) this.m_Parent).GetChildProperties(list, item);
				return; 
			}
			if ((this.m_Parent as Mobile) != null)
			{
				((Mobile) this.m_Parent).GetChildProperties(list, item);
 
			}
 
		} 

		public IPooledEnumerable GetClientsInRange(int range)
		{
			Map map1;
			map1 = this.m_Map;
			if (map1 == null)
			{
				return Map.NullEnumerable.Instance; 
			}
			if (this.m_Parent == null)
			{
				return map1.GetClientsInRange(this.m_Location, range); 
			}
			return map1.GetClientsInRange(this.GetWorldLocation(), range); 
		} 

		public virtual void GetContextMenuEntries(Mobile from, ArrayList list)
		{  
		} 

		public virtual int GetDropSound()
		{
			return -1; 
		} 

		private bool GetFlag(ImplFlag flag)
		{
			return ((this.m_Flags & flag) != 0); 
		} 

		public Rectangle2D GetGraphicBounds()
		{
			int num1;
			bool flag1;
			int num2;
			Rectangle2D rectangled1;
			num1 = this.m_ItemID;
			flag1 = (this.m_Amount > 1);
			if ((num1 >= 3818) && (num1 <= 3826))
			{
				num2 = ((num1 - 3818) / 3);
				num2 = (num2 * 3);
				num2 = (num2 + 3818);
				flag1 = false;
				if (this.m_Amount <= 1)
				{
					num1 = num2;
					goto L_0059;
 
				}
				if (this.m_Amount <= 5)
				{
					num1 = (num2 + 1);
					goto L_0059;
 
				}
				num1 = (num2 + 2);
 
			}
 
			L_0059:
				rectangled1 = ItemBounds.Table[(num1 & 16383)];
			if (flag1)
			{
				rectangled1.Set(rectangled1.X, rectangled1.Y, (rectangled1.Width + 5), (rectangled1.Height + 5));
 
			}
			return rectangled1; 
		} 

		public IPooledEnumerable GetItemsInRange(int range)
		{
			Map map1;
			map1 = this.m_Map;
			if (map1 == null)
			{
				return Map.NullEnumerable.Instance; 
			}
			if (this.m_Parent == null)
			{
				return map1.GetItemsInRange(this.m_Location, range); 
			}
			return map1.GetItemsInRange(this.GetWorldLocation(), range); 
		} 

		public virtual int GetLiftSound(Mobile from)
		{
			return 87; 
		} 

		public virtual int GetMaxUpdateRange()
		{
			return 18; 
		} 

		public IPooledEnumerable GetMobilesInRange(int range)
		{
			Map map1;
			map1 = this.m_Map;
			if (map1 == null)
			{
				return Map.NullEnumerable.Instance; 
			}
			if (this.m_Parent == null)
			{
				return map1.GetMobilesInRange(this.m_Location, range); 
			}
			return map1.GetMobilesInRange(this.GetWorldLocation(), range); 
		} 

		public IPooledEnumerable GetObjectsInRange(int range)
		{
			Map map1;
			map1 = this.m_Map;
			if (map1 == null)
			{
				return Map.NullEnumerable.Instance; 
			}
			if (this.m_Parent == null)
			{
				return map1.GetObjectsInRange(this.m_Location, range); 
			}
			return map1.GetObjectsInRange(this.GetWorldLocation(), range); 
		} 

		public virtual int GetPacketFlags()
		{
			int num1;
			num1 = 0;
			if (!this.Visible)
			{
				num1 = (num1 | 128);
 
			}
			if (this.Movable || this.ForceShowProperties)
			{
				num1 = (num1 | 32);
 
			}
			return num1; 
		} 

		public virtual void GetProperties(ObjectPropertyList list)
		{
			this.AddNameProperties(list);
			if ((this.m_Parent as Item) != null)
			{
				((Item) this.m_Parent).GetChildProperties(list, this);
				return; 
			}
			if ((this.m_Parent as Mobile) != null)
			{
				((Mobile) this.m_Parent).GetChildProperties(list, this);
 
			}
 
		} 

		public bool GetSavedFlag(int flag)
		{
			return ((this.m_SavedFlags & flag) != 0); 
		} 

		private static bool GetSaveFlag(SaveFlag flags, SaveFlag toGet)
		{
			return ((flags & toGet) != 0); 
		} 

		public SecureTradeContainer GetSecureTradeCont()
		{
			object obj1;
			for (obj1 = this; ((obj1 as Item) != null); obj1 = ((Item) obj1).m_Parent)
			{
				if ((obj1 as SecureTradeContainer) != null)
				{
					return ((SecureTradeContainer) obj1); 
				}
 
			}
			return null; 
		} 

		public Point3D GetSurfaceTop()
		{
			object obj1;
			ItemData itemdata1;

			obj1 = this.RootParent;
			if (obj1 == null) 
			{
				itemdata1 = this.ItemData;
				return new Point3D(this.m_Location.m_X, this.m_Location.m_Y,
					this.m_Location.m_Z + ((itemdata1.Surface) ? itemdata1.CalcHeight : 0)); 
			}
			return ((IEntity) obj1).Location;

			/*
			 * TOxDO: Write me:   XXX: Verify the code above is correct.
			 * .maxstack 6
	.locals (object V_0, ItemData V_1)
	L_0000: ldarg.0 
	L_0001: call Item.get_RootParent
	L_0006: stloc.0 
	L_0007: ldloc.0 
	L_0008: brtrue.s L_0053
	L_000a: ldarg.0 
	L_000b: ldflda Item.m_Location
	L_0010: ldfld Point3D.m_X
	L_0015: ldarg.0 
	L_0016: ldflda Item.m_Location
	L_001b: ldfld Point3D.m_Y
	L_0020: ldarg.0 
	L_0021: ldflda Item.m_Location
	L_0026: ldfld Point3D.m_Z
	L_002b: ldarg.0 
	L_002c: call Item.get_ItemData
	L_0031: stloc.1 
	L_0032: ldloca.s V_1
	L_0034: call ItemData.get_Surface
	L_0039: brtrue.s L_003e
	L_003b: ldc.i4.0 
	L_003c: br.s L_004c
	L_003e: ldarg.0 
	L_003f: call Item.get_ItemData
	L_0044: stloc.1 
	L_0045: ldloca.s V_1
	L_0047: call ItemData.get_CalcHeight
	L_004c: add 
	L_004d: newobj Point3D..ctor
	L_0052: ret 
	L_0053: ldloc.0 
	L_0054: castclass IEntity
	L_0059: callvirt IEntity.get_Location
	L_005e: ret 
			 */
		}

		public bool GetTempFlag(int flag)
		{
			return ((this.m_TempFlags & flag) != 0); 
		} 

		public virtual int GetUpdateRange(Mobile m)
		{
			return 18; 
		} 

		public Point3D GetWorldLocation()
		{
			object obj1;
			obj1 = this.RootParent;
			if (obj1 == null)
			{
				return this.m_Location; 
			}
			return ((IEntity) obj1).Location; 
		} 

		public Point3D GetWorldTop()
		{
			object obj1;
			ItemData data1;
			obj1 = this.RootParent;
			if (obj1 == null)
			{
				data1 = this.ItemData;
				return new Point3D(this.m_Location.m_X, this.m_Location.m_Y, (this.m_Location.m_Z + data1.CalcHeight)); 
			}
			return ((IEntity) obj1).Location; 
		} 

		public void Internalize()
		{
			this.MoveToWorld(Point3D.Zero, Map.Internal);
 
		} 

		public void InvalidateProperties()
		{
			ObjectPropertyList list1;
			ObjectPropertyList list2;
			if (!Core.AOS)
			{
				return; 
			}
			if ((this.m_Map != null) && (this.m_Map != Map.Internal))
			{
				list1 = this.m_PropertyList;
				this.m_PropertyList = null;
				list2 = this.PropertyList;
				if ((list1 != null) && (list1.Hash == list2.Hash))
				{
					goto L_0060;
 
				}
				this.m_OPLPacket = null;
				this.Delta(ItemDelta.Properties);
				return; 
			}
			this.m_PropertyList = null;
			this.m_OPLPacket = null;
 
			L_0060:
				return; 
		} 

		public virtual bool IsAccessibleTo(Mobile check)
		{
			Region region1;
			if ((this.m_Parent as Item) != null)
			{
				return ((Item) this.m_Parent).IsAccessibleTo(check); 
			}
			region1 = Region.Find(this.GetWorldLocation(), this.m_Map);
			return region1.CheckAccessibility(this, check); 
		} 

		public bool IsChildOf(object o)
		{
			return this.IsChildOf(o, false); 
		} 

		public bool IsChildOf(object o, bool allowNull)
		{
			object obj1;
			Item item1;
			obj1 = this.m_Parent;
			if (((obj1 == null) || (o == null)) && !allowNull)
			{
				return false; 
			}
			if (obj1 != o)
			{
				goto L_0034;
 
			}
			return true; 
			L_0018:
				item1 = ((Item) obj1);
			if (item1.m_Parent == null)
			{
				goto L_003C;
 
			}
			obj1 = item1.m_Parent;
			if (obj1 == o)
			{
				return true; 
			}
 
			L_0034:
				if ((obj1 as Item) != null)
				{
					goto L_0018;
 
				}
 
			L_003C:
				return false; 
		} 

		public virtual bool IsStandardLoot()
		{
			if (Mobile.InsuranceEnabled && this.Insured)
			{
				return false; 
			}
			if (this.m_BlessedFor != null)
			{
				return false; 
			}
			return (this.m_LootType == 0); 
		} 

		public virtual void LabelLootTypeTo(Mobile to)
		{
			if (this.m_LootType == LootType.Blessed)
			{
				this.LabelTo(to, 1041362);
				return; 
			}
			if (this.m_LootType == LootType.Cursed)
			{
				this.LabelTo(to, "(cursed)");
 
			}
 
		} 

		public void LabelTo(Mobile to, int number)
		{
			to.Send(new MessageLocalized(this.m_Serial, this.m_ItemID, MessageType.Label, 946, 3, number, "", ""));
 
		} 

		public void LabelTo(Mobile to, int number, string args)
		{
			to.Send(new MessageLocalized(this.m_Serial, this.m_ItemID, MessageType.Label, 946, 3, number, "", args));
 
		} 

		public void LabelTo(Mobile to, string text)
		{
			to.Send(new UnicodeMessage(this.m_Serial, this.m_ItemID, MessageType.Label, 946, 3, "ENU", "", text));
 
		} 

		public void LabelTo(Mobile to, string format, params object[] args)
		{
			this.LabelTo(to, string.Format(format, args));
 
		} 

		public void LabelToAffix(Mobile to, int number, AffixType type, string affix)
		{
			to.Send(new MessageLocalizedAffix(this.m_Serial, this.m_ItemID, MessageType.Label, 946, 3, number, "", type, affix, ""));
 
		} 

		public void LabelToAffix(Mobile to, int number, AffixType type, string affix, string args)
		{
			to.Send(new MessageLocalizedAffix(this.m_Serial, this.m_ItemID, MessageType.Label, 946, 3, number, "", type, affix, args));
 
		} 

		public virtual void MoveToWorld(Point3D location)
		{
			this.MoveToWorld(location, this.m_Map);
 
		} 

		public void MoveToWorld(Point3D location, Map map)
		{
			Point3D pointd1;
			Point3D pointd2;
			Map map1;
			Packet packet1;
			IPooledEnumerable enumerable1;
			NetState state1;
			Mobile mobile1;
			int num1;
			IPooledEnumerable enumerable2;
			NetState state2;
			Mobile mobile2;
			IPooledEnumerable enumerable3;
			Packet packet2;
			NetState state3;
			Mobile mobile3;
			Point3D pointd3;
			NetState state4;
			Mobile mobile4;
			IEnumerator enumerator1;
			IDisposable disposable1;
			if (this.Deleted)
			{
				return; 
			}
			pointd1 = this.GetWorldLocation();
			pointd2 = this.m_Location;
			this.SetLastMoved();
			if ((this.Parent as Mobile) != null)
			{
				((Mobile) this.Parent).RemoveItem(this);
 
			}
			else
			{
				if ((this.Parent as Item) != null)
				{
					((Item) this.Parent).RemoveItem(this);
 
				}
 
			}
			if (this.m_Map != map)
			{
				map1 = this.m_Map;
				if (this.m_Map != null)
				{
					this.m_Map.OnLeave(this);
					if (pointd1.m_X != 0)
					{
						packet1 = null;
						enumerable1 = this.m_Map.GetClientsInRange(pointd1, this.GetMaxUpdateRange());
						enumerator1 = enumerable1.GetEnumerator();
						try
						{
							while (enumerator1.MoveNext())
							{
								state1 = ((NetState) enumerator1.Current);
								mobile1 = state1.Mobile;
								if (mobile1.InRange(pointd1, this.GetUpdateRange(mobile1)))
								{
									if (packet1 == null)
									{
										packet1 = this.RemovePacket;
 
									}
									state1.Send(packet1);
 
								}
 
							}
 
						}
						finally
						{
							disposable1 = (enumerator1 as IDisposable);
							if (disposable1 != null)
							{
								disposable1.Dispose();
 
							}
 
						}
						enumerable1.Free();
 
					}
 
				}
				for (num1 = 0; (num1 < this.m_Items.Count); num1 = (num1 + 1))
				{
					((Item) this.m_Items[num1]).Map = map;
 
				}
				this.m_Location = location;
				this.OnLocationChange(pointd2);
				this.m_WorldPacket = null;
				this.m_Map = map;
				if (this.m_Map != null)
				{
					this.m_Map.OnEnter(this);
 
				}
				this.OnMapChange();
				if (this.m_Map != null)
				{
					enumerable2 = this.m_Map.GetClientsInRange(this.m_Location, this.GetMaxUpdateRange());
					enumerator1 = enumerable2.GetEnumerator();
					try
					{
						while (enumerator1.MoveNext())
						{
							state2 = ((NetState) enumerator1.Current);
							mobile2 = state2.Mobile;
							if (mobile2.CanSee(this) && mobile2.InRange(this.m_Location, this.GetUpdateRange(mobile2)))
							{
								this.SendInfoTo(state2);
 
							}
 
						}
 
					}
					finally
					{
						disposable1 = (enumerator1 as IDisposable);
						if (disposable1 != null)
						{
							disposable1.Dispose();
 
						}
 
					}
					enumerable2.Free();
 
				}
				this.RemDelta(ItemDelta.Update);
				if ((map1 != null) && (map1 != Map.Internal))
				{
					goto L_0398;
 
				}
				this.InvalidateProperties();
				return; 
			}
			if (this.m_Map != null)
			{
				if (pointd1.m_X != 0)
				{
					packet2 = null;
					enumerable3 = this.m_Map.GetClientsInRange(pointd1, this.GetMaxUpdateRange());
					enumerator1 = enumerable3.GetEnumerator();
					try
					{
						while (enumerator1.MoveNext())
						{
							state3 = ((NetState) enumerator1.Current);
							mobile3 = state3.Mobile;
							if (!mobile3.InRange(location, this.GetUpdateRange(mobile3)))
							{
								if (packet2 == null)
								{
									packet2 = this.RemovePacket;
 
								}
								state3.Send(packet2);
 
							}
 
						}
 
					}
					finally
					{
						disposable1 = (enumerator1 as IDisposable);
						if (disposable1 != null)
						{
							disposable1.Dispose();
 
						}
 
					}
					enumerable3.Free();
 
				}
				pointd3 = this.m_Location;
				this.m_Location = location;
				this.OnLocationChange(pointd2);
				this.m_WorldPacket = null;
				enumerable3 = this.m_Map.GetClientsInRange(this.m_Location, this.GetMaxUpdateRange());
				enumerator1 = enumerable3.GetEnumerator();
				try
				{
					while (enumerator1.MoveNext())
					{
						state4 = ((NetState) enumerator1.Current);
						mobile4 = state4.Mobile;
						if (mobile4.CanSee(this) && mobile4.InRange(this.m_Location, this.GetUpdateRange(mobile4)))
						{
							this.SendInfoTo(state4);
 
						}
 
					}
 
				}
				finally
				{
					disposable1 = (enumerator1 as IDisposable);
					if (disposable1 != null)
					{
						disposable1.Dispose();
 
					}
 
				}
				enumerable3.Free();
				this.m_Map.OnMove(pointd3, this);
				this.RemDelta(ItemDelta.Update);
				return; 
			}
			this.Map = map;
			this.Location = location;
 
			L_0398:
				return; 
		}
		
		public virtual void OnAdded(object parent)
		{  
		} 

		public virtual void OnAfterDelete()
		{  
		} 

		protected virtual void OnAmountChange(int oldValue)
		{  
		} 

		public virtual void OnAosSingleClick(Mobile from)
		{
			ObjectPropertyList list1;
			list1 = this.PropertyList;
			if (list1.Header > 0)
			{
				from.Send(new MessageLocalized(this.m_Serial, this.m_ItemID, MessageType.Label, 946, 3, list1.Header, this.m_Name, list1.HeaderArgs));
 
			}
 
		} 

		public virtual bool OnDecay()
		{
			if ((this.Decays && (this.Parent == null)) && (this.Map != Map.Internal))
			{
				return Region.Find(this.Location, this.Map).OnDecay(this); 
			}
			return false; 
		} 

		public virtual void OnDelete()
		{  
		} 

		public virtual void OnDoubleClick(Mobile from)
		{  
		} 

		public virtual void OnDoubleClickCantSee(Mobile from)
		{  
		} 

		public virtual void OnDoubleClickDead(Mobile from)
		{
			from.LocalOverheadMessage(0, 946, 1019048);
 
		} 

		public virtual void OnDoubleClickNotAccessible(Mobile from)
		{
			from.SendLocalizedMessage(500447);
 
		} 

		public virtual void OnDoubleClickOutOfRange(Mobile from)
		{  
		} 

		public virtual void OnDoubleClickSecureTrade(Mobile from)
		{
			from.SendLocalizedMessage(500447);
 
		} 

		public virtual bool OnDragDrop(Mobile from, Item dropped)
		{
			if ((this.Parent as Container) != null)
			{
				return ((Container) this.Parent).OnStackAttempt(from, this, dropped); 
			}
			return this.StackWith(from, dropped); 
		} 

		public virtual bool OnDragLift(Mobile from)
		{
			return true; 
		} 

		public virtual bool OnDroppedInto(Mobile from, Container target, Point3D p)
		{
			if (!from.OnDroppedItemInto(this, target, p))
			{
				return false; 
			}
			return target.OnDragDropInto(from, this, p); 
		} 

		public virtual bool OnDroppedOnto(Mobile from, Item target)
		{
			if (((this.Deleted || from.Deleted) || (target.Deleted || (from.Map != target.Map))) || ((from.Map == null) || (target.Map == null)))
			{
				return false; 
			}
			if ((from.AccessLevel < AccessLevel.GameMaster) && !from.InRange(target.GetWorldLocation(), 2))
			{
				return false; 
			}
			if (!from.CanSee(target) || !from.InLOS(target))
			{
				return false; 
			}
			if (!target.IsAccessibleTo(from))
			{
				return false; 
			}
			if (!from.OnDroppedItemOnto(this, target))
			{
				return false; 
			}
			return target.OnDragDrop(from, this); 
		} 

		public virtual bool OnDroppedToMobile(Mobile from, Mobile target)
		{
			return true; 
		} 

		public virtual bool OnDroppedToWorld(Mobile from, Point3D p)
		{
			return true; 
		} 

		public virtual bool OnEquip(Mobile from)
		{
			return true; 
		} 

		public virtual void OnHelpRequest(Mobile from)
		{  
		} 

		public virtual DeathMoveResult OnInventoryDeath(Mobile parent)
		{
			if (!this.Movable)
			{
				return DeathMoveResult.MoveToBackpack; 
			}
			if (parent.KeepsItemsOnDeath)
			{
				return DeathMoveResult.MoveToBackpack; 
			}
			if (this.CheckBlessed(parent))
			{
				return DeathMoveResult.MoveToBackpack; 
			}
			if (this.CheckNewbied() && (parent.Kills < 5))
			{
				return DeathMoveResult.MoveToBackpack; 
			}
			return DeathMoveResult.MoveToCorpse; 
		} 

		public virtual void OnItemAdded(Item item)
		{
			if ((this.m_Parent as Item) != null)
			{
				((Item) this.m_Parent).OnSubItemAdded(item);
				return; 
			}
			if ((this.m_Parent as Mobile) != null)
			{
				((Mobile) this.m_Parent).OnSubItemAdded(item);
 
			}
 
		} 

		public virtual void OnItemLifted(Mobile from, Item item)
		{
			if ((this.m_Parent as Item) != null)
			{
				((Item) this.m_Parent).OnItemLifted(from, item);
				return; 
			}
			if ((this.m_Parent as Mobile) != null)
			{
				((Mobile) this.m_Parent).OnItemLifted(from, item);
 
			}
 
		} 

		public virtual void OnItemRemoved(Item item)
		{
			if ((this.m_Parent as Item) != null)
			{
				((Item) this.m_Parent).OnSubItemRemoved(item);
				return; 
			}
			if ((this.m_Parent as Mobile) != null)
			{
				((Mobile) this.m_Parent).OnSubItemRemoved(item);
 
			}
 
		} 

		public virtual void OnItemUsed(Mobile from, Item item)
		{
			if ((this.m_Parent as Item) != null)
			{
				((Item) this.m_Parent).OnItemUsed(from, item);
				return; 
			}
			if ((this.m_Parent as Mobile) != null)
			{
				((Mobile) this.m_Parent).OnItemUsed(from, item);
 
			}
 
		} 

		public virtual void OnLocationChange(Point3D oldLocation)
		{  
		} 

		public virtual void OnMapChange()
		{  
		} 

		public virtual void OnMovement(Mobile m, Point3D oldLocation)
		{  
		} 

		public virtual bool OnMoveOff(Mobile m)
		{
			return true; 
		} 

		public virtual bool OnMoveOver(Mobile m)
		{
			return true; 
		} 

		public virtual DeathMoveResult OnParentDeath(Mobile parent)
		{
			if (!this.Movable)
			{
				return DeathMoveResult.RemainEquiped; 
			}
			if (parent.KeepsItemsOnDeath)
			{
				return DeathMoveResult.MoveToBackpack; 
			}
			if (this.CheckBlessed(parent))
			{
				return DeathMoveResult.MoveToBackpack; 
			}
			if (this.CheckNewbied() && (parent.Kills < 5))
			{
				return DeathMoveResult.MoveToBackpack; 
			}
			return DeathMoveResult.MoveToCorpse; 
		} 

		public virtual void OnParentDeleted(object parent)
		{
			this.Delete();
 
		} 

		public virtual void OnRemoved(object parent)
		{  
		} 

		public virtual void OnSectorActivate()
		{  
		} 

		public virtual void OnSectorDeactivate()
		{  
		} 

		public virtual void OnSecureTrade(Mobile from, Mobile to, Mobile newOwner, bool accepted)
		{  
		} 

		public virtual void OnSingleClick(Mobile from)
		{
			NetState state1;
			if (this.Deleted || !from.CanSee(this))
			{
				return; 
			}
			if (this.DisplayLootType)
			{
				this.LabelLootTypeTo(from);
 
			}
			state1 = from.NetState;
			if (state1 != null)
			{
				if (this.m_Name == null)
				{
					if (this.m_Amount <= 1)
					{
						state1.Send(new MessageLocalized(this.m_Serial, this.m_ItemID, MessageType.Label, 946, 3, this.LabelNumber, "", ""));
						return; 
					}
					state1.Send(new MessageLocalizedAffix(this.m_Serial, this.m_ItemID, MessageType.Label, 946, 3, this.LabelNumber, "", 0, string.Format(" : {0}", this.m_Amount), ""));
					return; 
				}
				state1.Send(new UnicodeMessage(this.m_Serial, this.m_ItemID, MessageType.Label, 946, 3, "ENU", "", string.Concat(this.m_Name, ((this.m_Amount > 1) ? string.Concat(" : ", this.m_Amount) : ""))));
 
			}
 
		} 

		public virtual void OnSingleClickContained(Mobile from, Item item)
		{
			if ((this.m_Parent as Item) != null)
			{
				((Item) this.m_Parent).OnSingleClickContained(from, item);
 
			}
 
		} 

		public virtual void OnSnoop(Mobile from)
		{  
		} 

		public virtual void OnSpeech(SpeechEventArgs e)
		{  
		} 

		public virtual void OnSubItemAdded(Item item)
		{
			if ((this.m_Parent as Item) != null)
			{
				((Item) this.m_Parent).OnSubItemAdded(item);
				return; 
			}
			if ((this.m_Parent as Mobile) != null)
			{
				((Mobile) this.m_Parent).OnSubItemAdded(item);
 
			}
 
		} 

		public virtual void OnSubItemRemoved(Item item)
		{
			if ((this.m_Parent as Item) != null)
			{
				((Item) this.m_Parent).OnSubItemRemoved(item);
				return; 
			}
			if ((this.m_Parent as Mobile) != null)
			{
				((Mobile) this.m_Parent).OnSubItemRemoved(item);
 
			}
 
		} 

		public void ProcessDelta()
		{
			ItemDelta delta1;
			Map map1;
			bool flag1;
			Container container1;
			Packet packet1;
			Point3D pointd1;
			Mobile mobile1;
			Mobile mobile2;
			NetState state1;
			SecureTradeContainer container2;
			SecureTrade trade1;
			Mobile mobile3;
			NetState state2;
			ArrayList list1;
			int num1;
			Mobile mobile4;
			int num2;
			NetState state3;
			Packet packet2;
			Point3D pointd2;
			IPooledEnumerable enumerable1;
			NetState state4;
			Mobile mobile5;
			Packet packet3;
			Point3D pointd3;
			IPooledEnumerable enumerable2;
			NetState state5;
			Mobile mobile6;
			Point3D pointd4;
			IPooledEnumerable enumerable3;
			NetState state6;
			Mobile mobile7;
			IEnumerator enumerator1;
			IDisposable disposable1;
			delta1 = this.m_DeltaFlags;
			this.SetFlag(ImplFlag.InQueue, false);
			this.m_DeltaFlags = 0;
			map1 = this.m_Map;
			if ((map1 != null) && !this.Deleted)
			{
				flag1 = (!ObjectPropertyList.Enabled ? false : (((delta1 & ItemDelta.Properties) == 0) == false));
				container1 = (this.m_Parent as Container);
				if (((container1 != null) && !container1.IsPublicContainer) && ((delta1 & ItemDelta.Update) != 0))
				{
					packet1 = null;
					pointd1 = this.GetWorldLocation();
					mobile1 = (container1.RootParent as Mobile);
					mobile2 = null;
					if (mobile1 != null)
					{
						state1 = mobile1.NetState;
						if (((state1 != null) && mobile1.CanSee(this)) && mobile1.InRange(pointd1, this.GetUpdateRange(mobile1)))
						{
							if (packet1 == null)
							{
								packet1 = new ContainerContentUpdate(this);
 
							}
							state1.Send(packet1);
							if (ObjectPropertyList.Enabled)
							{
								state1.Send(this.OPLPacket);
 
							}
 
						}
 
					}
					container2 = this.GetSecureTradeCont();
					if (container2 != null)
					{
						trade1 = container2.Trade;
						if (trade1 != null)
						{
							mobile3 = trade1.From.Mobile;
							if ((mobile3 != null) && (mobile3 != mobile1))
							{
								mobile2 = mobile3;
 
							}
							mobile3 = trade1.To.Mobile;
							if ((mobile3 != null) && (mobile3 != mobile1))
							{
								mobile2 = mobile3;
 
							}
							if (mobile2 != null)
							{
								state2 = mobile2.NetState;
								if (((state2 != null) && mobile2.CanSee(this)) && mobile2.InRange(pointd1, this.GetUpdateRange(mobile2)))
								{
									if (packet1 == null)
									{
										packet1 = new ContainerContentUpdate(this);
 
									}
									state2.Send(packet1);
									if (ObjectPropertyList.Enabled)
									{
										state2.Send(this.OPLPacket);
 
									}
 
								}
 
							}
 
						}
 
					}
					list1 = container1.Openers;
					if (list1 != null)
					{
						for (num1 = 0; (num1 < list1.Count); num1 = (num1 + 1))
						{
							mobile4 = ((Mobile) list1[num1]);
							num2 = this.GetUpdateRange(mobile4);
							if ((mobile4.Map != map1) || !mobile4.InRange(pointd1, num2))
							{
								num1 = (num1 - 1);
								list1.RemoveAt(num1);
 
							}
							else
							{
								if ((mobile4 != mobile1) && (mobile4 != mobile2))
								{
									state3 = mobile4.NetState;
									if ((state3 != null) && mobile4.CanSee(this))
									{
										if (packet1 == null)
										{
											packet1 = new ContainerContentUpdate(this);
 
										}
										state3.Send(packet1);
										if (ObjectPropertyList.Enabled)
										{
											state3.Send(this.OPLPacket);
 
										}
 
									}
 
								}
 
							}
 
						}
						if (list1.Count == 0)
						{
							container1.Openers = null;
 
						}
 
					}
					return; 
				}
				if ((delta1 & ItemDelta.Update) != 0)
				{
					packet2 = null;
					pointd2 = this.GetWorldLocation();
					enumerable1 = map1.GetClientsInRange(pointd2, this.GetMaxUpdateRange());
					enumerator1 = enumerable1.GetEnumerator();
					try
					{
						while (enumerator1.MoveNext())
						{
							state4 = ((NetState) enumerator1.Current);
							mobile5 = state4.Mobile;
							if (mobile5.CanSee(this) && mobile5.InRange(pointd2, this.GetUpdateRange(mobile5)))
							{
								if (this.m_Parent == null)
								{
									this.SendInfoTo(state4);
 
								}
								else
								{
									if (packet2 == null)
									{
										if ((this.m_Parent as Item) != null)
										{
											packet2 = new ContainerContentUpdate(this);
 
										}
										else
										{
											if ((this.m_Parent as Mobile) != null)
											{
												packet2 = new EquipUpdate(this);
 
											}
 
										}
 
									}
									state4.Send(packet2);
									if (ObjectPropertyList.Enabled)
									{
										state4.Send(this.OPLPacket);
 
									}
 
								}
 
							}
 
						}
 
					}
					finally
					{
						disposable1 = (enumerator1 as IDisposable);
						if (disposable1 != null)
						{
							disposable1.Dispose();
 
						}
 
					}
					enumerable1.Free();
					flag1 = false;
 
				}
				else
				{
					if (((delta1 & ItemDelta.EquipOnly) != 0) && ((this.m_Parent as Mobile) != null))
					{
						packet3 = null;
						pointd3 = this.GetWorldLocation();
						enumerable2 = map1.GetClientsInRange(pointd3, this.GetMaxUpdateRange());
						enumerator1 = enumerable2.GetEnumerator();
						try
						{
							while (enumerator1.MoveNext())
							{
								state5 = ((NetState) enumerator1.Current);
								mobile6 = state5.Mobile;
								if (mobile6.CanSee(this) && mobile6.InRange(pointd3, this.GetUpdateRange(mobile6)))
								{
									if (packet3 == null)
									{
										packet3 = new EquipUpdate(this);
 
									}
									state5.Send(packet3);
									if (ObjectPropertyList.Enabled)
									{
										state5.Send(this.OPLPacket);
 
									}
 
								}
 
							}
 
						}
						finally
						{
							disposable1 = (enumerator1 as IDisposable);
							if (disposable1 != null)
							{
								disposable1.Dispose();
 
							}
 
						}
						enumerable2.Free();
						flag1 = false;
 
					}
 
				}
				if (flag1)
				{
					pointd4 = this.GetWorldLocation();
					enumerable3 = map1.GetClientsInRange(pointd4, this.GetMaxUpdateRange());
					enumerator1 = enumerable3.GetEnumerator();
					try
					{
						while (enumerator1.MoveNext())
						{
							state6 = ((NetState) enumerator1.Current);
							mobile7 = state6.Mobile;
							if (mobile7.CanSee(this) && mobile7.InRange(pointd4, this.GetUpdateRange(mobile7)))
							{
								state6.Send(this.OPLPacket);
 
							}
 
						}
 
					}
					finally
					{
						disposable1 = (enumerator1 as IDisposable);
						if (disposable1 != null)
						{
							disposable1.Dispose();
 
						}
 
					}
					enumerable3.Free();
 
				}
 
			}
 
		}
		
		public static void ProcessDeltaQueue()
		{
			int num1;
			int num2;
			num1 = Item.m_DeltaQueue.Count;
			for (num2 = 0; (num2 < Item.m_DeltaQueue.Count); num2 = (num2 + 1))
			{
				((Item) Item.m_DeltaQueue[num2]).ProcessDelta();
				if (num2 >= num1)
				{
					goto L_0039;
 
				}
 
			}
 
			L_0039:
				if (Item.m_DeltaQueue.Count > 0)
				{
					Item.m_DeltaQueue.Clear();
 
				}
 
		} 


		public void PublicOverheadMessage(MessageType type, int hue, bool ascii, string text)
		{
			Packet packet1;
			Point3D pointd1;
			IPooledEnumerable enumerable1;
			NetState state1;
			Mobile mobile1;
			IEnumerator enumerator1;
			IDisposable disposable1;
			if (this.m_Map != null)
			{
				packet1 = null;
				pointd1 = this.GetWorldLocation();
				enumerable1 = this.m_Map.GetClientsInRange(pointd1, this.GetMaxUpdateRange());
				enumerator1 = enumerable1.GetEnumerator();
				try
				{
					while (enumerator1.MoveNext())
					{
						state1 = ((NetState) enumerator1.Current);
						mobile1 = state1.Mobile;
						if (mobile1.CanSee(this) && mobile1.InRange(pointd1, this.GetUpdateRange(mobile1)))
						{
							if (packet1 == null)
							{
								if (ascii)
								{
									packet1 = new AsciiMessage(this.m_Serial, this.m_ItemID, type, hue, 3, this.m_Name, text);
 
								}
								else
								{
									packet1 = new UnicodeMessage(this.m_Serial, this.m_ItemID, type, hue, 3, "ENU", this.m_Name, text);
 
								}
 
							}
							state1.Send(packet1);
 
						}
 
					}
 
				}
				finally
				{
					disposable1 = (enumerator1 as IDisposable);
					if (disposable1 != null)
					{
						disposable1.Dispose();
 
					}
 
				}
				enumerable1.Free();
 
			}
 
		} 

		public void PublicOverheadMessage(MessageType type, int hue, int number)
		{
			this.PublicOverheadMessage(type, hue, number, "");
 
		} 

		public void PublicOverheadMessage(MessageType type, int hue, int number, string args)
		{
			Packet packet1;
			Point3D pointd1;
			IPooledEnumerable enumerable1;
			NetState state1;
			Mobile mobile1;
			IEnumerator enumerator1;
			IDisposable disposable1;
			if (this.m_Map != null)
			{
				packet1 = null;
				pointd1 = this.GetWorldLocation();
				enumerable1 = this.m_Map.GetClientsInRange(pointd1, this.GetMaxUpdateRange());
				enumerator1 = enumerable1.GetEnumerator();
				try
				{
					while (enumerator1.MoveNext())
					{
						state1 = ((NetState) enumerator1.Current);
						mobile1 = state1.Mobile;
						if (mobile1.CanSee(this) && mobile1.InRange(pointd1, this.GetUpdateRange(mobile1)))
						{
							if (packet1 == null)
							{
								packet1 = new MessageLocalized(this.m_Serial, this.m_ItemID, type, hue, 3, number, this.m_Name, args);
 
							}
							state1.Send(packet1);
 
						}
 
					}
 
				}
				finally
				{
					disposable1 = (enumerator1 as IDisposable);
					if (disposable1 != null)
					{
						disposable1.Dispose();
 
					}
 
				}
				enumerable1.Free();
 
			}
 
		} 

		public void RecordBounce()
		{
			this.m_Bounce = new BounceInfo(this);
 
		} 

		public void RemDelta(ItemDelta flags)
		{
			//TRF this.m_DeltaFlags = (this.m_DeltaFlags & !flags);
			this.m_DeltaFlags = (this.m_DeltaFlags & ~flags);
			if (this.GetFlag(ImplFlag.InQueue) && (this.m_DeltaFlags == 0))
			{
				this.SetFlag(ImplFlag.InQueue, false);
				Item.m_DeltaQueue.Remove(this);
 
			}
 
		} 

		public virtual void RemoveItem(Item item)
		{
			int num1;
			if (this.m_Items.Contains(item))
			{
				item.SendRemovePacket();
				num1 = this.m_Items.Count;
				this.m_Items.Remove(item);
				this.TotalItems = ((((this.TotalItems - num1) + this.m_Items.Count) - item.TotalItems) + (item.IsVirtualItem ? 1 : 0));
				this.TotalWeight = (this.TotalWeight - (item.TotalWeight + item.PileWeight));
				this.TotalGold = (this.TotalGold - item.TotalGold);
				item.Parent = null;
				item.OnRemoved(this);
				this.OnItemRemoved(item);
 
			}
 
		} 

		public virtual void ScissorHelper(Mobile from, Item newItem, int amountPerOldItem)
		{
			this.ScissorHelper(from, newItem, amountPerOldItem, true);
 
		} 

		public virtual void ScissorHelper(Mobile from, Item newItem, int amountPerOldItem, bool carryHue)
		{
			int num1;
			int num2;
			Map map1;
			object obj1;
			Point3D pointd1;
			LootType type1;
			num1 = this.Amount;
			if (num1 > (60000 / amountPerOldItem))
			{
				num1 = (60000 / amountPerOldItem);
 
			}
			this.Amount = (this.Amount - num1);
			num2 = this.Hue;
			map1 = this.Map;
			obj1 = this.m_Parent;
			pointd1 = this.GetWorldLocation();
			type1 = this.LootType;
			if (this.Amount == 0)
			{
				this.Delete();
 
			}
			newItem.Amount = (num1 * amountPerOldItem);
			if (carryHue)
			{
				newItem.Hue = num2;
 
			}
			if (Item.m_ScissorCopyLootType)
			{
				newItem.LootType = type1;
 
			}
			if (((obj1 as Container) == null) || !((Container) obj1).TryDropItem(from, newItem, false))
			{
				newItem.MoveToWorld(pointd1, map1);
 
			}
 
		} 

		public virtual void SendInfoTo(NetState state)
		{
			state.Send(this.WorldPacket);
			if (ObjectPropertyList.Enabled)
			{
				state.Send(this.OPLPacket);
 
			}
 
		} 

		public void SendLocalizedMessageTo(Mobile to, int number)
		{
			if (this.Deleted || !to.CanSee(this))
			{
				return; 
			}
			to.Send(new MessageLocalized(this.Serial, this.ItemID, 0, 946, 3, number, "", ""));
 
		} 

		public void SendLocalizedMessageTo(Mobile to, int number, AffixType affixType, string affix, string args)
		{
			if (this.Deleted || !to.CanSee(this))
			{
				return; 
			}
			to.Send(new MessageLocalizedAffix(this.Serial, this.ItemID, 0, 946, 3, number, "", affixType, affix, args));
 
		} 

		public void SendLocalizedMessageTo(Mobile to, int number, string args)
		{
			if (this.Deleted || !to.CanSee(this))
			{
				return; 
			}
			to.Send(new MessageLocalized(this.Serial, this.ItemID, 0, 946, 3, number, "", args));
 
		} 

		public virtual void SendPropertiesTo(Mobile from)
		{
			from.Send(this.PropertyList);
 
		} 

		public void SendRemovePacket()
		{
			Packet packet1;
			Point3D pointd1;
			IPooledEnumerable enumerable1;
			NetState state1;
			Mobile mobile1;
			IEnumerator enumerator1;
			IDisposable disposable1;
			if (!this.Deleted && (this.m_Map != null))
			{
				packet1 = null;
				pointd1 = this.GetWorldLocation();
				enumerable1 = this.m_Map.GetClientsInRange(pointd1, this.GetMaxUpdateRange());
				enumerator1 = enumerable1.GetEnumerator();
				try
				{
					while (enumerator1.MoveNext())
					{
						state1 = ((NetState) enumerator1.Current);
						mobile1 = state1.Mobile;
						if (mobile1.InRange(pointd1, this.GetUpdateRange(mobile1)))
						{
							if (packet1 == null)
							{
								packet1 = this.RemovePacket;
 
							}
							state1.Send(packet1);
 
						}
 
					}
 
				}
				finally
				{
					disposable1 = (enumerator1 as IDisposable);
					if (disposable1 != null)
					{
						disposable1.Dispose();
 
					}
 
				}
				enumerable1.Free();
 
			}
 
		} 

		public virtual void Serialize(GenericWriter writer)
		{
			SaveFlag flag1;
			int num1;
			int num2;
			int num3;
			ImplFlag flag2;
			long num4;
			long num5;
			TimeSpan span1;
			double num6;
			DateTime time1;
			writer.Write(7);
			flag1 = 0;
			num1 = this.m_Location.m_X;
			num2 = this.m_Location.m_Y;
			num3 = this.m_Location.m_Z;
			if (((num1 != 0) || (num2 != 0)) || (num3 != 0))
			{
				if (((num1 >= -32768) && ((num1 <= 32767) && (num2 >= -32768))) && (((num2 <= 32767) && (num3 >= -128)) && (num3 <= 127)))
				{
					if ((num1 != 0) || (num2 != 0))
					{
						if ((num1 >= 0) && (((num1 <= 255) && (num2 >= 0)) && (num2 <= 255)))
						{
							flag1 = (flag1 | SaveFlag.LocationByteXY);
 
						}
						else
						{
							flag1 = (flag1 | SaveFlag.LocationShortXY);
 
						}
 
					}
					if (num3 == 0)
					{
						goto L_00A1;
 
					}
					flag1 = (flag1 | SaveFlag.LocationSByteZ);
 
				}
				else
				{
					flag1 = (flag1 | SaveFlag.LocationFull);
 
				}
 
			}
 
			L_00A1:
				if (this.m_Direction != 0)
				{
					flag1 = (flag1 | SaveFlag.Direction);
 
				}
			if (this.m_Bounce != null)
			{
				flag1 = (flag1 | SaveFlag.Bounce);
 
			}
			if (this.m_LootType != 0)
			{
				flag1 = (flag1 | SaveFlag.LootType);
 
			}
			if (this.m_ItemID != 0)
			{
				flag1 = (flag1 | SaveFlag.ItemID);
 
			}
			if (this.m_Hue != 0)
			{
				flag1 = (flag1 | SaveFlag.Hue);
 
			}
			if (this.m_Amount != 1)
			{
				flag1 = (flag1 | SaveFlag.Amount);
 
			}
			if (this.m_Layer != 0)
			{
				flag1 = (flag1 | SaveFlag.Layer);
 
			}
			if (this.m_Name != null)
			{
				flag1 = (flag1 | SaveFlag.Name);
 
			}
			if (this.m_Parent != null)
			{
				flag1 = (flag1 | SaveFlag.Parent);
 
			}
			if ((this.m_Items != null) && (this.m_Items.Count > 0))
			{
				flag1 = (flag1 | SaveFlag.Items);
 
			}
			if (this.m_Map != Map.Internal)
			{
				flag1 = (flag1 | SaveFlag.Map);
 
			}
			if ((this.m_BlessedFor != null) && !this.m_BlessedFor.Deleted)
			{
				flag1 = (flag1 | SaveFlag.BlessedFor);
 
			}
			if ((this.m_HeldBy != null) && !this.m_HeldBy.Deleted)
			{
				flag1 = (flag1 | SaveFlag.HeldBy);
 
			}
			if (this.m_SavedFlags != 0)
			{
				flag1 = (flag1 | SaveFlag.SavedFlags);
 
			}
			if (this.m_Weight == 0)
			{
				flag1 = (flag1 | SaveFlag.WeightIs0);
 
			}
			else
			{
				if (this.m_Weight != 1)
				{
					if (this.m_Weight == ((double) ((int) this.m_Weight)))
					{
						flag1 = (flag1 | SaveFlag.IntWeight);
						goto L_01E8;
 
					}
					flag1 = (flag1 | SaveFlag.WeightNot1or0);
 
				}
 
			}
 
			L_01E8:
				flag2 = (this.m_Flags & (ImplFlag.PayedInsurance | ImplFlag.Insured | ImplFlag.Stackable | ImplFlag.Movable | ImplFlag.Visible));
			if (flag2 != (ImplFlag.Visible | ImplFlag.Movable))
			{
				flag1 = (flag1 | SaveFlag.ImplFlags);
 
			}
			writer.Write((byte)flag1);
			num4 = this.m_LastMovedTime.Ticks;
			time1 = DateTime.Now;
			num5 = time1.Ticks;
			try
			{
				span1 = new TimeSpan(num4 - num5);
 
			}
			catch /*(Exception ignored)*/
			{
				if (num4 < num5)
				{
					span1 = TimeSpan.MaxValue;
				}
				else
				{
					span1 = TimeSpan.MaxValue;
				}
			}
			num6 = -span1.TotalMinutes;
			if (num6 < -2147483648)
			{
				num6 = -2147483648;
			}
			else
			{
				if (num6 > 2147483647)
				{
					num6 = 2147483647;
				}
			}
			writer.WriteEncodedInt(((int) num6));
			if (Item.GetSaveFlag(flag1, SaveFlag.Direction))
			{
				writer.Write((byte)this.m_Direction);
			}
			if (Item.GetSaveFlag(flag1, SaveFlag.Bounce))
			{
				BounceInfo.Serialize(this.m_Bounce, writer);
 
			}
			if (Item.GetSaveFlag(flag1, SaveFlag.LootType))
			{
				writer.Write((byte)this.m_LootType);
 
			}
			if (Item.GetSaveFlag(flag1, SaveFlag.LocationFull))
			{
				writer.WriteEncodedInt(num1);
				writer.WriteEncodedInt(num2);
				writer.WriteEncodedInt(num3);
 
			}
			else
			{
				if (Item.GetSaveFlag(flag1, SaveFlag.LocationByteXY))
				{
					writer.Write(((byte) num1));
					writer.Write(((byte) num2));
 
				}
				else
				{
					if (Item.GetSaveFlag(flag1, SaveFlag.LocationShortXY))
					{
						writer.Write(((short) num1));
						writer.Write(((short) num2));
 
					}
 
				}
				if (Item.GetSaveFlag(flag1, SaveFlag.LocationSByteZ))
				{
					writer.Write(((sbyte) num3));
 
				}
 
			}
			if (Item.GetSaveFlag(flag1, SaveFlag.ItemID))
			{
				writer.WriteEncodedInt(this.m_ItemID);
 
			}
			if (Item.GetSaveFlag(flag1, SaveFlag.Hue))
			{
				writer.WriteEncodedInt(this.m_Hue);
 
			}
			if (Item.GetSaveFlag(flag1, SaveFlag.Amount))
			{
				writer.WriteEncodedInt(this.m_Amount);
 
			}
			if (Item.GetSaveFlag(flag1, SaveFlag.Layer))
			{
				writer.Write((byte)this.m_Layer);
 
			}
			if (Item.GetSaveFlag(flag1, SaveFlag.Name))
			{
				writer.Write(this.m_Name);
 
			}
			if (Item.GetSaveFlag(flag1, SaveFlag.Parent))
			{
				if (((this.m_Parent as Mobile) != null) && !((Mobile) this.m_Parent).Deleted)
				{
					writer.Write(((Mobile) this.m_Parent).Serial);
 
				}
				else
				{
					if (((this.m_Parent as Item) != null) && !((Item) this.m_Parent).Deleted)
					{
						writer.Write(((Item) this.m_Parent).Serial);
						goto L_044C;
 
					}
					writer.Write(Serial.MinusOne);
 
				}
 
			}
 
			L_044C:
				if (Item.GetSaveFlag(flag1, SaveFlag.Items))
				{
					writer.WriteItemList(this.m_Items, false);
 
				}
			if (Item.GetSaveFlag(flag1, SaveFlag.IntWeight))
			{
				writer.WriteEncodedInt(((int) this.m_Weight));
 
			}
			else
			{
				if (Item.GetSaveFlag(flag1, SaveFlag.WeightNot1or0))
				{
					writer.Write(this.m_Weight);
 
				}
 
			}
			if (Item.GetSaveFlag(flag1, SaveFlag.Map))
			{
				writer.Write(this.m_Map);
 
			}
			if (Item.GetSaveFlag(flag1, SaveFlag.ImplFlags))
			{
				writer.WriteEncodedInt((byte)flag2);
 
			}
			if (Item.GetSaveFlag(flag1, SaveFlag.InsuredFor))
			{
				writer.Write((Mobile)null);
			}
			if (Item.GetSaveFlag(flag1, SaveFlag.BlessedFor))
			{
				writer.Write(this.m_BlessedFor);
 
			}
			if (Item.GetSaveFlag(flag1, SaveFlag.HeldBy))
			{
				writer.Write(this.m_HeldBy);
 
			}
			if (Item.GetSaveFlag(flag1, SaveFlag.SavedFlags))
			{
				writer.WriteEncodedInt(this.m_SavedFlags);
 
			}
 
		}
		
		private void SetFlag(ImplFlag flag, bool value)
		{
			if (value)
			{
				this.m_Flags = (this.m_Flags | flag);
				return; 
			}
			//TRF this.m_Flags = (this.m_Flags & !flag);
			this.m_Flags = (this.m_Flags & ~flag);
		} 

		public void SetLastMoved()
		{
			this.m_LastMovedTime = DateTime.Now;
 
		} 

		public void SetSavedFlag(int flag, bool value)
		{
			if (value)
			{
				this.m_SavedFlags = (this.m_SavedFlags | flag);
				return; 
			}
			//TRF this.m_SavedFlags = (this.m_SavedFlags & !flag);
			this.m_SavedFlags = (this.m_SavedFlags & ~flag);
		} 

		private static void SetSaveFlag(ref SaveFlag flags, SaveFlag toSet, bool setIf)
		{
			if (setIf)
			{
				flags = (flags | toSet);
			}
		} 

		public void SetTempFlag(int flag, bool value)
		{
			if (value)
			{
				this.m_TempFlags = (this.m_TempFlags | flag);
				return; 
			}
			//TRF this.m_TempFlags = (this.m_TempFlags & !flag);
			this.m_TempFlags = (this.m_TempFlags & ~flag);
		} 

		public void SetTotalGold(int value)
		{
			this.m_TotalGold = value;
		} 

		public void SetTotalItems(int value)
		{
			this.m_TotalItems = value;
		} 

		public void SetTotalWeight(int value)
		{
			this.m_TotalWeight = value;
		} 

		public virtual bool StackWith(Mobile from, Item dropped)
		{
			return this.StackWith(from, dropped, true); 
		} 

		public virtual bool StackWith(Mobile from, Item dropped, bool playSound)
		{
			int num1;
			if (((this.Stackable && dropped.Stackable) && ((dropped.GetType() == base.GetType()) && (dropped.ItemID == this.ItemID))) && ((dropped.Hue == this.Hue) && ((dropped.Amount + this.Amount) <= 60000)))
			{
				if (this.m_LootType != dropped.m_LootType)
				{
					this.m_LootType = 0;
 
				}
				this.Amount = (this.Amount + dropped.Amount);
				dropped.Delete();
				if (playSound)
				{
					num1 = this.GetDropSound();
					if (num1 == -1)
					{
						num1 = 66;
 
					}
					from.SendSound(num1, this.GetWorldLocation());
 
				}
				return true; 
			}
			return false; 
		} 

		public override string ToString()
		{
			return string.Format("0x{0:X} \"{1}\"", this.m_Serial.Value, base.GetType().Name); 
		} 

		public virtual void UpdateTotals()
		{
			int num1;
			Item item1;
			if (this.m_Items == null)
			{
				return; 
			}
			this.m_TotalGold = 0;
			this.m_TotalItems = 0;
			this.m_TotalWeight = 0;
			for (num1 = 0; (num1 < this.m_Items.Count); num1 = (num1 + 1))
			{
				item1 = ((Item) this.m_Items[num1]);
				item1.UpdateTotals();
				this.m_TotalGold = (this.m_TotalGold + item1.TotalGold);
				this.m_TotalItems = (this.m_TotalItems + item1.TotalItems);
				this.m_TotalWeight = (this.m_TotalWeight + (item1.TotalWeight + item1.PileWeight));
				if (item1.IsVirtualItem)
				{
					this.m_TotalItems = (this.m_TotalItems - 1);
 
				}
 
			}
			this.m_TotalItems = (this.m_TotalItems + this.m_Items.Count);
 
		} 

		public virtual bool VerifyMove(Mobile from)
		{
			return this.Movable; 
		} 

		public int Amount 
		{ 
			get
			{
				return this.m_Amount;
			}
			set
			{
				int num1;
				int num2;
				Item item1;
				Mobile mobile1;
				num1 = this.m_Amount;
				if (num1 != value)
				{
					num2 = this.PileWeight;
					this.m_Amount = value;
					this.m_WorldPacket = null;
					if ((this.m_Parent as Item) != null)
					{
						item1 = ((Item) this.m_Parent);
						item1.TotalWeight = ((item1.TotalWeight - num2) + this.PileWeight);
 
					}
					else
					{
						if (((this.m_Parent as Mobile) != null) && ((this as BankBox) == null))
						{
							mobile1 = ((Mobile) this.m_Parent);
							mobile1.TotalWeight = ((mobile1.TotalWeight - num2) + this.PileWeight);
 
						}
 
					}
					this.OnAmountChange(num1);
					this.Delta(ItemDelta.Update);
					if ((num1 > 1) || (value > 1))
					{
						this.InvalidateProperties();
 
					}
					if (!this.Stackable && (this.m_Amount > 1))
					{
						Console.WriteLine("Warning: 0x{0:X}: Amount changed for non-stackable item '{2}'. ({1})", this.m_Serial.Value, this.m_Amount, base.GetType().Name);
 
					}
 
				}
			}
		}
		public Mobile BlessedFor 
		{ 
			get
			{
				return this.m_BlessedFor;
			}
			set
			{
				this.m_BlessedFor = value;
				this.InvalidateProperties();
			}
		}
		public virtual bool BlocksFit 
		{ 
			get
			{
				return false;

			}
		}
		public virtual bool CanTarget 
		{ 
			get
			{
				return true;
			}
		}
		public virtual int ColdResistance 
		{ 
			get
			{
				return 0;
			}
		}
		public virtual bool Decays 
		{ 
			get
			{
				if (this.Movable)
				{
					return this.Visible;
				}
				return false;
			}
		}
		public virtual TimeSpan DecayTime 
		{ 
			get
			{
				return Item.m_DDT;
			}
		}
		public static TimeSpan DefaultDecayTime 
		{ 
			get
			{
				return Item.m_DDT;
			}
			set
			{
				Item.m_DDT = value;
			}
		}
		public bool Deleted 
		{ 
			get
			{
				return this.GetFlag(ImplFlag.Deleted);
			}
		}
		public Direction Direction 
		{ 
			get
			{
				return this.m_Direction;
			}
			set
			{
				if (this.m_Direction != value)
				{
					this.m_Direction = value;
					this.m_WorldPacket = null;
					this.Delta(ItemDelta.Update);
				}
			}
		}
		public virtual bool DisplayLootType 
		{ 
			get
			{
				return true;
			}
		}
		public virtual int EnergyResistance 
		{ 
			get
			{
				return 0;
			}
		}
		public virtual int FireResistance 
		{ 
			get
			{
				return 0;
			}
		}
		public virtual bool ForceShowProperties 
		{ 
			get
			{
				return false;
			}
		}
		public virtual bool HandlesOnMovement 
		{ 
			get
			{
				return false;
			}
		}
		public virtual bool HandlesOnSpeech 
		{ 
			get
			{
				return false;
			}
		}
		public Mobile HeldBy 
		{ 
			get
			{
				return this.m_HeldBy;
			}
			set
			{
				this.m_HeldBy = value;
			}
		}
		public virtual int Hue 
		{ 
			get
			{
				return this.m_Hue;
			}
			set
			{
				if (this.m_Hue != value)
				{
					this.m_Hue = value;
					this.m_WorldPacket = null;
					this.Delta(ItemDelta.Update);
				}
			}
		}
		public virtual int HuedItemID 
		{ 
			get
			{
				return (this.m_ItemID & 16383);
			}
		}
		public bool InSecureTrade 
		{ 
			get
			{
				return (this.GetSecureTradeCont() != null);
			}
		}
		public bool Insured 
		{ 
			get
			{
				return this.GetFlag(ImplFlag.Insured);
			}
			set
			{
				this.SetFlag(ImplFlag.Insured, value);
				this.InvalidateProperties();
			}
		}
		public bool IsLockedDown 
		{ 
			get
			{
				return this.GetTempFlag(Item.m_LockedDownFlag);
			}
			set
			{
				this.SetTempFlag(Item.m_LockedDownFlag, value);
				this.InvalidateProperties();
			}
		}
		public bool IsSecure 
		{ 
			get
			{
				return this.GetTempFlag(Item.m_SecureFlag);
			}
			set
			{
				this.SetTempFlag(Item.m_SecureFlag, value);
				this.InvalidateProperties();
			}
		}
		public virtual bool IsVirtualItem 
		{ 
			get
			{
				return false;
			}
		}
		public ItemData ItemData 
		{ 
			get
			{
				return TileData.ItemTable[(this.m_ItemID & 16383)];
			}
		}
		public virtual int ItemID 
		{ 
			get
			{
				return this.m_ItemID;
			}
			set
			{
				if (this.m_ItemID != value)
				{
					this.m_ItemID = value;
					this.m_WorldPacket = null;
					this.InvalidateProperties();
					this.Delta(ItemDelta.Update);
				}
			}
		}
		public ArrayList Items 
		{ 
			get
			{
				return this.m_Items;
			}
		}
		public virtual int LabelNumber 
		{ 
			get
			{
				return (1020000 + (this.m_ItemID & 16383)); 
			}
		}
		public DateTime LastMoved 
		{ 
			get
			{
				return this.m_LastMovedTime;
			}
			set
			{
				this.m_LastMovedTime = value;
			}
		}
		public virtual Layer Layer 
		{ 
			get
			{
				return this.m_Layer;
			}
			set
			{
				if (this.m_Layer != value)
				{
					this.m_Layer = value;
					this.Delta(ItemDelta.EquipOnly);
				}
			}
		}
		public LightType Light 
		{ 
			get
			{
				return (LightType)this.m_Direction;
			}
			set
			{
				if ((LightType)this.m_Direction != value)
				{
					this.m_Direction = ((Direction) value);
					this.m_WorldPacket = null;
					this.Delta(ItemDelta.Update);
				}
			}
		}
		public virtual Point3D Location 
		{ 
			get
			{
				return this.m_Location;
			}
			set
			{
				Point3D pointd1;
				IPooledEnumerable enumerable1;
				Packet packet1;
				NetState state1;
				Mobile mobile1;
				NetState state2;
				Mobile mobile2;
				IEnumerator enumerator1;
				IDisposable disposable1;
				pointd1 = this.m_Location;
				if (pointd1 == value)
				{
					goto L_01B6;
 
				}
				if (this.m_Map != null)
				{
					if (this.m_Parent == null)
					{
						if (this.m_Location.m_X != 0)
						{
							packet1 = null;
							enumerable1 = this.m_Map.GetClientsInRange(pointd1, this.GetMaxUpdateRange());
							enumerator1 = enumerable1.GetEnumerator();
							try
							{
								while (enumerator1.MoveNext())
								{
									state1 = ((NetState) enumerator1.Current);
									mobile1 = state1.Mobile;
									if (!mobile1.InRange(value, this.GetUpdateRange(mobile1)))
									{
										if (packet1 == null)
										{
											packet1 = this.RemovePacket;
 
										}
										state1.Send(packet1);
 
									}
 
								}
 
							}
							finally
							{
								disposable1 = (enumerator1 as IDisposable);
								if (disposable1 != null)
								{
									disposable1.Dispose();
 
								}
 
							}
							enumerable1.Free();
 
						}
						this.m_Location = value;
						this.m_WorldPacket = null;
						this.SetLastMoved();
						enumerable1 = this.m_Map.GetClientsInRange(this.m_Location, this.GetMaxUpdateRange());
						enumerator1 = enumerable1.GetEnumerator();
						try
						{
							while (enumerator1.MoveNext())
							{
								state2 = ((NetState) enumerator1.Current);
								mobile2 = state2.Mobile;
								if (mobile2.CanSee(this) && mobile2.InRange(this.m_Location, this.GetUpdateRange(mobile2)))
								{
									this.SendInfoTo(state2);
 
								}
 
							}
 
						}
						finally
						{
							disposable1 = (enumerator1 as IDisposable);
							if (disposable1 != null)
							{
								disposable1.Dispose();
 
							}
 
						}
						enumerable1.Free();
						this.RemDelta(ItemDelta.Update);
 
					}
					else
					{
						if ((this.m_Parent as Item) != null)
						{
							this.m_Location = value;
							this.m_WorldPacket = null;
							this.Delta(ItemDelta.Update);
						}
						else
						{
							this.m_Location = value;
							this.m_WorldPacket = null;
						}
					}
					if (this.m_Parent != null)
					{
						goto L_01AF;
 
					}
					this.m_Map.OnMove(pointd1, this);
 
				}
				else
				{
					this.m_Location = value;
					this.m_WorldPacket = null;
 
				}
 
				L_01AF:
					this.OnLocationChange(pointd1);
 
				L_01B6:
					return; 

			}
		}
		public static int LockedDownFlag 
		{ 
			get
			{
				return Item.m_LockedDownFlag;
			}
			set
			{
				Item.m_LockedDownFlag = value;
			}
		}
		public LootType LootType 
		{ 
			get
			{
				return this.m_LootType;
			}
			set
			{
				if (this.m_LootType != value)
				{
					this.m_LootType = value;
					if (this.DisplayLootType)
					{
						this.InvalidateProperties();
 
					}
 
				}

			}
		}
		public Map Map 
		{ 
			get
			{
				return this.m_Map;
			}
			set
			{
				Map map1;
				int num1;
				if (this.m_Map != value)
				{
					map1 = this.m_Map;
					if (this.m_Map != null)
					{
						this.m_Map.OnLeave(this);
						if (this.m_Parent == null)
						{
							this.SendRemovePacket();
 
						}
 
					}
					if (this.m_Items == null)
					{
						throw new Exception(string.Format("Items array is null--are you calling the serialization constructor? Type={0}", base.GetType()));
 
					}
					for (num1 = 0; (num1 < this.m_Items.Count); num1 = (num1 + 1))
					{
						((Item) this.m_Items[num1]).Map = value;
 
					}
					this.m_Map = value;
					if ((this.m_Map != null) && (this.m_Parent == null))
					{
						this.m_Map.OnEnter(this);
 
					}
					this.Delta(ItemDelta.Update);
					this.OnMapChange();
					if ((map1 == null) || (map1 == Map.Internal))
					{
						this.InvalidateProperties();
					}
				}
			}
		}
		public bool Movable 
		{ 
			get
			{
				return this.GetFlag(ImplFlag.Movable);
			}
			set
			{
				if (this.GetFlag(ImplFlag.Movable) != value)
				{
					this.SetFlag(ImplFlag.Movable, value);
					this.m_WorldPacket = null;
					this.Delta(ItemDelta.Update);
				}
			}
		}
		public virtual string Name 
		{ 
			get
			{
				return this.m_Name;
			}
			set
			{
				this.m_Name = value;
				this.InvalidateProperties();

			}
		}
		public bool Newbied 
		{ 
			get
			{
				return (this.m_LootType == LootType.Newbied);
			}
			set
			{
				this.m_LootType = (value ? LootType.Newbied : 0);
			}
		}
		public Packet OPLPacket 
		{ 
			get
			{
				if (this.m_OPLPacket == null)
				{
					this.m_OPLPacket = new OPLInfo(this.PropertyList);
 
				}
				return this.m_OPLPacket;
			}
		}
		public virtual object Parent 
		{ 
			get
			{
				return this.m_Parent; 

			}
			set
			{
				object obj1;
				if (this.m_Parent == value)
				{
					return; 
				}
				obj1 = this.m_Parent;
				this.m_Parent = value;
				if (this.m_Map != null)
				{
					if ((obj1 != null) && (this.m_Parent == null))
					{
						this.m_Map.OnEnter(this);
						return; 
					}
					if (this.m_Parent != null)
					{
						this.m_Map.OnLeave(this);
 
					}
 
				}			
			}
		}
		public bool PayedInsurance 
		{ 
			get
			{
				return this.GetFlag(ImplFlag.PayedInsurance);
			}
			set
			{
				this.SetFlag(ImplFlag.PayedInsurance,value);
			}
		}
		public virtual int PhysicalResistance 
		{ 
			get
			{
				return 0;
			}
		}
		public int PileWeight 
		{ 
			get
			{
				return ((int) Math.Ceiling((this.m_Weight * ((double) this.m_Amount))));
			}
		}
		public virtual int PoisonResistance 
		{ 
			get
			{
				return 0;
			}
		}
		public ObjectPropertyList PropertyList 
		{ 
			get
			{
				if (this.m_PropertyList == null)
				{
					this.m_PropertyList = new ObjectPropertyList(this);
					this.GetProperties(this.m_PropertyList);
					this.m_PropertyList.Terminate();
 
				}
				return this.m_PropertyList;
			}
		}
		public Packet RemovePacket 
		{ 
			get
			{
				if (this.m_RemovePacket == null)
				{
					this.m_RemovePacket = new Network.RemoveItem(this);
 
				}
				return this.m_RemovePacket;
			}
		}
		public object RootParent 
		{ 
			get
			{
				object obj1;
				Item item1;
				for (obj1 = this.m_Parent; ((obj1 as Item) != null); obj1 = item1.m_Parent)
				{
					item1 = ((Item) obj1);
					if (item1.m_Parent == null)
					{
						goto L_0027;
 
					}
 
				}
 
				L_0027:
					return obj1; 

			}
		}
		public int SavedFlags 
		{ 
			get
			{
				return this.m_SavedFlags;
			}
			set
			{
				this.m_SavedFlags = value;
			}
		}
		public static bool ScissorCopyLootType 
		{ 
			get
			{
				return Item.m_ScissorCopyLootType;
			}
			set
			{
				Item.m_ScissorCopyLootType = value;
			}
		}
		public static int SecureFlag 
		{ 
			get
			{
				return Item.m_SecureFlag;
			}
			set
			{
				Item.m_SecureFlag = value;
			}
		}
		public Serial Serial 
		{ 
			get
			{
				return this.m_Serial;
			}
		}

		/*TRF: private*/ Point3D Server.IEntity.Location 
						 { 
							 get
							 {
								 return this.m_Location;
							 }
						 }

		public bool Stackable 
		{ 
			get
			{
				return this.GetFlag(ImplFlag.Stackable);
			}
			set
			{
				this.SetFlag(ImplFlag.Stackable, value);
			}
		}
		public int TempFlags 
		{ 
			get
			{
				return this.m_TempFlags;
			}
			set
			{
				this.m_TempFlags = value;
			}
		}
		public int TotalGold 
		{ 
			get
			{
				return this.m_TotalGold;
			}
			set
			{
				Item item1;
				Mobile mobile1;
				if (this.m_TotalGold != value)
				{
					if ((this.m_Parent as Item) != null)
					{
						item1 = ((Item) this.m_Parent);
						item1.TotalGold = ((item1.TotalGold - this.m_TotalGold) + value);
 
					}
					else
					{
						if (((this.m_Parent as Mobile) != null) && ((this as BankBox) == null))
						{
							mobile1 = ((Mobile) this.m_Parent);
							mobile1.TotalGold = ((mobile1.TotalGold - this.m_TotalGold) + value);
 
						}
 
					}
					this.m_TotalGold = value;
 
				}
			}
		}
		public int TotalItems 
		{ 
			get
			{
				return this.m_TotalItems;
			}
			set
			{
				Item item1;
				if (this.m_TotalItems != value)
				{
					if ((this.m_Parent as Item) != null)
					{
						item1 = ((Item) this.m_Parent);
						item1.TotalItems = ((item1.TotalItems - this.m_TotalItems) + value);
						item1.InvalidateProperties();
 
					}
					this.m_TotalItems = value;
					this.InvalidateProperties();
 
				}			
			}
		}
		public int TotalWeight 
		{ 
			get
			{
				return this.m_TotalWeight;
			}
			set
			{
				Item item1;
				Mobile mobile1;
				if (this.m_TotalWeight != value)
				{
					if ((this.m_Parent as Item) != null)
					{
						item1 = ((Item) this.m_Parent);
						item1.TotalWeight = ((item1.TotalWeight - this.m_TotalWeight) + value);
						item1.InvalidateProperties();
 
					}
					else
					{
						if (((this.m_Parent as Mobile) != null) && ((this as BankBox) == null))
						{
							mobile1 = ((Mobile) this.m_Parent);
							mobile1.TotalWeight = ((mobile1.TotalWeight - this.m_TotalWeight) + value);
 
						}
 
					}
					this.m_TotalWeight = value;
					this.InvalidateProperties();
 
				}
			}
		}
		public bool Visible 
		{ 
			get
			{
				return this.GetFlag(ImplFlag.Visible);
			}
			set
			{
				Packet packet1;
				Point3D pointd1;
				IPooledEnumerable enumerable1;
				NetState state1;
				Mobile mobile1;
				IEnumerator enumerator1;
				IDisposable disposable1;
				if (this.GetFlag(ImplFlag.Visible) != value)
				{
					this.SetFlag(ImplFlag.Visible, value);
					this.m_WorldPacket = null;
					if (this.m_Map != null)
					{
						packet1 = null;
						pointd1 = this.GetWorldLocation();
						enumerable1 = this.m_Map.GetClientsInRange(pointd1, this.GetMaxUpdateRange());
						enumerator1 = enumerable1.GetEnumerator();
						try
						{
							while (enumerator1.MoveNext())
							{
								state1 = ((NetState) enumerator1.Current);
								mobile1 = state1.Mobile;
								if (!mobile1.CanSee(this) && mobile1.InRange(pointd1, this.GetUpdateRange(mobile1)))
								{
									if (packet1 == null)
									{
										packet1 = this.RemovePacket;
 
									}
									state1.Send(packet1);
 
								}
 
							}
 
						}
						finally
						{
							disposable1 = (enumerator1 as IDisposable);
							if (disposable1 != null)
							{
								disposable1.Dispose();
 
							}
 
						}
						enumerable1.Free();
					}
					this.Delta(ItemDelta.Update);
				}
			}
		}
		public double Weight 
		{ 
			get
			{
				return this.m_Weight;
			}
			set
			{
				int num1;
				Item item1;
				Mobile mobile1;
				if (this.m_Weight != value)
				{
					num1 = this.PileWeight;
					this.m_Weight = value;
					if ((this.m_Parent as Item) != null)
					{
						item1 = ((Item) this.m_Parent);
						item1.TotalWeight = ((item1.TotalWeight - num1) + this.PileWeight);
						item1.InvalidateProperties();
 
					}
					else
					{
						if (((this.m_Parent as Mobile) != null) && ((this as BankBox) == null))
						{
							mobile1 = ((Mobile) this.m_Parent);
							mobile1.TotalWeight = ((mobile1.TotalWeight - num1) + this.PileWeight);
 
						}
 
					}
					this.InvalidateProperties();
 
				}
			}
		}
		public Packet WorldPacket 
		{ 
			get
			{
				if (this.m_WorldPacket == null)
				{
					this.m_WorldPacket = new WorldItem(this);
 
				}
				return this.m_WorldPacket; 

			}
		}
		public virtual int X 
		{ 
			get
			{
				return this.m_Location.m_X;
			}
			set
			{
				this.Location = new Point3D(value, this.m_Location.m_Y, this.m_Location.m_Z);
			}
		}
		public virtual int Y 
		{ 
			get
			{
				return this.m_Location.m_Y;
			}
			set
			{
				this.Location = new Point3D(this.m_Location.m_X, value, this.m_Location.m_Z);
			}
		}
		public virtual int Z 
		{ 
			get
			{
				return this.m_Location.m_Z;
			}
			set
			{
				this.Location = new Point3D(this.m_Location.m_X, this.m_Location.m_Y, value);
			}
		}

		private int m_Amount;
		private Mobile m_BlessedFor;
		private BounceInfo m_Bounce;
		private static TimeSpan m_DDT = TimeSpan.FromHours(1);
		private ItemDelta m_DeltaFlags;
		private static ArrayList m_DeltaQueue = new ArrayList();
		private Direction m_Direction;
		private ImplFlag m_Flags;
		private Mobile m_HeldBy;
		private int m_Hue;
		private int m_ItemID;
		private ArrayList m_Items;
		private DateTime m_LastMovedTime;
		private Layer m_Layer;
		private Point3D m_Location;
		private static int m_LockedDownFlag;
		private LootType m_LootType;
		private Map m_Map;
		private string m_Name;
		private static Int32 m_OpenSlots;
		private Packet m_OPLPacket;
		private object m_Parent;
		private ObjectPropertyList m_PropertyList;
		private Packet m_RemovePacket;
		private int m_SavedFlags;
		private static bool m_ScissorCopyLootType;
		private static int m_SecureFlag;
		private Serial m_Serial;
		private int m_TempFlags;
		private int m_TotalGold;
		private int m_TotalItems;
		private int m_TotalWeight;
		private double m_Weight;
		private Packet m_WorldPacket;

		private enum ImplFlag
		{
			None = 0,
			Visible = 1,
			Movable = 2,
			Deleted = 4,
			Stackable = 8,
			InQueue = 16,
			Insured = 32,
			PayedInsurance = 64,
		}

		private enum SaveFlag
		{
			None = 0,
			Direction = 1,
			Bounce = 2,
			LootType = 4,
			LocationFull = 8,
			ItemID = 16,
			Hue = 32,
			Amount = 64,
			Layer = 128,
			Name = 256,
			Parent = 512,
			Items = 1024,
			WeightNot1or0 = 2048,
			Map = 4096,
			Visible = 8192,
			Movable = 16384,
			Stackable = 32768,
			WeightIs0 = 65536,
			LocationSByteZ = 131072,
			LocationShortXY = 262144,
			LocationByteXY = 524288,
			ImplFlags = 1048576,
			InsuredFor = 2097152,
			BlessedFor = 4194304,
			HeldBy = 8388608,
			IntWeight = 16777216,
			SavedFlags = 33554432,
		}
	}
}