using System;
using System.Collections;

using Server;
using Server.Network;

namespace Server.Items
{
	public delegate int CheckItemGroup(Item a, Item b);
	public delegate void ContainerSnoopHandler(Container cont, Mobile from);
	public delegate void OnItemConsumed(Item item, int amount);

	public class BankBox : Container
	{
		public BankBox(Mobile owner) : base(3649)
		{
			base.Layer = Layer.Bank;
			base.Movable = false;
			this.m_Owner = owner;
		}

		public BankBox(Serial serial) : base(serial)
		{
		}

		public void Close()
		{
			this.m_Open = false;
			if ((this.m_Owner != null) && BankBox.m_SendRemovePacket)
			{
				this.m_Owner.Send(base.RemovePacket);
 
			}
 
		} 

		public override void Deserialize(GenericReader reader)
		{
			int num1;
			int num2;
			base.Deserialize(reader);
			num1 = reader.ReadInt();
			num2 = num1;
			if (num2 != 0)
			{
				return; 
			}
			this.m_Owner = reader.ReadMobile();
			this.m_Open = reader.ReadBool();
			if (this.m_Owner == null)
			{
				base.Delete();
 
			}
 
		} 

		public override void OnDoubleClick(Mobile from)
		{  
		} 

		public override bool OnDragDrop(Mobile from, Item dropped)
		{
			if ((from != this.m_Owner) || !this.m_Open)
			{
				return false; 
			}
			return base.OnDragDrop(from, dropped); 
		} 

		public override DeathMoveResult OnParentDeath(Mobile parent)
		{
			return DeathMoveResult.RemainEquiped; 
		} 

		public override void OnSingleClick(Mobile from)
		{  
		} 

		public void Open()
		{
			this.m_Open = true;
			if (this.m_Owner != null)
			{
				this.m_Owner.PrivateOverheadMessage(0, 946, 1, string.Format("Bank container has {0} items, {1} stones", base.TotalItems, base.TotalWeight), this.m_Owner.NetState);
				this.m_Owner.Send(new EquipUpdate(this));
				base.DisplayTo(this.m_Owner);
 
			}
 
		} 

		public override void Serialize(GenericWriter writer)
		{
			base.Serialize(writer);
			writer.Write(0);
			writer.Write(this.m_Owner);
			writer.Write(this.m_Open);
 
		} 
		
		public override Rectangle2D Bounds 
		{ 
			get
			{
				return new Rectangle2D(18, 105, 144, 73); 
			}
		}

		public override int DefaultDropSound 
		{ 
			get
			{
				return 66;
			}
		}

		public override int DefaultGumpID 
		{ 
			get
			{
				return 74;
			}
		}

		public override int MaxWeight 
		{ 
			get
			{
				return 0;
			}
		}

		public bool Opened 
		{ 
			get
			{
				return this.m_Open;
			}
		}

		public Mobile Owner 
		{ 
			get
			{
				return this.m_Owner;
			}
		}

		public static bool SendDeleteOnClose 
		{ 
			get
			{
				return BankBox.m_SendRemovePacket;
			}
			set
			{
				BankBox.m_SendRemovePacket = value;
			}
		}
		
		private bool m_Open;
		private Mobile m_Owner;
		private static bool m_SendRemovePacket;
	}
 
	public class BaseMulti : Item
	{
		public BaseMulti(int itemID) : base(itemID)
		{
			base.Movable = false;
		}

		public BaseMulti(Serial serial) : base(serial)
		{
		}

		public virtual bool Contains(int x, int y)
		{
			MultiComponentList list1;
			Point2D pointd1;
			list1 = this.Components;
			pointd1 = list1.Min;
			x = (x - (base.X + pointd1.m_X));
			pointd1 = list1.Min;
			y = (y - (base.Y + pointd1.m_Y));
			if (((x >= 0) && (x < list1.Width)) && ((y >= 0) && (y < list1.Height)))
			{
				return (list1.Tiles[x][y].Length > 0); 
			}
			return false; 
		} 

		public virtual bool Contains(IPoint3D p)
		{
			return this.Contains(p.X, p.Y); 
		}
		
		public bool Contains(Item item)
		{
			if (item.Map == base.Map)
			{
				return this.Contains(item.X, item.Y); 
			}
			return false; 
		} 

		public bool Contains(Mobile m)
		{
			if (m.Map == base.Map)
			{
				return this.Contains(m.X, m.Y); 
			}
			return false; 
		} 

		public virtual bool Contains(Point2D p)
		{
			return this.Contains(p.m_X, p.m_Y); 
		} 

		public virtual bool Contains(Point3D p)
		{
			return this.Contains(p.m_X, p.m_Y); 
		} 

		public override void Deserialize(GenericReader reader)
		{
			base.Deserialize(reader);
			reader.ReadInt();
 
		} 

		public override int GetMaxUpdateRange()
		{
			return 22; 
		} 

		public override int GetUpdateRange(Mobile m)
		{
			return 22; 
		} 

		public virtual void RefreshComponents()
		{
			Map map1;
			if (base.Parent != null)
			{
				return; 
			}
			map1 = base.Map;
			if (map1 != null)
			{
				map1.OnLeave(this);
				map1.OnEnter(this);
 
			}
 
		} 

		public override void Serialize(GenericWriter writer)
		{
			base.Serialize(writer);
			writer.Write(0);
 
		} 

		public virtual MultiComponentList Components 
		{ 
			get
			{
				return MultiData.GetComponents(base.ItemID);
			}
		}

		public override int LabelNumber 
		{ 
			get
			{
				MultiComponentList list1;
				list1 = this.Components;
				if (list1.List.Length > 0)
				{
					return (1020000 + (list1.List[0].m_ItemID & 16383)); 
				}
				return base.LabelNumber;
			}
		}
	}
 
	public class Container : Item
	{
		public Container(int itemID) : base(itemID)
		{
			this.m_GumpID = -1;
			this.m_DropSound = -1;
			this.m_MaxItems = -1;
		}

		public Container(Serial serial) : base(serial)
		{
		}

		public virtual bool CheckContentDisplay(Mobile from)
		{
			object obj1;
			if (!this.DisplaysContent)
			{
				return false; 
			}
			obj1 = base.RootParent;
			if (((obj1 == null) || ((obj1 as Item) != null)) || ((obj1 == from) || (from.AccessLevel > 0)))
			{
				return true; 
			}
			return false; 
		} 

		public virtual bool CheckHold(Mobile m, Item item, bool message)
		{
			return this.CheckHold(m, item, message, true); 
		} 

		public virtual bool CheckHold(Mobile m, Item item, bool message, bool checkItems)
		{
			object obj1;
			Container container1;
			int num1;
			int num2;
			if (m.AccessLevel >= AccessLevel.GameMaster)
			{
				return true; 
			}
			obj1 = this;
			while ((obj1 != null))
			{
				if ((obj1 as Container) != null)
				{
					container1 = (obj1 as Container);
					num1 = container1.MaxItems;
					if ((checkItems && (num1 != 0)) && (((container1.TotalItems + item.TotalItems) + (item.IsVirtualItem ? 0 : 1)) > num1))
					{
						if (message)
						{
							this.SendFullItemsMessage(m, item);
 
						}
						return false; 
					}
					num2 = container1.MaxWeight;
					if ((num2 != 0) && (((container1.TotalWeight + item.TotalWeight) + item.PileWeight) > num2))
					{
						if (message)
						{
							this.SendFullWeightMessage(m, item);
 
						}
						return false; 
					}
					obj1 = container1.Parent;
 
				}
				else
				{
					if ((obj1 as Item) == null)
					{
						goto L_00AA;
 
					}
					obj1 = ((Item) obj1).Parent;
 
				}
 
			}
 
			L_00AA:
				return true; 
		} 

		public bool ConsumeTotal(Type type, int amount)
		{
			return this.ConsumeTotal(type, amount, true, null); 
		} 

		public bool ConsumeTotal(Type type, int amount, bool recurse)
		{
			return this.ConsumeTotal(type, amount, recurse, null); 
		} 

		public bool ConsumeTotal(Type type, int amount, bool recurse, OnItemConsumed callback)
		{
			Item[] array1;
			int num1;
			int num2;
			int num3;
			int num4;
			Item item1;
			int num5;
			array1 = this.FindItemsByType(type, recurse);
			num1 = 0;
			for (num2 = 0; (num2 < array1.Length); num2 = (num2 + 1))
			{
				num1 = (num1 + array1[num2].Amount);
 
			}
			if (num1 >= amount)
			{
				num3 = amount;
				for (num4 = 0; (num4 < array1.Length); num4 = (num4 + 1))
				{
					item1 = array1[num4];
					num5 = item1.Amount;
					if (num5 < num3)
					{
						if (callback != null)
						{
							callback(item1, num5);
 
						}
						item1.Delete();
						num3 = (num3 - num5);
 
					}
					else
					{
						if (callback != null)
						{
							callback(item1, num3);
 
						}
						item1.Consume(num3);
						return true; 
					}
 
				}
 
			}
			return false; 
		} 

		public int ConsumeTotal(Type[] types, int[] amounts)
		{
			return this.ConsumeTotal(types, amounts, true, null); 
		} 

		public int ConsumeTotal(Type[] types, int[] amounts, bool recurse)
		{
			return this.ConsumeTotal(types, amounts, recurse, null); 
		} 

		public int ConsumeTotal(Type[] types, int[] amounts, bool recurse, OnItemConsumed callback)
		{
			Item[][] array1;
			int[] array2;
			int num1;
			int num2;
			int num3;
			int num4;
			int num5;
			Item item1;
			int num6;
			int[] array3;
			IntPtr ptr1;
			if (types.Length != amounts.Length)
			{
				throw new ArgumentException();
 
			}
			//TRF array1 = new Item[][types.Length];
			array1 = new Item[types.Length][];
			array2 = new int[types.Length];
			for (num1 = 0; (num1 < types.Length); num1 = (num1 + 1))
			{
				array1[num1] = this.FindItemsByType(types[num1], recurse);
				for (num2 = 0; (num2 < array1[num1].Length); num2 = (num2 + 1))
				{
					array3 = array2;
					ptr1 = (IntPtr)num1;
					array2[num1] = (array3[(int)ptr1] + array1[num1][num2].Amount);
 
				}
				if (array2[num1] < amounts[num1])
				{
					return num1; 
				}
 
			}
			num3 = 0;
			goto L_00E5;
 
			L_0073:
				num4 = amounts[num3];
			for (num5 = 0; (num5 < array1[num3].Length); num5 = (num5 + 1))
			{
				item1 = array1[num3][num5];
				num6 = item1.Amount;
				if (num6 < num4)
				{
					if (callback != null)
					{
						callback(item1, num6);
 
					}
					item1.Delete();
					num4 = (num4 - num6);
 
				}
				else
				{
					if (callback != null)
					{
						callback(item1, num4);
 
					}
					item1.Consume(num4);
					goto L_00DF;
 
				}
 
			}
 
			L_00DF:
				num3 = (num3 + 1);
 
			L_00E5:
				if (num3 < types.Length)
				{
					goto L_0073;
 
				}
			return -1; 
		}
		
		public int ConsumeTotal(Type[][] types, int[] amounts)
		{
			return this.ConsumeTotal(types, amounts, true, null); 
		} 

		public int ConsumeTotal(Type[][] types, int[] amounts, bool recurse)
		{
			return this.ConsumeTotal(types, amounts, recurse, null); 
		} 

		public int ConsumeTotal(Type[][] types, int[] amounts, bool recurse, OnItemConsumed callback)
		{
			Item[][] array1;
			int[] array2;
			int num1;
			int num2;
			int num3;
			int num4;
			int num5;
			Item item1;
			int num6;
			int[] array3;
			IntPtr ptr1;
			if (types.Length != amounts.Length)
			{
				throw new ArgumentException();
 
			}
			// TRF array1 = new Item[][types.Length];
			array1 = new Item[][] { new Item[types.Length] };
			array2 = new int[types.Length];
			for (num1 = 0; (num1 < types.Length); num1 = (num1 + 1))
			{
				array1[num1] = this.FindItemsByType(types[num1], recurse);
				for (num2 = 0; (num2 < array1[num1].Length); num2 = (num2 + 1))
				{
					array3 = array2;
					ptr1 = (IntPtr)num1;
					array2[num1] = (array3[(int)ptr1] + array1[num1][num2].Amount);
 
				}
				if (array2[num1] < amounts[num1])
				{
					return num1; 
				}
 
			}
			num3 = 0;
			goto L_00E5;
 
			L_0073:
				num4 = amounts[num3];
			for (num5 = 0; (num5 < array1[num3].Length); num5 = (num5 + 1))
			{
				item1 = array1[num3][num5];
				num6 = item1.Amount;
				if (num6 < num4)
				{
					if (callback != null)
					{
						callback(item1, num6);
 
					}
					item1.Delete();
					num4 = (num4 - num6);
 
				}
				else
				{
					if (callback != null)
					{
						callback(item1, num4);
 
					}
					item1.Consume(num4);
					goto L_00DF;
 
				}
 
			}
 
			L_00DF:
				num3 = (num3 + 1);
 
			L_00E5:
				if (num3 < types.Length)
				{
					goto L_0073;
 
				}
			return -1; 
		}
		
		public bool ConsumeTotalGrouped(Type type, int amount, bool recurse, OnItemConsumed callback, CheckItemGroup grouper)
		{
			Item[] array1;
			ArrayList list1;
			int num1;
			Item item1;
			ArrayList list2;
			Item item2;
			int num2;
			Item[][] array2;
			int[] array3;
			bool flag1;
			int num3;
			int num4;
			int num5;
			int num6;
			int num7;
			Item item3;
			int num8;
			int[] array4;
			IntPtr ptr1;
			if (grouper == null)
			{
				throw new ArgumentNullException();
 
			}
			array1 = this.FindItemsByType(type, recurse);
			list1 = new ArrayList();
			num1 = 0;
			goto L_0069;
 
			L_001D:
				num1 = (num1 + 1);
			item1 = array1[num1];
			list2 = new ArrayList();
			list2.Add(item1);
			while ((num1 < array1.Length))
			{
				item2 = array1[num1];
				num2 = grouper(item1, item2);
				if (num2 != 0)
				{
					goto L_0060;
 
				}
				list2.Add(item2);
				num1 = (num1 + 1);
 
			}
 
			L_0060:
				list1.Add(list2);
 
			L_0069:
				if (num1 < array1.Length)
				{
					goto L_001D;
 
				}
			//TRF array2 = new Item[][list1.Count];
			array2 = new Item[list1.Count][];
			array3 = new int[list1.Count];
			flag1 = false;
			for (num3 = 0; (num3 < list1.Count); num3 = (num3 + 1))
			{
				array2[num3] = ((Item[]) ((ArrayList)list1[num3]).ToArray(typeof(Item)));
				for (num4 = 0; (num4 < array2[num3].Length); num4 = (num4 + 1))
				{
					array4 = array3;
					ptr1 = (IntPtr)num3;
					array3[num3] = (array4[(int)ptr1] + array2[num3][num4].Amount);
 
				}
				if (array3[num3] >= amount)
				{
					flag1 = true;
 
				}
 
			}
			if (!flag1)
			{
				return false; 
			}
			for (num5 = 0; (num5 < array2.Length); num5 = (num5 + 1))
			{
				if (array3[num5] >= amount)
				{
					num6 = amount;
					for (num7 = 0; (num7 < array2[num5].Length); num7 = (num7 + 1))
					{
						item3 = array2[num5][num7];
						num8 = item3.Amount;
						if (num8 < num6)
						{
							if (callback != null)
							{
								callback(item3, num8);
 
							}
							item3.Delete();
							num6 = (num6 - num8);
 
						}
						else
						{
							if (callback != null)
							{
								callback(item3, num6);
 
							}
							item3.Consume(num6);
							goto L_0197;
 
						}
 
					}
					goto L_0197;
 
				}
 
			}
 
			L_0197:
				return true; 
		}
		
		public int ConsumeTotalGrouped(Type[] types, int[] amounts, bool recurse, OnItemConsumed callback, CheckItemGroup grouper)
		{
			Item[][][] array1;
			int[][] array2;
			int num1;
			Item[] array3;
			ArrayList list1;
			int num2;
			Item item1;
			ArrayList list2;
			Item item2;
			int num3;
			bool flag1;
			int num4;
			int num5;
			int num6;
			int num7;
			int num8;
			int num9;
			Item item3;
			int num10;
			int[] array4;
			IntPtr ptr1;
			if (types.Length != amounts.Length)
			{
				throw new ArgumentException();
 
			}
			if (grouper == null)
			{
				throw new ArgumentNullException();
 
			}
			//TRF array1 = new Item[][][types.Length];
			array1 = new Item[types.Length][][];
			//TRF array2 = new int[][types.Length];
			array2 = new int[types.Length][];
			num1 = 0;
			goto L_0153;
 
			L_0031:
				array3 = this.FindItemsByType(types[num1], recurse);
			list1 = new ArrayList();
			num2 = 0;
			goto L_009E;
 
			L_0048:
				num2 = (num2 + 1);
			item1 = array3[num2];
			list2 = new ArrayList();
			list2.Add(item1);
			while ((num2 < array3.Length))
			{
				item2 = array3[num2];
				num3 = grouper(item1, item2);
				if (num3 != 0)
				{
					goto L_0094;
 
				}
				list2.Add(item2);
				num2 = (num2 + 1);
 
			}
 
			L_0094:
				list1.Add(list2);
 
			L_009E:
				if (num2 < array3.Length)
				{
					goto L_0048;
 
				}
			//TRF array1[num1] = new Item[][list1.Count];
			array1[num1] = new Item[list1.Count][];
			array2[num1] = new int[list1.Count];
			flag1 = false;
			for (num4 = 0; (num4 < list1.Count); num4 = (num4 + 1))
			{
				array1[num1][num4] = ((Item[]) ((ArrayList)list1[num4]).ToArray(typeof(Item)));
				for (num5 = 0; (num5 < array1[num1][num4].Length); num5 = (num5 + 1))
				{
					array4 = array2[num1];
					ptr1 = (IntPtr)num4;
					array2[num1][num4] = (array4[(int)ptr1] + array1[num1][num4][num5].Amount);
 
				}
				if (array2[num1][num4] >= amounts[num1])
				{
					flag1 = true;
 
				}
 
			}
			if (!flag1)
			{
				return num1; 
			}
			num1 = (num1 + 1);
 
			L_0153:
				if (num1 < types.Length)
				{
					goto L_0031;
 
				}
			num6 = 0;
			goto L_0206;
 
			L_0164:
				num7 = 0;
			while ((num7 < array1[num6].Length))
			{
				if (array2[num6][num7] >= amounts[num6])
				{
					num8 = amounts[num6];
					for (num9 = 0; (num9 < array1[num6][num7].Length); num9 = (num9 + 1))
					{
						item3 = array1[num6][num7][num9];
						num10 = item3.Amount;
						if (num10 < num8)
						{
							if (callback != null)
							{
								callback(item3, num10);
 
							}
							item3.Delete();
							num8 = (num8 - num10);
 
						}
						else
						{
							if (callback != null)
							{
								callback(item3, num8);
 
							}
							item3.Consume(num8);
							goto L_0200;
 
						}
 
					}
					goto L_0200;
 
				}
				num7 = (num7 + 1);
 
			}
 
			L_0200:
				num6 = (num6 + 1);
 
			L_0206:
				if (num6 < array1.Length)
				{
					goto L_0164;
 
				}
			return -1; 
		} 

		public int ConsumeTotalGrouped(Type[][] types, int[] amounts, bool recurse, OnItemConsumed callback, CheckItemGroup grouper)
		{
			Item[][][] array1;
			int[][] array2;
			int num1;
			Item[] array3;
			ArrayList list1;
			int num2;
			Item item1;
			ArrayList list2;
			Item item2;
			int num3;
			bool flag1;
			int num4;
			int num5;
			int num6;
			int num7;
			int num8;
			int num9;
			Item item3;
			int num10;
			int[] array4;
			IntPtr ptr1;
			if (types.Length != amounts.Length)
			{
				throw new ArgumentException();
 
			}
			if (grouper == null)
			{
				throw new ArgumentNullException();
 
			}
			//TRF array1 = new Item[][][types.Length];
			array1 = new Item[types.Length][][];
			//TRF array2 = new int[][types.Length];
			array2 = new int[types.Length][];
			num1 = 0;
			goto L_0153;
 
			L_0031:
				array3 = this.FindItemsByType(types[num1], recurse);
			list1 = new ArrayList();
			num2 = 0;
			goto L_009E;
 
			L_0048:
				num2 = (num2 + 1);
			item1 = array3[num2];
			list2 = new ArrayList();
			list2.Add(item1);
			while ((num2 < array3.Length))
			{
				item2 = array3[num2];
				num3 = grouper(item1, item2);
				if (num3 != 0)
				{
					goto L_0094;
 
				}
				list2.Add(item2);
				num2 = (num2 + 1);
 
			}
 
			L_0094:
				list1.Add(list2);
 
			L_009E:
				if (num2 < array3.Length)
				{
					goto L_0048;
 
				}
			//TRF array1[num1] = new Item[][list1.Count];
			array1[num1] = new Item[list1.Count][];
			array2[num1] = new int[list1.Count];
			flag1 = false;
			for (num4 = 0; (num4 < list1.Count); num4 = (num4 + 1))
			{
				array1[num1][num4] = ((Item[]) ((ArrayList)list1[num4]).ToArray(typeof(Item)));
				for (num5 = 0; (num5 < array1[num1][num4].Length); num5 = (num5 + 1))
				{
					array4 = array2[num1];
					ptr1 = (IntPtr)num4;
					array2[num1][num4] = (array4[(int)ptr1] + array1[num1][num4][num5].Amount);
 
				}
				if (array2[num1][num4] >= amounts[num1])
				{
					flag1 = true;
 
				}
 
			}
			if (!flag1)
			{
				return num1; 
			}
			num1 = (num1 + 1);
 
			L_0153:
				if (num1 < types.Length)
				{
					goto L_0031;
 
				}
			num6 = 0;
			goto L_0206;
 
			L_0164:
				num7 = 0;
			while ((num7 < array1[num6].Length))
			{
				if (array2[num6][num7] >= amounts[num6])
				{
					num8 = amounts[num6];
					for (num9 = 0; (num9 < array1[num6][num7].Length); num9 = (num9 + 1))
					{
						item3 = array1[num6][num7][num9];
						num10 = item3.Amount;
						if (num10 < num8)
						{
							if (callback != null)
							{
								callback(item3, num10);
 
							}
							item3.Delete();
							num8 = (num8 - num10);
 
						}
						else
						{
							if (callback != null)
							{
								callback(item3, num8);
 
							}
							item3.Consume(num8);
							goto L_0200;
 
						}
 
					}
					goto L_0200;
 
				}
				num7 = (num7 + 1);
 
			}
 
			L_0200:
				num6 = (num6 + 1);
 
			L_0206:
				if (num6 < array1.Length)
				{
					goto L_0164;
 
				}
			return -1; 
		}
		
		public int ConsumeUpTo(Type type, int amount)
		{
			return this.ConsumeUpTo(type, amount, true); 
		} 

		public int ConsumeUpTo(Type type, int amount, bool recurse)
		{
			int num1;
			Queue queue1;
			num1 = 0;
			queue1 = new Queue();
			Container.RecurseConsumeUpTo(this, type, amount, recurse, ref num1, queue1);
			while ((queue1.Count > 0))
			{
				((Item) queue1.Dequeue()).Delete();
 
			}
			return num1; 
		} 

		public override void Deserialize(GenericReader reader)
		{
			int num1;
			SaveFlag flag1;
			int num2;
			base.Deserialize(reader);
			num1 = reader.ReadInt();
			num2 = num1;
			switch (num2)
			{
				case 0:
				{
					goto L_0090;
 
				}
				case 1:
				{
					goto L_0084;
 
				}
				case 2:
				{
					flag1 = (SaveFlag)reader.ReadByte();
					if (Container.GetSaveFlag(flag1, SaveFlag.MaxItems) == false)
					{
						goto L_0041;
 
					}
					this.m_MaxItems = reader.ReadEncodedInt();
					goto L_0048;
 
				}
 
			}
			return; 
			L_0041:
				this.m_MaxItems = -1;
 
			L_0048:
				if (Container.GetSaveFlag(flag1, SaveFlag.GumpID))
				{
					this.m_GumpID = reader.ReadEncodedInt();
 
				}
				else
				{
					this.m_GumpID = -1;
 
				}
			if (Container.GetSaveFlag(flag1, SaveFlag.DropSound))
			{
				this.m_DropSound = reader.ReadEncodedInt();
				return; 
			}
			this.m_DropSound = -1;
			return; 
			L_0084:
				this.m_MaxItems = reader.ReadInt();
 
			L_0090:
				if (num1 < 1)
				{
					this.m_MaxItems = Container.m_GlobalMaxItems;
 
				}
			this.m_GumpID = reader.ReadInt();
			this.m_DropSound = reader.ReadInt();
			if (this.m_GumpID == this.DefaultGumpID)
			{
				this.m_GumpID = -1;
 
			}
			if (this.m_DropSound == this.DefaultDropSound)
			{
				this.m_DropSound = -1;
 
			}
			if (this.m_MaxItems == this.DefaultMaxItems)
			{
				this.m_MaxItems = -1;
 
			}
			reader.ReadPoint2D();
			reader.ReadPoint2D();
 
		}
		
		public virtual void Destroy()
		{
			Point3D pointd1;
			Map map1;
			int num1;
			pointd1 = base.GetWorldLocation();
			map1 = base.Map;
			for (num1 = (base.Items.Count - 1); (num1 >= 0); num1 = (num1 - 1))
			{
				if (num1 < base.Items.Count)
				{
					((Item) base.Items[num1]).SetLastMoved();
					((Item) base.Items[num1]).MoveToWorld(pointd1, map1);
 
				}
 
			}
			base.Delete();
 
		} 

		public virtual void DisplayTo(Mobile to)
		{
			bool flag1;
			Point3D pointd1;
			Map map1;
			int num1;
			Mobile mobile1;
			int num2;
			ArrayList list1;
			int num3;
			if (!this.IsPublicContainer)
			{
				flag1 = false;
				if (this.m_Openers != null)
				{
					pointd1 = base.GetWorldLocation();
					map1 = base.Map;
					for (num1 = 0; (num1 < this.m_Openers.Count); num1 = (num1 + 1))
					{
						mobile1 = ((Mobile) this.m_Openers[num1]);
						if (mobile1 == to)
						{
							flag1 = true;
 
						}
						else
						{
							num2 = base.GetUpdateRange(mobile1);
							if ((mobile1.Map != map1) || !mobile1.InRange(pointd1, num2))
							{
								num1 = (num1 - 1);
								this.m_Openers.RemoveAt(num1);
 
							}
 
						}
 
					}
 
				}
				if (!flag1)
				{
					if (this.m_Openers == null)
					{
						this.m_Openers = new ArrayList(4);
 
					}
					this.m_Openers.Add(to);
 
				}
				else
				{
					if ((this.m_Openers != null) && (this.m_Openers.Count == 0))
					{
						this.m_Openers = null;
 
					}
 
				}
 
			}
			to.Send(new ContainerDisplay(this));
			to.Send(new ContainerContent(to, this));
			if (ObjectPropertyList.Enabled)
			{
				list1 = base.Items;
				for (num3 = 0; (num3 < list1.Count); num3 = (num3 + 1))
				{
					to.Send(((Item) list1[num3]).OPLPacket);
 
				}
 
			}
 
		} 

		public virtual void DropItem(Item dropped)
		{
			Rectangle2D rectangled1;
			Rectangle2D rectangled2;
			int num1;
			int num2;
			base.AddItem(dropped);
			rectangled1 = dropped.GetGraphicBounds();
			rectangled2 = this.Bounds;
			if (rectangled1.Width >= rectangled2.Width)
			{
				num1 = ((rectangled2.Width - rectangled1.Width) / 2);
 
			}
			else
			{
				num1 = Utility.Random((rectangled2.Width - rectangled1.Width));
 
			}
			if (rectangled1.Height >= rectangled2.Height)
			{
				num2 = ((rectangled2.Height - rectangled1.Height) / 2);
 
			}
			else
			{
				num2 = Utility.Random((rectangled2.Height - rectangled1.Height));
 
			}
			num1 = (num1 + rectangled2.X);
			num1 = (num1 - rectangled1.X);
			num2 = (num2 + rectangled2.Y);
			num2 = (num2 - rectangled1.Y);
			dropped.Location = new Point3D(num1, num2, 0);
 
		} 

		public Item FindItemByType(Type type)
		{
			return this.FindItemByType(type, true); 
		} 

		public Item FindItemByType(Type type, bool recurse)
		{
			return Container.RecurseFindItemByType(this, type, recurse); 
		} 

		public Item FindItemByType(Type[] types)
		{
			return this.FindItemByType(types, true); 
		} 

		public Item FindItemByType(Type[] types, bool recurse)
		{
			return Container.RecurseFindItemByType(this, types, recurse); 
		} 

		public Item[] FindItemsByType(Type type)
		{
			return this.FindItemsByType(type, true); 
		} 

		public Item[] FindItemsByType(Type type, bool recurse)
		{
			if (Container.m_FindItemsList.Count > 0)
			{
				Container.m_FindItemsList.Clear();
 
			}
			Container.RecurseFindItemsByType(this, type, recurse, Container.m_FindItemsList);
			return ((Item[]) Container.m_FindItemsList.ToArray(typeof(Item))); 
		} 

		public Item[] FindItemsByType(Type[] types)
		{
			return this.FindItemsByType(types, true); 
		} 

		public Item[] FindItemsByType(Type[] types, bool recurse)
		{
			if (Container.m_FindItemsList.Count > 0)
			{
				Container.m_FindItemsList.Clear();
 
			}
			Container.RecurseFindItemsByType(this, types, recurse, Container.m_FindItemsList);
			return ((Item[]) Container.m_FindItemsList.ToArray(typeof(Item))); 
		} 

		public int GetAmount(Type type)
		{
			return this.GetAmount(type, true); 
		} 

		public int GetAmount(Type type, bool recurse)
		{
			Item[] array1;
			int num1;
			int num2;
			array1 = this.FindItemsByType(type, recurse);
			num1 = 0;
			for (num2 = 0; (num2 < array1.Length); num2 = (num2 + 1))
			{
				num1 = (num1 + array1[num2].Amount);
 
			}
			return num1; 
		} 

		public int GetAmount(Type[] types)
		{
			return this.GetAmount(types, true); 
		} 

		public int GetAmount(Type[] types, bool recurse)
		{
			Item[] array1;
			int num1;
			int num2;
			array1 = this.FindItemsByType(types, recurse);
			num1 = 0;
			for (num2 = 0; (num2 < array1.Length); num2 = (num2 + 1))
			{
				num1 = (num1 + array1[num2].Amount);
 
			}
			return num1; 
		} 

		public int GetBestGroupAmount(Type type, bool recurse, CheckItemGroup grouper)
		{
			int num1;
			Item[] array1;
			ArrayList list1;
			int num2;
			Item item1;
			ArrayList list2;
			Item item2;
			int num3;
			int num4;
			Item[] array2;
			int num5;
			int num6;
			if (grouper == null)
			{
				throw new ArgumentNullException();
 
			}
			num1 = 0;
			array1 = this.FindItemsByType(type, recurse);
			list1 = new ArrayList();
			num2 = 0;
			goto L_006C;
 
			L_001E:
				num2 = (num2 + 1);
			item1 = array1[num2];
			list2 = new ArrayList();
			list2.Add(item1);
			while ((num2 < array1.Length))
			{
				item2 = array1[num2];
				num3 = grouper(item1, item2);
				if (num3 != 0)
				{
					goto L_0063;
 
				}
				list2.Add(item2);
				num2 = (num2 + 1);
 
			}
 
			L_0063:
				list1.Add(list2);
 
			L_006C:
				if (num2 < array1.Length)
				{
					goto L_001E;
 
				}
			for (num4 = 0; (num4 < list1.Count); num4 = (num4 + 1))
			{
				array2 = ((Item[]) ((ArrayList)list1[num4]).ToArray(typeof(Item)));
				num5 = 0;
				for (num6 = 0; (num6 < array2.Length); num6 = (num6 + 1))
				{
					num5 = (num5 + array2[num6].Amount);
 
				}
				if (num5 >= num1)
				{
					num1 = num5;
 
				}
 
			}
			return num1; 
		} 

		public int GetBestGroupAmount(Type[] types, bool recurse, CheckItemGroup grouper)
		{
			int num1;
			Item[] array1;
			ArrayList list1;
			int num2;
			Item item1;
			ArrayList list2;
			Item item2;
			int num3;
			int num4;
			Item[] array2;
			int num5;
			int num6;
			if (grouper == null)
			{
				throw new ArgumentNullException();
 
			}
			num1 = 0;
			array1 = this.FindItemsByType(types, recurse);
			list1 = new ArrayList();
			num2 = 0;
			goto L_006C;
 
			L_001E:
				num2 = (num2 + 1);
			item1 = array1[num2];
			list2 = new ArrayList();
			list2.Add(item1);
			while ((num2 < array1.Length))
			{
				item2 = array1[num2];
				num3 = grouper(item1, item2);
				if (num3 != 0)
				{
					goto L_0063;
 
				}
				list2.Add(item2);
				num2 = (num2 + 1);
 
			}
 
			L_0063:
				list1.Add(list2);
 
			L_006C:
				if (num2 < array1.Length)
				{
					goto L_001E;
 
				}
			for (num4 = 0; (num4 < list1.Count); num4 = (num4 + 1))
			{
				array2 = ((Item[]) ((ArrayList)list1[num4]).ToArray(typeof(Item)));
				num5 = 0;
				for (num6 = 0; (num6 < array2.Length); num6 = (num6 + 1))
				{
					num5 = (num5 + array2[num6].Amount);
 
				}
				if (num5 >= num1)
				{
					num1 = num5;
 
				}
 
			}
			return num1; 
		}
		
		public int GetBestGroupAmount(Type[][] types, bool recurse, CheckItemGroup grouper)
		{
			int num1;
			int num2;
			Item[] array1;
			ArrayList list1;
			int num3;
			Item item1;
			ArrayList list2;
			Item item2;
			int num4;
			int num5;
			Item[] array2;
			int num6;
			int num7;
			if (grouper == null)
			{
				throw new ArgumentNullException();
 
			}
			num1 = 0;
			num2 = 0;
			goto L_00EC;
 
			L_0012:
				array1 = this.FindItemsByType(types[num2], recurse);
			list1 = new ArrayList();
			num3 = 0;
			goto L_007C;
 
			L_0028:
				num3 = (num3 + 1);
			item1 = array1[num3];
			list2 = new ArrayList();
			list2.Add(item1);
			while ((num3 < array1.Length))
			{
				item2 = array1[num3];
				num4 = grouper(item1, item2);
				if (num4 != 0)
				{
					goto L_0073;
 
				}
				list2.Add(item2);
				num3 = (num3 + 1);
 
			}
 
			L_0073:
				list1.Add(list2);
 
			L_007C:
				if (num3 < array1.Length)
				{
					goto L_0028;
 
				}
			for (num5 = 0; (num5 < list1.Count); num5 = (num5 + 1))
			{
				array2 = ((Item[]) ((ArrayList)list1[num5]).ToArray(typeof(Item)));
				num6 = 0;
				for (num7 = 0; (num7 < array2.Length); num7 = (num7 + 1))
				{
					num6 = (num6 + array2[num7].Amount);
 
				}
				if (num6 >= num1)
				{
					num1 = num6;
 
				}
 
			}
			num2 = (num2 + 1);
 
			L_00EC:
				if (num2 < types.Length)
				{
					goto L_0012;
 
				}
			return num1; 
		}
		
		public virtual int GetDroppedSound(Item item)
		{
			int num1;
			num1 = item.GetDropSound();
			if (num1 == -1)
			{
				return this.DropSound; 
			}
			return num1; 
		} 

		public override void GetProperties(ObjectPropertyList list)
		{
			base.GetProperties(list);
			if (this.DisplaysContent)
			{
				list.Add(1050044, "{0} {1}", base.TotalItems, base.TotalWeight);
 
			}
 
		} 

		private static bool GetSaveFlag(SaveFlag flags, SaveFlag toGet)
		{
			return ((flags & toGet) != 0); 
		} 

		private static bool InTypeList(Item item, Type[] types)
		{
			Type type1;
			int num1;
			type1 = item.GetType();
			for (num1 = 0; (num1 < types.Length); num1 = (num1 + 1))
			{
				if (types[num1].IsAssignableFrom(type1))
				{
					return true; 
				}
 
			}
			return false; 
		} 

		public override void OnDelete()
		{
			base.OnDelete();
			this.m_Openers = null;
 
		} 

		public override void OnDoubleClick(Mobile from)
		{
			if ((from.AccessLevel > 0) || from.InRange(base.GetWorldLocation(), 2))
			{
				this.DisplayTo(from);
				return; 
			}
			from.SendLocalizedMessage(500446);
 
		} 

		public override void OnDoubleClickSecureTrade(Mobile from)
		{
			SecureTradeContainer container1;
			SecureTrade trade1;
			if (from.InRange(base.GetWorldLocation(), 2))
			{
				this.DisplayTo(from);
				container1 = base.GetSecureTradeCont();
				if (container1 == null)
				{
					goto L_0078;
 
				}
				trade1 = container1.Trade;
				if ((trade1 != null) && (trade1.From.Mobile == from))
				{
					this.DisplayTo(trade1.To.Mobile);
					return; 
				}
				if ((trade1 == null) || (trade1.To.Mobile != from))
				{
					goto L_0078;
 
				}
				this.DisplayTo(trade1.From.Mobile);
				return; 
			}
			from.SendLocalizedMessage(500446);
 
			L_0078:
				return; 
		} 

		public override bool OnDragDrop(Mobile from, Item dropped)
		{
			if (this.TryDropItem(from, dropped, true))
			{
				from.SendSound(this.GetDroppedSound(dropped), base.GetWorldLocation());
				return true; 
			}
			return false; 
		} 

		public virtual bool OnDragDropInto(Mobile from, Item item, Point3D p)
		{
			if (!this.CheckHold(from, item, true, true))
			{
				return false; 
			}
			item.Location = new Point3D(p.m_X, p.m_Y, 0);
			base.AddItem(item);
			from.SendSound(this.GetDroppedSound(item), base.GetWorldLocation());
			return true; 
		} 

		public override void OnSingleClick(Mobile from)
		{
			object[] array1;
			base.OnSingleClick(from);
			if (this.CheckContentDisplay(from))
			{
				array1 = new object[2];
				array1[0] = base.TotalItems;
				array1[1] = base.TotalWeight;
				base.LabelTo(from, "({0} items, {1} stones)", array1);
 
			}
 
		} 

		public override void OnSnoop(Mobile from)
		{
			if (Container.m_SnoopHandler != null)
			{
				Container.m_SnoopHandler(this, from);
 
			}
 
		} 

		public virtual bool OnStackAttempt(Mobile from, Item stack, Item dropped)
		{
			if (!this.CheckHold(from, dropped, true, false))
			{
				return false; 
			}
			return stack.StackWith(from, dropped); 
		} 

		private static void RecurseConsumeUpTo(Item current, Type type, int amount, bool recurse, ref int consumed, Queue toDelete)
		{
			ArrayList list1;
			int num1;
			Item item1;
			int num2;
			int num3;
			if ((current == null) || (current.Items.Count <= 0))
			{
				goto L_009D;
 
			}
			list1 = current.Items;
			num1 = 0;
			goto L_0094;
 
			L_0022:
				item1 = ((Item) list1[num1]);
			if (type.IsAssignableFrom(item1.GetType()))
			{
				num2 = (amount - consumed);
				num3 = item1.Amount;
				if (num3 <= num2)
				{
					toDelete.Enqueue(item1);
					consumed = (consumed + num3);
					goto L_0090;
 
				}
				item1.Amount = (item1.Amount - num2);
				consumed = (consumed + num2);
				return; 
			}
			if (recurse && ((item1 as Container) != null))
			{
				Container.RecurseConsumeUpTo(item1, type, amount, recurse, ref consumed, toDelete);
			}
 
			L_0090:
				num1 = (num1 + 1);
 
			L_0094:
				if (num1 < list1.Count)
				{
					goto L_0022;
 
				}
 
			L_009D:
				return; 
		} 

		private static Item RecurseFindItemByType(Item current, Type type, bool recurse)
		{
			ArrayList list1;
			int num1;
			Item item1;
			Item item2;
			if ((current != null) && (current.Items.Count > 0))
			{
				list1 = current.Items;
				for (num1 = 0; (num1 < list1.Count); num1 = (num1 + 1))
				{
					item1 = ((Item) list1[num1]);
					if (type.IsAssignableFrom(item1.GetType()))
					{
						return item1; 
					}
					if (recurse && ((item1 as Container) != null))
					{
						item2 = Container.RecurseFindItemByType(item1, type, recurse);
						if (item2 != null)
						{
							return item2; 
						}
 
					}
 
				}
 
			}
			return null; 
		} 

		private static Item RecurseFindItemByType(Item current, Type[] types, bool recurse)
		{
			ArrayList list1;
			int num1;
			Item item1;
			Item item2;
			if ((current != null) && (current.Items.Count > 0))
			{
				list1 = current.Items;
				for (num1 = 0; (num1 < list1.Count); num1 = (num1 + 1))
				{
					item1 = ((Item) list1[num1]);
					if (Container.InTypeList(item1, types))
					{
						return item1; 
					}
					if (recurse && ((item1 as Container) != null))
					{
						item2 = Container.RecurseFindItemByType(item1, types, recurse);
						if (item2 != null)
						{
							return item2; 
						}
 
					}
 
				}
 
			}
			return null; 
		} 

		private static void RecurseFindItemsByType(Item current, Type type, bool recurse, ArrayList list)
		{
			ArrayList list1;
			int num1;
			Item item1;
			if ((current != null) && (current.Items.Count > 0))
			{
				list1 = current.Items;
				for (num1 = 0; (num1 < list1.Count); num1 = (num1 + 1))
				{
					item1 = ((Item) list1[num1]);
					if (type.IsAssignableFrom(item1.GetType()))
					{
						list.Add(item1);
 
					}
					if (recurse && ((item1 as Container) != null))
					{
						Container.RecurseFindItemsByType(item1, type, recurse, list);
 
					}
 
				}
 
			}
 
		} 

		private static void RecurseFindItemsByType(Item current, Type[] types, bool recurse, ArrayList list)
		{
			ArrayList list1;
			int num1;
			Item item1;
			if ((current != null) && (current.Items.Count > 0))
			{
				list1 = current.Items;
				for (num1 = 0; (num1 < list1.Count); num1 = (num1 + 1))
				{
					item1 = ((Item) list1[num1]);
					if (Container.InTypeList(item1, types))
					{
						list.Add(item1);
 
					}
					if (recurse && ((item1 as Container) != null))
					{
						Container.RecurseFindItemsByType(item1, types, recurse, list);
 
					}
 
				}
 
			}
 
		} 

		public virtual void SendFullItemsMessage(Mobile to, Item item)
		{
			to.SendMessage("That container cannot hold more items.");
 
		} 

		public virtual void SendFullWeightMessage(Mobile to, Item item)
		{
			to.SendMessage("That container cannot hold more weight.");
 
		} 

		public override void Serialize(GenericWriter writer)
		{
			SaveFlag flag1;
			base.Serialize(writer);
			writer.Write(2);
			flag1 = 0;
			Container.SetSaveFlag(ref flag1, SaveFlag.MaxItems, ((this.m_MaxItems == -1) == false));
			Container.SetSaveFlag(ref flag1, SaveFlag.GumpID, ((this.m_GumpID == -1) == false));
			Container.SetSaveFlag(ref flag1, SaveFlag.DropSound, ((this.m_DropSound == -1) == false));
			writer.Write((byte)flag1);
			if (Container.GetSaveFlag(flag1, SaveFlag.MaxItems))
			{
				writer.WriteEncodedInt(this.m_MaxItems);
 
			}
			if (Container.GetSaveFlag(flag1, SaveFlag.GumpID))
			{
				writer.WriteEncodedInt(this.m_GumpID);
 
			}
			if (Container.GetSaveFlag(flag1, SaveFlag.DropSound))
			{
				writer.WriteEncodedInt(this.m_DropSound);
 
			}
 
		} 

		private static void SetSaveFlag(ref SaveFlag flags, SaveFlag toSet, bool setIf)
		{
			if (setIf)
			{
				flags = (flags | toSet);
 
			}
 
		} 

		public virtual bool TryDropItem(Mobile from, Item dropped, bool sendFullMessage)
		{
			ArrayList list1;
			int num1;
			Item item1;
			if (!this.CheckHold(from, dropped, sendFullMessage, true))
			{
				return false; 
			}
			list1 = base.Items;
			for (num1 = 0; (num1 < list1.Count); num1 = (num1 + 1))
			{
				item1 = ((Item) list1[num1]);
				if (((item1 as Container) == null) && item1.StackWith(from, dropped, false))
				{
					return true; 
				}
 
			}
			this.DropItem(dropped);
			return true; 
		} 

		public virtual Rectangle2D Bounds 
		{ 
			get
			{
				return new Rectangle2D(44, 65, 142, 94);
			}
		}

		public virtual int DefaultDropSound 
		{ 
			get
			{
				return 72;
			}
		}

		public virtual int DefaultGumpID 
		{ 
			get
			{
				return 60;
			}
		}

		public virtual int DefaultMaxItems 
		{ 
			get
			{
				return Container.m_GlobalMaxItems;
			}
		}

		public virtual int DefaultMaxWeight 
		{ 
			get
			{
				return Container.m_GlobalMaxWeight;
			}
		}

		public virtual bool DisplaysContent 
		{ 
			get
			{
				return true;
			}
		}

		public int DropSound 
		{ 
			get
			{
				if (this.m_DropSound != -1)
				{
					return this.m_DropSound; 
				}
				return this.DefaultDropSound; 
			}
			set
			{
				this.m_DropSound = value;
			}
		}

		public static int GlobalMaxItems 
		{ 
			get
			{
				return Container.m_GlobalMaxItems;
			}
			set
			{
				Container.m_GlobalMaxItems = value;
			}
		}

		public static int GlobalMaxWeight 
		{ 
			get
			{
				return Container.m_GlobalMaxWeight;
			}
			set
			{
				Container.m_GlobalMaxWeight = value;
			}
		}

		public int GumpID 
		{ 
			get
			{
				if (this.m_GumpID != -1)
				{
					return this.m_GumpID; 
				}
				return this.DefaultGumpID; 
			}
			set
			{
				this.m_GumpID = value;
			}
		}

		public virtual bool IsPublicContainer 
		{ 
			get
			{
				return false;
			}
		}

		public int MaxItems 
		{ 
			get
			{
				if (this.m_MaxItems != -1)
				{
					return this.m_MaxItems; 
				}
				return this.DefaultMaxItems;
			}
			set
			{
				this.m_MaxItems = value;
			}
		}

		public virtual int MaxWeight 
		{ 
			get
			{
				if (((base.Parent as Container) != null) && (((Container) base.Parent).MaxWeight == 0))
				{
					return 0; 
				}
				if (base.Movable || (((base.Parent as Mobile) != null) && ((Mobile) base.Parent).Player))
				{
					return this.DefaultMaxWeight; 
				}
				return 0;
			}
		}

		public ArrayList Openers 
		{ 
			get
			{
				return this.m_Openers;
			}
			set
			{
				this.m_Openers = value;
			}
		}

		public static ContainerSnoopHandler SnoopHandler 
		{ 
			get
			{
				return Container.m_SnoopHandler;
			}
			set
			{
				Container.m_SnoopHandler = value;
			}
		}

		
		private int m_DropSound;
		private static ArrayList m_FindItemsList = new ArrayList();
		private static int m_GlobalMaxItems = 125;
		private static int m_GlobalMaxWeight = 400;
		private int m_GumpID;
		private int m_MaxItems;
		private ArrayList m_Openers;
		private static ContainerSnoopHandler m_SnoopHandler;

		private class GroupComparer : IComparer
		{
			public GroupComparer(CheckItemGroup grouper) : base()
			{
				this.m_Grouper = grouper;
			}

			public int Compare(object x, object y)
			{
				Item item1;
				Item item2;
				item1 = ((Item) x);
				item2 = ((Item) y);
				return this.m_Grouper(item1, item2); 
			} 

			private CheckItemGroup m_Grouper;
		}

		private enum SaveFlag
		{
			None = 0,
			MaxItems = 1,
			GumpID = 2,
			DropSound = 4,
		}
	}

	public class SecureTradeContainer : Container
	{
		public SecureTradeContainer(SecureTrade trade) : base(7774)
		{
			this.m_Trade = trade;
			base.Movable = false;
		} 

		public SecureTradeContainer(Serial serial) : base(serial)
		{
		}

		public override bool CheckLift(Mobile from, Item item)
		{
			return false; 
		} 

		public void ClearChecks()
		{
			if (this.m_Trade != null)
			{
				this.m_Trade.From.Accepted = false;
				this.m_Trade.To.Accepted = false;
				this.m_Trade.Update();
 
			}
 
		} 

		public override void Deserialize(GenericReader reader)
		{
			base.Deserialize(reader);
			reader.ReadInt();
 
		} 

		public override bool IsAccessibleTo(Mobile check)
		{
			if (!base.IsChildOf(check))
			{
				return false; 
			}
			return base.IsAccessibleTo(check); 
		} 

		public override void OnItemAdded(Item item)
		{
			this.ClearChecks();
 
		} 

		public override void OnItemRemoved(Item item)
		{
			this.ClearChecks();
 
		} 

		public override void OnSubItemAdded(Item item)
		{
			this.ClearChecks();
 
		} 

		public override void OnSubItemRemoved(Item item)
		{
			this.ClearChecks();
 
		} 

		public override void Serialize(GenericWriter writer)
		{
			base.Serialize(writer);
			writer.Write(0);
 
		} 

		public override Rectangle2D Bounds 
		{ 
			get
			{
				return new Rectangle2D(0, 0, 110, 62);
			}
		}

		public override int DefaultDropSound 
		{ 
			get
			{
				return 66;
			}
		}

		public override int DefaultGumpID 
		{ 
			get
			{
				return 82;
			}
		}

		public SecureTrade Trade 
		{ 
			get
			{
				return this.m_Trade;
			}
		}

		private SecureTrade m_Trade;
	}
}