using System;
using System.Collections;
using System.IO;
using System.Net;
using System.Net.Sockets;

namespace Server
{
	public struct MultiTileEntry
	{
		public MultiTileEntry(short itemID, short xOffset, short yOffset, short zOffset, int flags)
		{
			this.m_ItemID = itemID;
			this.m_OffsetX = xOffset;
			this.m_OffsetY = yOffset;
			this.m_OffsetZ = zOffset;
			this.m_Flags = flags;
		} 

		public int m_Flags;
		public short m_ItemID;
		public short m_OffsetX;
		public short m_OffsetY;
		public short m_OffsetZ;
	}

	public class TypeTable
	{
		public TypeTable(int capacity) : base()
		{
			this.m_Sensitive = new Hashtable(capacity);
			this.m_Insensitive = new Hashtable(capacity, CaseInsensitiveHashCodeProvider.Default, CaseInsensitiveComparer.Default);
		} 

		public void Add(string key, Type type)
		{
				this.m_Sensitive[key] = type;
			this.m_Insensitive[key] = type;
 
		} 

		public Type Get(string key, bool ignoreCase)
		{
				if (ignoreCase)
		  {
			  return ((Type) this.m_Insensitive[key]); 
		  }
			return ((Type) this.m_Sensitive[key]); 
		} 

		private Hashtable m_Insensitive;
		private Hashtable m_Sensitive;
	}

	public class ItemBounds
	{
		static ItemBounds()
		{
			FileStream stream1;
			BinaryReader reader1;
			int num1;
			int num2;
			int num3;
			int num4;
			int num5;
			if (File.Exists("Data/Binary/Bounds.bin"))
			{
				using (stream1 = new FileStream("Data/Binary/Bounds.bin", FileMode.Open, FileAccess.Read, FileShare.Read)) {
					reader1 = new BinaryReader(stream1);
					ItemBounds.m_Bounds = new Rectangle2D[16384];
					for (num1 = 0; (num1 < 16384); num1 = (num1 + 1))
					{
						num2 = reader1.ReadInt16();
						num3 = reader1.ReadInt16();
						num4 = reader1.ReadInt16();
						num5 = reader1.ReadInt16();
						ItemBounds.m_Bounds[num1].Set(num2, num3, ((num4 - num2) + 1), ((num5 - num3) + 1));
 
					}
					goto L_00A4;
				}
 
			}
			Console.WriteLine("Warning: Data/Binary/Bounds.bin does not exist");
			ItemBounds.m_Bounds = new Rectangle2D[16384];
 
			L_00A4:
				return; 
		}
		
		public ItemBounds() : base()
		{
		}

		public static Rectangle2D[] Table { 
			get
			{
				return ItemBounds.m_Bounds;
			}
		}
		
		private static Rectangle2D[] m_Bounds;
	}

	public class OPLInfo : Network.Packet
	{
		public OPLInfo(ObjectPropertyList list)
			: base(191)
		{
			base.EnsureCapacity(13);
			base.m_Stream.Write(16);
			base.m_Stream.Write(list.Entity.Serial);
			base.m_Stream.Write(list.Hash);
		} 
	}

	public class MultiData
	{
		static MultiData()
		{
			string text1;
			string text2;
			string text3;
			FileStream stream1;
			BinaryReader reader1;
			int num1;
			int num2;
			int num3;
			int num4;
			int num5;
			int num6;
			text1 = Core.FindDataFile("Multi.idx");
			text2 = Core.FindDataFile("Multi.mul");
			if (File.Exists(text1) && File.Exists(text2))
			{
				MultiData.m_Index = new FileStream(text1, FileMode.Open, FileAccess.Read, FileShare.Read);
				MultiData.m_IndexReader = new BinaryReader(MultiData.m_Index);
				MultiData.m_Stream = new FileStream(text2, FileMode.Open, FileAccess.Read, FileShare.Read);
				MultiData.m_StreamReader = new BinaryReader(MultiData.m_Stream);
				MultiData.m_Components = new MultiComponentList[((int) (MultiData.m_Index.Length / ((long) 12)))];
				text3 = Core.FindDataFile("Verdata.mul");
				if (!File.Exists(text3))
				{
					goto L_0170;
 
				}
				using (stream1 = new FileStream(text3, FileMode.Open, FileAccess.Read, FileShare.Read)) {
					reader1 = new BinaryReader(stream1);
					num1 = reader1.ReadInt32();
					for (num2 = 0; (num2 < num1); num2 = (num2 + 1))
					{
						num3 = reader1.ReadInt32();
						num4 = reader1.ReadInt32();
						num5 = reader1.ReadInt32();
						num6 = reader1.ReadInt32();
						reader1.ReadInt32();
						if (((num3 == 14) && (num4 >= 0)) && (((num4 < MultiData.m_Components.Length) && (num5 >= 0)) && (num6 > 0)))
						{
							reader1.BaseStream.Seek(((long) num5), 0);
							MultiData.m_Components[num4] = new MultiComponentList(reader1, (num6 / 12));
							reader1.BaseStream.Seek(((long) (24 + (num2 * 20))), 0);
 
						}
 
					}
					goto L_0170;
				}
 
			}
			Console.WriteLine("Warning: Multi data files not found");
			MultiData.m_Components = new MultiComponentList[0];
 
			L_0170:
				return; 
		}
		
		public MultiData() : base()
		{
		}

		public static MultiComponentList GetComponents(int multiID)
		{
				MultiComponentList list1;
			multiID = (multiID & 16383);
			if ((multiID >= 0) && (multiID < MultiData.m_Components.Length))
			{
				list1 = MultiData.m_Components[multiID];
				if (list1 != null)
				{
					goto L_0039;
 
				}
				list1 = MultiData.Load(multiID);
				MultiData.m_Components[multiID] = MultiData.Load(multiID);
 
			}
			else
			{
					list1 = MultiComponentList.Empty;
 
			}
 
			L_0039:
				return list1; 
		} 

		public static MultiComponentList Load(int multiID)
		{
				int num1;
			int num2;
			MultiComponentList list1;
			try
			{
				MultiData.m_IndexReader.BaseStream.Seek(((long) (multiID * 12)), 0);
				num1 = MultiData.m_IndexReader.ReadInt32();
				num2 = MultiData.m_IndexReader.ReadInt32();
				if ((num1 < 0) || (num2 <= 0))
				{
					list1 = MultiComponentList.Empty;
					goto L_0069;
 
				}
				MultiData.m_StreamReader.BaseStream.Seek(((long) num1), 0);
				list1 = new MultiComponentList(MultiData.m_StreamReader, (num2 / 12));
 
			}
			catch /*(Exception ignored)*/
			{
				list1 = MultiComponentList.Empty;
 
			}
 
			L_0069:
				return list1; 
		} 

		private static MultiComponentList[] m_Components;
		private static FileStream m_Index;
		private static BinaryReader m_IndexReader;
		private static FileStream m_Stream;
		private static BinaryReader m_StreamReader;
	}

	public class Entity : IEntity
	{
		public Entity(Serial serial, Point3D loc, Map map) : base()
		{
			this.m_Serial = serial;
			this.m_Location = loc;
			this.m_Map = map;
		} 
		
		public Point3D Location 
		{ 
			get
			{
				return this.m_Location;
			}
		}

		public Map Map 
		{ 
			get
			{
				return this.m_Map;
			}
		}

		public Serial Serial 
		{ 
			get
			{
				return this.m_Serial;
			}
		}

		public int X 
		{ 
			get
			{
				return this.m_Location.X;
			}
		}

		public int Y 
		{ 
			get
			{
				return this.m_Location.Y;
			}
		}

		public int Z 
		{ 
			get
			{
				return this.m_Location.Z;
			}
		}

		private Point3D m_Location;
		private Map m_Map;
		private Serial m_Serial;
	}

	public class Firewall
	{
		static Firewall()
		{
			string text1;
			StreamReader reader1;
			string text2;
			object obj1;
			Firewall.m_Blocked = new ArrayList();
			text1 = "firewall.cfg";
			if (File.Exists(text1))
			{
				reader1 = new StreamReader(text1);
				try
				{
					goto L_004F;
 
				L_0021:
					text2 = text2.Trim();
					if (text2.Length != 0)
					{
						try
						{
							obj1 = IPAddress.Parse(text2);
 
						}
						catch (object obj2)
						{
							obj1 = text2;
 
						}
						Firewall.m_Blocked.Add(obj1.ToString());
 
					}
 
				L_004F:
					text2 = reader1.ReadLine();
					if (reader1.ReadLine() != null)
					{
						goto L_0021;
 
					}
 
				}
				finally
				{
					if (reader1 != null)
					{
						reader1.Dispose();
 
					}
 
				}
 
			}
 
		}
		
		public Firewall() : base()
		{
		}

		public static void Add(IPAddress ip)
		{
				if (!Firewall.m_Blocked.Contains(ip))
		  {
			  Firewall.m_Blocked.Add(ip);
 
		  }
			Firewall.Save();
 
		} 

		public static void Add(object obj)
		{
				if (((obj as IPAddress) == null) && ((obj as string) == null))
		  {
			  return; 
		  }
			if (!Firewall.m_Blocked.Contains(obj))
			{
				Firewall.m_Blocked.Add(obj);
 
			}
			Firewall.Save();
 
		} 

		public static void Add(string pattern)
		{
				if (!Firewall.m_Blocked.Contains(pattern))
		  {
			  Firewall.m_Blocked.Add(pattern);
 
		  }
			Firewall.Save();
 
		} 

		public static bool IsBlocked(IPAddress ip)
		{
				bool flag1;
			int num1;
			flag1 = false;
			for (num1 = 0; (!flag1 && (num1 < Firewall.m_Blocked.Count)); num1 = (num1 + 1))
			{
				if ((Firewall.m_Blocked[num1] as IPAddress) != null)
				{
					flag1 = ip.Equals(Firewall.m_Blocked[num1]);
 
				}
				else
				{
						if ((Firewall.m_Blocked[num1] as string) != null)
				  {
					  flag1 = Utility.IPMatch(((string) Firewall.m_Blocked[num1]), ip);
 
				  }
 
				}
 
			}
			return flag1; 
		} 

		public static void Remove(IPAddress ip)
		{
				Firewall.m_Blocked.Remove(ip);
			Firewall.Save();
 
		} 

		public static void Remove(string pattern)
		{
				Firewall.m_Blocked.Remove(pattern);
			Firewall.Save();
 
		} 

		public static void RemoveAt(int index)
		{
				Firewall.m_Blocked.RemoveAt(index);
			Firewall.Save();
 
		} 

		public static void Save()
		{
				string text1;
			StreamWriter writer1;
			int num1;
			text1 = "firewall.cfg";
			writer1 = new StreamWriter(text1);
			try
			{
				for (num1 = 0; (num1 < Firewall.m_Blocked.Count); num1 = (num1 + 1))
				{
					writer1.WriteLine(Firewall.m_Blocked[num1]);
 
				}
 
			}
			finally
			{
				if (writer1 != null)
				{
					writer1.Dispose();
 
				}
 
			}
 
		} 

		public static ArrayList List 
		{ 
			get
			{
				return Firewall.m_Blocked;
			}
		}

		private static ArrayList m_Blocked;
	}
}