using System;
using System.Collections;

using Server.Network;
using Server.Targeting;
using Server.Items;

namespace Server
{
	public sealed class Map
	{
		public Map(int mapID, int mapIndex, int fileIndex, int width, int height, int season, string name, MapRules rules)
			: base()
		{
			this.m_MapID = mapID;
			this.m_MapIndex = mapIndex;
			this.m_FileIndex = fileIndex;
			this.m_Width = width;
			this.m_Height = height;
			this.m_Season = season;
			this.m_Name = name;
			this.m_Rules = rules;
			this.m_Regions = new ArrayList();
			this.m_InvalidSector = new Sector(0, 0, this);
			this.m_SectorsWidth = (width >> 4);
			this.m_SectorsHeight = (height >> 4);
			//TRF this.m_Sectors = new Sector[][this.m_SectorsWidth];
			this.m_Sectors = new Sector[this.m_SectorsWidth][];
		} 

		public void ActivateSectors(int cx, int cy)
		{
			int num1;
			int num2;
			Sector sector1;
			for (num1 = (cx - Map.SectorActiveRange); (num1 <= (cx + Map.SectorActiveRange)); num1 = (num1 + 1))
			{
				for (num2 = (cy - Map.SectorActiveRange); (num2 <= (cy + Map.SectorActiveRange)); num2 = (num2 + 1))
				{
					sector1 = this.GetRealSector(num1, num2);
					if (sector1 != this.m_InvalidSector)
					{
						sector1.Activate();
 
					}
 
				}
 
			}
 
		}
		
		public void AddMulti(Item item, Sector start, Sector end)
		{
			int num1;
			int num2;
			if (this == Map.Internal)
			{
				return; 
			}
			for (num1 = start.X; (num1 <= end.X); num1 = (num1 + 1))
			{
				for (num2 = start.Y; (num2 <= end.Y); num2 = (num2 + 1))
				{
					this.InternalGetSector(num1, num2).OnMultiEnter(item);
 
				}
 
			}
 
		} 

		public void Bound(int x, int y, ref int newX, ref int newY)
		{
			if (x < 0)
			{
				newX = 0;
 
			}
			else
			{
				if (x >= this.m_Width)
				{
					newX = (this.m_Width - 1);
					goto L_0021;
 
				}
				newX = x;
 
			}
 
			L_0021:
				if (y < 0)
				{
					newY = 0;
					return; 
				}
			if (y >= this.m_Height)
			{
				newY = (this.m_Height - 1);
				return; 
			}
			newY = y;
 
		} 

		public Point2D Bound(Point2D p)
		{
			int num1;
			int num2;
			num1 = p.m_X;
			num2 = p.m_Y;
			if (num1 < 0)
			{
				num1 = 0;
 
			}
			else
			{
				if (num1 >= this.m_Width)
				{
					num1 = (this.m_Width - 1);
 
				}
 
			}
			if (num2 < 0)
			{
				num2 = 0;
 
			}
			else
			{
				if (num2 >= this.m_Height)
				{
					num2 = (this.m_Height - 1);
 
				}
 
			}
			return new Point2D(num1, num2); 
		} 

		public bool CanFit(int x, int y, int z, int height)
		{
			return this.CanFit(x, y, z, height, false, true, true); 
		} 

		public bool CanFit(int x, int y, int z, int height, bool checksBlocksFit)
		{
			return this.CanFit(x, y, z, height, false, true, true); 
		} 

		public bool CanFit(int x, int y, int z, int height, bool checkBlocksFit, bool checkMobiles)
		{
			return this.CanFit(x, y, z, height, checkBlocksFit, checkMobiles, true); 
		} 

		public bool CanFit(int x, int y, int z, int height, bool checkBlocksFit, bool checkMobiles, bool requireSurface)
		{
			bool flag1;
			Tile tile1;
			int num1;
			int num2;
			int num3;
			TileFlag flag2;
			Tile[] array1;
			bool flag3;
			bool flag4;
			int num4;
			ItemData data1;
			Sector sector1;
			ArrayList list1;
			ArrayList list2;
			int num5;
			Item item1;
			ItemData data2;
			int num6;
			Mobile mobile1;
			Point3D pointd1;
			if (this == Map.Internal)
			{
				return false; 
			}
			if ((x < 0) || (((y < 0) || (x >= this.m_Width)) || (y >= this.m_Height)))
			{
				return false; 
			}
			flag1 = false;
			tile1 = this.Tiles.GetLandTile(x, y);
			num1 = 0;
			num2 = 0;
			num3 = 0;
			this.GetAverageZ(x, y, ref num1, ref num2, ref num3);
			flag2 = TileData.LandTable[(tile1.ID & 16383)].Flags;
			if ((((flag2 & TileFlag.Impassable) != 0) && (num2 > z)) && ((z + height) > num1))
			{
				return false; 
			}
			if (((flag2 & TileFlag.Impassable) == 0) && ((z == num2) && !tile1.Ignored))
			{
				flag1 = true;
 
			}
			array1 = this.Tiles.GetStaticTiles(x, y, true);
			for (num4 = 0; (num4 < array1.Length); num4 = (num4 + 1))
			{
				data1 = TileData.ItemTable[(array1[num4].ID & 16383)];
				flag3 = data1.Surface;
				flag4 = data1.Impassable;
				if ((flag3 || flag4) && (((array1[num4].Z + data1.CalcHeight) > z) && ((z + height) > array1[num4].Z)))
				{
					return false; 
				}
				if (flag3 && (!flag4 && (z == (array1[num4].Z + data1.CalcHeight))))
				{
					flag1 = true;
 
				}
 
			}
			sector1 = this.GetSector(x, y);
			list1 = sector1.Items;
			list2 = sector1.Mobiles;
			for (num5 = 0; (num5 < list1.Count); num5 = (num5 + 1))
			{
				item1 = ((Item) list1[num5]);
				if ((item1.ItemID < 16384) && item1.AtWorldPoint(x, y))
				{
					data2 = item1.ItemData;
					flag3 = data2.Surface;
					flag4 = data2.Impassable;
					if (((flag3 || flag4) || (checkBlocksFit && item1.BlocksFit)) && (((item1.Z + data2.CalcHeight) > z) && ((z + height) > item1.Z)))
					{
						return false; 
					}
					if (flag3 && (!flag4 && (z == (item1.Z + data2.CalcHeight))))
					{
						flag1 = true;
 
					}
 
				}
 
			}
			if (checkMobiles)
			{
				for (num6 = 0; (num6 < list2.Count); num6 = (num6 + 1))
				{
					mobile1 = ((Mobile) list2[num6]);
					pointd1 = mobile1.Location;
					if (pointd1.m_X == x)
					{
						pointd1 = mobile1.Location;
						if (((pointd1.m_Y == y) && ((mobile1.Z + 16) > z)) && ((z + height) > mobile1.Z))
						{
							return false; 
						}
 
					}
 
				}
 
			}
			if (requireSurface)
			{
				return flag1; 
			}
			return true; 
		}
		
		public bool CanFit(Point2D p, int z, int height)
		{
			return this.CanFit(p.m_X, p.m_Y, z, height, false, true, true); 
		} 

		public bool CanFit(Point2D p, int z, int height, bool checkBlocksFit)
		{
			return this.CanFit(p.m_X, p.m_Y, z, height, checkBlocksFit, true, true); 
		} 

		public bool CanFit(Point3D p, int height)
		{
			return this.CanFit(p.m_X, p.m_Y, p.m_Z, height, false, true, true); 
		} 

		public bool CanFit(Point3D p, int height, bool checkBlocksFit)
		{
			return this.CanFit(p.m_X, p.m_Y, p.m_Z, height, checkBlocksFit, true, true); 
		} 

		public bool CanFit(Point3D p, int height, bool checkBlocksFit, bool checkMobiles)
		{
			return this.CanFit(p.m_X, p.m_Y, p.m_Z, height, checkBlocksFit, checkMobiles, true); 
		} 

		public bool CanSpawnMobile(int x, int y, int z)
		{
			if (!Region.Find(new Point3D(x, y, z), this).AllowSpawn())
			{
				return false; 
			}
			return this.CanFit(x, y, z, 16); 
		} 

		public bool CanSpawnMobile(Point2D p, int z)
		{
			return this.CanSpawnMobile(p.m_X, p.m_Y, z); 
		} 

		public bool CanSpawnMobile(Point3D p)
		{
			return this.CanSpawnMobile(p.m_X, p.m_Y, p.m_Z); 
		} 

		private static void CheckNamesAndValues()
		{
			int num1;
			Map map1;
			if ((Map.m_MapNames != null) && (Map.m_MapNames.Length == Map.m_AllMaps.Count))
			{
				return; 
			}
			Map.m_MapNames = new string[Map.m_AllMaps.Count];
			Map.m_MapValues = new Map[Map.m_AllMaps.Count];
			for (num1 = 0; (num1 < Map.m_AllMaps.Count); num1 = (num1 + 1))
			{
				map1 = ((Map) Map.m_AllMaps[num1]);
				Map.m_MapNames[num1] = map1.Name;
				Map.m_MapValues[num1] = map1;
 
			}
 
		} 

		public void DeactivateSectors(int cx, int cy)
		{
			int num1;
			int num2;
			Sector sector1;
			for (num1 = (cx - Map.SectorActiveRange); (num1 <= (cx + Map.SectorActiveRange)); num1 = (num1 + 1))
			{
				for (num2 = (cy - Map.SectorActiveRange); (num2 <= (cy + Map.SectorActiveRange)); num2 = (num2 + 1))
				{
					sector1 = this.GetRealSector(num1, num2);
					if ((sector1 != this.m_InvalidSector) && !this.PlayersInRange(sector1, Map.SectorActiveRange))
					{
						sector1.Deactivate();
 
					}
 
				}
 
			}
 
		} 

		public void FixColumn(int x, int y)
		{
			Tile tile1;
			int num1;
			int num2;
			int num3;
			Tile[] array1;
			ArrayList list1;
			IPooledEnumerable enumerable1;
			Item item1;
			int num4;
			Item item2;
			int num5;
			int num6;
			int num7;
			Tile tile2;
			ItemData data1;
			int num8;
			int num9;
			int num10;
			Item item3;
			ItemData data2;
			int num11;
			int num12;
			IEnumerator enumerator1;
			IDisposable disposable1;
			tile1 = this.Tiles.GetLandTile(x, y);
			num1 = 0;
			num2 = 0;
			num3 = 0;
			this.GetAverageZ(x, y, ref num1, ref num2, ref num3);
			array1 = this.Tiles.GetStaticTiles(x, y, true);
			list1 = new ArrayList();
			enumerable1 = this.GetItemsInRange(new Point3D(x, y, 0), 0);
			enumerator1 = enumerable1.GetEnumerator();
			try
			{
				while (enumerator1.MoveNext())
				{
					item1 = ((Item) enumerator1.Current);
					if (item1.ItemID < 16384)
					{
						list1.Add(item1);
						if (list1.Count > 100)
						{
							goto L_008F;
 
						}
 
					}
 
				}
 
			L_008F:
				goto L_00A6;
 
			}
			finally
			{
				disposable1 = (enumerator1 as IDisposable);
				if (disposable1 != null)
				{
					disposable1.Dispose();
 
				}
 
			}
 
			L_00A6:
				enumerable1.Free();
			if (list1.Count > 100)
			{
				return; 
			}
			list1.Sort(ZComparer.Default);
			for (num4 = 0; (num4 < list1.Count); num4 = (num4 + 1))
			{
				item2 = ((Item) list1[num4]);
				if (item2.Movable)
				{
					num5 = -2147483648;
					num6 = item2.Z;
					if (!tile1.Ignored && (num2 <= num6))
					{
						num5 = num2;
 
					}
					for (num7 = 0; (num7 < array1.Length); num7 = (num7 + 1))
					{
						tile2 = array1[num7];
						data1 = TileData.ItemTable[(tile2.ID & 16383)];
						num8 = tile2.Z;
						num9 = (num8 + data1.CalcHeight);
						if ((num9 == num8) && !data1.Surface)
						{
							num9 = (num9 + 1);
 
						}
						if ((num9 > num5) && (num9 <= num6))
						{
							num5 = num9;
 
						}
 
					}
					for (num10 = 0; (num10 < list1.Count); num10 = (num10 + 1))
					{
						if (num10 != num4)
						{
							item3 = ((Item) list1[num10]);
							data2 = item3.ItemData;
							num11 = item3.Z;
							num12 = (num11 + data2.CalcHeight);
							if ((num12 == num11) && !data2.Surface)
							{
								num12 = (num12 + 1);
 
							}
							if ((num12 > num5) && (num12 <= num6))
							{
								num5 = num12;
 
							}
 
						}
 
					}
					if (num5 != -2147483648)
					{
						item2.Location = new Point3D(item2.X, item2.Y, num5);
 
					}
 
				}
 
			}
 
		}
		
		public int GetAverageZ(int x, int y)
		{
			int num1;
			int num2;
			int num3;
			num1 = 0;
			num2 = 0;
			num3 = 0;
			this.GetAverageZ(x, y, ref num1, ref num2, ref num3);
			return num2; 
		} 

		public void GetAverageZ(int x, int y, ref int z, ref int avg, ref int top)
		{
			int num1;
			int num2;
			int num3;
			int num4;
			Tile tile1;
			tile1 = this.Tiles.GetLandTile(x, y);
			num1 = tile1.Z;
			tile1 = this.Tiles.GetLandTile(x, (y + 1));
			num2 = tile1.Z;
			tile1 = this.Tiles.GetLandTile((x + 1), y);
			num3 = tile1.Z;
			tile1 = this.Tiles.GetLandTile((x + 1), (y + 1));
			num4 = tile1.Z;
			z = num1;
			if (num2 < z)
			{
				z = num2;
 
			}
			if (num3 < z)
			{
				z = num3;
 
			}
			if (num4 < z)
			{
				z = num4;
 
			}
			top = num1;
			if (num2 > top)
			{
				top = num2;
 
			}
			if (num3 > top)
			{
				top = num3;
 
			}
			if (num4 > top)
			{
				top = num4;
 
			}
			if (Math.Abs((num1 - num4)) > Math.Abs((num2 - num3)))
			{
				avg = ((int) Math.Floor((((double) (num2 + num3)) / 2)));
				return; 
			}
			avg = ((int) Math.Floor((((double) (num1 + num4)) / 2)));
 
		} 

		public IPooledEnumerable GetClientsInBounds(Rectangle2D bounds)
		{
			if (this == Map.Internal)
			{
				return NullEnumerable.Instance; 
			}
			return PooledEnumerable.Instantiate(TypedEnumerator.Instantiate(this, bounds, SectorEnumeratorType.Clients)); 
		} 

		public IPooledEnumerable GetClientsInRange(Point3D p)
		{
			if (this == Map.Internal)
			{
				return NullEnumerable.Instance; 
			}
			return PooledEnumerable.Instantiate(TypedEnumerator.Instantiate(this, new Rectangle2D((p.m_X - 18), (p.m_Y - 18), 37, 37), SectorEnumeratorType.Clients)); 
		} 

		public IPooledEnumerable GetClientsInRange(Point3D p, int range)
		{
			if (this == Map.Internal)
			{
				return NullEnumerable.Instance; 
			}
			return PooledEnumerable.Instantiate(TypedEnumerator.Instantiate(this, new Rectangle2D((p.m_X - range), (p.m_Y - range), ((range * 2) + 1), ((range * 2) + 1)), SectorEnumeratorType.Clients)); 
		} 

		public IPooledEnumerable GetItemsInBounds(Rectangle2D bounds)
		{
			if (this == Map.Internal)
			{
				return NullEnumerable.Instance; 
			}
			return PooledEnumerable.Instantiate(TypedEnumerator.Instantiate(this, bounds, SectorEnumeratorType.Items)); 
		} 

		public IPooledEnumerable GetItemsInRange(Point3D p)
		{
			if (this == Map.Internal)
			{
				return NullEnumerable.Instance; 
			}
			return PooledEnumerable.Instantiate(TypedEnumerator.Instantiate(this, new Rectangle2D((p.m_X - 18), (p.m_Y - 18), 37, 37), SectorEnumeratorType.Items)); 
		} 

		public IPooledEnumerable GetItemsInRange(Point3D p, int range)
		{
			if (this == Map.Internal)
			{
				return NullEnumerable.Instance; 
			}
			return PooledEnumerable.Instantiate(TypedEnumerator.Instantiate(this, new Rectangle2D((p.m_X - range), (p.m_Y - range), ((range * 2) + 1), ((range * 2) + 1)), SectorEnumeratorType.Items)); 
		} 

		public static string[] GetMapNames()
		{
			Map.CheckNamesAndValues();
			return Map.m_MapNames; 
		} 

		public static Map[] GetMapValues()
		{
			Map.CheckNamesAndValues();
			return Map.m_MapValues; 
		} 

		public IPooledEnumerable GetMobilesInBounds(Rectangle2D bounds)
		{
			if (this == Map.Internal)
			{
				return NullEnumerable.Instance; 
			}
			return PooledEnumerable.Instantiate(TypedEnumerator.Instantiate(this, bounds, 0)); 
		} 

		public IPooledEnumerable GetMobilesInRange(Point3D p)
		{
			if (this == Map.Internal)
			{
				return NullEnumerable.Instance; 
			}
			return PooledEnumerable.Instantiate(TypedEnumerator.Instantiate(this, new Rectangle2D((p.m_X - 18), (p.m_Y - 18), 37, 37), 0)); 
		} 

		public IPooledEnumerable GetMobilesInRange(Point3D p, int range)
		{
			if (this == Map.Internal)
			{
				return NullEnumerable.Instance; 
			}
			return PooledEnumerable.Instantiate(TypedEnumerator.Instantiate(this, new Rectangle2D((p.m_X - range), (p.m_Y - range), ((range * 2) + 1), ((range * 2) + 1)), 0)); 
		} 

		public Sector GetMultiMaxSector(Point3D loc, MultiComponentList mcl)
		{
			Point2D pointd1;
			pointd1 = mcl.Max;
			pointd1 = mcl.Max;
			return this.GetSector(this.Bound(new Point2D((loc.m_X + pointd1.m_X), (loc.m_Y + pointd1.m_Y)))); 
		} 

		public Sector GetMultiMinSector(Point3D loc, MultiComponentList mcl)
		{
			Point2D pointd1;
			pointd1 = mcl.Min;
			pointd1 = mcl.Min;
			return this.GetSector(this.Bound(new Point2D((loc.m_X + pointd1.m_X), (loc.m_Y + pointd1.m_Y)))); 
		} 

		public IPooledEnumerable GetMultiTilesAt(int x, int y)
		{
			Sector sector1;
			if (this == Map.Internal)
			{
				return NullEnumerable.Instance; 
			}
			sector1 = this.GetSector(x, y);
			if (sector1.Multis.Count == 0)
			{
				return NullEnumerable.Instance; 
			}
			return PooledEnumerable.Instantiate(MultiTileEnumerator.Instantiate(sector1, new Point2D(x, y))); 
		} 

		public IPooledEnumerable GetObjectsInBounds(Rectangle2D bounds)
		{
			if (this == Map.Internal)
			{
				return NullEnumerable.Instance; 
			}
			return PooledEnumerable.Instantiate(ObjectEnumerator.Instantiate(this, bounds)); 
		} 

		public IPooledEnumerable GetObjectsInRange(Point3D p)
		{
			if (this == Map.Internal)
			{
				return NullEnumerable.Instance; 
			}
			return PooledEnumerable.Instantiate(ObjectEnumerator.Instantiate(this, new Rectangle2D((p.m_X - 18), (p.m_Y - 18), 37, 37))); 
		} 

		public IPooledEnumerable GetObjectsInRange(Point3D p, int range)
		{
			if (this == Map.Internal)
			{
				return NullEnumerable.Instance; 
			}
			return PooledEnumerable.Instantiate(ObjectEnumerator.Instantiate(this, new Rectangle2D((p.m_X - range), (p.m_Y - range), ((range * 2) + 1), ((range * 2) + 1)))); 
		} 

		public Point3D GetPoint(object o, bool eye)
		{
			Point3D pointd1;
			int num1;
			int num2;
			int num3;
			StaticTarget target1;
			ItemData data1;
			ItemData data2;
			if ((o as Mobile) != null)
			{
				pointd1 = ((Mobile) o).Location;
				pointd1.Z = (pointd1.Z + (eye ? 15 : 10));
 
			}
			else
			{
				if ((o as Item) != null)
				{
					pointd1 = ((Item) o).GetWorldLocation();
					data2 = ((Item) o).ItemData;
					pointd1.Z = (pointd1.Z + ((data2.Height / 2) + 1));
					goto L_0153;
 
				}
				if ((o as Point3D) != 0)
				{
					pointd1 = ((Point3D) o);
					goto L_0153;
 
				}
				if ((o as LandTarget) != null)
				{
					pointd1 = ((LandTarget) o).Location;
					num1 = 0;
					num2 = 0;
					num3 = 0;
					this.GetAverageZ(pointd1.X, pointd1.Y, ref num1, ref num2, ref num3);
					pointd1.Z = (num3 + 1);
					goto L_0153;
 
				}
				if ((o as StaticTarget) != null)
				{
					target1 = ((StaticTarget) o);
					data1 = TileData.ItemTable[(target1.ItemID & 16383)];
					pointd1 = new Point3D(target1.X, target1.Y, (((target1.Z - data1.CalcHeight) + (data1.Height / 2)) + 1));
					goto L_0153;
 
				}
				if ((o as IPoint3D) != null)
				{
					pointd1 = new Point3D(((IPoint3D) o));
					goto L_0153;
 
				}
				Console.WriteLine("Warning: Invalid object ({0}) in line of sight", o);
				pointd1 = Point3D.Zero;
 
			}
 
			L_0153:
				return pointd1; 
		}
		
		public Sector GetRealSector(int x, int y)
		{
			return this.InternalGetSector(x, y); 
		} 

		public Sector GetSector(int x, int y)
		{
			return this.InternalGetSector((x >> 4), (y >> 4)); 
		} 

		public Sector GetSector(IPoint2D p)
		{
			return this.InternalGetSector((p.X >> 4), (p.Y >> 4)); 
		} 

		public Sector GetSector(Point2D p)
		{
			return this.InternalGetSector((p.m_X >> 4), (p.m_Y >> 4)); 
		} 

		public Sector GetSector(Point3D p)
		{
			return this.InternalGetSector((p.m_X >> 4), (p.m_Y >> 4)); 
		} 

		public ArrayList GetTilesAt(Point2D p, bool items, bool land, bool statics)
		{
			ArrayList list1;
			Sector sector1;
			Item item1;
			IEnumerator enumerator1;
			IDisposable disposable1;
			list1 = new ArrayList();
			if (this == Map.Internal)
			{
				return list1; 
			}
			if (land)
			{
				list1.Add(this.Tiles.GetLandTile(p.m_X, p.m_Y));
 
			}
			if (statics)
			{
				list1.AddRange(this.Tiles.GetStaticTiles(p.m_X, p.m_Y, true));
 
			}
			if (items)
			{
				sector1 = this.GetSector(p);
				enumerator1 = sector1.Items.GetEnumerator();
				try
				{
					while (enumerator1.MoveNext())
					{
						item1 = ((Item) enumerator1.Current);
						if (item1.AtWorldPoint(p.m_X, p.m_Y))
						{
							list1.Add(new Tile(((short) ((item1.ItemID & 16383) + 16384)), ((sbyte) item1.Z)));
 
						}
 
					}
 
				}
				finally
				{
					disposable1 = (enumerator1 as IDisposable);
					if (disposable1 != null)
					{
						disposable1.Dispose();
 
					}
 
				}
 
			}
			return list1; 
		} 

		private Sector InternalGetSector(int x, int y)
		{
			Sector[] array1;
			Sector sector1;
			if (((x >= 0) && (x < this.m_SectorsWidth)) && ((y >= 0) && (y < this.m_SectorsHeight)))
			{
				array1 = this.m_Sectors[x];
				if (array1 == null)
				{
					array1 = new Sector[this.m_SectorsHeight];
					this.m_Sectors[x] = new Sector[this.m_SectorsHeight];
 
				}
				sector1 = array1[y];
				if (sector1 == null)
				{
					sector1 = new Sector(x, y, this);
					array1[y] = new Sector(x, y, this);
 
				}
				return sector1; 
			}
			return this.m_InvalidSector; 
		} 

		public bool LineOfSight(Mobile from, Mobile to)
		{
			Point3D pointd1;
			Point3D pointd2;
			if ((from == to) || (from.AccessLevel > 0))
			{
				return true; 
			}
			pointd1 = from.Location;
			pointd2 = to.Location;
			pointd1.Z = (pointd1.Z + 14);
			pointd2.Z = (pointd2.Z + 10);
			return this.LineOfSight(pointd1, pointd2); 
		} 

		public bool LineOfSight(Mobile from, Point3D target)
		{
			Point3D pointd1;
			if (from.AccessLevel > 0)
			{
				return true; 
			}
			pointd1 = from.Location;
			pointd1.Z = (pointd1.Z + 14);
			return this.LineOfSight(pointd1, target); 
		} 

		public bool LineOfSight(object from, object dest)
		{
			if ((from == dest) || (((from as Mobile) != null) && (((Mobile) from).AccessLevel > 0)))
			{
				return true; 
			}
			if (((dest as Item) != null) && (((from as Mobile) != null) && (((Item) dest).RootParent == from)))
			{
				return true; 
			}
			return this.LineOfSight(this.GetPoint(from, true), this.GetPoint(dest, false)); 
		} 

		public bool LineOfSight(Point3D org, Point3D dest)
		{
			double num1;
			double num2;
			double num3;
			double num4;
			double num5;
			double num6;
			double num7;
			int num8;
			int num9;
			int num10;
			int num11;
			int num12;
			int num13;
			int num14;
			bool flag1;
			Point3D pointd1;
			Point3DList list1;
			TileFlag flag2;
			Point3D pointd2;
			Point3D pointd3;
			int num15;
			int num16;
			Point3D pointd4;
			Tile tile1;
			int num17;
			int num18;
			int num19;
			Tile[] array1;
			bool flag3;
			int num20;
			int num21;
			IPooledEnumerable enumerable1;
			Item item1;
			int num22;
			Tile tile2;
			ItemData data1;
			Rectangle2D rectangled1;
			IPooledEnumerable enumerable2;
			Item item2;
			ItemData data2;
			int num23;
			int num24;
			Point3D pointd5;
			Point3D pointd6;
			bool flag4;
			IEnumerator enumerator1;
			IDisposable disposable1;
			if (this == Map.Internal)
			{
				return false; 
			}
			if (!Utility.InRange(org, dest, Map.m_MaxLOSDistance))
			{
				return false; 
			}
			list1 = Map.m_PathList;
			if (org == dest)
			{
				return true; 
			}
			if (list1.Count > 0)
			{
				list1.Clear();
 
			}
			num8 = (dest.m_X - org.m_X);
			num9 = (dest.m_Y - org.m_Y);
			num10 = (dest.m_Z - org.m_Z);
			num3 = Math.Sqrt(((double) ((num8 * num8) + (num9 * num9))));
			if (num10 != 0)
			{
				num4 = Math.Sqrt(((num3 * num3) + ((double) (num10 * num10))));
 
			}
			else
			{
				num4 = num3;
 
			}
			num1 = (((double) ((float) num9)) / num4);
			num2 = (((double) ((float) num8)) / num4);
			num3 = (((double) ((float) num10)) / num4);
			num6 = ((double) org.m_Y);
			num7 = ((double) org.m_Z);
			num5 = ((double) org.m_X);
			goto L_0152;
 
			L_00D2:
				num11 = ((int) Math.Round(num5));
			num12 = ((int) Math.Round(num6));
			num13 = ((int) Math.Round(num7));
			if (list1.Count > 0)
			{
				pointd1 = list1.Last;
				if (((pointd1.m_X == num11) && (pointd1.m_Y == num12)) && (pointd1.m_Z == num13))
				{
					goto L_0140;
 
				}
				list1.Add(num11, num12, num13);
 
			}
			else
			{
				list1.Add(num11, num12, num13);
 
			}
 
			L_0140:
				num5 = (num5 + num2);
			num6 = (num6 + num1);
			num7 = (num7 + num3);
 
			L_0152:
				if (Utility.NumberBetween(num5, dest.m_X, org.m_X, 0.5) && (Utility.NumberBetween(num6, dest.m_Y, org.m_Y, 0.5) && Utility.NumberBetween(num7, dest.m_Z, org.m_Z, 0.5)))
				{
					goto L_00D2;
 
				}
			if (list1.Count == 0)
			{
				return true; 
			}
			pointd1 = list1.Last;
			if (pointd1 != dest)
			{
				list1.Add(dest);
 
			}
			pointd2 = org;
			pointd3 = dest;
			Utility.FixPoints(ref pointd2, ref pointd3);
			num15 = list1.Count;
			num16 = 0;
			goto L_0427;
 
			L_01FB:
				pointd4 = list1[num16];
			tile1 = this.Tiles.GetLandTile(pointd4.X, pointd4.Y);
			num17 = 0;
			num18 = 0;
			num19 = 0;
			this.GetAverageZ(pointd4.m_X, pointd4.m_Y, ref num17, ref num18, ref num19);
			if (((num17 <= pointd4.m_Z) && (num19 >= pointd4.m_Z)) && ((((pointd4.m_X != dest.m_X) || (pointd4.m_Y != dest.m_Y)) || ((num17 > dest.m_Z) || (num19 < dest.m_Z))) && !tile1.Ignored))
			{
				return false; 
			}
			array1 = this.Tiles.GetStaticTiles(pointd4.m_X, pointd4.m_Y, true);
			flag3 = false;
			num20 = tile1.ID;
			for (num21 = 0; (!flag3 && (num21 < Map.m_InvalidLandTiles.Length)); num21 = (num21 + 1))
			{
				flag3 = (num20 == Map.m_InvalidLandTiles[num21]);
 
			}
			if (!flag3 || (array1.Length != 0))
			{
				goto L_0356;
 
			}
			enumerable1 = this.GetItemsInRange(pointd4, 0);
			enumerator1 = enumerable1.GetEnumerator();
			try
			{
				while (enumerator1.MoveNext())
				{
					item1 = ((Item) enumerator1.Current);
					if (item1.Visible)
					{
						flag3 = false;
 
					}
					if (!flag3)
					{
						goto L_0332;
 
					}
 
				}
 
			L_0332:
				goto L_0349;
 
			}
			finally
			{
				disposable1 = (enumerator1 as IDisposable);
				if (disposable1 != null)
				{
					disposable1.Dispose();
 
				}
 
			}
 
			L_0349:
				enumerable1.Free();
			if (flag3)
			{
				return false; 
			}
 
			L_0356:
				num22 = 0;
			while ((num22 < array1.Length))
			{
				tile2 = array1[num22];
				data1 = TileData.ItemTable[(tile2.ID & 16383)];
				flag2 = data1.Flags;
				num14 = data1.CalcHeight;
				if ((((tile2.Z <= pointd4.Z) && ((tile2.Z + num14) >= pointd4.Z)) && ((flag2 & (TileFlag.NoShoot | TileFlag.Window)) != 0)) && (((pointd4.m_X != dest.m_X) || (pointd4.m_Y != dest.m_Y)) || ((tile2.Z > dest.m_Z) || ((tile2.Z + num14) < dest.m_Z))))
				{
					return false; 
				}
				num22 = (num22 + 1);
 
			}
			num16 = (num16 + 1);
 
			L_0427:
				if (num16 < num15)
				{
					goto L_01FB;
 
				}
			rectangled1 = new Rectangle2D(pointd2.m_X, pointd2.m_Y, ((pointd3.m_X - pointd2.m_X) + 1), ((pointd3.m_Y - pointd2.m_Y) + 1));
			enumerable2 = this.GetItemsInBounds(rectangled1);
			enumerator1 = enumerable2.GetEnumerator();
			try
			{
				goto L_05A4;
 
			L_047F:
				item2 = ((Item) enumerator1.Current);
				if (!item2.Visible || (item2.ItemID >= 16384))
				{
					goto L_05A4;
 
				}
				data2 = item2.ItemData;
				flag2 = data2.Flags;
				if ((flag2 & (TileFlag.NoShoot | TileFlag.Window)) == 0)
				{
					goto L_05A4;
 
				}
				num14 = data2.CalcHeight;
				flag1 = false;
				num23 = list1.Count;
				for (num24 = 0; (num24 < num23); num24 = (num24 + 1))
				{
					pointd5 = list1[num24];
					pointd6 = item2.Location;
					if ((((pointd6.m_X == pointd5.m_X) && (pointd6.m_Y == pointd5.m_Y)) && ((pointd6.m_Z <= pointd5.m_Z) && ((pointd6.m_Z + num14) >= pointd5.m_Z))) && (((pointd6.m_X != dest.m_X) || (pointd6.m_Y != dest.m_Y)) || ((pointd6.m_Z > dest.m_Z) || ((pointd6.m_Z + num14) < dest.m_Z))))
					{
						flag1 = true;
						goto L_0594;
 
					}
 
				}
 
			L_0594:
				if (flag1)
				{
					enumerable2.Free();
					flag4 = false;
					goto L_05D0;
 
				}
 
			L_05A4:
				if (enumerator1.MoveNext())
				{
					goto L_047F;
 
				}
 
			}
			finally
			{
				disposable1 = (enumerator1 as IDisposable);
				if (disposable1 != null)
				{
					disposable1.Dispose();
 
				}
 
			}
			enumerable2.Free();
			return true; 
			L_05D0:
				return flag4; 
		}
		
		public void OnClientChange(NetState oldState, NetState newState, Mobile m)
		{
			if (this == Map.Internal)
			{
				return; 
			}
			this.GetSector(m).OnClientChange(oldState, newState);
 
		} 

		public void OnEnter(Item item)
		{
			MultiComponentList list1;
			Sector sector1;
			Sector sector2;
			if (this == Map.Internal)
			{
				return; 
			}
			this.GetSector(item).OnEnter(item);
			if ((item as BaseMulti) != null)
			{
				list1 = ((BaseMulti) item).Components;
				sector1 = this.GetMultiMinSector(item.Location, list1);
				sector2 = this.GetMultiMaxSector(item.Location, list1);
				this.AddMulti(item, sector1, sector2);
 
			}
 
		} 

		public void OnEnter(Mobile m)
		{
			if (this == Map.Internal)
			{
				return; 
			}
			this.GetSector(m).OnEnter(m);
 
		} 

		public void OnLeave(Item item)
		{
			MultiComponentList list1;
			Sector sector1;
			Sector sector2;
			if (this == Map.Internal)
			{
				return; 
			}
			this.GetSector(item).OnLeave(item);
			if ((item as BaseMulti) != null)
			{
				list1 = ((BaseMulti) item).Components;
				sector1 = this.GetMultiMinSector(item.Location, list1);
				sector2 = this.GetMultiMaxSector(item.Location, list1);
				this.RemoveMulti(item, sector1, sector2);
 
			}
 
		} 

		public void OnLeave(Mobile m)
		{
			if (this == Map.Internal)
			{
				return; 
			}
			this.GetSector(m).OnLeave(m);
 
		} 

		public void OnMove(Point3D oldLocation, Item item)
		{
			Sector sector1;
			Sector sector2;
			MultiComponentList list1;
			Sector sector3;
			Sector sector4;
			Sector sector5;
			Sector sector6;
			if (this == Map.Internal)
			{
				return; 
			}
			sector1 = this.GetSector(oldLocation);
			sector2 = this.GetSector(item.Location);
			if (sector1 != sector2)
			{
				sector1.OnLeave(item);
				sector2.OnEnter(item);
 
			}
			if ((item as BaseMulti) != null)
			{
				list1 = ((BaseMulti) item).Components;
				sector3 = this.GetMultiMinSector(item.Location, list1);
				sector4 = this.GetMultiMaxSector(item.Location, list1);
				sector5 = this.GetMultiMinSector(oldLocation, list1);
				sector6 = this.GetMultiMaxSector(oldLocation, list1);
				if ((sector5 != sector3) || (sector6 != sector4))
				{
					this.RemoveMulti(item, sector5, sector6);
					this.AddMulti(item, sector3, sector4);
 
				}
 
			}
 
		} 

		public void OnMove(Point3D oldLocation, Mobile m)
		{
			Sector sector1;
			Sector sector2;
			if (this == Map.Internal)
			{
				return; 
			}
			sector1 = this.GetSector(oldLocation);
			sector2 = this.GetSector(m.Location);
			if (sector1 != sector2)
			{
				sector1.OnLeave(m);
				sector2.OnEnter(m);
 
			}
 
		} 

		public static Map Parse(string value)
		{
			int num1;
			int num2;
			Map map1;
			Map.CheckNamesAndValues();
			for (num1 = 0; (num1 < Map.m_MapNames.Length); num1 = (num1 + 1))
			{
				if (Insensitive.Equals(Map.m_MapNames[num1], value))
				{
					return Map.m_MapValues[num1]; 
				}
 
			}
			try
			{
				num2 = int.Parse(value);
				if (((num2 >= 0) && (num2 < Map.m_Maps.Length)) && (Map.m_Maps[num2] != null))
				{
					map1 = Map.m_Maps[num2];
					goto L_0066;
 
				}
 
			}
			catch /*(Exception ignored)*/
			{
  
			}
			throw new Exception("Invalid map name");
 
			L_0066:
				return map1; 
		} 

		private bool PlayersInRange(Sector sect, int range)
		{
			int num1;
			int num2;
			Sector sector1;
			for (num1 = (sect.X - range); (num1 <= (sect.X + range)); num1 = (num1 + 1))
			{
				for (num2 = (sect.Y - range); (num2 <= (sect.Y + range)); num2 = (num2 + 1))
				{
					sector1 = this.GetRealSector(num1, num2);
					if ((sector1 != this.m_InvalidSector) && (sector1.Players.Count > 0))
					{
						return true; 
					}
 
				}
 
			}
			return false; 
		} 

		public void RemoveMulti(Item item, Sector start, Sector end)
		{
			int num1;
			int num2;
			if (this == Map.Internal)
			{
				return; 
			}
			for (num1 = start.X; (num1 <= end.X); num1 = (num1 + 1))
			{
				for (num2 = start.Y; (num2 <= end.Y); num2 = (num2 + 1))
				{
					this.InternalGetSector(num1, num2).OnMultiLeave(item);
 
				}
 
			}
 
		} 

		public override string ToString()
		{
			return this.m_Name; 
		} 

		public static ArrayList AllMaps 
		{ 
			get
			{
				return Map.m_AllMaps;
			}
		}

		public Region DefaultRegion 
		{ 
			get
			{
				if (this.m_DefaultRegion == null)
				{
					this.m_DefaultRegion = new Region("", "", this);
 
				}
				return this.m_DefaultRegion; 
			}
			set
			{
				this.m_DefaultRegion = value;
			}
		}

		public static Map Felucca 
		{ 
			get
			{
				return Map.m_Maps[0];
			}
		}

		public int Height 
		{ 
			get
			{
				return this.m_Height;
			}
		}

		public static Map Ilshenar 
		{ 
			get
			{
				return Map.m_Maps[2];
			}
		}

		public static Map Internal 
		{ 
			get
			{
				return Map.m_Maps[127];
			}
		}

		public static int[] InvalidLandTiles 
		{ 
			get
			{
				return Map.m_InvalidLandTiles;
			}
			set
			{
				Map.m_InvalidLandTiles = value;
			}
		}

		public Sector InvalidSector 
		{ 
			get
			{
				return this.m_InvalidSector;
			}
		}

		public static Map Malas 
		{ 
			get
			{
				return Map.m_Maps[3];
			}
		}

		public int MapID 
		{ 
			get
			{
				return this.m_MapID;
			}
		}

		public int MapIndex 
		{ 
			get
			{
				return this.m_MapIndex;
			}
		}

		public static Map[] Maps 
		{ 
			get
			{
				return Map.m_Maps;
			}
		}

		public static int MaxLOSDistance 
		{ 
			get
			{
				return Map.m_MaxLOSDistance;
			}
			set
			{
				Map.m_MaxLOSDistance = value;
			}
		}

		public string Name 
		{ 
			get
			{
				return this.m_Name;
			}
			set
			{
				this.m_Name = value;
			}
		}

		public ArrayList Regions 
		{ 
			get
			{
				return this.m_Regions;
			}
		}

		public MapRules Rules 
		{ 
			get
			{
				return this.m_Rules;
			}
			set
			{
				this.m_Rules = value;
			}
		}

		public int Season 
		{ 
			get
			{
				return this.m_Season;
			}
			set
			{
				this.m_Season = value;
			}
		}

		public TileMatrix Tiles 
		{ 
			get
			{
				if (this.m_Tiles == null)
				{
					this.m_Tiles = new TileMatrix(this, this.m_FileIndex, this.m_MapID, this.m_Width, this.m_Height);
 
				}
				return this.m_Tiles;
			}
		}

		public static Map Trammel 
		{ 
			get
			{
				return Map.m_Maps[1];
			}
		}

		public int Width 
		{ 
			get
			{
				return this.m_Width;
			}
		}

		
		private static ArrayList m_AllMaps = new ArrayList();
		private Region m_DefaultRegion;
		private int m_FileIndex;
		private int m_Height;
		private static int[] m_InvalidLandTiles = new int[1] { 580 };
		private Sector m_InvalidSector;
		private int m_MapID;
		private int m_MapIndex;
		private static string[] m_MapNames;
		private static Map[] m_Maps = new Map[256];
		private static Map[] m_MapValues;
		private static int m_MaxLOSDistance = 25;
		private string m_Name;
		private static Point3DList m_PathList = new Point3DList();
		private ArrayList m_Regions;
		private MapRules m_Rules;
		private int m_Season;
		private Sector[][] m_Sectors;
		private int m_SectorsHeight;
		private int m_SectorsWidth;
		private TileMatrix m_Tiles;
		private int m_Width;
		public static int SectorActiveRange = 2;
		public const int SectorShift = 4;
		public const int SectorSize = 16;

		private class MultiTileEnumerator : IPooledEnumerator, IDisposable
		{
			private MultiTileEnumerator(Sector sector, Point2D loc) : base()
			{
				this.m_List = sector.Multis;
				this.m_Location = loc;
				this.Reset();
			} 

			public void Dispose()
			{
				this.Free();
			}

			public void Free()
			{
				if (this.m_List == null)
				{
					return; 
				}
				MultiTileEnumerator.m_InstancePool.Enqueue(this);
				this.m_List = null;
				if (this.m_Enumerable != null)
				{
					this.m_Enumerable.Free();
 
				}
 
			} 

			public static MultiTileEnumerator Instantiate(Sector sector, Point2D loc)
			{
				MultiTileEnumerator enumerator1;
				if (MultiTileEnumerator.m_InstancePool.Count > 0)
				{
					enumerator1 = ((MultiTileEnumerator) MultiTileEnumerator.m_InstancePool.Dequeue());
					enumerator1.m_List = sector.Multis;
					enumerator1.m_Location = loc;
					enumerator1.Reset();
 
				}
				else
				{
					enumerator1 = new MultiTileEnumerator(sector, loc);
 
				}
				return enumerator1; 
			} 

			public bool MoveNext()
			{
				BaseMulti multi1;
				MultiComponentList list1;
				int num1;
				int num2;
				Tile[] array1;
				Tile[] array2;
				int num3;
				Point3D pointd1;
				Point2D pointd2;
				int num4;
				goto L_011B;
 
				L_0005:
					multi1 = (this.m_List[this.m_Index] as BaseMulti);
				if ((multi1 != null) && !multi1.Deleted)
				{
					list1 = multi1.Components;
					pointd1 = multi1.Location;
					pointd2 = list1.Min;
					num1 = (this.m_Location.m_X - (pointd1.m_X + pointd2.m_X));
					pointd1 = multi1.Location;
					pointd2 = list1.Min;
					num2 = (this.m_Location.m_Y - (pointd1.m_Y + pointd2.m_Y));
					if (((num1 >= 0) && (num1 < list1.Width)) && ((num2 >= 0) && (num2 < list1.Height)))
					{
						array1 = list1.Tiles[num1][num2];
						if (array1.Length > 0)
						{
							array2 = new Tile[array1.Length];
							for (num3 = 0; (num3 < array2.Length); num3 = (num3 + 1))
							{
								array2[num3] = array1[num3];
								array2[num3].Z = (array2[num3].Z + multi1.Z);
 
							}
							this.m_Current = array2;
							return true; 
						}
 
					}
 
				}
 
				L_011B:
					num4 = (this.m_Index + 1);
				this.m_Index = (this.m_Index + 1);
				if (num4 < this.m_List.Count)
				{
					goto L_0005;
 
				}
				return false; 
			} 

			public void Reset()
			{
				this.m_Current = null;
				this.m_Index = -1;
 
			} 

			public object Current 
			{ 
				get 
				{
					return this.m_Current;
				}
			}

			public IPooledEnumerable Enumerable 
			{ 
				get
				{
					return this.m_Enumerable;
				}
				set
				{
					this.m_Enumerable = value;
				}
			}
			
			private object m_Current;
			private IPooledEnumerable m_Enumerable;
			private int m_Index;
			private static Queue m_InstancePool = new Queue();
			private ArrayList m_List;
			private Point2D m_Location;
		}

		public class NullEnumerable : IPooledEnumerable
		{
			private NullEnumerable() : base()
			{
				this.m_Enumerator = new InternalEnumerator();
			}
			
			public void Free()
			{
			}

			public IEnumerator GetEnumerator()
			{
				return this.m_Enumerator;
			}

			public static readonly NullEnumerable Instance = new NullEnumerable();
			private InternalEnumerator m_Enumerator;

			private class InternalEnumerator : IEnumerator
			{
				public InternalEnumerator()
				{
				}

				public bool MoveNext()
				{
					return false;
				}

				public void Reset()
				{
				}

				public object Current 
				{ 
					get
					{
						return null;
					}
				}
			}
		}

		private class ObjectEnumerator : IPooledEnumerator, IDisposable
		{
			private ObjectEnumerator(Map map, Rectangle2D bounds) : base()
			{
				this.m_Map = map;
				this.m_Bounds = bounds;
				this.Reset();
			} 

			public void Dispose()
			{
				this.Free();
 
			} 

			public void Free()
			{
				if (this.m_Map == null)
				{
					return; 
				}
				ObjectEnumerator.m_InstancePool.Enqueue(this);
				this.m_Map = null;
				if (this.m_Enumerator != null)
				{
					this.m_Enumerator.Free();
					this.m_Enumerator = null;
 
				}
				if (this.m_Enumerable != null)
				{
					this.m_Enumerable.Free();
 
				}
 
			} 

			public static ObjectEnumerator Instantiate(Map map, Rectangle2D bounds)
			{
				ObjectEnumerator enumerator1;
				if (ObjectEnumerator.m_InstancePool.Count > 0)
				{
					enumerator1 = ((ObjectEnumerator) ObjectEnumerator.m_InstancePool.Dequeue());
					enumerator1.m_Map = map;
					enumerator1.m_Bounds = bounds;
					enumerator1.Reset();
 
				}
				else
				{
					enumerator1 = new ObjectEnumerator(map, bounds);
 
				}
				return enumerator1; 
			} 

			public bool MoveNext()
			{
				object obj1;
				Mobile mobile1;
				Item item1;
 
				L_0000:
					if (this.m_Enumerator.MoveNext())
					{
						obj1 = this.m_Enumerator.Current;
						if ((obj1 as Mobile) != null)
						{
							mobile1 = ((Mobile) obj1);
							if (!this.m_Bounds.Contains(mobile1.Location))
							{
								goto L_0000;
 
							}
							this.m_Current = obj1;
							return true; 
						}
						if ((obj1 as Item) == null)
						{
							goto L_0000;
 
						}
						item1 = ((Item) obj1);
						if ((item1.Parent != null) || !this.m_Bounds.Contains(item1.Location))
						{
							goto L_0000;
 
						}
						this.m_Current = obj1;
						return true; 
					}
				if (this.m_Stage == 0)
				{
					this.m_Enumerator.Free();
					this.m_Enumerator = SectorEnumerator.Instantiate(this.m_Map, this.m_Bounds, 0);
					this.m_Current = null;
					this.m_Stage = 1;
					goto L_0000;
 
				}
				this.m_Enumerator.Free();
				this.m_Enumerator = null;
				this.m_Current = null;
				this.m_Stage = -1;
				return false; 
			} 

			public void Reset()
			{
				this.m_Stage = 0;
				this.m_Current = null;
				if (this.m_Enumerator != null)
				{
					this.m_Enumerator.Free();
 
				}
				this.m_Enumerator = SectorEnumerator.Instantiate(this.m_Map, this.m_Bounds, SectorEnumeratorType.Items);
 
			} 

			public object Current 
			{ 
				get
				{
					return this.m_Current;
				}
			}
			public IPooledEnumerable Enumerable 
			{ 
				get
				{
					return this.m_Enumerable;
				}
				set
				{
					this.m_Enumerable = value;
				}
			}
			
			private Rectangle2D m_Bounds;
			private object m_Current;
			private IPooledEnumerable m_Enumerable;
			private SectorEnumerator m_Enumerator;
			private static Queue m_InstancePool = new Queue();
			private Map m_Map;
			private int m_Stage;
		}

		private class PooledEnumerable : IPooledEnumerable, IDisposable
		{
			private PooledEnumerable(IPooledEnumerator etor) : base()
			{
				this.m_Enumerator = etor;
			}

			public void Dispose()
			{
				this.Free();
			}

			public void Free()
			{
				if (this.m_Enumerator != null)
				{
					PooledEnumerable.m_InstancePool.Enqueue(this);
					this.m_Enumerator.Free();
					this.m_Enumerator = null;
					PooledEnumerable.m_Depth = (PooledEnumerable.m_Depth - 1);
 
				}
 
			} 

			public IEnumerator GetEnumerator()
			{
				if (this.m_Enumerator == null)
				{
					throw new ObjectDisposedException("PooledEnumerable", "GetEnumerator() called after Free()");
 
				}
				return this.m_Enumerator; 
			} 

			public static PooledEnumerable Instantiate(IPooledEnumerator etor)
			{
				PooledEnumerable enumerable1;
				PooledEnumerable.m_Depth = (PooledEnumerable.m_Depth + 1);
				if (PooledEnumerable.m_Depth >= 5)
				{
					Console.WriteLine("Warning: Make sure to call .Free() on pooled enumerables.");
 
				}
				if (PooledEnumerable.m_InstancePool.Count > 0)
				{
					enumerable1 = ((PooledEnumerable) PooledEnumerable.m_InstancePool.Dequeue());
					enumerable1.m_Enumerator = etor;
 
				}
				else
				{
					enumerable1 = new PooledEnumerable(etor);
 
				}
				etor.Enumerable = enumerable1;
				return enumerable1; 
			} 

			private static int m_Depth = 0;
			private IPooledEnumerator m_Enumerator;
			private static Queue m_InstancePool = new Queue();
		}

		private class SectorEnumerator : IPooledEnumerator, IDisposable
		{
			private SectorEnumerator(Map map, Rectangle2D bounds, SectorEnumeratorType type) : base()
			{
				this.m_Map = map;
				this.m_Bounds = bounds;
				this.m_Type = type;
				this.Reset();
			} 

			public void Dispose()
			{
				this.Free();
			}

			public void Free()
			{
				if (this.m_Map == null)
				{
					return; 
				}
				SectorEnumerator.m_InstancePool.Enqueue(this);
				this.m_Map = null;
				if (this.m_Enumerable != null)
				{
					this.m_Enumerable.Free();
 
				}
 
			} 

			private ArrayList GetListForSector(Sector sector)
			{
				SectorEnumeratorType type1;
				type1 = this.m_Type;
				switch (type1)
				{
					case SectorEnumeratorType.Mobiles:
					{
						goto L_0022;
 
					}
					case SectorEnumeratorType.Items:
					{
						goto L_0029;
 
					}
					case SectorEnumeratorType.Clients:
					{
						goto L_001B;
 
					}
 
				}
				goto L_0030;
 
				L_001B:
					return sector.Clients; 
				L_0022:
					return sector.Mobiles; 
				L_0029:
					return sector.Items; 
				L_0030:
					throw new Exception("Invalid SectorEnumeratorType");
 
			} 

			public static SectorEnumerator Instantiate(Map map, Rectangle2D bounds, SectorEnumeratorType type)
			{
				SectorEnumerator enumerator1;
				if (SectorEnumerator.m_InstancePool.Count > 0)
				{
					enumerator1 = ((SectorEnumerator) SectorEnumerator.m_InstancePool.Dequeue());
					enumerator1.m_Map = map;
					enumerator1.m_Bounds = bounds;
					enumerator1.m_Type = type;
					enumerator1.Reset();
 
				}
				else
				{
					enumerator1 = new SectorEnumerator(map, bounds, type);
 
				}
				return enumerator1; 
			} 

			public bool MoveNext()
			{  
				L_0000:
					this.m_CurrentIndex = (this.m_CurrentIndex + 1);
				if (this.m_CurrentIndex == this.m_CurrentList.Count)
				{
					this.m_ySector = (this.m_ySector + 1);
					if (this.m_ySector > this.m_ySectorEnd)
					{
						this.m_ySector = this.m_ySectorStart;
						this.m_xSector = (this.m_xSector + 1);
						if (this.m_xSector > this.m_xSectorEnd)
						{
							this.m_CurrentIndex = -1;
							this.m_CurrentList = null;
							return false; 
						}
 
					}
					this.m_CurrentIndex = -1;
					this.m_CurrentList = this.GetListForSector(this.m_Map.InternalGetSector(this.m_xSector, this.m_ySector));
					goto L_0000;
 
				}
				return true; 
			} 

			public void Reset()
			{
				Point2D pointd1;
				int num1;
				pointd1 = this.m_Bounds.Start;
				pointd1 = this.m_Bounds.Start;
				this.m_Map.Bound(pointd1.m_X, pointd1.m_Y, ref this.m_xSectorStart, ref this.m_ySectorStart);
				pointd1 = this.m_Bounds.End;
				pointd1 = this.m_Bounds.End;
				this.m_Map.Bound((pointd1.m_X - 1), (pointd1.m_Y - 1), ref this.m_xSectorEnd, ref this.m_ySectorEnd);
				num1 = (this.m_xSectorStart >> 4);
				this.m_xSectorStart = (this.m_xSectorStart >> 4);
				this.m_xSector = num1;
				num1 = (this.m_ySectorStart >> 4);
				this.m_ySectorStart = (this.m_ySectorStart >> 4);
				this.m_ySector = num1;
				this.m_xSectorEnd = (this.m_xSectorEnd >> 4);
				this.m_ySectorEnd = (this.m_ySectorEnd >> 4);
				this.m_CurrentIndex = -1;
				this.m_CurrentList = this.GetListForSector(this.m_Map.InternalGetSector(this.m_xSector, this.m_ySector));
 
			} 
			
			public object Current 
			{ 
				get
				{
					return this.m_CurrentList[this.m_CurrentIndex];
				}
			}

			public IPooledEnumerable Enumerable 
			{ 
				get
				{
					return this.m_Enumerable;
				}
				set
				{
					this.m_Enumerable = value;
				}
			}

			private Rectangle2D m_Bounds;
			private int m_CurrentIndex;
			private ArrayList m_CurrentList;
			private IPooledEnumerable m_Enumerable;
			private static Queue m_InstancePool = new Queue();
			private Map m_Map;
			private SectorEnumeratorType m_Type;
			private int m_xSector;
			private int m_xSectorEnd;
			private int m_xSectorStart;
			private int m_ySector;
			private int m_ySectorEnd;
			private int m_ySectorStart;
		}

		private enum SectorEnumeratorType
		{
			Mobiles = 0,
			Items = 1,
			Clients = 2,
		}

		private class TypedEnumerator : IPooledEnumerator, IDisposable
		{
			public TypedEnumerator(Map map, Rectangle2D bounds, SectorEnumeratorType type) : base()
			{
				this.m_Map = map;
				this.m_Bounds = bounds;
				this.m_Type = type;
				this.Reset();
			} 

			public void Dispose()
			{
				this.Free();
			}

			public void Free()
			{
				if (this.m_Map == null)
				{
					return; 
				}
				TypedEnumerator.m_InstancePool.Enqueue(this);
				this.m_Map = null;
				if (this.m_Enumerator != null)
				{
					this.m_Enumerator.Free();
					this.m_Enumerator = null;
 
				}
				if (this.m_Enumerable != null)
				{
					this.m_Enumerable.Free();
 
				}
			}

			public static TypedEnumerator Instantiate(Map map, Rectangle2D bounds, SectorEnumeratorType type)
			{
				TypedEnumerator enumerator1;
				if (TypedEnumerator.m_InstancePool.Count > 0)
				{
					enumerator1 = ((TypedEnumerator) TypedEnumerator.m_InstancePool.Dequeue());
					enumerator1.m_Map = map;
					enumerator1.m_Bounds = bounds;
					enumerator1.m_Type = type;
					enumerator1.Reset();
 
				}
				else
				{
					enumerator1 = new TypedEnumerator(map, bounds, type);
 
				}
				return enumerator1; 
			} 

			public bool MoveNext()
			{
				object obj1;
				Mobile mobile1;
				Item item1;
				Mobile mobile2;
 
				L_0000:
					if (this.m_Enumerator.MoveNext())
					{
						obj1 = this.m_Enumerator.Current;
						if ((obj1 as Mobile) != null)
						{
							mobile1 = ((Mobile) obj1);
							if (mobile1.Deleted || !this.m_Bounds.Contains(mobile1.Location))
							{
								goto L_0000;
 
							}
							this.m_Current = obj1;
							return true; 
						}
						if ((obj1 as Item) != null)
						{
							item1 = ((Item) obj1);
							if ((item1.Deleted || (item1.Parent != null)) || !this.m_Bounds.Contains(item1.Location))
							{
								goto L_0000;
 
							}
							this.m_Current = obj1;
							return true; 
						}
						if ((obj1 as NetState) == null)
						{
							goto L_0000;
 
						}
						mobile2 = ((NetState) obj1).Mobile;
						if (((mobile2 == null) || mobile2.Deleted) || !this.m_Bounds.Contains(mobile2.Location))
						{
							goto L_0000;
 
						}
						this.m_Current = obj1;
						return true; 
					}
				this.m_Current = null;
				this.m_Enumerator.Free();
				this.m_Enumerator = null;
				return false; 
			} 

			public void Reset()
			{
				this.m_Current = null;
				if (this.m_Enumerator != null)
				{
					this.m_Enumerator.Free();
 
				}
				this.m_Enumerator = SectorEnumerator.Instantiate(this.m_Map, this.m_Bounds, this.m_Type);
 
			} 
			
			public object Current 
			{ 
				get
				{
					return this.m_Current;
				}
			}

			public IPooledEnumerable Enumerable 
			{ 
				get
				{
					return this.m_Enumerable;
				}
				set
				{
					this.m_Enumerable = value;
				}
			}

			private Rectangle2D m_Bounds;
			private object m_Current;
			private IPooledEnumerable m_Enumerable;
			private SectorEnumerator m_Enumerator;
			private static Queue m_InstancePool = new Queue();
			private Map m_Map;
			private SectorEnumeratorType m_Type;
		}

		private class ZComparer : IComparer
		{
			public ZComparer() : base()
			{
			}

			public int Compare(object x, object y)
			{
				Item item1;
				Item item2;
				int num1;
				item1 = ((Item) x);
				item2 = ((Item) y);
				num1 = item1.Z;
				return num1.CompareTo(item2.Z); 
			} 

			public static readonly IComparer Default = new ZComparer();
		}
 
	}

}