using System;

namespace Server
{
	public class ResistanceMod
	{
		public ResistanceMod(ResistanceType type, int offset) : base()
		{
			this.m_Type = type;
			this.m_Offset = offset;
		} 

		public int Offset { 
			get
			{
				return this.m_Offset;
			}
			set
			{
				if (this.m_Offset != value)
				{
					this.m_Offset = value;
					if (this.m_Owner != null)
					{
						this.m_Owner.UpdateResistances();
					}
				}
			}
		}

		public Mobile Owner { 
			get
			{
				return this.m_Owner;
			}
			set
			{
				this.m_Owner = value;
			}
		}

		public ResistanceType Type { 
			get
			{
				return this.m_Type;
			}
			set
			{
				if (this.m_Type != value)
				{
					this.m_Type = value;
					if (this.m_Owner != null)
					{
						this.m_Owner.UpdateResistances();
					}
				}			
			}
		}

		private int m_Offset;
		private Mobile m_Owner;
		private ResistanceType m_Type;
	}

	public abstract class SkillMod
	{
		public SkillMod(SkillName skill, bool relative, double value) : base()
		{
			this.m_Skill = skill;
			this.m_Relative = relative;
			this.m_Value = value;
		} 

		public abstract bool CheckCondition();

		public void Remove()
		{
			this.Owner = null;
		}

		public bool Absolute { 
			get
			{
				return !this.m_Relative;
			}
			set
			{
				Skill skill1;
				if (this.m_Relative == value)
				{
					this.m_Relative = !value;
					if (this.m_Owner != null)
					{
						skill1 = this.m_Owner.Skills[this.m_Skill];
						if (skill1 != null)
						{
							skill1.Update();
 
						}
 
					}
 
				}
			}
		}

		public bool ObeyCap 
		{ 
			get
			{
				return this.m_ObeyCap;
			}
			set
			{
				Skill skill1;
				this.m_ObeyCap = value;
				if (this.m_Owner != null)
				{
					skill1 = this.m_Owner.Skills[this.m_Skill];
					if (skill1 != null)
					{
						skill1.Update();
 
					}
 
				}
			}
		}

		public Mobile Owner 
		{ 
			get
			{
				return this.m_Owner;
			}
			set
			{
				if (this.m_Owner != value)
				{
					if (this.m_Owner != null)
					{
						this.m_Owner.RemoveSkillMod(this);
 
					}
					this.m_Owner = value;
					if (this.m_Owner != value)
					{
						this.m_Owner.AddSkillMod(this);
 
					}
 
				} 

			}
		}

		public bool Relative 
		{ 
			get
			{
				return this.m_Relative;
			}
			set
			{
				Skill skill1;
				if (this.m_Relative != value)
				{
					this.m_Relative = value;
					if (this.m_Owner != null)
					{
						skill1 = this.m_Owner.Skills[this.m_Skill];
						if (skill1 != null)
						{
							skill1.Update();
 
						}
 
					}
 
				}
			}
		}

		public SkillName Skill 
		{ 
			get
			{
				return this.m_Skill;
			}
			set
			{
				Skill skill1;
				Skill skill2;
				if (this.m_Skill != value)
				{
					skill1 = ((this.m_Owner == null) ? this.m_Owner.Skills[this.m_Skill] : null);
					this.m_Skill = value;
					if (this.m_Owner != null)
					{
						skill2 = this.m_Owner.Skills[this.m_Skill];
						if (skill2 != null)
						{
							skill2.Update();
 
						}
 
					}
					if (skill1 != null)
					{
						skill1.Update();
 
					}
 
				}
			}
		}

		public double Value 
		{ 
			get
			{
				return this.m_Value;
			}
			set
			{
				Skill skill1;
				if (this.m_Value != value)
				{
					this.m_Value = value;
					if (this.m_Owner != null)
					{
						skill1 = this.m_Owner.Skills[this.m_Skill];
						if (skill1 != null)
						{
							skill1.Update();
 
						}
 
					}
 
				}
			}
		}


		private bool m_ObeyCap;
		private Mobile m_Owner;
		private bool m_Relative;
		private SkillName m_Skill;
		private double m_Value;
	}

	public class StatMod
	{
		public StatMod(StatType type, string name, int offset, TimeSpan duration) : base()
		{
			this.m_Type = type;
			this.m_Name = name;
			this.m_Offset = offset;
			this.m_Duration = duration;
			this.m_Added = DateTime.Now;
		} 

		public bool HasElapsed()
		{
				if (this.m_Duration == TimeSpan.Zero)
		  {
			  return false; 
		  }
			return (DateTime.Now - this.m_Added) >= this.m_Duration; 
		} 

		public string Name { 
			get
			{
				return this.m_Name;
			}
		}

		public int Offset { 
			get
			{
				return this.m_Offset;
			}
		}

		public StatType Type { 
			get
			{
				return this.m_Type;
			}
		}

		private DateTime m_Added;
		private TimeSpan m_Duration;
		private string m_Name;
		private int m_Offset;
		private StatType m_Type;
	}
}