using System;
using System.IO;
using System.Text;
using System.Collections;

using Server.Gumps;
using Server.Network;

namespace Server
{
	public sealed class DisplayGump : Packet
	{
		public DisplayGump(Gump g, string layout, string[] text) : base(176)
		{
			int num1;
			string text1;
			int num2;
			if (layout == null)
			{
				layout = "";
			}
			base.EnsureCapacity(256);
			base.m_Stream.Write(g.Serial);
			base.m_Stream.Write(g.TypeID);
			base.m_Stream.Write(g.X);
			base.m_Stream.Write(g.Y);
			base.m_Stream.Write(((ushort) (layout.Length + 1)));
			base.m_Stream.WriteAsciiNull(layout);
			base.m_Stream.Write(((ushort) text.Length));
			for (num1 = 0; (num1 < text.Length); num1 = (num1 + 1))
			{
				text1 = text[num1];
				if (text1 == null)
				{
					text1 = "";
				}
				num2 = ((ushort) text1.Length);
				base.m_Stream.Write(((ushort) num2));
				base.m_Stream.WriteBigUniFixed(text1, num2);
			}
		} 
	}
 
	public sealed class DisplayGumpFast : Packet
	{
		static DisplayGumpFast()
		{
			DisplayGumpFast.m_Buffer = new byte[48];
			DisplayGumpFast.m_Buffer[0] = 32;
		}

		public DisplayGumpFast(Gump g) : base(176)
		{
			this.m_True = Gump.StringToBuffer(" 1");
			this.m_False = Gump.StringToBuffer(" 0");
			base.EnsureCapacity(1024);
			base.m_Stream.Write(g.Serial);
			base.m_Stream.Write(g.TypeID);
			base.m_Stream.Write(g.X);
			base.m_Stream.Write(g.Y);
			base.m_Stream.Write(65535);
		} 

		public void AppendLayout(bool val)
		{
			this.AppendLayout((val ? this.m_True : this.m_False));
		} 

		public void AppendLayout(byte[] buffer)
		{
			int num1;
			num1 = buffer.Length;
			base.m_Stream.Write(buffer, 0, num1);
			this.m_LayoutLength = (this.m_LayoutLength + num1);
		} 

		public void AppendLayout(int val)
		{
			string text1;
			int num1;
			text1 = val.ToString();
			num1 = (Encoding.ASCII.GetBytes(text1, 0, text1.Length, DisplayGumpFast.m_Buffer, 1) + 1);
			base.m_Stream.Write(DisplayGumpFast.m_Buffer, 0, num1);
			this.m_LayoutLength = (this.m_LayoutLength + num1);
		} 

		public void AppendLayoutNS(int val)
		{
			string text1;
			int num1;
			text1 = val.ToString();
			num1 = Encoding.ASCII.GetBytes(text1, 0, text1.Length, DisplayGumpFast.m_Buffer, 1);
			base.m_Stream.Write(DisplayGumpFast.m_Buffer, 1, num1);
			this.m_LayoutLength = (this.m_LayoutLength + num1);
		} 

		public void WriteText(ArrayList text)
		{
			int num1;
			string text1;
			int num2;
			base.m_Stream.Seek(((long) 19), 0);
			base.m_Stream.Write(((ushort) this.m_LayoutLength));
			base.m_Stream.Seek(((long) 0), SeekOrigin.End);
			base.m_Stream.Write(((ushort) text.Count));
			for (num1 = 0; (num1 < text.Count); num1 = (num1 + 1))
			{
				text1 = ((string) text[num1]);
				if (text1 == null)
				{
					text1 = "";
				}
				num2 = ((ushort) text1.Length);
				base.m_Stream.Write(((ushort) num2));
				base.m_Stream.WriteBigUniFixed(text1, num2);
			}
		} 

		private static byte[] m_Buffer;
		private byte[] m_False;
		private int m_LayoutLength;
		private byte[] m_True;
	}
}