using System;

using Server;
using Server.ContextMenus;
using Server.Items;
using Server.HuePickers;
using Server.Prompts;

namespace Server.Network
{
	public sealed class MobileMoving : Packet
	{
		public MobileMoving(Mobile m, int noto) : base(119, 17)
		{
			Point3D pointd1;
			int num1;
			pointd1 = m.Location;
			num1 = m.Hue;
			if (m.SolidHueOverride >= 0)
			{
				num1 = m.SolidHueOverride;
			}
			base.m_Stream.Write(Serial.op_Implicit(m.Serial));
			base.m_Stream.Write(((short) Body.op_Implicit(m.Body)));
			base.m_Stream.Write(((short) pointd1.m_X));
			base.m_Stream.Write(((short) pointd1.m_Y));
			base.m_Stream.Write(((sbyte) pointd1.m_Z));
			base.m_Stream.Write(m.Direction);
			base.m_Stream.Write(((short) num1));
			base.m_Stream.Write(((byte) m.GetPacketFlags()));
			base.m_Stream.Write(((byte) noto));
		} 
	}

	public sealed class DisplayHuePicker : Packet
	{
		public DisplayHuePicker(HuePicker huePicker) : base(149, 9)
		{
			base.m_Stream.Write(huePicker.Serial);
			base.m_Stream.Write(0);
			base.m_Stream.Write(((short) huePicker.ItemID));
		}
	}

	public sealed class MessageLocalized : Packet
	{
		public MessageLocalized(Serial serial, int graphic, MessageType type, int hue, int font, int number, string name, string args)
			: base(193)
		{
			if (name == null)
			{
				name = "";
			}
			if (args == null)
			{
				args = "";
			}
			if (hue == 0)
			{
				hue = 946;
			}
			base.EnsureCapacity((50 + (args.Length * 2)));
			base.m_Stream.Write(serial);
			base.m_Stream.Write(((short) graphic));
			base.m_Stream.Write(((byte) type));
			base.m_Stream.Write(((short) hue));
			base.m_Stream.Write(((short) font));
			base.m_Stream.Write(number);
			base.m_Stream.WriteAsciiFixed(name, 30);
			base.m_Stream.WriteLittleUniNull(args);
		} 

		public static MessageLocalized InstantiateGeneric(int number)
		{
			MessageLocalized[] array1;
			int num1;
			MessageLocalized localized1;
			array1 = null;
			num1 = 0;
			if (number >= 3000000)
			{
				array1 = MessageLocalized.m_Cache_IntLoc;
				num1 = (number - 3000000);
 
			}
			else
			{
				if (number >= 1000000)
				{
					array1 = MessageLocalized.m_Cache_CliLoc;
					num1 = (number - 1000000);
					goto L_004A;
 
				}
				if (number >= 500000)
				{
					array1 = MessageLocalized.m_Cache_CliLocCmp;
					num1 = (number - 500000);
 
				}
 
			}
 
			L_004A:
				if ((array1 != null) && ((num1 >= 0) && (num1 < array1.Length)))
				{
					localized1 = array1[num1];
					if (localized1 != null)
					{
						goto L_00A0;
 
					}
					localized1 = new MessageLocalized(Serial.MinusOne, -1, 0, 946, 3, number, "System", "");
					array1[num1] = new MessageLocalized(Serial.MinusOne, -1, 0, 946, 3, number, "System", "");
 
				}
				else
				{
					localized1 = new MessageLocalized(Serial.MinusOne, -1, 0, 946, 3, number, "System", "");
 
				}
 
			L_00A0:
				return localized1; 
		}

		private static MessageLocalized[] m_Cache_CliLoc = new MessageLocalized[100000];
		private static MessageLocalized[] m_Cache_CliLocCmp = new MessageLocalized[5000];
		private static MessageLocalized[] m_Cache_IntLoc = new MessageLocalized[15000];
	}

	public sealed class UnicodeMessage : Packet
	{
		public UnicodeMessage(Serial serial, int graphic, MessageType type, int hue, int font, string lang, string name, string text)
			: base(174)
		{
			if ((lang == null) || (lang == ""))
			{
				lang = "ENU";
			}
			if (name == null)
			{
				name = "";
			}
			if (text == null)
			{
				text = "";
			}
			if (hue == 0)
			{
				hue = 946;
			}
			base.EnsureCapacity((50 + (text.Length * 2)));
			base.m_Stream.Write(serial);
			base.m_Stream.Write(((short) graphic));
			base.m_Stream.Write(((byte) type));
			base.m_Stream.Write(((short) hue));
			base.m_Stream.Write(((short) font));
			base.m_Stream.WriteAsciiFixed(lang, 4);
			base.m_Stream.WriteAsciiFixed(name, 30);
			base.m_Stream.WriteBigUniNull(text);
		} 
	}

	public sealed class MessageLocalizedAffix : Packet
	{
		public MessageLocalizedAffix(Serial serial, int graphic, MessageType messageType, int hue, int font, int number, string name, AffixType affixType, string affix, string args)
			: base(204)
		{
			if (name == null)
			{
				name = "";
			}
			if (affix == null)
			{
				affix = "";
			}
			if (args == null)
			{
				args = "";
			}
			if (hue == 0)
			{
				hue = 946;
			}
			base.EnsureCapacity(((52 + affix.Length) + (args.Length * 2)));
			base.m_Stream.Write(serial);
			base.m_Stream.Write(((short) graphic));
			base.m_Stream.Write(((byte) messageType));
			base.m_Stream.Write(((short) hue));
			base.m_Stream.Write(((short) font));
			base.m_Stream.Write(number);
			base.m_Stream.Write(affixType);
			base.m_Stream.WriteAsciiFixed(name, 30);
			base.m_Stream.WriteAsciiNull(affix);
			base.m_Stream.WriteBigUniNull(args);
		} 
	}

	public sealed class ContainerContentUpdate : Packet
	{
		public ContainerContentUpdate(Item item)
			: base(37, 20)
		{
			Serial serial1;
			ushort num1;
			if ((item.Parent as Item) != null)
			{
				serial1 = ((Item) item.Parent).Serial;
			}
			else
			{
				Console.WriteLine("Warning: ContainerContentUpdate on item with !(parent is Item)");
				serial1 = Serial.Zero;
			}
			num1 = ((ushort) item.ItemID);
			if (num1 > 16383)
			{
				num1 = 2519;
			}
			base.m_Stream.Write(Serial.op_Implicit(item.Serial));
			base.m_Stream.Write(((short) num1));
			base.m_Stream.Write(0);
			base.m_Stream.Write(((ushort) item.Amount));
			base.m_Stream.Write(((short) item.X));
			base.m_Stream.Write(((short) item.Y));
			base.m_Stream.Write(Serial.op_Implicit(serial1));
			base.m_Stream.Write(((ushort) item.Hue));
		} 
	}

	public sealed class EquipUpdate : Packet
	{
		public EquipUpdate(Item item)
			: base(46, 15)
		{
			Serial serial1;
			int num1;
			Mobile mobile1;
			if ((item.Parent as Mobile) != null)
			{
				serial1 = ((Mobile) item.Parent).Serial;
			}
			else
			{
				Console.WriteLine("Warning: EquipUpdate on item with !(parent is Mobile)");
				serial1 = Serial.Zero;
			}
			num1 = item.Hue;
			if ((item.Parent as Mobile) != null)
			{
				mobile1 = ((Mobile) item.Parent);
				if (mobile1.SolidHueOverride >= 0)
				{
					num1 = mobile1.SolidHueOverride;
				}
			}
			base.m_Stream.Write(item.Serial);
			base.m_Stream.Write(((short) item.ItemID));
			base.m_Stream.Write(0);
			base.m_Stream.Write(item.Layer);
			base.m_Stream.Write(serial1);
			base.m_Stream.Write(((short) num1));
		} 
	}

	public sealed class AsciiMessage : Packet
	{
		public AsciiMessage(Serial serial, int graphic, MessageType type, int hue, int font, string name, string text)
			: base(28)
		{
			if (name == null)
			{
				name = "";
			}
			if (text == null)
			{
				text = "";
			}
			if (hue == 0)
			{
				hue = 946;
			}
			base.EnsureCapacity((45 + text.Length));
			base.m_Stream.Write(serial);
			base.m_Stream.Write(((short) graphic));
			base.m_Stream.Write(((byte) type));
			base.m_Stream.Write(((short) hue));
			base.m_Stream.Write(((short) font));
			base.m_Stream.WriteAsciiFixed(name, 30);
			base.m_Stream.WriteAsciiNull(text);
		} 
	}

	public sealed class RemoveItem : Packet
	{
		public RemoveItem(Item item) : base(29, 5)
		{
			base.m_Stream.Write(item.Serial);
		}
	}

	public sealed class WorldItem : Packet
	{
		public WorldItem(Item item) : base(26)
		{
			uint num1;
			int num2;
			int num3;
			Point3D pointd1;
			int num4;
			int num5;
			int num6;
			int num7;
			int num8;
			Serial serial1;
			base.EnsureCapacity(20);
			serial1 = item.Serial;
			num1 = serial1.Value;
			num2 = item.ItemID;
			num3 = item.Amount;
			pointd1 = item.Location;
			num4 = pointd1.m_X;
			num5 = pointd1.m_Y;
			num6 = item.Hue;
			num7 = item.GetPacketFlags();
			num8 = item.Direction;
			if (num3 != 0)
			{
				num1 = (num1 | -2147483648);
 
			}
			else
			{
				num1 = (num1 & 2147483647);
 
			}
			base.m_Stream.Write(num1);
			base.m_Stream.Write(((short) (num2 & 32767)));
			if (num3 != 0)
			{
				base.m_Stream.Write(((short) num3));
 
			}
			num4 = (num4 & 32767);
			if (num8 != 0)
			{
				num4 = (num4 | 32768);
 
			}
			base.m_Stream.Write(((short) num4));
			num5 = (num5 & 16383);
			if (num6 != 0)
			{
				num5 = (num5 | 32768);
 
			}
			if (num7 != 0)
			{
				num5 = (num5 | 16384);
 
			}
			base.m_Stream.Write(((short) num5));
			if (num8 != 0)
			{
				base.m_Stream.Write(((byte) num8));
 
			}
			base.m_Stream.Write(((sbyte) pointd1.m_Z));
			if (num6 != 0)
			{
				base.m_Stream.Write(((ushort) num6));
 
			}
			if (num7 != 0)
			{
				base.m_Stream.Write(((byte) num7));
			}
		}	
	}

	public sealed class ContainerDisplay : Packet
	{
		public ContainerDisplay(Container c) : base(36, 7)
		{
			base.m_Stream.Write(c.Serial);
			base.m_Stream.Write(((short) c.GumpID));
		} 
	}

	public sealed class ContainerContent : Packet
	{
		public ContainerContent(Mobile beholder, Item beheld) : base(60)
		{
			ArrayList list1;
			int num1;
			long num2;
			int num3;
			int num4;
			Item item1;
			Point3D pointd1;
			ushort num5;
			list1 = beheld.Items;
			num1 = list1.Count;
			base.EnsureCapacity((5 + (num1 * 19)));
			num2 = base.m_Stream.Position;
			num3 = 0;
			base.m_Stream.Write(0);
			for (num4 = 0; (num4 < num1); num4 = (num4 + 1))
			{
				item1 = ((Item) list1[num4]);
				if (!item1.Deleted && beholder.CanSee(item1))
				{
					pointd1 = item1.Location;
					num5 = ((ushort) item1.ItemID);
					if (num5 > 16383)
					{
						num5 = 2519;
					}
					base.m_Stream.Write(item1.Serial);
					base.m_Stream.Write(num5);
					base.m_Stream.Write(0);
					base.m_Stream.Write(((ushort) item1.Amount));
					base.m_Stream.Write(((short) pointd1.m_X));
					base.m_Stream.Write(((short) pointd1.m_Y));
					base.m_Stream.Write(beheld.Serial);
					base.m_Stream.Write(((ushort) item1.Hue));
					num3 = (num3 + 1);
				}
			}
			base.m_Stream.Seek(num2, 0);
			base.m_Stream.Write(((ushort) num3));
		}	
	}

	public sealed class MobileIncoming : Packet
	{
		public MobileIncoming(Mobile beholder, Mobile beheld) : base(120)
		{
			ArrayList list1;
			int num1;
			int num2;
			int num3;
			Item item1;
			byte num4;
			int num5;
			bool flag1;
			this.m_Beheld = beheld;
			MobileIncoming.m_Version = (MobileIncoming.m_Version + 1);
			list1 = beheld.Items;
			num1 = list1.Count;
			base.EnsureCapacity((23 + (num1 * 9)));
			num2 = beheld.Hue;
			if (beheld.SolidHueOverride >= 0)
			{
				num2 = beheld.SolidHueOverride;
 
			}
			base.m_Stream.Write(Serial.op_Implicit(beheld.Serial));
			base.m_Stream.Write(((short) Body.op_Implicit(beheld.Body)));
			base.m_Stream.Write(((short) beheld.X));
			base.m_Stream.Write(((short) beheld.Y));
			base.m_Stream.Write(((sbyte) beheld.Z));
			base.m_Stream.Write(beheld.Direction);
			base.m_Stream.Write(((short) num2));
			base.m_Stream.Write(((byte) beheld.GetPacketFlags()));
			base.m_Stream.Write(((byte) Notoriety.Compute(beholder, beheld)));
			for (num3 = 0; (num3 < num1); num3 = (num3 + 1))
			{
				item1 = ((Item) list1[num3]);
				num4 = item1.Layer;
				if ((!item1.Deleted && beholder.CanSee(item1)) && (MobileIncoming.m_DupedLayers[num4] != MobileIncoming.m_Version))
				{
					MobileIncoming.m_DupedLayers[num4] = MobileIncoming.m_Version;
					num2 = item1.Hue;
					if (beheld.SolidHueOverride >= 0)
					{
						num2 = beheld.SolidHueOverride;
 
					}
					num5 = (item1.ItemID & 16383);
					flag1 = (num2 != 0);
					if (flag1)
					{
						num5 = (num5 | 32768);
 
					}
					base.m_Stream.Write(Serial.op_Implicit(item1.Serial));
					base.m_Stream.Write(((short) num5));
					base.m_Stream.Write(num4);
					if (flag1)
					{
						base.m_Stream.Write(((short) num2));
 
					}
 
				}
 
			}
			base.m_Stream.Write(0);
 
		}

		public Mobile m_Beheld;
		private static int[] m_DupedLayers = new int[256];
		private static int m_Version;
	}

	public sealed class MobileAnimation : Packet
	{
		public MobileAnimation(Mobile m, int action, int frameCount, int repeatCount, bool forward, bool repeat, int delay)
			: base(110, 14)
		{
			base.m_Stream.Write(m.Serial);
			base.m_Stream.Write(((short) action));
			base.m_Stream.Write(((short) frameCount));
			base.m_Stream.Write(((short) repeatCount));
			base.m_Stream.Write((forward == 0));
			base.m_Stream.Write(repeat);
			base.m_Stream.Write(((byte) delay));
		} 
	}

	public sealed class CloseGump : Packet
	{
		public CloseGump(int typeID, int buttonID) : base(191)
		{
			base.EnsureCapacity(13);
			base.m_Stream.Write(4);
			base.m_Stream.Write(typeID);
			base.m_Stream.Write(buttonID);
		} 
	}

	public sealed class DamagePacket : Packet
	{
		public DamagePacket(Mobile m, int amount) : base(191)
		{
			base.EnsureCapacity(11);
			base.m_Stream.Write(34);
			base.m_Stream.Write(1);
			base.m_Stream.Write(Serial.op_Implicit(m.Serial));
			if (amount > 255)
			{
				amount = 255;
			}
			else
			{
				if (amount < 0)
				{
					amount = 0;
				}
			}
			base.m_Stream.Write(((byte) amount));
		} 
	}

	public sealed class DeathAnimation : Packet
	{
		public DeathAnimation(Mobile killed, Item corpse) : base(175, 13)
		{
			base.m_Stream.Write(killed.Serial);
			base.m_Stream.Write(((corpse == null) ? Serial.Zero : corpse.Serial));
			base.m_Stream.Write(0);
		} 
	}

	public sealed class DragEffect : Packet
	{
		public DragEffect(IEntity src, IEntity trg, int itemID, int hue, int amount) : base(35, 26)
		{
			base.m_Stream.Write(((short) itemID));
			base.m_Stream.Write(0);
			base.m_Stream.Write(((short) hue));
			base.m_Stream.Write(((short) amount));
			base.m_Stream.Write(src.Serial);
			base.m_Stream.Write(((short) src.X));
			base.m_Stream.Write(((short) src.Y));
			base.m_Stream.Write(((sbyte) src.Z));
			base.m_Stream.Write(trg.Serial);
			base.m_Stream.Write(((short) trg.X));
			base.m_Stream.Write(((short) trg.Y));
			base.m_Stream.Write(((sbyte) trg.Z));
		} 
	}

	public sealed class PlaySound : Packet
	{
		public PlaySound(int soundID, IPoint3D target) : base(84, 12)
		{
			base.m_Stream.Write(1);
			base.m_Stream.Write(((short) soundID));
			base.m_Stream.Write(0);
			base.m_Stream.Write(((short) target.X));
			base.m_Stream.Write(((short) target.Y));
			base.m_Stream.Write(((short) target.Z));
		} 
	}

	public sealed class LiftRej : Packet
	{
		public LiftRej(LRReason reason) : base(39, 2)
		{
			base.m_Stream.Write(reason);
		} 
	} 

	public sealed class MovementAck : Packet
	{
		static MovementAck()
		{
			MovementAck[][] array1;
			//TRF array1 = new MovementAck[][8];
			array1 = new MovementAck[8][];
			array1[0] = new MovementAck[256];
			array1[1] = new MovementAck[256];
			array1[2] = new MovementAck[256];
			array1[3] = new MovementAck[256];
			array1[4] = new MovementAck[256];
			array1[5] = new MovementAck[256];
			array1[6] = new MovementAck[256];
			array1[7] = new MovementAck[256];
			MovementAck.m_Cache = array1;
		} 

		private MovementAck(int seq, int noto) : base(34, 3)
		{
			base.m_Stream.Write(((byte) seq));
			base.m_Stream.Write(((byte) noto));
		} 

		public static MovementAck Instantiate(int seq, Mobile m)
		{
			int num1;
			MovementAck ack1;
			num1 = Notoriety.Compute(m, m);
			ack1 = MovementAck.m_Cache[num1][seq];
			if (ack1 == null)
			{
				ack1 = new MovementAck(seq, num1);
				MovementAck.m_Cache[num1][seq] = new MovementAck(seq, num1);
 
			}
			return ack1; 
		} 

		private static MovementAck[][] m_Cache;
	}

	public sealed class MapChange : Packet
	{
		public MapChange(Mobile m) : base(191)
		{
			base.EnsureCapacity(6);
			base.m_Stream.Write(8);
			base.m_Stream.Write(((byte) ((m.Map == null) ? 0 : m.Map.MapID)));
		} 
	}

	public sealed class MapPatches : Packet
	{
		public MapPatches() : base(191)
		{
			base.EnsureCapacity(33);
			base.m_Stream.Write(24);
			base.m_Stream.Write(4);
			base.m_Stream.Write(Map.Felucca.Tiles.Patch.StaticBlocks);
			base.m_Stream.Write(Map.Felucca.Tiles.Patch.LandBlocks);
			base.m_Stream.Write(Map.Trammel.Tiles.Patch.StaticBlocks);
			base.m_Stream.Write(Map.Trammel.Tiles.Patch.LandBlocks);
			base.m_Stream.Write(Map.Ilshenar.Tiles.Patch.StaticBlocks);
			base.m_Stream.Write(Map.Ilshenar.Tiles.Patch.LandBlocks);
			base.m_Stream.Write(Map.Malas.Tiles.Patch.StaticBlocks);
			base.m_Stream.Write(Map.Malas.Tiles.Patch.LandBlocks);
		} 
	}

	public sealed class SeasonChange : Packet
	{
		static SeasonChange()
		{
			SeasonChange[][] array1;
			//TRF array1 = new SeasonChange[][5];
			array1 = new SeasonChange[5][];
			array1[0] = new SeasonChange[2];
			array1[1] = new SeasonChange[2];
			array1[2] = new SeasonChange[2];
			array1[3] = new SeasonChange[2];
			array1[4] = new SeasonChange[2];
			SeasonChange.m_Cache = array1;
		} 

		public SeasonChange(int season) : this(season, 1)
		{
		}

		public SeasonChange(int season, bool playSound) : base(188, 3)
		{
			base.m_Stream.Write(((byte) season));
			base.m_Stream.Write(playSound);
		} 

		public static SeasonChange Instantiate(int season)
		{
			return SeasonChange.Instantiate(season, 1); 
		} 

		public static SeasonChange Instantiate(int season, bool playSound)
		{
			int num1;
			SeasonChange change1;
			if ((season >= 0) && (season < SeasonChange.m_Cache.Length))
			{
				num1 = (playSound ? 1 : 0);
				change1 = SeasonChange.m_Cache[season][num1];
				if (change1 == null)
				{
					change1 = new SeasonChange(season, playSound);
					SeasonChange.m_Cache[season][num1] = new SeasonChange(season, playSound);
 
				}
				return change1; 
			}
			return new SeasonChange(season, playSound); 
		} 

		private static SeasonChange[][] m_Cache;
	}

	public sealed class MobileUpdate : Packet
	{
		public MobileUpdate(Mobile m) : base(32, 19)
		{
			int num1;
			num1 = m.Hue;
			if (m.SolidHueOverride >= 0)
			{
				num1 = m.SolidHueOverride;
			}
			base.m_Stream.Write(m.Serial);
			base.m_Stream.Write(((short) Body.op_Implicit(m.Body)));
			base.m_Stream.Write(0);
			base.m_Stream.Write(((short) num1));
			base.m_Stream.Write(((byte) m.GetPacketFlags()));
			base.m_Stream.Write(((short) m.X));
			base.m_Stream.Write(((short) m.Y));
			base.m_Stream.Write(0);
			base.m_Stream.Write(m.Direction);
			base.m_Stream.Write(((sbyte) m.Z));
		} 
	}

	public sealed class ServerChange : Packet
	{
		public ServerChange(Mobile m, Map map) : base(118, 16)
		{
			base.m_Stream.Write(((short) m.X));
			base.m_Stream.Write(((short) m.Y));
			base.m_Stream.Write(((short) m.Z));
			base.m_Stream.Write(0);
			base.m_Stream.Write(0);
			base.m_Stream.Write(0);
			base.m_Stream.Write(((short) map.Width));
			base.m_Stream.Write(((short) map.Height));
		} 
	}

	public sealed class SupportedFeatures : Packet
	{
		public SupportedFeatures() : base(185, 3)
		{
			base.m_Stream.Write(((ushort) SupportedFeatures.m_Value));
		}

		public static SupportedFeatures Instantiate()
		{
			if (SupportedFeatures.m_Instance == null)
			{
				SupportedFeatures.m_Instance = new SupportedFeatures();
 
			}
			return SupportedFeatures.m_Instance; 
		} 

		public static int Value 
		{ 
			get
			{
				return SupportedFeatures.m_Value;
			}
			set
			{
				SupportedFeatures.m_Value = value;
				SupportedFeatures.m_Instance = null;
			}
		}
		
		private static SupportedFeatures m_Instance;
		private static int m_Value = 32799;
	}

	public sealed class MobileAttributes : Packet
	{
		public MobileAttributes(Mobile m) : base(45, 17)
		{
			base.m_Stream.Write(m.Serial);
			base.m_Stream.Write(((short) m.HitsMax));
			base.m_Stream.Write(((short) m.Hits));
			base.m_Stream.Write(((short) m.ManaMax));
			base.m_Stream.Write(((short) m.Mana));
			base.m_Stream.Write(((short) m.StamMax));
			base.m_Stream.Write(((short) m.Stam));
		} 
	}

	public sealed class DeathStatus : Packet
	{
		public DeathStatus(bool dead) : base(44, 2)
		{
			base.m_Stream.Write(((byte) (dead ? 0 : 2)));
		}

		public static DeathStatus Instantiate(bool dead)
		{
			if (!dead)
			{
				return DeathStatus.Alive; 
			}
			return DeathStatus.Dead; 
		} 

		public static readonly DeathStatus Alive = new DeathStatus(false);
		public static readonly DeathStatus Dead = new DeathStatus(true);
	}

	public sealed class SkillUpdate : Packet
	{
		public SkillUpdate(Skills skills) : base(58)
		{
			int num1;
			Skill skill1;
			double num2;
			int num3;
			base.EnsureCapacity((6 + (skills.Length * 9)));
			base.m_Stream.Write(2);
			for (num1 = 0; (num1 < skills.Length); num1 = (num1 + 1))
			{
				skill1 = skills[num1];
				num2 = skill1.Value;
				num3 = ((int) (num2 * 10));
				if (num3 < 0)
				{
					num3 = 0;
 
				}
				else
				{
						if (num3 >= 65536)
				  {
					  num3 = 65535;
 
				  }
 
				}
				base.m_Stream.Write(((ushort) (skill1.Info.SkillID + 1)));
				base.m_Stream.Write(((ushort) num3));
				base.m_Stream.Write(((ushort) skill1.BaseFixedPoint));
				base.m_Stream.Write(skill1.Lock);
				base.m_Stream.Write(((ushort) skill1.CapFixedPoint));
 
			}
			base.m_Stream.Write(0);
		} 
	}

	public sealed class MobileStatus : Packet
	{
		public MobileStatus(Mobile beholder, Mobile beheld) : base(17)
		{
			string text1;
			IWeapon weapon1;
			int num1;
			int num2;
			text1 = beheld.Name;
			if (text1 == null)
			{
				text1 = "";
 
			}
			base.EnsureCapacity((43 + ((beholder == beheld) ? 45 : 0)));
			base.m_Stream.Write(Serial.op_Implicit(beheld.Serial));
			base.m_Stream.WriteAsciiFixed(text1, 30);
			if (beholder == beheld)
			{
				this.WriteAttr(beheld.Hits, beheld.HitsMax);
 
			}
			else
			{
					this.WriteAttrNorm(beheld.Hits, beheld.HitsMax);
 
			}
			base.m_Stream.Write(beheld.CanBeRenamedBy(beholder));
			if (beholder == beheld)
			{
				base.m_Stream.Write(((byte) (MobileStatus.m_SendAosInfo ? 4 : 3)));
				base.m_Stream.Write(beheld.Female);
				base.m_Stream.Write(((short) beheld.Str));
				base.m_Stream.Write(((short) beheld.Dex));
				base.m_Stream.Write(((short) beheld.Int));
				this.WriteAttr(beheld.Stam, beheld.StamMax);
				this.WriteAttr(beheld.Mana, beheld.ManaMax);
				base.m_Stream.Write(beheld.TotalGold);
				base.m_Stream.Write(((short) (Core.AOS ? beheld.PhysicalResistance : ((int) (beheld.ArmorRating + 0.5)))));
				base.m_Stream.Write(((short) (Mobile.BodyWeight + beheld.TotalWeight)));
				base.m_Stream.Write(((short) beheld.StatCap));
				base.m_Stream.Write(((byte) beheld.Followers));
				base.m_Stream.Write(((byte) beheld.FollowersMax));
				if (!MobileStatus.m_SendAosInfo)
				{
					goto L_0253;
 
				}
				base.m_Stream.Write(((short) beheld.FireResistance));
				base.m_Stream.Write(((short) beheld.ColdResistance));
				base.m_Stream.Write(((short) beheld.PoisonResistance));
				base.m_Stream.Write(((short) beheld.EnergyResistance));
				base.m_Stream.Write(((short) beheld.Luck));
				weapon1 = beheld.Weapon;
				num1 = 0;
				num2 = 0;
				if (weapon1 != null)
				{
					weapon1.GetStatusDamage(beheld, &(num1), &(num2));
 
				}
				base.m_Stream.Write(((short) num1));
				base.m_Stream.Write(((short) num2));
				base.m_Stream.Write(beheld.TithingPoints);
				return; 
			}
			base.m_Stream.Write(0);
 
			L_0253:
				return; 
		}
		
		private void WriteAttr(int current, int maximum)
		{
				base.m_Stream.Write(((short) current));
			base.m_Stream.Write(((short) maximum));
 
		} 

		private void WriteAttrNorm(int current, int maximum)
		{
				AttributeNormalizer.WriteReverse(base.m_Stream, current, maximum);
 
		} 

		public static bool SendAosInfo 
		{ 
			get
			{
				return MobileStatus.m_SendAosInfo;
			}
			set
			{
				MobileStatus.m_SendAosInfo = value;
			}
		}
		
		private static bool m_SendAosInfo;
	}

	public sealed class StatLockInfo : Packet
	{
		public StatLockInfo(Mobile m) : base(191)
		{
			int num1;
			base.EnsureCapacity(12);
			base.m_Stream.Write(25);
			base.m_Stream.Write(2);
			base.m_Stream.Write(m.Serial);
			base.m_Stream.Write(0);
			num1 = 0;
			num1 = (num1 | (m.StrLock << 4));
			num1 = (num1 | (m.DexLock << 2));
			num1 = (num1 | m.IntLock);
			base.m_Stream.Write(((byte) num1));
		} 
	}

	public sealed class MobileStatusExtended : Packet
	{
		public MobileStatusExtended(Mobile m) : base(17)
		{
			string text1;
			IWeapon weapon1;
			int num1;
			int num2;
			text1 = m.Name;
			if (text1 == null)
			{
				text1 = "";
 
			}
			base.EnsureCapacity(88);
			base.m_Stream.Write(Serial.op_Implicit(m.Serial));
			base.m_Stream.WriteAsciiFixed(text1, 30);
			base.m_Stream.Write(((short) m.Hits));
			base.m_Stream.Write(((short) m.HitsMax));
			base.m_Stream.Write(m.CanBeRenamedBy(m));
			base.m_Stream.Write(((byte) (MobileStatus.SendAosInfo ? 4 : 3)));
			base.m_Stream.Write(m.Female);
			base.m_Stream.Write(((short) m.Str));
			base.m_Stream.Write(((short) m.Dex));
			base.m_Stream.Write(((short) m.Int));
			base.m_Stream.Write(((short) m.Stam));
			base.m_Stream.Write(((short) m.StamMax));
			base.m_Stream.Write(((short) m.Mana));
			base.m_Stream.Write(((short) m.ManaMax));
			base.m_Stream.Write(m.TotalGold);
			base.m_Stream.Write(((short) (Core.AOS ? m.PhysicalResistance : ((int) (m.ArmorRating + 0.5)))));
			base.m_Stream.Write(((short) (Mobile.BodyWeight + m.TotalWeight)));
			base.m_Stream.Write(((short) m.StatCap));
			base.m_Stream.Write(((byte) m.Followers));
			base.m_Stream.Write(((byte) m.FollowersMax));
			if (MobileStatus.SendAosInfo)
			{
				base.m_Stream.Write(((short) m.FireResistance));
				base.m_Stream.Write(((short) m.ColdResistance));
				base.m_Stream.Write(((short) m.PoisonResistance));
				base.m_Stream.Write(((short) m.EnergyResistance));
				base.m_Stream.Write(((short) m.Luck));
				weapon1 = m.Weapon;
				num1 = 0;
				num2 = 0;
				if (weapon1 != null)
				{
					weapon1.GetStatusDamage(m, &(num1), &(num2));
				}
				base.m_Stream.Write(((short) num1));
				base.m_Stream.Write(((short) num2));
				base.m_Stream.Write(m.TithingPoints);
 			}
 		}	
	}

	public sealed class MobileHits : Packet
	{
		public MobileHits(Mobile m) : base(161, 9)
		{
			base.m_Stream.Write(m.Serial);
			base.m_Stream.Write(((short) m.HitsMax));
			base.m_Stream.Write(((short) m.Hits));
		} 
	}

	public sealed class MobileStam : Packet
	{
		public MobileStam(Mobile m) : base(163, 9)
		{
			base.m_Stream.Write(m.Serial);
			base.m_Stream.Write(((short) m.StamMax));
			base.m_Stream.Write(((short) m.Stam));
		} 
	}

	public sealed class MobileMana : Packet
	{
		public MobileMana(Mobile m) : base(162, 9)
		{
			base.m_Stream.Write(m.Serial);
			base.m_Stream.Write(((short) m.ManaMax));
			base.m_Stream.Write(((short) m.Mana));
		} 
	}

	public sealed class BondedStatus : Packet
	{
		public BondedStatus(int val1, Serial serial, int val2) : base(191)
		{
			base.EnsureCapacity(11);
			base.m_Stream.Write(25);
			base.m_Stream.Write(((byte) val1));
			base.m_Stream.Write(serial);
			base.m_Stream.Write(((byte) val2));
		} 
	}

	public sealed class MobileStatusCompact : Packet
	{
		public MobileStatusCompact(bool canBeRenamed, Mobile m) : base(17)
		{
			string text1;
			text1 = m.Name;
			if (text1 == null)
			{
				text1 = "";
			}
			base.EnsureCapacity(43);
			base.m_Stream.Write(Serial.op_Implicit(m.Serial));
			base.m_Stream.WriteAsciiFixed(text1, 30);
			AttributeNormalizer.WriteReverse(base.m_Stream, m.Hits, m.HitsMax);
			base.m_Stream.Write(canBeRenamed);
			base.m_Stream.Write(0);
		} 
	}

	public sealed class MobileHitsN : Packet
	{
		public MobileHitsN(Mobile m) : base(161, 9)
		{
			base.m_Stream.Write(Serial.op_Implicit(m.Serial));
			AttributeNormalizer.Write(base.m_Stream, m.Hits, m.HitsMax);
		} 
	}

	public sealed class ChangeCombatant : Packet
	{
		public ChangeCombatant(Mobile combatant) : base(170, 5)
		{
			base.m_Stream.Write(((combatant != null) ? combatant.Serial : Serial.Zero));
		} 
	}

	public sealed class DisplayContextMenu : Packet
	{
		public DisplayContextMenu(ContextMenu menu) : base(191)
		{
			ContextMenuEntry[] array1;
			int num1;
			IEntity entity1;
			Point3D pointd1;
			int num2;
			ContextMenuEntry entry1;
			int num3;
			CMEFlags flags1;
			int num4;
			array1 = menu.Entries;
			num1 = ((byte) array1.Length);
			base.EnsureCapacity((12 + (num1 * 8)));
			base.m_Stream.Write(20);
			base.m_Stream.Write(1);
			entity1 = (menu.Target as IEntity);
			base.m_Stream.Write(Serial.op_Implicit(((entity1 == null) ? Serial.MinusOne : entity1.Serial)));
			base.m_Stream.Write(((byte) num1));
			if ((entity1 as Mobile) != null)
			{
				pointd1 = entity1.Location;
 
			}
			else
			{
					if ((entity1 as Item) != null)
			  {
				  pointd1 = ((Item) entity1).GetWorldLocation();
				  goto L_00A2;
 
			  }
				pointd1 = Point3D.Zero;
 
			}
 
			L_00A2:
				num2 = 0;
			while ((num2 < num1))
			{
				entry1 = array1[num2];
				base.m_Stream.Write(((short) num2));
				base.m_Stream.Write(((ushort) entry1.Number));
				num3 = entry1.Range;
				if (num3 == -1)
				{
					num3 = 18;
				}
				flags1 = ((entry1.Enabled && menu.From.InRange(pointd1, num3)) ? 0 : 1);
				num4 = (entry1.Color & 65535);
				if (num4 != 65535)
				{
					flags1 = (flags1 | 32);
				}
				flags1 = (flags1 | entry1.Flags);
				base.m_Stream.Write(((short) flags1));
				if ((flags1 & 32) != 0)
				{
					base.m_Stream.Write(((short) num4));
 				}
				num2 = (num2 + 1);
 			}
 		}	
	}

	public sealed class UnicodePrompt : Packet
	{
		public UnicodePrompt(Prompt prompt) : base(194)
		{
			base.EnsureCapacity(21);
			base.m_Stream.Write(prompt.Serial);
			base.m_Stream.Write(prompt.Serial);
			base.m_Stream.Write(0);
			base.m_Stream.Write(0);
			base.m_Stream.Write(0);
		} 
	}

	public sealed class RemoveMobile : Packet
	{
		public RemoveMobile(Mobile m) : base(29, 5)
		{
			base.m_Stream.Write(m.Serial);
		} 
	}

	public sealed class SetWarMode : Packet
	{
		public SetWarMode(bool mode) : base(114, 5)
		{
			base.m_Stream.Write(mode);
			base.m_Stream.Write(0);
			base.m_Stream.Write(50);
			base.m_Stream.Write(0);
		} 

		public static SetWarMode Instantiate(bool mode)
		{
			if (!mode)
			{
				return SetWarMode.InPeaceMode; 
			}
			return SetWarMode.InWarMode; 
		} 

		public static readonly SetWarMode InPeaceMode = new SetWarMode(false);
		public static readonly SetWarMode InWarMode = new SetWarMode(true);
	}
}