using System;
using System.IO;
using System.Collections;

namespace Server.Network
{
	public class PacketWriter
	{
		public PacketWriter() : this(32)
		{
		}

		public PacketWriter(int capacity) : base()
		{
			this.m_Stream = new MemoryStream(capacity);
			this.m_Capacity = capacity;
		}

		public static PacketWriter CreateInstance()
		{
				return PacketWriter.CreateInstance(32); 
		} 

		public static PacketWriter CreateInstance(int capacity)
		{
				PacketWriter writer1;
			Stack stack1;
			writer1 = null;
			stack1 = PacketWriter.m_Pool;
			Monitor.Enter(PacketWriter.m_Pool);
			try
			{
				if (PacketWriter.m_Pool.Count > 0)
				{
					writer1 = ((PacketWriter) PacketWriter.m_Pool.Pop());
					if (writer1 != null)
					{
						writer1.m_Capacity = capacity;
						writer1.m_Stream.SetLength(((long) 0));
 
					}
 
				}
 
			}
			finally
			{
				Monitor.Exit(stack1);
 
			}
			if (writer1 == null)
			{
				writer1 = new PacketWriter(capacity);
 
			}
			return writer1; 
		} 

		public void Fill()
		{
				this.Fill(((int) (((long) this.m_Capacity) - this.m_Stream.Length)));
 
		} 

		public void Fill(int length)
		{
				if (this.m_Stream.Position == this.m_Stream.Length)
		  {
			  this.m_Stream.SetLength((this.m_Stream.Length + ((long) length)));
			  this.m_Stream.Seek(((long) 0), 2);
			  return; 
		  }
			this.m_Stream.Write(new byte[length], 0, length);
 
		} 

		public static void ReleaseInstance(PacketWriter pw)
		{
				StreamWriter writer1;
			Stack stack1;
			stack1 = PacketWriter.m_Pool;
			Monitor.Enter(PacketWriter.m_Pool);
			try
			{
				if (!PacketWriter.m_Pool.Contains(pw))
				{
					PacketWriter.m_Pool.Push(pw);
 
				}
				else
				{
						try
				  {
					  writer1 = new StreamWriter("neterr.log");
					  try
					  {
						  writer1.WriteLine("{0} Instance pool contains writer", DateTime.Now);
						  goto L_0061;
 
					  }
					  finally
					  {
						  if (writer1 != null)
						  {
							  writer1.Dispose();
 
						  }
 
					  }
					  goto L_006A;
 
				  }
				  catch (object obj1)
				  {
					  Console.WriteLine("net error");
					  goto L_006A;
 
				  }
 
				}
 
			L_0061:
				goto L_006A;
 
			}
			finally
			{
				Monitor.Exit(stack1);
 
			}
 
			L_006A:
				return; 
		} 

		public long Seek(long offset, SeekOrigin origin)
		{
				return this.m_Stream.Seek(offset, origin); 
		} 

		public byte[] ToArray()
		{
				return this.m_Stream.ToArray(); 
		} 

		public void Write(bool value)
		{
				this.m_Stream.WriteByte(((byte) (value ? 1 : 0)));
 
		} 

		public void Write(byte value)
		{
				this.m_Stream.WriteByte(value);
 
		} 

		public void Write(byte[] buffer, int offset, int size)
		{
				this.m_Stream.Write(buffer, offset, size);
 
		} 

		public void Write(short value)
		{
				PacketWriter.m_Buffer[0] = ((byte) (value >> 8));
			PacketWriter.m_Buffer[1] = ((byte) value);
			this.m_Stream.Write(PacketWriter.m_Buffer, 0, 2);
 
		} 

		public void Write(int value)
		{
				PacketWriter.m_Buffer[0] = ((byte) (value >> 24));
			PacketWriter.m_Buffer[1] = ((byte) (value >> 16));
			PacketWriter.m_Buffer[2] = ((byte) (value >> 8));
			PacketWriter.m_Buffer[3] = ((byte) value);
			this.m_Stream.Write(PacketWriter.m_Buffer, 0, 4);
 
		} 

		public void Write(sbyte value)
		{
				this.m_Stream.WriteByte(((byte) value));
 
		} 

		public void Write(ushort value)
		{
				PacketWriter.m_Buffer[0] = ((byte) (value >> 8));
			PacketWriter.m_Buffer[1] = ((byte) value);
			this.m_Stream.Write(PacketWriter.m_Buffer, 0, 2);
 
		} 

		public void Write(uint value)
		{
				PacketWriter.m_Buffer[0] = ((byte) (value >> 24));
			PacketWriter.m_Buffer[1] = ((byte) (value >> 16));
			PacketWriter.m_Buffer[2] = ((byte) (value >> 8));
			PacketWriter.m_Buffer[3] = ((byte) value);
			this.m_Stream.Write(PacketWriter.m_Buffer, 0, 4);
 
		} 

		public void WriteAsciiFixed(string value, int size)
		{
				byte[] array1;
			if (value == null)
			{
				Console.WriteLine("Network: Attempted to WriteAsciiFixed() with null value");
				value = string.Empty;
 
			}
			array1 = Encoding.ASCII.GetBytes(value);
			if (array1.Length >= size)
			{
				this.m_Stream.Write(array1, 0, size);
				return; 
			}
			this.m_Stream.Write(array1, 0, array1.Length);
			this.Fill((size - array1.Length));
 
		} 

		public void WriteAsciiNull(string value)
		{
				byte[] array1;
			if (value == null)
			{
				Console.WriteLine("Network: Attempted to WriteAsciiNull() with null value");
				value = string.Empty;
 
			}
			array1 = Encoding.ASCII.GetBytes(value);
			this.m_Stream.Write(array1, 0, array1.Length);
			this.m_Stream.WriteByte(0);
 
		} 

		public void WriteBigUniFixed(string value, int size)
		{
				byte[] array1;
			if (value == null)
			{
				Console.WriteLine("Network: Attempted to WriteBigUniFixed() with null value");
				value = string.Empty;
 
			}
			size = (size * 2);
			array1 = Encoding.BigEndianUnicode.GetBytes(value);
			if (array1.Length >= size)
			{
				this.m_Stream.Write(array1, 0, size);
				return; 
			}
			this.m_Stream.Write(array1, 0, array1.Length);
			this.Fill((size - array1.Length));
 
		} 

		public void WriteBigUniNull(string value)
		{
				byte[] array1;
			if (value == null)
			{
				Console.WriteLine("Network: Attempted to WriteBigUniNull() with null value");
				value = string.Empty;
 
			}
			array1 = Encoding.BigEndianUnicode.GetBytes(value);
			this.m_Stream.Write(array1, 0, array1.Length);
			PacketWriter.m_Buffer[0] = 0;
			PacketWriter.m_Buffer[1] = 0;
			this.m_Stream.Write(PacketWriter.m_Buffer, 0, 2);
 
		} 

		public void WriteLittleUniFixed(string value, int size)
		{
				byte[] array1;
			if (value == null)
			{
				Console.WriteLine("Network: Attempted to WriteLittleUniFixed() with null value");
				value = string.Empty;
 
			}
			size = (size * 2);
			array1 = Encoding.Unicode.GetBytes(value);
			if (array1.Length >= size)
			{
				this.m_Stream.Write(array1, 0, size);
				return; 
			}
			this.m_Stream.Write(array1, 0, array1.Length);
			this.Fill((size - array1.Length));
 
		} 

		public void WriteLittleUniNull(string value)
		{
				byte[] array1;
			if (value == null)
			{
				Console.WriteLine("Network: Attempted to WriteLittleUniNull() with null value");
				value = string.Empty;
 
			}
			array1 = Encoding.Unicode.GetBytes(value);
			this.m_Stream.Write(array1, 0, array1.Length);
			PacketWriter.m_Buffer[0] = 0;
			PacketWriter.m_Buffer[1] = 0;
			this.m_Stream.Write(PacketWriter.m_Buffer, 0, 2);
 
		} 

		public long Length { 
			get
			{
				return this.m_Stream.Length;
			}
		}

		public long Position { 
			get
			{
				return this.m_Stream.Position;
			}
			set
			{
				this.m_Stream.Position = value;
			}
		}

		public MemoryStream UnderlyingStream { 
			get
			{
				return this.m_Stream;
			}
		}
		
		private static byte[] m_Buffer = new byte[4];
		private int m_Capacity;
		private static Stack m_Pool = new Stack();
		private MemoryStream m_Stream;
	}

	public class EncodedReader
	{
		public EncodedReader(PacketReader reader)
		{
			this.m_Reader = reader;
		}

		public int ReadInt32()
		{
			if (this.m_Reader.ReadByte() != 0)
			{
				return 0; 
			}
			return this.m_Reader.ReadInt32(); 
		}

		public Point3D ReadPoint3D()
		{
				if (this.m_Reader.ReadByte() != 3)
		  {
			  return Point3D.Zero; 
		  }
			return new Point3D(this.m_Reader.ReadInt16(), this.m_Reader.ReadInt16(), this.m_Reader.ReadByte()); 
		} 

		public string ReadUnicodeString()
		{
				int num1;
			if (this.m_Reader.ReadByte() != 2)
			{
				return ""; 
			}
			num1 = this.m_Reader.ReadUInt16();
			return this.m_Reader.ReadUnicodeString(num1); 
		} 

		public string ReadUnicodeStringSafe()
		{
				int num1;
			if (this.m_Reader.ReadByte() != 2)
			{
				return ""; 
			}
			num1 = this.m_Reader.ReadUInt16();
			return this.m_Reader.ReadUnicodeStringSafe(num1); 
		} 

		public void Trace(NetState state)
		{
				this.m_Reader.Trace(state);
 
		} 

		public byte[] Buffer { 
			get
			{
				return this.m_Reader.Buffer;
			}
		}
		
		private PacketReader m_Reader;
	}

	public class PacketReader
	{
		public PacketReader(byte[] data, bool fixedSize) : base()
		{
			this.m_Data = data;
			this.m_Index = (fixedSize ? 1 : 3);
		} 

		public bool IsSafeChar(int c)
		{
				if (c >= 32)
		  {
			  return (c < 65534); 
		  }
			return false; 
		} 

		public bool ReadBoolean()
		{
				int num1;
			if ((this.m_Index + 1) > this.m_Data.Length)
			{
				return false; 
			}
			num1 = this.m_Index;
			this.m_Index = (this.m_Index + 1);
			return (this.m_Data[num1] != 0); 
		} 

		public byte ReadByte()
		{
				int num1;
			if ((this.m_Index + 1) > this.m_Data.Length)
			{
				return 0; 
			}
			num1 = this.m_Index;
			this.m_Index = (this.m_Index + 1);
			return this.m_Data[num1]; 
		} 

		public short ReadInt16()
		{
				int num1;
			if ((this.m_Index + 2) > this.m_Data.Length)
			{
				return 0; 
			}
			num1 = this.m_Index;
			this.m_Index = (this.m_Index + 1);
			num1 = this.m_Index;
			this.m_Index = (this.m_Index + 1);
			return ((short) ((this.m_Data[num1] << 8) | this.m_Data[num1])); 
		} 

		public int ReadInt32()
		{
				int num1;
			if ((this.m_Index + 4) > this.m_Data.Length)
			{
				return 0; 
			}
			num1 = this.m_Index;
			this.m_Index = (this.m_Index + 1);
			num1 = this.m_Index;
			this.m_Index = (this.m_Index + 1);
			num1 = this.m_Index;
			this.m_Index = (this.m_Index + 1);
			num1 = this.m_Index;
			this.m_Index = (this.m_Index + 1);
			return ((((this.m_Data[num1] << 24) | (this.m_Data[num1] << 16)) | (this.m_Data[num1] << 8)) | this.m_Data[num1]); 
		} 

		public sbyte ReadSByte()
		{
				int num1;
			if ((this.m_Index + 1) > this.m_Data.Length)
			{
				return 0; 
			}
			num1 = this.m_Index;
			this.m_Index = (this.m_Index + 1);
			return ((sbyte) this.m_Data[num1]); 
		} 

		public string ReadString()
		{
				StringBuilder builder1;
			int num1;
			int num2;
			builder1 = new StringBuilder();
			goto L_0011;
 
			L_0008:
				builder1.Append(((ushort) num1));
 
			L_0011:
				if (this.m_Index < this.m_Data.Length)
				{
					num2 = this.m_Index;
					this.m_Index = (this.m_Index + 1);
					num1 = this.m_Data[num2];
					if (this.m_Data[num2] != 0)
					{
						goto L_0008;
 
					}
 
				}
			return builder1.ToString(); 
		} 

		public string ReadString(int fixedLength)
		{
				int num1;
			int num2;
			StringBuilder builder1;
			int num3;
			int num4;
			num1 = (this.m_Index + fixedLength);
			num2 = num1;
			if (num1 > this.m_Data.Length)
			{
				num1 = this.m_Data.Length;
 
			}
			builder1 = new StringBuilder();
			goto L_0030;
 
			L_0027:
				builder1.Append(((ushort) num3));
 
			L_0030:
				if (this.m_Index < num1)
				{
					num4 = this.m_Index;
					this.m_Index = (this.m_Index + 1);
					num3 = this.m_Data[num4];
					if (this.m_Data[num4] != 0)
					{
						goto L_0027;
 
					}
 
				}
			this.m_Index = num2;
			return builder1.ToString(); 
		} 

		public string ReadStringSafe()
		{
				StringBuilder builder1;
			int num1;
			int num2;
			builder1 = new StringBuilder();
			goto L_001A;
 
			L_0008:
				if (this.IsSafeChar(num1))
				{
					builder1.Append(((ushort) num1));
 
				}
 
			L_001A:
				if (this.m_Index < this.m_Data.Length)
				{
					num2 = this.m_Index;
					this.m_Index = (this.m_Index + 1);
					num1 = this.m_Data[num2];
					if (this.m_Data[num2] != 0)
					{
						goto L_0008;
 
					}
 
				}
			return builder1.ToString(); 
		} 

		public string ReadStringSafe(int fixedLength)
		{
				int num1;
			int num2;
			StringBuilder builder1;
			int num3;
			int num4;
			num1 = (this.m_Index + fixedLength);
			num2 = num1;
			if (num1 > this.m_Data.Length)
			{
				num1 = this.m_Data.Length;
 
			}
			builder1 = new StringBuilder();
			goto L_0039;
 
			L_0027:
				if (this.IsSafeChar(num3))
				{
					builder1.Append(((ushort) num3));
 
				}
 
			L_0039:
				if (this.m_Index < num1)
				{
					num4 = this.m_Index;
					this.m_Index = (this.m_Index + 1);
					num3 = this.m_Data[num4];
					if (this.m_Data[num4] != 0)
					{
						goto L_0027;
 
					}
 
				}
			this.m_Index = num2;
			return builder1.ToString(); 
		} 

		public ushort ReadUInt16()
		{
				int num1;
			if ((this.m_Index + 2) > this.m_Data.Length)
			{
				return 0; 
			}
			num1 = this.m_Index;
			this.m_Index = (this.m_Index + 1);
			num1 = this.m_Index;
			this.m_Index = (this.m_Index + 1);
			return ((ushort) ((this.m_Data[num1] << 8) | this.m_Data[num1])); 
		} 

		public uint ReadUInt32()
		{
				int num1;
			if ((this.m_Index + 4) > this.m_Data.Length)
			{
				return 0; 
			}
			num1 = this.m_Index;
			this.m_Index = (this.m_Index + 1);
			num1 = this.m_Index;
			this.m_Index = (this.m_Index + 1);
			num1 = this.m_Index;
			this.m_Index = (this.m_Index + 1);
			num1 = this.m_Index;
			this.m_Index = (this.m_Index + 1);
			return ((((this.m_Data[num1] << 24) | (this.m_Data[num1] << 16)) | (this.m_Data[num1] << 8)) | this.m_Data[num1]); 
		} 

		public string ReadUnicodeString()
		{
				StringBuilder builder1;
			int num1;
			int num2;
			builder1 = new StringBuilder();
			goto L_0011;
 
			L_0008:
				builder1.Append(((ushort) num1));
 
			L_0011:
				if ((this.m_Index + 1) < this.m_Data.Length)
				{
					num2 = this.m_Index;
					this.m_Index = (this.m_Index + 1);
					num2 = this.m_Index;
					this.m_Index = (this.m_Index + 1);
					num1 = ((this.m_Data[num2] << 8) | this.m_Data[num2]);
					if (((this.m_Data[num2] << 8) | this.m_Data[num2]) != 0)
					{
						goto L_0008;
 
					}
 
				}
			return builder1.ToString(); 
		} 

		public string ReadUnicodeString(int fixedLength)
		{
				int num1;
			int num2;
			StringBuilder builder1;
			int num3;
			int num4;
			num1 = (this.m_Index + (fixedLength << 1));
			num2 = num1;
			if (num1 > this.m_Data.Length)
			{
				num1 = this.m_Data.Length;
 
			}
			builder1 = new StringBuilder();
			goto L_0032;
 
			L_0029:
				builder1.Append(((ushort) num3));
 
			L_0032:
				if ((this.m_Index + 1) < num1)
				{
					num4 = this.m_Index;
					this.m_Index = (this.m_Index + 1);
					num4 = this.m_Index;
					this.m_Index = (this.m_Index + 1);
					num3 = ((this.m_Data[num4] << 8) | this.m_Data[num4]);
					if (((this.m_Data[num4] << 8) | this.m_Data[num4]) != 0)
					{
						goto L_0029;
 
					}
 
				}
			this.m_Index = num2;
			return builder1.ToString(); 
		} 

		public string ReadUnicodeStringLE()
		{
				StringBuilder builder1;
			int num1;
			int num2;
			builder1 = new StringBuilder();
			goto L_0011;
 
			L_0008:
				builder1.Append(((ushort) num1));
 
			L_0011:
				if ((this.m_Index + 1) < this.m_Data.Length)
				{
					num2 = this.m_Index;
					this.m_Index = (this.m_Index + 1);
					num2 = this.m_Index;
					this.m_Index = (this.m_Index + 1);
					num1 = (this.m_Data[num2] | (this.m_Data[num2] << 8));
					if ((this.m_Data[num2] | (this.m_Data[num2] << 8)) != 0)
					{
						goto L_0008;
 
					}
 
				}
			return builder1.ToString(); 
		} 

		public string ReadUnicodeStringLESafe()
		{
				StringBuilder builder1;
			int num1;
			int num2;
			builder1 = new StringBuilder();
			goto L_001A;
 
			L_0008:
				if (this.IsSafeChar(num1))
				{
					builder1.Append(((ushort) num1));
 
				}
 
			L_001A:
				if ((this.m_Index + 1) < this.m_Data.Length)
				{
					num2 = this.m_Index;
					this.m_Index = (this.m_Index + 1);
					num2 = this.m_Index;
					this.m_Index = (this.m_Index + 1);
					num1 = (this.m_Data[num2] | (this.m_Data[num2] << 8));
					if ((this.m_Data[num2] | (this.m_Data[num2] << 8)) != 0)
					{
						goto L_0008;
 
					}
 
				}
			return builder1.ToString(); 
		} 

		public string ReadUnicodeStringLESafe(int fixedLength)
		{
				int num1;
			int num2;
			StringBuilder builder1;
			int num3;
			int num4;
			num1 = (this.m_Index + (fixedLength << 1));
			num2 = num1;
			if (num1 > this.m_Data.Length)
			{
				num1 = this.m_Data.Length;
 
			}
			builder1 = new StringBuilder();
			goto L_003B;
 
			L_0029:
				if (this.IsSafeChar(num3))
				{
					builder1.Append(((ushort) num3));
 
				}
 
			L_003B:
				if ((this.m_Index + 1) < num1)
				{
					num4 = this.m_Index;
					this.m_Index = (this.m_Index + 1);
					num4 = this.m_Index;
					this.m_Index = (this.m_Index + 1);
					num3 = (this.m_Data[num4] | (this.m_Data[num4] << 8));
					if ((this.m_Data[num4] | (this.m_Data[num4] << 8)) != 0)
					{
						goto L_0029;
 
					}
 
				}
			this.m_Index = num2;
			return builder1.ToString(); 
		} 

		public string ReadUnicodeStringSafe()
		{
				StringBuilder builder1;
			int num1;
			int num2;
			builder1 = new StringBuilder();
			goto L_001A;
 
			L_0008:
				if (this.IsSafeChar(num1))
				{
					builder1.Append(((ushort) num1));
 
				}
 
			L_001A:
				if ((this.m_Index + 1) < this.m_Data.Length)
				{
					num2 = this.m_Index;
					this.m_Index = (this.m_Index + 1);
					num2 = this.m_Index;
					this.m_Index = (this.m_Index + 1);
					num1 = ((this.m_Data[num2] << 8) | this.m_Data[num2]);
					if (((this.m_Data[num2] << 8) | this.m_Data[num2]) != 0)
					{
						goto L_0008;
 
					}
 
				}
			return builder1.ToString(); 
		} 

		public string ReadUnicodeStringSafe(int fixedLength)
		{
				int num1;
			int num2;
			StringBuilder builder1;
			int num3;
			int num4;
			num1 = (this.m_Index + (fixedLength << 1));
			num2 = num1;
			if (num1 > this.m_Data.Length)
			{
				num1 = this.m_Data.Length;
 
			}
			builder1 = new StringBuilder();
			goto L_003B;
 
			L_0029:
				if (this.IsSafeChar(num3))
				{
					builder1.Append(((ushort) num3));
 
				}
 
			L_003B:
				if ((this.m_Index + 1) < num1)
				{
					num4 = this.m_Index;
					this.m_Index = (this.m_Index + 1);
					num4 = this.m_Index;
					this.m_Index = (this.m_Index + 1);
					num3 = ((this.m_Data[num4] << 8) | this.m_Data[num4]);
					if (((this.m_Data[num4] << 8) | this.m_Data[num4]) != 0)
					{
						goto L_0029;
 
					}
 
				}
			this.m_Index = num2;
			return builder1.ToString(); 
		} 

		public string ReadUTF8String()
		{
				int num1;
			int num2;
			byte[] array1;
			int num3;
			int num4;
			if (this.m_Index >= this.m_Data.Length)
			{
				return string.Empty; 
			}
			num1 = 0;
			num2 = this.m_Index;
			goto L_0025;
 
			L_0021:
				num1 = (num1 + 1);
 
			L_0025:
				if (num2 < this.m_Data.Length)
				{
					num2 = (num2 + 1);
					if (this.m_Data[num2] != 0)
					{
						goto L_0021;
 
					}
 
				}
			num2 = 0;
			array1 = new byte[num1];
			num3 = 0;
			goto L_0054;
 
			L_004B:
				num2 = (num2 + 1);
			array1[num2] = ((byte) num3);
 
			L_0054:
				if (this.m_Index < this.m_Data.Length)
				{
					num4 = this.m_Index;
					this.m_Index = (this.m_Index + 1);
					num3 = this.m_Data[num4];
					if (this.m_Data[num4] != 0)
					{
						goto L_004B;
 
					}
 
				}
			return Utility.UTF8.GetString(array1); 
		} 

		public string ReadUTF8StringSafe()
		{
				int num1;
			int num2;
			byte[] array1;
			int num3;
			string text1;
			bool flag1;
			int num4;
			StringBuilder builder1;
			int num5;
			int num6;
			if (this.m_Index >= this.m_Data.Length)
			{
				return string.Empty; 
			}
			num1 = 0;
			num2 = this.m_Index;
			goto L_0025;
 
			L_0021:
				num1 = (num1 + 1);
 
			L_0025:
				if (num2 < this.m_Data.Length)
				{
					num2 = (num2 + 1);
					if (this.m_Data[num2] != 0)
					{
						goto L_0021;
 
					}
 
				}
			num2 = 0;
			array1 = new byte[num1];
			num3 = 0;
			goto L_0054;
 
			L_004B:
				num2 = (num2 + 1);
			array1[num2] = ((byte) num3);
 
			L_0054:
				if (this.m_Index < this.m_Data.Length)
				{
					num6 = this.m_Index;
					this.m_Index = (this.m_Index + 1);
					num3 = this.m_Data[num6];
					if (this.m_Data[num6] != 0)
					{
						goto L_004B;
 
					}
 
				}
			text1 = Utility.UTF8.GetString(array1);
			flag1 = true;
			for (num4 = 0; (flag1 && (num4 < text1.Length)); num4 = (num4 + 1))
			{
				flag1 = this.IsSafeChar(text1.Chars[num4]);
 
			}
			if (flag1)
			{
				return text1; 
			}
			builder1 = new StringBuilder(text1.Length);
			for (num5 = 0; (num5 < text1.Length); num5 = (num5 + 1))
			{
				if (this.IsSafeChar(text1.Chars[num5]))
				{
					builder1.Append(text1.Chars[num5]);
 
				}
 
			}
			return builder1.ToString(); 
		}
		
		public string ReadUTF8StringSafe(int fixedLength)
		{
				int num1;
			int num2;
			int num3;
			int num4;
			byte[] array1;
			int num5;
			string text1;
			bool flag1;
			int num6;
			StringBuilder builder1;
			int num7;
			int num8;
			if (this.m_Index >= this.m_Data.Length)
			{
				this.m_Index = (this.m_Index + fixedLength);
				return string.Empty; 
			}
			num1 = (this.m_Index + fixedLength);
			if (num1 > this.m_Data.Length)
			{
				num1 = this.m_Data.Length;
 
			}
			num2 = 0;
			num3 = this.m_Index;
			num4 = this.m_Index;
			goto L_0057;
 
			L_0053:
				num2 = (num2 + 1);
 
			L_0057:
				if (num3 < num1)
				{
					num3 = (num3 + 1);
					if (this.m_Data[num3] != 0)
					{
						goto L_0053;
 
					}
 
				}
			num3 = 0;
			array1 = new byte[num2];
			num5 = 0;
			goto L_0083;
 
			L_0078:
				num3 = (num3 + 1);
			array1[num3] = ((byte) num5);
 
			L_0083:
				if (this.m_Index < num1)
				{
					num8 = this.m_Index;
					this.m_Index = (this.m_Index + 1);
					num5 = this.m_Data[num8];
					if (this.m_Data[num8] != 0)
					{
						goto L_0078;
 
					}
 
				}
			text1 = Utility.UTF8.GetString(array1);
			flag1 = true;
			for (num6 = 0; (flag1 && (num6 < text1.Length)); num6 = (num6 + 1))
			{
				flag1 = this.IsSafeChar(text1.Chars[num6]);
 
			}
			this.m_Index = (num4 + fixedLength);
			if (flag1)
			{
				return text1; 
			}
			builder1 = new StringBuilder(text1.Length);
			for (num7 = 0; (num7 < text1.Length); num7 = (num7 + 1))
			{
				if (this.IsSafeChar(text1.Chars[num7]))
				{
					builder1.Append(text1.Chars[num7]);
 
				}
 
			}
			return builder1.ToString(); 
		}
		
		public int Seek(int offset, SeekOrigin origin)
		{
				SeekOrigin origin1;
			origin1 = origin;
			switch (origin1)
			{
				case 0:
				{
					this.m_Index = offset;
					goto L_003F;
 
				}
				case 1:
				{
					this.m_Index = (this.m_Index + offset);
					goto L_003F;
 
				}
				case 2:
				{
					goto L_002F;
 
				}
 
			}
			goto L_003F;
 
			L_002F:
				this.m_Index = (this.m_Data.Length - offset);
 
			L_003F:
				return this.m_Index; 
		} 

		public void Trace(NetState state)
		{
				StreamWriter writer1;
			byte[] array1;
			try
			{
				writer1 = new StreamWriter("Packets.log", 1);
				try
				{
					array1 = this.m_Data;
					if (array1.Length > 0)
					{
						writer1.WriteLine("Client: {0}: Unhandled packet 0x{1:X2}", state, array1[0]);
 
					}
					Utility.FormatBuffer(writer1, new MemoryStream(array1), array1.Length);
					writer1.WriteLine();
					writer1.WriteLine();
					goto L_0059;
 
				}
				finally
				{
					if (writer1 != null)
					{
						writer1.Dispose();
 
					}
 
				}
 
			}
			catch (object obj1)
			{
  
			}
 
			L_0059:
				return; 
		} 

		public byte[] Buffer { 
			get
			{
				return this.m_Data;
			}
		}
		
		private byte[] m_Data;
		private int m_Index;
	}

	public class PacketProfile
	{
		public PacketProfile(bool outgoing) : base()
		{
			this.m_Outgoing = outgoing;
		}

		public static PacketProfile GetIncomingProfile(int packetID)
		{
				PacketProfile profile1;
			if (!Core.Profiling)
			{
				return null; 
			}
			profile1 = PacketProfile.m_IncomingProfiles[packetID];
			if (profile1 == null)
			{
				profile1 = new PacketProfile(0);
				PacketProfile.m_IncomingProfiles[packetID] = new PacketProfile(0);
 
			}
			return profile1; 
		} 

		public static PacketProfile GetOutgoingProfile(int packetID)
		{
				PacketProfile profile1;
			if (!Core.Profiling)
			{
				return null; 
			}
			profile1 = PacketProfile.m_OutgoingProfiles[packetID];
			if (profile1 == null)
			{
				profile1 = new PacketProfile(1);
				PacketProfile.m_OutgoingProfiles[packetID] = new PacketProfile(1);
 
			}
			return profile1; 
		} 

		public void Record(int byteLength, TimeSpan processTime)
		{
				this.m_Count = (this.m_Count + 1);
			this.m_TotalByteLength = (this.m_TotalByteLength + byteLength);
			this.m_TotalProcTime = TimeSpan.op_Addition(this.m_TotalProcTime, processTime);
 
		} 

		public void RegConstruct()
		{
				this.m_Constructed = (this.m_Constructed + 1);
 
		} 
		
		public double AverageByteLength 
		{ 
			get
			{
				if (this.m_Count == 0)
				{
					return 0; 
				}
				return Math.Round((((double) this.m_TotalByteLength) / ((double) this.m_Count)), 2);
			}
		}

		public TimeSpan AverageProcTime 
		{ 
			get
			{
				if (this.m_Count == 0)
				{
					return TimeSpan.Zero; 
				}
				return TimeSpan.FromTicks((this.m_TotalProcTime.Ticks / ((long) this.m_Count))); 
			}
		}

		public int Constructed 
		{ 
			get
			{
				return this.m_Constructed; 
			}
		}

		public int Count 
		{ 
			get
			{
				return this.m_Count;
			}
		}

		public static PacketProfile[] IncomingProfiles 
		{ 
			get
			{
				return PacketProfile.m_IncomingProfiles;
			}
		}

		public bool Outgoing 
		{ 
			get
			{
				return this.m_Outgoing;
			}
		}

		public static PacketProfile[] OutgoingProfiles 
		{ 
			get
			{
				return PacketProfile.m_OutgoingProfiles;
			}
		}

		public int TotalByteLength 
		{ 
			get
			{
				return this.m_TotalByteLength;
			}
		}

		public TimeSpan TotalProcTime 
		{ 
			get
			{
				return this.m_TotalProcTime;
			}
		}

		private int m_Constructed;
		private int m_Count;
		private static PacketProfile[] m_IncomingProfiles = new PacketProfile[256];
		private bool m_Outgoing;
		private static PacketProfile[] m_OutgoingProfiles = new PacketProfile[256];
		private int m_TotalByteLength;
		private TimeSpan m_TotalProcTime;
	}

	public class PacketHandler
	{
		public PacketHandler(int packetID, int length, bool ingame, OnPacketReceive onReceive) : base()
		{
			this.m_PacketID = packetID;
			this.m_Length = length;
			this.m_Ingame = ingame;
			this.m_OnReceive = onReceive;
		} 
		
		public bool Ingame 
		{ 
			get
			{
				return this.m_Ingame;
			}
		}

		public int Length 
		{ 
			get
			{
				return this.m_Length;
			}
		}

		public OnPacketReceive OnReceive 
		{ 
			get
			{
				return this.m_OnReceive;
			}
		}

		public int PacketID 
		{ 
			get
			{
				return this.m_PacketID;
			}
		}

		public ThrottlePacketCallback ThrottleCallback 
		{ 
			get
			{
				return this.m_ThrottleCallback;
			}
			set
			{
				this.m_ThrottleCallback = value;
			}
		}

		private bool m_Ingame;
		private int m_Length;
		private OnPacketReceive m_OnReceive;
		private int m_PacketID;
		private ThrottlePacketCallback m_ThrottleCallback;

	}
}