using System;
using System.IO;
using System.Collections;

namespace Server.Network
{
	public class ByteQueue
	{
		public ByteQueue() : base()
		{
			this.m_Length = 0;
			this.m_Capacity = 0;
			this.m_Buffer = new byte[this.m_Capacity];
		} 

		public void Dequeue(byte[] buffer)
		{
				this.Dequeue(buffer, 0, buffer.Length);
		} 

		public void Dequeue(byte[] buffer, int offset, int count)
		{
				int num1;
			int num2;
			Buffer.BlockCopy(this.m_Buffer, 0, buffer, offset, count);
			this.m_Length = (this.m_Length - count);
			num1 = count;
			for (num2 = 0; (num2 < this.m_Length); num2 = (num2 + 1))
			{
				this.m_Buffer[num2] = this.m_Buffer[num1];
				num1 = (num1 + 1);
 
			}
 
		} 

		public byte[] Dequeue(int count)
		{
			byte[] array1;
			array1 = new byte[count];
			this.Dequeue(array1, 0, count);
			return array1; 
		} 

		public void Enqueue(byte[] buffer)
		{
				this.Enqueue(buffer, 0, buffer.Length);
		} 

		public void Enqueue(byte[] buffer, int offset, int count)
		{
				this.EnsureCapacity((this.m_Length + count));
			Buffer.BlockCopy(buffer, offset, this.m_Buffer, this.m_Length, count);
			this.m_Length = (this.m_Length + count);
 
		} 

		public void Enqueue(Stream stream, int count)
		{
				byte[] array1;
			array1 = new byte[count];
			count = stream.Read(array1, 0, count);
			this.Enqueue(array1, 0, count);
 
		} 

		private void EnsureCapacity(int length)
		{
				byte[] array1;
			if (length > this.m_Capacity)
			{
				array1 = this.m_Buffer;
				this.m_Capacity = ((length + 2047) & -2048);
				this.m_Buffer = new byte[this.m_Capacity];
				Buffer.BlockCopy(array1, 0, this.m_Buffer, 0, this.m_Length);
 
			}
 
		} 

		public void Peek(byte[] buffer)
		{
				this.Peek(buffer, 0, buffer.Length);
 
		} 

		public void Peek(byte[] buffer, int offset, int count)
		{
				Buffer.BlockCopy(this.m_Buffer, 0, buffer, offset, count);
 
		} 

		public byte[] Peek(int count)
		{
				byte[] array1;
			array1 = new byte[count];
			this.Peek(array1, 0, count);
			return array1; 
		} 
		
		public int Capacity { 
			get
			{
				return this.m_Capacity;
			}
		}

		public int Length { 
			get
			{
				return this.m_Length;
			}
		}

		private byte[] m_Buffer;
		private int m_Capacity;
		private int m_Length;
	}

	public class SendQueue
	{
		public SendQueue() : base()
		{
			this.m_Queue = new Queue();
		}

		public byte[] Dequeue(ref int length)
		{
			Entry entry1;
			Entry.Release(((Entry) this.m_Queue.Dequeue()));
			if ((this.m_Queue.Count == 0) && (this.m_Buffered != null))
			{
				this.m_Queue.Enqueue(this.m_Buffered);
				this.m_Buffered = null;
 
			}
			if (this.m_Queue.Count > 0)
			{
				entry1 = ((Entry) this.m_Queue.Peek());
				length = entry1.m_Length;
				return entry1.m_Buffer; 
			}
			return null; 
		} 

		public bool Enqueue(byte[] buffer, int length)
		{
				if (buffer == null)
		  {
			  Console.WriteLine("Warning: Attempting to send null packet buffer");
			  return false; 
		  }
			if (this.m_Queue.Count == 0)
			{
				this.m_Queue.Enqueue(Entry.Pool(buffer, length, 0));
				return true; 
			}
			if ((this.m_Buffered != null) && ((this.m_Buffered.m_Length + length) > this.m_Buffered.m_Buffer.Length))
			{
				this.m_Queue.Enqueue(this.m_Buffered);
				this.m_Buffered = null;
 
			}
			if (length >= SendQueue.m_CoalesceBufferSize)
			{
				this.m_Queue.Enqueue(Entry.Pool(buffer, length, 0));
 
			}
			else
			{
					if (this.m_Buffered == null)
			  {
				  this.m_Buffered = Entry.Pool(SendQueue.GetUnusedBuffer(), 0, 1);
 
			  }
				Buffer.BlockCopy(buffer, 0, this.m_Buffered.m_Buffer, this.m_Buffered.m_Length, length);
				this.m_Buffered.m_Length = (this.m_Buffered.m_Length + length);
 
			}
			return false; 
		} 

		public static byte[] GetUnusedBuffer()
		{
				byte[] array1;
			Stack stack1;
			stack1 = SendQueue.m_UnusedBuffers;
			Monitor.Enter(SendQueue.m_UnusedBuffers);
			try
			{
				if (SendQueue.m_UnusedBuffers.Count == 0)
				{
					array1 = new byte[SendQueue.m_CoalesceBufferSize];
					goto L_003E;
 
				}
				array1 = ((byte[]) SendQueue.m_UnusedBuffers.Pop());
 
			}
			finally
			{
				Monitor.Exit(stack1);
 
			}
 
			L_003E:
				return array1; 
		} 

		public static void ReleaseBuffer(byte[] buffer)
		{
				Stack stack1;
			stack1 = SendQueue.m_UnusedBuffers;
			Monitor.Enter(SendQueue.m_UnusedBuffers);
			try
			{
				if (buffer == null)
				{
					Console.WriteLine("Warning: Attempting to release null packet buffer");
 
				}
				else
				{
						if (buffer.Length == SendQueue.m_CoalesceBufferSize)
				  {
					  SendQueue.m_UnusedBuffers.Push(buffer);
 
				  }
 
				}
 
			}
			finally
			{
				Monitor.Exit(stack1);
 
			}
 
		} 
		
		public static int CoalesceBufferSize { 
			get
			{
				return SendQueue.m_CoalesceBufferSize;
			}
			set
			{
				Stack stack1;
				stack1 = SendQueue.m_UnusedBuffers;
				Monitor.Enter(SendQueue.m_UnusedBuffers);
				try
				{
					SendQueue.m_UnusedBuffers.Clear();
					SendQueue.m_CoalesceBufferSize = value;
				}
				finally
				{
					Monitor.Exit(stack1);
				}
			}
		}

		private Entry m_Buffered;
		private static int m_CoalesceBufferSize = 512;
		private Queue m_Queue;
		private static Stack m_UnusedBuffers = new Stack();

		private class Entry
		{
			private Entry(byte[] buffer, int length, bool managed) : base()
			{
				this.m_Buffer = buffer;
				this.m_Length = length;
				this.m_Managed = managed;
			}

			public static Entry Pool(byte[] buffer, int length, bool managed)
			{
					Entry entry1;
				Entry entry2;
				Stack stack1;
				stack1 = Entry.m_Pool;
				Monitor.Enter(Entry.m_Pool);
				try
				{
					if (Entry.m_Pool.Count == 0)
					{
						entry2 = new Entry(buffer, length, managed);
						goto L_0053;
 
					}
					entry1 = ((Entry) Entry.m_Pool.Pop());
					entry1.m_Buffer = buffer;
					entry1.m_Length = length;
					entry1.m_Managed = managed;
					entry2 = entry1;
 
				}
				finally
				{
					Monitor.Exit(stack1);
 
				}
 
				L_0053:
					return entry2; 
			} 

			public static void Release(Entry e)
			{
				Stack stack1;
				stack1 = Entry.m_Pool;
				Monitor.Enter(Entry.m_Pool);
				try
				{
					Entry.m_Pool.Push(e);
					if (e.m_Managed)
					{
						SendQueue.ReleaseBuffer(e.m_Buffer);
					}
				}
				finally
				{
					Monitor.Exit(stack1);
				}
			} 

			public byte[] m_Buffer;
			public int m_Length;
			public bool m_Managed;
			private static Stack m_Pool = new Stack();
		}
	}
}