using System;
using System.Collections;
using System.CodeDom.Compiler;
using System.Reflection;

namespace Server
{
	public class TypeCache
	{
		public TypeCache(Assembly asm) : base()
		{
			Type type1;
			int num1;
			Type type2;
			object[] array1;
			TypeAliasAttribute attribute1;
			int num2;
			if (asm == null)
			{
				this.m_Types = Type.EmptyTypes;
 
			}
			else
			{
				this.m_Types = asm.GetTypes();
 
			}
			this.m_Names = new TypeTable(this.m_Types.Length);
			this.m_FullNames = new TypeTable(this.m_Types.Length);
			type1 = typeof(TypeAliasAttribute);
			for (num1 = 0; (num1 < this.m_Types.Length); num1 = (num1 + 1))
			{
				type2 = this.m_Types[num1];
				this.m_Names.Add(type2.Name, type2);
				this.m_FullNames.Add(type2.FullName, type2);
				if (type2.IsDefined(type1, 0))
				{
					array1 = type2.GetCustomAttributes(type1, 0);
					if ((array1 != null) && (array1.Length > 0))
					{
						attribute1 = (array1[0] as TypeAliasAttribute);
						if (attribute1 != null)
						{
							for (num2 = 0; (num2 < attribute1.Aliases.Length); num2 = (num2 + 1))
							{
								this.m_FullNames.Add(attribute1.Aliases[num2], type2);
 
							}
 
						}
 
					}
 
				}
 
			}
 
		} 

		public Type GetTypeByFullName(string fullName, bool ignoreCase)
		{
			return this.m_FullNames.Get(fullName, ignoreCase); 
		} 

		public Type GetTypeByName(string name, bool ignoreCase)
		{
			return this.m_Names.Get(name, ignoreCase); 
		} 

		public TypeTable FullNames 
		{ 
			get
			{
				return this.m_FullNames;
			}
		}

		public TypeTable Names 
		{ 
			get
			{
				return this.m_Names;
			}
		}

		public Type[] Types 
		{ 
			get
			{
				return this.m_Types;
			}
		}

		private TypeTable m_FullNames;
		private TypeTable m_Names;
		private Type[] m_Types;
	}

	public class ScriptCompiler
	{
		public static bool Compile()
		{
			return ScriptCompiler.Compile(0);
		}

		public static bool Compile(bool debug)
		{
			CompilerResults results1;
			CompilerResults results2;
			int num1;
			ArrayList list1;
			Type[] array1;
			int num2;
			MethodInfo info1;
			int num3;
			Type[] array2;
			int num4;
			MethodInfo info2;
			int num5;
			ScriptCompiler.EnsureDirectory("Scripts/");
			ScriptCompiler.EnsureDirectory("Scripts/Output/");
			ScriptCompiler.DeleteFiles("Scripts.CS*.dll");
			ScriptCompiler.DeleteFiles("Scripts.VB*.dll");
			ScriptCompiler.DeleteFiles("Scripts*.dll");
			if (ScriptCompiler.m_AdditionalReferences.Count > 0)
			{
				ScriptCompiler.m_AdditionalReferences.Clear();
 
			}
			results1 = null;
			results2 = null;
			results1 = ScriptCompiler.CompileCSScripts(debug);
			if ((results1 == null) || !results1.Errors.HasErrors)
			{
				results2 = ScriptCompiler.CompileVBScripts(debug);
 
			}
			if (((results1 == null) || !results1.Errors.HasErrors) && (((results2 == null) || !results2.Errors.HasErrors) && ((results2 != null) || (results1 != null))))
			{
				num1 = 0;
				if ((results1 == null) || (results2 == null))
				{
					ScriptCompiler.m_Assemblies = new Assembly[1];
 
				}
				else
				{
					ScriptCompiler.m_Assemblies = new Assembly[2];
 
				}
				if (results1 != null)
				{
					num1 = (num1 + 1);
					ScriptCompiler.m_Assemblies[num1] = results1.CompiledAssembly;
 
				}
				if (results2 != null)
				{
					num1 = (num1 + 1);
					ScriptCompiler.m_Assemblies[num1] = results2.CompiledAssembly;
 
				}
				Console.Write("Scripts: Verifying...");
				Core.VerifySerialization();
				Console.WriteLine("done ({0} items, {1} mobiles)", Core.ScriptItems, Core.ScriptMobiles);
				list1 = new ArrayList();
				for (num1 = 0; (num1 < ScriptCompiler.m_Assemblies.Length); num1 = (num1 + 1))
				{
					array1 = ScriptCompiler.m_Assemblies[num1].GetTypes();
					for (num2 = 0; (num2 < array1.Length); num2 = (num2 + 1))
					{
						info1 = array1[num2].GetMethod("Configure", 24);
						if (info1 != null)
						{
							list1.Add(info1);
 
						}
 
					}
 
				}
				list1.Sort(new CallPriorityComparer());
				for (num3 = 0; (num3 < list1.Count); num3 = (num3 + 1))
				{
					((MethodInfo) list1[num3]).Invoke(null, null);
 
				}
				list1.Clear();
				World.Load();
				for (num1 = 0; (num1 < ScriptCompiler.m_Assemblies.Length); num1 = (num1 + 1))
				{
					array2 = ScriptCompiler.m_Assemblies[num1].GetTypes();
					for (num4 = 0; (num4 < array2.Length); num4 = (num4 + 1))
					{
						info2 = array2[num4].GetMethod("Initialize", 24);
						if (info2 != null)
						{
							list1.Add(info2);
 
						}
 
					}
 
				}
				list1.Sort(new CallPriorityComparer());
				for (num5 = 0; (num5 < list1.Count); num5 = (num5 + 1))
				{
					((MethodInfo) list1[num5]).Invoke(null, null);
 
				}
				return true; 
			}
			return false; 
		}

		private static CompilerResults CompileCSScripts()
		{
			return ScriptCompiler.CompileCSScripts(0); 
		} 

		private static CompilerResults CompileCSScripts(bool debug)
		{
			CSharpCodeProvider provider1;
			ICodeCompiler compiler1;
			string[] array1;
			string text1;
			CompilerResults results1;
			int num1;
			int num2;
			CompilerError error1;
			CompilerError error2;
			IEnumerator enumerator1;
			IDisposable disposable1;
			object[] array2;
			provider1 = new CSharpCodeProvider();
			compiler1 = provider1.CreateCompiler();
			Console.Write("Scripts: Compiling C# scripts...");
			array1 = ScriptCompiler.GetScripts("*.cs");
			if (array1.Length == 0)
			{
				Console.WriteLine("no files found.");
				return null; 
			}
			text1 = ScriptCompiler.GetUnusedPath("Scripts.CS");
			results1 = compiler1.CompileAssemblyFromFileBatch(new CompilerParameters(ScriptCompiler.GetReferenceAssemblies(), text1, debug), array1);
			ScriptCompiler.m_AdditionalReferences.Add(text1);
			if (results1.Errors.Count > 0)
			{
				num1 = 0;
				num2 = 0;
				enumerator1 = results1.Errors.GetEnumerator();
				try
				{
					while (enumerator1.MoveNext())
					{
						error1 = ((CompilerError) enumerator1.Current);
						if (error1.IsWarning)
						{
							num2 = (num2 + 1);
 
						}
						else
						{
							num1 = (num1 + 1);
 
						}
 
					}
 
				}
				finally
				{
					disposable1 = (enumerator1 as IDisposable);
					if (disposable1 != null)
					{
						disposable1.Dispose();
 
					}
 
				}
				if (num1 > 0)
				{
					Console.WriteLine("failed ({0} errors, {1} warnings)", num1, num2);
 
				}
				else
				{
					Console.WriteLine("done ({0} errors, {1} warnings)", num1, num2);
 
				}
				enumerator1 = results1.Errors.GetEnumerator();
				try
				{
					while (enumerator1.MoveNext())
					{
						error2 = ((CompilerError) enumerator1.Current);
						array2 = new object[6];
						array2[0] = (error2.IsWarning ? "Warning" : "Error");
						array2[1] = error2.FileName;
						array2[2] = error2.ErrorNumber;
						array2[3] = error2.Line;
						array2[4] = error2.Column;
						array2[5] = error2.ErrorText;
						Console.WriteLine(" - {0}: {1}: {2}: (line {3}, column {4}) {5}", array2);
 
					}
					goto L_01BC;
 
				}
				finally
				{
					disposable1 = (enumerator1 as IDisposable);
					if (disposable1 != null)
					{
						disposable1.Dispose();
 
					}
 
				}
 
			}
			Console.WriteLine("done (0 errors, 0 warnings)");
 
			L_01BC:
				return results1; 
		}
		
		private static CompilerResults CompileVBScripts()
		{
			return ScriptCompiler.CompileVBScripts(0); 
		} 

		private static CompilerResults CompileVBScripts(bool debug)
		{
			VBCodeProvider provider1;
			ICodeCompiler compiler1;
			string[] array1;
			string text1;
			CompilerResults results1;
			int num1;
			int num2;
			CompilerError error1;
			CompilerError error2;
			IEnumerator enumerator1;
			IDisposable disposable1;
			object[] array2;
			provider1 = new VBCodeProvider();
			compiler1 = provider1.CreateCompiler();
			Console.Write("Scripts: Compiling VB.net scripts...");
			array1 = ScriptCompiler.GetScripts("*.vb");
			if (array1.Length == 0)
			{
				Console.WriteLine("no files found.");
				return null; 
			}
			text1 = ScriptCompiler.GetUnusedPath("Scripts.VB");
			results1 = compiler1.CompileAssemblyFromFileBatch(new CompilerParameters(ScriptCompiler.GetReferenceAssemblies(), text1, 1), array1);
			ScriptCompiler.m_AdditionalReferences.Add(text1);
			if (results1.Errors.Count > 0)
			{
				num1 = 0;
				num2 = 0;
				enumerator1 = results1.Errors.GetEnumerator();
				try
				{
					while (enumerator1.MoveNext())
					{
						error1 = ((CompilerError) enumerator1.Current);
						if (error1.IsWarning)
						{
							num2 = (num2 + 1);
 
						}
						else
						{
							num1 = (num1 + 1);
 
						}
 
					}
 
				}
				finally
				{
					disposable1 = (enumerator1 as IDisposable);
					if (disposable1 != null)
					{
						disposable1.Dispose();
 
					}
 
				}
				if (num1 > 0)
				{
					Console.WriteLine("failed ({0} errors, {1} warnings)", num1, num2);
 
				}
				else
				{
					Console.WriteLine("done ({0} errors, {1} warnings)", num1, num2);
 
				}
				enumerator1 = results1.Errors.GetEnumerator();
				try
				{
					while (enumerator1.MoveNext())
					{
						error2 = ((CompilerError) enumerator1.Current);
						array2 = new object[6];
						array2[0] = (error2.IsWarning ? "Warning" : "Error");
						array2[1] = error2.FileName;
						array2[2] = error2.ErrorNumber;
						array2[3] = error2.Line;
						array2[4] = error2.Column;
						array2[5] = error2.ErrorText;
						Console.WriteLine(" - {0}: {1}: {2}: (line {3}, column {4}) {5}", array2);
 
					}
					goto L_01BC;
 
				}
				finally
				{
					disposable1 = (enumerator1 as IDisposable);
					if (disposable1 != null)
					{
						disposable1.Dispose();
 
					}
 
				}
 
			}
			Console.WriteLine("done (0 errors, 0 warnings)");
 
			L_01BC:
				return results1; 
		}
		
		private static void DeleteFiles(string mask)
		{
			string[] array1;
			string text1;
			string[] array2;
			int num1;
			try
			{
				array1 = Directory.GetFiles(Path.Combine(Core.BaseDirectory, "Scripts/Output"), mask);
				array2 = array1;
				for (num1 = 0; (num1 < array2.Length); num1 = (num1 + 1))
				{
					text1 = array2[num1];
					try
					{
						File.Delete(text1);
 
					}
					catch (object obj1)
					{
  
					}
 
				}
 
			}
			catch (object obj2)
			{
  
			}
 
		} 

		private static void EnsureDirectory(string dir)
		{
			string text1;
			text1 = Path.Combine(Core.BaseDirectory, dir);
			if (!Directory.Exists(text1))
			{
				Directory.CreateDirectory(text1);
 
			}
 
		} 

		public static Type FindTypeByFullName(string fullName)
		{
			return ScriptCompiler.FindTypeByFullName(fullName, 1); 
		} 

		public static Type FindTypeByFullName(string fullName, bool ignoreCase)
		{
			Type type1;
			int num1;
			type1 = null;
			for (num1 = 0; ((type1 == null) && (num1 < ScriptCompiler.m_Assemblies.Length)); num1 = (num1 + 1))
			{
				type1 = ScriptCompiler.GetTypeCache(ScriptCompiler.m_Assemblies[num1]).GetTypeByFullName(fullName, ignoreCase);
 
			}
			if (type1 == null)
			{
				type1 = ScriptCompiler.GetTypeCache(Core.Assembly).GetTypeByFullName(fullName, ignoreCase);
 
			}
			return type1; 
		} 

		public static Type FindTypeByName(string name)
		{
			return ScriptCompiler.FindTypeByName(name, 1); 
		} 

		public static Type FindTypeByName(string name, bool ignoreCase)
		{
			Type type1;
			int num1;
			type1 = null;
			for (num1 = 0; ((type1 == null) && (num1 < ScriptCompiler.m_Assemblies.Length)); num1 = (num1 + 1))
			{
				type1 = ScriptCompiler.GetTypeCache(ScriptCompiler.m_Assemblies[num1]).GetTypeByName(name, ignoreCase);
 
			}
			if (type1 == null)
			{
				type1 = ScriptCompiler.GetTypeCache(Core.Assembly).GetTypeByName(name, ignoreCase);
 
			}
			return type1; 
		} 

		public static string[] GetReferenceAssemblies()
		{
			ArrayList list1;
			string text1;
			StreamReader reader1;
			string text2;
			list1 = new ArrayList();
			text1 = Path.Combine(Core.BaseDirectory, "Data/Assemblies.cfg");
			if (File.Exists(text1))
			{
				reader1 = new StreamReader(text1);
				try
				{
					goto L_0045;
 
				L_0027:
					if ((text2.Length > 0) && !text2.StartsWith("#"))
					{
						list1.Add(text2);
 
					}
 
				L_0045:
					text2 = reader1.ReadLine();
					if (reader1.ReadLine() != null)
					{
						goto L_0027;
 
					}
 
				}
				finally
				{
					if (reader1 != null)
					{
						reader1.Dispose();
 
					}
 
				}
 
			}
			list1.Add(Core.ExePath);
			list1.AddRange(ScriptCompiler.m_AdditionalReferences);
			return ((string[]) list1.ToArray(typeof(string))); 
		} 

		private static void GetScripts(ArrayList list, string path, string type)
		{
			string text1;
			string[] array1;
			int num1;
			array1 = Directory.GetDirectories(path);
			for (num1 = 0; (num1 < array1.Length); num1 = (num1 + 1))
			{
				text1 = array1[num1];
				ScriptCompiler.GetScripts(list, text1, type);
 
			}
			list.AddRange(Directory.GetFiles(path, type));
 
		} 

		private static string[] GetScripts(string type)
		{
			ArrayList list1;
			list1 = new ArrayList();
			ScriptCompiler.GetScripts(list1, Path.Combine(Core.BaseDirectory, "Scripts"), type);
			return ((string[]) list1.ToArray(typeof(string))); 
		} 

		public static TypeCache GetTypeCache(Assembly asm)
		{
			TypeCache cache1;
			if (asm == null)
			{
				if (ScriptCompiler.m_NullCache == null)
				{
					ScriptCompiler.m_NullCache = new TypeCache(null);
 
				}
				return ScriptCompiler.m_NullCache; 
			}
			cache1 = ((TypeCache) ScriptCompiler.m_TypeCaches[asm]);
			if (cache1 == null)
			{
				cache1 = new TypeCache(asm);
				ScriptCompiler.m_TypeCaches[asm] = new TypeCache(asm);
 
			}
			return cache1; 
		} 

		private static string GetUnusedPath(string name)
		{
			string text1;
			int num1;
			text1 = Path.Combine(Core.BaseDirectory, string.Format("Scripts/Output/{0}.dll", name));
			for (num1 = 2; (File.Exists(text1) && (num1 <= 1000)); num1 = (num1 + 1))
			{
				text1 = Path.Combine(Core.BaseDirectory, string.Format("Scripts/Output/{0}.{1}.dll", name, num1));
 
			}
			return text1; 
		} 

		public static Assembly[] Assemblies 
		{ 
			get
			{
				return ScriptCompiler.m_Assemblies;
			}
			set
			{
				ScriptCompiler.m_Assemblies = value;
			}
		}

		private static ArrayList m_AdditionalReferences = new ArrayList();
		private static Assembly[] m_Assemblies;
		private static TypeCache m_NullCache;
		private static Hashtable m_TypeCaches = new Hashtable();
	}
}