using System;

using Server.Items;

namespace Server
{
	public class SecureTrade
	{
		public SecureTrade(Mobile from, Mobile to) : base()
		{
			this.m_Valid = true;
			this.m_From = new SecureTradeInfo(this, from, new SecureTradeContainer(this));
			this.m_To = new SecureTradeInfo(this, to, new SecureTradeContainer(this));
			from.Send(new MobileStatus(from, to));
			from.Send(new UpdateSecureTrade(this.m_From.Container, 0, 0));
			from.Send(new SecureTradeEquip(this.m_To.Container, to));
			from.Send(new UpdateSecureTrade(this.m_From.Container, 0, 0));
			from.Send(new SecureTradeEquip(this.m_From.Container, from));
			from.Send(new DisplaySecureTrade(to, this.m_From.Container, this.m_To.Container, to.Name));
			from.Send(new UpdateSecureTrade(this.m_From.Container, 0, 0));
			to.Send(new MobileStatus(to, from));
			to.Send(new UpdateSecureTrade(this.m_To.Container, 0, 0));
			to.Send(new SecureTradeEquip(this.m_From.Container, from));
			to.Send(new UpdateSecureTrade(this.m_To.Container, 0, 0));
			to.Send(new SecureTradeEquip(this.m_To.Container, to));
			to.Send(new DisplaySecureTrade(from, this.m_To.Container, this.m_From.Container, from.Name));
			to.Send(new UpdateSecureTrade(this.m_To.Container, 0, 0));
		} 

		public void Cancel()
		{
				ArrayList list1;
			int num1;
			Item item1;
			int num2;
			Item item2;
			if (!this.m_Valid)
			{
				return; 
			}
			list1 = this.m_From.Container.Items;
			for (num1 = (list1.Count - 1); (num1 >= 0); num1 = (num1 - 1))
			{
				if (num1 < list1.Count)
				{
					item1 = ((Item) list1[num1]);
					item1.OnSecureTrade(this.m_From.Mobile, this.m_To.Mobile, this.m_From.Mobile, 0);
					if (!item1.Deleted)
					{
						this.m_From.Mobile.AddToBackpack(item1);
 
					}
 
				}
 
			}
			list1 = this.m_To.Container.Items;
			for (num2 = (list1.Count - 1); (num2 >= 0); num2 = (num2 - 1))
			{
				if (num2 < list1.Count)
				{
					item2 = ((Item) list1[num2]);
					item2.OnSecureTrade(this.m_To.Mobile, this.m_From.Mobile, this.m_To.Mobile, 0);
					if (!item2.Deleted)
					{
						this.m_To.Mobile.AddToBackpack(item2);
 
					}
 
				}
 
			}
			this.Close();
 
		}
		
		public void Close()
		{
				NetState state1;
			if (!this.m_Valid)
			{
				return; 
			}
			this.m_From.Mobile.Send(new CloseSecureTrade(this.m_From.Container));
			this.m_To.Mobile.Send(new CloseSecureTrade(this.m_To.Container));
			this.m_Valid = false;
			state1 = this.m_From.Mobile.NetState;
			if (state1 != null)
			{
				state1.RemoveTrade(this);
 
			}
			state1 = this.m_To.Mobile.NetState;
			if (state1 != null)
			{
				state1.RemoveTrade(this);
 
			}
			this.m_From.Container.Delete();
			this.m_To.Container.Delete();
 
		} 

		public void Update()
		{
				ArrayList list1;
			bool flag1;
			int num1;
			Item item1;
			int num2;
			Item item2;
			int num3;
			Item item3;
			int num4;
			Item item4;
			if (!this.m_Valid)
			{
				return; 
			}
			if (this.m_From.Accepted && this.m_To.Accepted)
			{
				list1 = this.m_From.Container.Items;
				flag1 = true;
				for (num1 = (list1.Count - 1); (flag1 && (num1 >= 0)); num1 = (num1 - 1))
				{
					if (num1 < list1.Count)
					{
						item1 = ((Item) list1[num1]);
						if (!item1.AllowSecureTrade(this.m_From.Mobile, this.m_To.Mobile, this.m_To.Mobile, 1))
						{
							flag1 = false;
 
						}
 
					}
 
				}
				list1 = this.m_To.Container.Items;
				for (num2 = (list1.Count - 1); (flag1 && (num2 >= 0)); num2 = (num2 - 1))
				{
					if (num2 < list1.Count)
					{
						item2 = ((Item) list1[num2]);
						if (!item2.AllowSecureTrade(this.m_To.Mobile, this.m_From.Mobile, this.m_From.Mobile, 1))
						{
							flag1 = false;
 
						}
 
					}
 
				}
				if (!flag1)
				{
					this.m_From.Accepted = false;
					this.m_To.Accepted = false;
					this.m_From.Mobile.Send(new UpdateSecureTrade(this.m_From.Container, this.m_From.Accepted, this.m_To.Accepted));
					this.m_To.Mobile.Send(new UpdateSecureTrade(this.m_To.Container, this.m_To.Accepted, this.m_From.Accepted));
					return; 
				}
				list1 = this.m_From.Container.Items;
				for (num3 = (list1.Count - 1); (num3 >= 0); num3 = (num3 - 1))
				{
					if (num3 < list1.Count)
					{
						item3 = ((Item) list1[num3]);
						item3.OnSecureTrade(this.m_From.Mobile, this.m_To.Mobile, this.m_To.Mobile, 1);
						if (!item3.Deleted)
						{
							this.m_To.Mobile.AddToBackpack(item3);
 
						}
 
					}
 
				}
				list1 = this.m_To.Container.Items;
				for (num4 = (list1.Count - 1); (num4 >= 0); num4 = (num4 - 1))
				{
					if (num4 < list1.Count)
					{
						item4 = ((Item) list1[num4]);
						item4.OnSecureTrade(this.m_To.Mobile, this.m_From.Mobile, this.m_From.Mobile, 1);
						if (!item4.Deleted)
						{
							this.m_From.Mobile.AddToBackpack(item4);
 
						}
 
					}
 
				}
				this.Close();
				return; 
			}
			this.m_From.Mobile.Send(new UpdateSecureTrade(this.m_From.Container, this.m_From.Accepted, this.m_To.Accepted));
			this.m_To.Mobile.Send(new UpdateSecureTrade(this.m_To.Container, this.m_To.Accepted, this.m_From.Accepted));
 
		}

		public SecureTradeInfo From { 
			get
			{
				return this.m_From;
			}
		}

		public SecureTradeInfo To { 
			get
			{
				return this.m_To;
			}
		}

		public bool Valid { 
			get
			{
				return this.m_Valid;
			}
		}
		
		private SecureTradeInfo m_From;
		private SecureTradeInfo m_To;
		private bool m_Valid;
	}

	public class SecureTradeInfo
	{
		public SecureTradeInfo(SecureTrade owner, Mobile m, SecureTradeContainer c) : base()
		{
			this.m_Owner = owner;
			this.m_Mobile = m;
			this.m_Container = c;
			this.m_Mobile.AddItem(this.m_Container);
		} 
		
		public bool Accepted { 
			get
			{
				return this.m_Accepted;
			}
			set
			{
				this.m_Accepted = value;
			}
		}

		public SecureTradeContainer Container { 
			get
			{
				return this.m_Container;
			}
		}

		public Mobile Mobile { 
			get
			{
				return this.m_Mobile;
			}
		}

		public SecureTrade Owner { 
			get
			{
				return this.m_Owner;
			}
		}

		private bool m_Accepted;
		private SecureTradeContainer m_Container;
		private Mobile m_Mobile;
		private SecureTrade m_Owner;
	}
}