using System;
using System.IO;

namespace Server
{
	public struct Tile
	{
		public Tile(short id, sbyte z)
		{
			this.m_ID = id;
			this.m_Z = z;
		} 

		public void Set(short id, sbyte z)
		{
			this.m_ID = id;
			this.m_Z = z;
		} 

		public int Height 
		{ 
			get
			{
				if (this.m_ID < 16384)
				{
					return 0; 
				}
				return TileData.ItemTable[(this.m_ID & 16383)].Height; 
			}
		}

		public int ID 
		{ 
			get
			{
				return this.m_ID;
			}
		}

		public bool Ignored 
		{ 
			get
			{
				if ((this.m_ID != 2) && (this.m_ID != 475))
				{
					if (this.m_ID >= 430)
					{
						return !(this.m_ID > 437); 
					}
					return false; 
				}
				return true;
			}
		}

		public int Z 
		{ 
			get
			{
				return this.m_Z;
			}
			set
			{
				this.m_Z = ((sbyte) value);
			}
		}
		
		internal short m_ID;
		internal sbyte m_Z;
	}

	public class TileList
	{
		public TileList() : base()
		{
			this.m_Tiles = new Tile[8];
			this.m_Count = 0;
		}

		public void Add(short id, sbyte z)
		{
			Tile[] array1;
			int num1;
			if ((this.m_Count + 1) > this.m_Tiles.Length)
			{
				array1 = this.m_Tiles;
				this.m_Tiles = new Tile[(array1.Length * 2)];
				for (num1 = 0; (num1 < array1.Length); num1 = (num1 + 1))
				{
					this.m_Tiles[num1] = array1[num1];
 
				}
 
			}
			this.m_Tiles[this.m_Count].m_ID = id;
			this.m_Tiles[this.m_Count].m_Z = z;
			this.m_Count = (this.m_Count + 1);
 
		} 

		public void AddRange(Tile[] tiles)
		{
			Tile[] array1;
			int num1;
			int num2;
			int num3;
			if ((this.m_Count + tiles.Length) > this.m_Tiles.Length)
			{
				array1 = this.m_Tiles;
				this.m_Tiles = new Tile[((this.m_Count + tiles.Length) * 2)];
				for (num1 = 0; (num1 < array1.Length); num1 = (num1 + 1))
				{
					this.m_Tiles[num1] = array1[num1];
 
				}
 
			}
			for (num2 = 0; (num2 < tiles.Length); num2 = (num2 + 1))
			{
				num3 = this.m_Count;
				this.m_Count = (this.m_Count + 1);
				this.m_Tiles[num3] = tiles[num2];
 
			}
 
		} 

		public Tile[] ToArray()
		{
			Tile[] array1;
			int num1;
			if (this.m_Count == 0)
			{
				return TileList.m_EmptyTiles; 
			}
			array1 = new Tile[this.m_Count];
			for (num1 = 0; (num1 < this.m_Count); num1 = (num1 + 1))
			{
				array1[num1] = this.m_Tiles[num1];
 
			}
			this.m_Count = 0;
			return array1; 
		} 

		public int Count 
		{ 
			get
			{
				return this.m_Count;
			}
		}
		
		private int m_Count;
		private static Tile[] m_EmptyTiles = new Tile[0];
		private Tile[] m_Tiles;
	}

	public struct StaticTile
	{
		public short m_Hue;
		public short m_ID;
		public byte m_X;
		public byte m_Y;
		public sbyte m_Z;
	}

	public class TileData
	{
		static TileData()
		{
			string text1;
			FileStream stream1;
			BinaryReader reader1;
			int num1;
			TileFlag flag1;
			int num2;
			TileFlag flag2;
			int num3;
			int num4;
			int num5;
			int num6;
			int num7;
			TileData.m_StringBuffer = new byte[20];
			text1 = Core.FindDataFile("TileData.mul");
			if (File.Exists(text1))
			{
				stream1 = new FileStream(text1, 3, 1, 1);
				try
				{
					reader1 = new BinaryReader(stream1);
					TileData.m_LandData = new LandData[16384];
					for (num1 = 0; (num1 < 16384); num1 = (num1 + 1))
					{
						if ((num1 & 31) == 0)
						{
							reader1.ReadInt32();
 
						}
						flag1 = reader1.ReadInt32();
						reader1.ReadInt16();
						TileData.m_LandData[num1]..ctor(TileData.ReadNameString(reader1), flag1);
 
					}
					TileData.m_ItemData = new ItemData[16384];
					for (num2 = 0; (num2 < 16384); num2 = (num2 + 1))
					{
						if ((num2 & 31) == 0)
						{
							reader1.ReadInt32();
 
						}
						flag2 = reader1.ReadInt32();
						num3 = reader1.ReadByte();
						num4 = reader1.ReadByte();
						reader1.ReadInt16();
						reader1.ReadByte();
						num5 = reader1.ReadByte();
						reader1.ReadInt32();
						reader1.ReadByte();
						num6 = reader1.ReadByte();
						num7 = reader1.ReadByte();
						TileData.m_ItemData[num2]..ctor(TileData.ReadNameString(reader1), flag2, num3, num4, num5, num6, num7);
 
					}
					goto L_0167;
 
				}
				finally
				{
					if (stream1 != null)
					{
						stream1.Dispose();
 
					}
 
				}
 
			}
			Console.WriteLine("TileData.mul was not found");
			Console.WriteLine("Make sure your Scripts/Misc/DataPath.cs is properly configured");
			Console.WriteLine("After pressing return an exception will be thrown and the server will terminate");
			throw new Exception(string.Format("TileData: {0} not found", text1));
 
			L_0167:
				return; 
		} 

		public TileData() : base()
		{
		}

		private static string ReadNameString(BinaryReader bin)
		{
			int num1;
			bin.Read(TileData.m_StringBuffer, 0, 20);
			for (num1 = 0; ((num1 < 20) && (TileData.m_StringBuffer[num1] != 0)); num1 = (num1 + 1))
			{
  
			}
			return Encoding.ASCII.GetString(TileData.m_StringBuffer, 0, num1); 
		} 
		
		public static ItemData[] ItemTable 
		{ 
			get
			{
				return TileData.m_ItemData;
			}
		}

		public static LandData[] LandTable 
		{ 
			get
			{
				return TileData.m_LandData;
			}
		}

		private static ItemData[] m_ItemData;
		private static LandData[] m_LandData;
		private static byte[] m_StringBuffer;
	}
}