using System;
using System.IO;
using System.Collections;
using System.Threading;

namespace Server
{
	public class Timer
	{
		public Timer(TimeSpan delay) : this(delay, TimeSpan.Zero, 1)
		{
		}

		public Timer(TimeSpan delay, TimeSpan interval) : this(delay, interval, 0)
		{
		}

		public Timer(TimeSpan delay, TimeSpan interval, int count) : base()
		{
			this.m_Delay = delay;
			this.m_Interval = interval;
			this.m_Count = count;
			if (this.DefRegCreation)
			{
				this.RegCreation();
			}
		}

		public static TimerPriority ComputePriority(TimeSpan ts)
		{
				if (TimeSpan.op_GreaterThanOrEqual(ts, TimeSpan.FromMinutes(1)))
		  {
			  return 6; 
		  }
			if (TimeSpan.op_GreaterThanOrEqual(ts, TimeSpan.FromSeconds(10)))
			{
				return 5; 
			}
			if (TimeSpan.op_GreaterThanOrEqual(ts, TimeSpan.FromSeconds(5)))
			{
				return 4; 
			}
			if (TimeSpan.op_GreaterThanOrEqual(ts, TimeSpan.FromSeconds(2.5)))
			{
				return 3; 
			}
			if (TimeSpan.op_GreaterThanOrEqual(ts, TimeSpan.FromSeconds(1)))
			{
				return 2; 
			}
			if (TimeSpan.op_GreaterThanOrEqual(ts, TimeSpan.FromSeconds(0.5)))
			{
				return 1; 
			}
			return 0; 
		} 

		public static Timer DelayCall(TimeSpan delay, TimerCallback callback)
		{
				return Timer.DelayCall(delay, TimeSpan.Zero, 1, callback); 
		} 

		public static Timer DelayCall(TimeSpan delay, TimerStateCallback callback, object state)
		{
				return Timer.DelayCall(delay, TimeSpan.Zero, 1, callback, state); 
		} 

		public static Timer DelayCall(TimeSpan delay, TimeSpan interval, int count, TimerCallback callback)
		{
				Timer timer1;
			timer1 = new DelayCallTimer(delay, interval, count, callback);
			if (count == 1)
			{
				timer1.Priority = Timer.ComputePriority(delay);
 
			}
			else
			{
					timer1.Priority = Timer.ComputePriority(interval);
 
			}
			timer1.Start();
			return timer1; 
		} 

		public static Timer DelayCall(TimeSpan delay, TimeSpan interval, int count, TimerStateCallback callback, object state)
		{
				Timer timer1;
			timer1 = new DelayStateCallTimer(delay, interval, count, callback, state);
			if (count == 1)
			{
				timer1.Priority = Timer.ComputePriority(delay);
 
			}
			else
			{
					timer1.Priority = Timer.ComputePriority(interval);
 
			}
			timer1.Start();
			return timer1; 
		} 

		public static Timer DelayCall(TimeSpan delay, TimeSpan interval, TimerCallback callback)
		{
				return Timer.DelayCall(delay, interval, 0, callback); 
		} 

		public static Timer DelayCall(TimeSpan delay, TimeSpan interval, TimerStateCallback callback, object state)
		{
				return Timer.DelayCall(delay, interval, 0, callback, state); 
		} 

		public static void DumpInfo(TextWriter tw)
		{
				TimerThread.DumpInfo(tw);
 
		} 

		private static string FormatDelegate(Delegate callback)
		{
				if (callback == null)
		  {
			  return "null"; 
		  }
			return string.Format("{0}.{1}", callback.Method.DeclaringType.FullName, callback.Method.Name); 
		} 

		public TimerProfile GetProfile()
		{
				string text1;
			TimerProfile profile1;
			if (!Core.Profiling)
			{
				return null; 
			}
			text1 = this.ToString();
			if (text1 == null)
			{
				text1 = "null";
 
			}
			profile1 = ((TimerProfile) Timer.m_Profiles[text1]);
			if (profile1 == null)
			{
				profile1 = new TimerProfile();
				Timer.m_Profiles[text1] = new TimerProfile();
 
			}
			return profile1; 
		} 

		protected virtual void OnTick()
		{  
		} 

		public virtual void RegCreation()
		{
				TimerProfile profile1;
			profile1 = this.GetProfile();
			if (profile1 != null)
			{
				profile1.RegCreation();
 
			}
 
		} 

		public static void Slice()
		{
				int num1;
			DateTime time1;
			Timer timer1;
			TimerProfile profile1;
			Queue queue1;
			queue1 = Timer.m_Queue;
			Monitor.Enter(Timer.m_Queue);
			try
			{
				Timer.m_QueueCountAtSlice = Timer.m_Queue.Count;
				num1 = 0;
				time1 = DateTime.MinValue;
				while (((num1 < Timer.m_BreakCount) && (Timer.m_Queue.Count != 0)))
				{
					timer1 = ((Timer) Timer.m_Queue.Dequeue());
					profile1 = timer1.GetProfile();
					if (profile1 != null)
					{
						time1 = DateTime.Now;
 
					}
					timer1.OnTick();
					timer1.m_Queued = false;
					num1 = (num1 + 1);
					if (profile1 != null)
					{
						profile1.RegTicked(DateTime.op_Subtraction(DateTime.Now, time1));
 
					}
 
				}
 
			}
			finally
			{
				Monitor.Exit(queue1);
 
			}
 
		} 

		public void Start()
		{
				TimerProfile profile1;
			if (!this.m_Running)
			{
				this.m_Running = true;
				TimerThread.AddTimer(this);
				profile1 = this.GetProfile();
				if (profile1 != null)
				{
					profile1.RegStart();
 
				}
 
			}
 
		} 

		public void Stop()
		{
				TimerProfile profile1;
			if (this.m_Running)
			{
				this.m_Running = false;
				TimerThread.RemoveTimer(this);
				profile1 = this.GetProfile();
				if (profile1 != null)
				{
					profile1.RegStopped();
 
				}
 
			}
 
		} 

		public override string ToString()
		{
				return base.GetType().FullName; 
		} 

		public static int BreakCount { 
			get
			{
				return Timer.m_BreakCount;
			}
			set
			{
				Timer.m_BreakCount = value;
			}
		}

		public virtual bool DefRegCreation { 
			get
			{
				return true;
			}
		}

		public TimeSpan Delay 
		{ 
			get
			{
				return this.m_Delay;
			}
			set
			{
				this.m_Delay = value;
			}
		}

		public TimeSpan Interval 
		{ 
			get
			{
				return this.m_Interval;
			}
			set
			{
				this.m_Interval = value;
			}
		}

		public TimerPriority Priority 
		{ 
			get
			{
				return this.m_Priority;
			}
			set
			{
				if (this.m_Priority != value)
				{
					this.m_Priority = value;
					if (this.m_Running)
					{
						TimerThread.PriorityChange(this, this.m_Priority);
					}
				}
			}
		}

		public static Hashtable Profiles 
		{ 
			get
			{
				return Timer.m_Profiles;
			}
		}

		public static Queue Queue 
		{ 
			get
			{
				return Timer.m_Queue;
			}
		}

		public static int QueueCountAtSlice 
		{ 
			get
			{
				return Timer.m_QueueCountAtSlice;
			}
		}

		public bool Running 
		{ 
			get
			{
				return this.m_Running;
			}
			set
			{
				if (value)
				{
					this.Start();
					return; 
				}
				this.Stop();
			}
		}
		
		private static int m_BreakCount = 20000;
		private int m_Count;
		private TimeSpan m_Delay;
		private int m_Index;
		private TimeSpan m_Interval;
		private ArrayList m_List;
		private DateTime m_Next;
		private TimerPriority m_Priority;
		private static Hashtable m_Profiles = new Hashtable();
		private static Queue m_Queue = new Queue();
		private static int m_QueueCountAtSlice;
		private bool m_Queued;
		private bool m_Running;

		private class DelayCallTimer : Timer
		{
			public DelayCallTimer(TimeSpan delay, TimeSpan interval, int count, TimerCallback callback) : base(delay, interval, count)
			{
				this.m_Callback = callback;
				base.RegCreation();
			}

			protected override void OnTick()
			{
					if (this.m_Callback != null)
			  {
				  this.m_Callback.Invoke();
 
			  }
 
			} 

			public override string ToString()
			{
					return string.Format("DelayCallTimer[{0}]", Timer.FormatDelegate(this.m_Callback)); 
			} 

			public TimerCallback Callback { 
				get
				{
					return this.m_Callback;
				}
			}

			public override bool DefRegCreation { 
				get
				{
					return false;
				}
			}

			private TimerCallback m_Callback;
		}

		private class DelayStateCallTimer : Timer
		{
			public DelayStateCallTimer(TimeSpan delay, TimeSpan interval, int count, TimerStateCallback callback, object state)
				: base(delay, interval, count)
			{
				this.m_Callback = callback;
				this.m_State = state;
				base.RegCreation();
			}

			protected override void OnTick()
			{
					if (this.m_Callback != null)
			  {
				  this.m_Callback.Invoke(this.m_State);
 
			  }
 
			} 

			public override string ToString()
			{
					return string.Format("DelayStateCall[{0}]", Timer.FormatDelegate(this.m_Callback)); 
			} 
			
			public TimerStateCallback Callback { 
				get
				{
					return this.m_Callback;
				}
			}

			public override bool DefRegCreation { 
				get
				{
					return false;
				}
			}
			
			private TimerStateCallback m_Callback;
			private object m_State;
		}

		public class TimerThread
		{
			static TimerThread()
			{
				TimeSpan[] array1;
				ArrayList[] array2;
				TimerThread.m_ChangeQueue = Queue.Synchronized(new Queue());
				TimerThread.m_NextPriorities = new DateTime[8];
				array1 = new TimeSpan[8];
				array1[0] = TimeSpan.Zero;
				array1[1] = TimeSpan.FromMilliseconds(10);
				array1[2] = TimeSpan.FromMilliseconds(25);
				array1[3] = TimeSpan.FromMilliseconds(50);
				array1[4] = TimeSpan.FromMilliseconds(250);
				array1[5] = TimeSpan.FromSeconds(1);
				array1[6] = TimeSpan.FromSeconds(5);
				array1[7] = TimeSpan.FromMinutes(1);
				TimerThread.m_PriorityDelays = array1;
				array2 = new ArrayList[8];
				array2[0] = new ArrayList();
				array2[1] = new ArrayList();
				array2[2] = new ArrayList();
				array2[3] = new ArrayList();
				array2[4] = new ArrayList();
				array2[5] = new ArrayList();
				array2[6] = new ArrayList();
				array2[7] = new ArrayList();
				TimerThread.m_Timers = array2;
			}

			public TimerThread() : base()
			{
			}

			public static void AddTimer(Timer t)
			{
					TimerThread.Change(t, t.Priority, 1);
 
			} 

			public static void Change(Timer t, int newIndex, bool isAdd)
			{
					TimerThread.m_ChangeQueue.Enqueue(TimerChangeEntry.GetInstance(t, newIndex, isAdd));
 
			} 

			public static void DumpInfo(TextWriter tw)
			{
					int num1;
				Hashtable hashtable1;
				int num2;
				Timer timer1;
				string text1;
				ArrayList list1;
				DictionaryEntry entry1;
				string text2;
				ArrayList list2;
				IDictionaryEnumerator enumerator1;
				IDisposable disposable1;
				for (num1 = 0; (num1 < 8); num1 = (num1 + 1))
				{
					tw.WriteLine("Priority: {0}", num1);
					tw.WriteLine();
					hashtable1 = new Hashtable();
					for (num2 = 0; (num2 < TimerThread.m_Timers[num1].Count); num2 = (num2 + 1))
					{
						timer1 = ((Timer) TimerThread.m_Timers[num1][num2]);
						text1 = timer1.ToString();
						list1 = ((ArrayList) hashtable1[text1]);
						if (list1 == null)
						{
							list1 = new ArrayList();
							hashtable1[text1] = new ArrayList();
 
						}
						list1.Add(timer1);
 
					}
					enumerator1 = hashtable1.GetEnumerator();
					try
					{
						while (enumerator1.MoveNext())
						{
							entry1 = ((DictionaryEntry) enumerator1.Current);
							text2 = ((string) entry1.Key);
							list2 = ((ArrayList) entry1.Value);
							tw.WriteLine("Type: {0}; Count: {1}; Percent: {2}%", text2, list2.Count, ((int) (100 * (((double) list2.Count) / ((double) TimerThread.m_Timers[num1].Count)))));
 
						}
 
					}
					finally
					{
						disposable1 = (enumerator1 as IDisposable);
						if (disposable1 != null)
						{
							disposable1.Dispose();
 
						}
 
					}
					tw.WriteLine();
					tw.WriteLine();
 
				}
 
			} 

			public static void PriorityChange(Timer t, int newPrio)
			{
					TimerThread.Change(t, newPrio, 0);
 
			} 

			private static void ProcessChangeQueue()
			{
					TimerChangeEntry entry1;
				Timer timer1;
				int num1;
				while ((TimerThread.m_ChangeQueue.Count > 0))
				{
					entry1 = ((TimerChangeEntry) TimerThread.m_ChangeQueue.Dequeue());
					timer1 = entry1.m_Timer;
					num1 = entry1.m_NewIndex;
					if (timer1.m_List != null)
					{
						timer1.m_List.Remove(timer1);
 
					}
					if (entry1.m_IsAdd)
					{
						timer1.m_Next = DateTime.op_Addition(DateTime.Now, timer1.m_Delay);
						timer1.m_Index = 0;
 
					}
					if (num1 >= 0)
					{
						timer1.m_List = TimerThread.m_Timers[num1];
						timer1.m_List.Add(timer1);
 
					}
					else
					{
							timer1.m_List = null;
 
					}
 
				}
 
			} 

			public static void RemoveTimer(Timer t)
			{
					TimerThread.Change(t, -1, 0);
 
			} 

			public void TimerMain()
			{
					DateTime time1;
				int num1;
				int num2;
				Timer timer1;
				Queue queue1;
				int num3;
				goto L_011C;
 
				L_0005:
					Thread.Sleep(10);
				TimerThread.ProcessChangeQueue();
				num1 = 0;
				goto L_010F;
 
				L_0018:
					time1 = DateTime.Now;
				if (DateTime.op_LessThan(time1, TimerThread.m_NextPriorities[num1]))
				{
					goto L_011C;
 
				}
				TimerThread.m_NextPriorities[num1] = DateTime.op_Addition(time1, TimerThread.m_PriorityDelays[num1]);
				num2 = 0;
				goto L_00F9;
 
				L_0066:
					timer1 = ((Timer) TimerThread.m_Timers[num1][num2]);
				if (!timer1.m_Queued && DateTime.op_GreaterThan(time1, timer1.m_Next))
				{
					timer1.m_Queued = true;
					queue1 = Timer.m_Queue;
					Monitor.Enter(Timer.m_Queue);
					try
					{
						Timer.m_Queue.Enqueue(timer1);
 
					}
					finally
					{
						Monitor.Exit(queue1);
 
					}
					if (timer1.m_Count != 0)
					{
						num3 = (timer1.m_Index + 1);
						timer1.m_Index = (timer1.m_Index + 1);
						if (num3 >= timer1.m_Count)
						{
							timer1.Stop();
							goto L_00F5;
 
						}
 
					}
					timer1.m_Next = DateTime.op_Addition(time1, timer1.m_Interval);
 
				}
 
				L_00F5:
					num2 = (num2 + 1);
 
				L_00F9:
					if (num2 < TimerThread.m_Timers[num1].Count)
					{
						goto L_0066;
 
					}
				num1 = (num1 + 1);
 
				L_010F:
					if (num1 < TimerThread.m_Timers.Length)
					{
						goto L_0018;
 
					}
 
				L_011C:
					if (!Core.Closing)
					{
						goto L_0005;
 
					}
 
			}
			
			private static Queue m_ChangeQueue;
			private static DateTime[] m_NextPriorities;
			private static TimeSpan[] m_PriorityDelays;
			private static ArrayList[] m_Timers;

			private class TimerChangeEntry
			{
				private TimerChangeEntry(Timer t, int newIndex, bool isAdd) : base()
				{
					this.m_Timer = t;
					this.m_NewIndex = newIndex;
					this.m_IsAdd = isAdd;
				}

				public void Free()
				{
					TimerChangeEntry.m_InstancePool.Enqueue(this);
				}

				public static TimerChangeEntry GetInstance(Timer t, int newIndex, bool isAdd)
				{
						TimerChangeEntry entry1;
					if (TimerChangeEntry.m_InstancePool.Count > 0)
					{
						entry1 = ((TimerChangeEntry) TimerChangeEntry.m_InstancePool.Dequeue());
						entry1.m_Timer = t;
						entry1.m_NewIndex = newIndex;
						entry1.m_IsAdd = isAdd;
 
					}
					else
					{
							entry1 = new TimerChangeEntry(t, newIndex, isAdd);
 
					}
					return entry1; 
				} 
			
				private static Queue m_InstancePool = new Queue();
				public bool m_IsAdd;
				public int m_NewIndex;
				public Timer m_Timer;
			}
		}
	}

	public class TimerProfile
	{
		public TimerProfile() : base()
		{
		}

		public void RegCreation()
		{
			this.m_Created = (this.m_Created + 1);
		}

		public void RegStart()
		{
			this.m_Started = (this.m_Started + 1);
		}

		public void RegStopped()
		{
			this.m_Stopped = (this.m_Stopped + 1);
		}

		public void RegTicked(TimeSpan procTime)
		{
			this.m_Ticked = (this.m_Ticked + 1);
			this.m_TotalProcTime = TimeSpan.op_Addition(this.m_TotalProcTime, procTime);
		} 
		
		public TimeSpan AverageProcTime { 
			get
			{
				if (this.m_Ticked == 0)
				{
					return TimeSpan.Zero; 
				}
				return TimeSpan.FromTicks((this.m_TotalProcTime.Ticks / ((long) this.m_Ticked))); 
			}
		}

		public int Created 
		{ 
			get
			{
				return this.m_Created;
			}
		}

		public int Started 
		{ 
			get
			{
				return this.m_Started;
			}
		}

		public int Stopped 
		{ 
			get
			{
				return this.m_Stopped;
			}
		}

		public int Ticked 
		{ 
			get
			{
				return this.m_Ticked;
			}
		}

		public TimeSpan TotalProcTime 
		{ 
			get
			{
				return this.m_TotalProcTime;
			}
		}

		private int m_Created;
		private int m_Started;
		private int m_Stopped;
		private int m_Ticked;
		private TimeSpan m_TotalProcTime;
	}
}
